/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

/* Amulets are now nethack 3.3.0 compliant */

struct S_Amulet {
    char * name;
    char * desc;
    int prob;
} SAmulets[] = {

#define AMULET(name,desc,power,prob) \
	{name, desc, prob}

AMULET("amulet of ESP",           "circular",		TELEPAT,    175),
AMULET("amulet of life saving",   "spherical",		LIFESAVED,   75),
AMULET("amulet of strangulation", "oval",		STRANGLED,  135),
AMULET("amulet of restful sleep", "triangular",		SLEEPING,   135),
AMULET("amulet versus poison",    "pyramidal",		POISON_RES, 165),
AMULET("amulet of change",        "square",		0,          130),
                                                	/* POLYMORPH */
AMULET("amulet of unchanging",    "concave",		UNCHANGING,      45),
AMULET("amulet of reflection",    "hexagonal",		REFLECTING,  75),
AMULET("amulet of magical breathing", "octagonal",	MAGICAL_BREATHING, 65),
AMULET("cheap plastic imitation of the Amulet of Yendor",
				  "Amulet of Yendor",	0,	0),
AMULET("Amulet of Yendor",	  "Amulet of Yendor",	0,	0)
#undef AMULET
};

#define AMULETDESCS	11

static bool amuletKnown[AMULETDESCS][MAXPLAYERS];
static char amuletRandDesc[AMULETDESCS][MAXPLAYERS];

void initAmulets() {
    for (int x = 0; x < MAXPLAYERS; x++)
        quitAmulets(x);
}

void loadAmulets(ID user) {
}

void saveAmulets(ID user) {
    quitAmulets(user);
}

void quitAmulets(ID user) {
    for (int y = 0; y < AMULETDESCS; y++) {
        amuletKnown[y][user] = false;
        amuletRandDesc[y][user] = y;
    }
    for (int y = 0; y < AMULETDESCS-2; y++) {
        int r = rn2(AMULETDESCS-2);
        char temp = amuletRandDesc[y][user];
        amuletRandDesc[y][user] = amuletRandDesc[r][user];
        amuletRandDesc[r][user] = temp;
    }
}

CAmulet::CAmulet() : CObject(Amulet, '\"', 20, 150) {
    polymorph();
}

CAmulet::CAmulet(EAmuletType t) : CObject(Amulet, '\"', 20, 150) {
    amuletType = t;
    if (amuletType == AmYendor) setCost(3500);
    else if (amuletType == AmImitationYendor) setCost(0);
    int r = rn2(100);
    if (amuletType == AmChange || amuletType == AmRestfulSleep ||
        amuletType == AmStrangulation) {
        if (r < 90) curseIt();
    } else {
        if (r < 5) blessIt();
        else if (r >= 95) curseIt();
    }
}

void CAmulet::polymorph() {
    amuletType = (EAmuletType)0;
    for(int tprob = rnd(1000);
        (tprob -= SAmulets[amuletType].prob) > 0;
        amuletType = (EAmuletType)(amuletType+1));
    int r = rn2(100);
    if (amuletType == AmChange || amuletType == AmRestfulSleep ||
        amuletType == AmStrangulation) {
        if (r < 90) curseIt();
    } else {
        if (r < 5) blessIt();
        else if (r >= 95) curseIt();
    }
    setName("");
}

bool CAmulet::canEat() {
    if (amuletType == AmChange || amuletType == AmESP ||
        amuletType == AmMagicalBreathing || amuletType == AmRestfulSleep ||
        amuletType == AmStrangulation || amuletType == AmVersusPoison)
        return true;
    return false;
}

string CAmulet::getDescription(ID user) {
    string s;
    if (getNumber() > 1)
        s = string(number);
    else {
        if (isBlessed() || isCursed() || !isKnown(user))
            s = "a ";
        else s = "an ";
    }
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
    }
    if (isKnown(user)) {
        s += "amulet";
        if (getNumber() > 1) s += "s";
        s += " of ";
        s += SAmulets[amuletType].name;
    } else {
        s += SAmulets[amuletRandDesc[amuletType][user]].desc;
        s += " amulet";
        if (getNumber() > 1) s += "s";
    }
    return s;
}

int CAmulet::getColor(ID user) {
    return HI_METAL;
}

bool CAmulet::isKnown(ID user) {
    return amuletKnown[amuletType][user];
}

void CAmulet::identify(ID user) {
    amuletKnown[amuletType][user] = true;
    BCidentify(user);
}

/* Keep amulets isolated */
bool CAmulet::operator== (CAmulet *o) {
    return false;
}
