/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

/* Armor is now nethack 3.3.0 compliant */

struct S_Armor {
    char *name;
    char *desc;
    bool kn, magic, block;
    int prob;
    int delay;
    int weight;
    int cost;
    int ac;
    int can;
    int subtype;
    int metal;
    int color;
} SArmor[] = {

#define ARM_HELM	1
#define ARM_CLOAK	2
#define ARM_SHIELD	3
#define ARM_GLOVES	4
#define ARM_BOOTS	5
#define ARM_SUIT	6
#define ARM_SHIRT	7
#define DRAGON_HIDE	8

#define LEATHER		1
#define IRON		2
#define WOOD		3
#define CLOTH		4
#define SILVER		5
#define GLASS		6
#define COPPER		7
#define MITHRIL		8

#define ARMOR(name,desc,kn,mgc,blk,power,prob,delay,wt,cost,ac,can,sub,metal,c) \
	{name,desc,kn,mgc,blk,prob,delay,wt,cost,ac,can,ARM_SUIT,metal,c}
#define SHIRT(name,desc,kn,mgc,blk,power,prob,delay,wt,cost,ac,can,sub,metal,c) \
	{name,desc,kn,mgc,blk,prob,delay,wt,cost,ac,can,ARM_SHIRT,metal,c}
#define HELM(name,desc,kn,mgc,power,prob,delay,wt,cost,ac,can,metal,c) \
	{name,desc,kn,mgc,0,prob,delay,wt,cost,ac,can,ARM_HELM,metal,c}
#define CLOAK(name,desc,kn,mgc,power,prob,delay,wt,cost,ac,can,metal,c) \
	{name,desc,kn,mgc,0,prob,delay,wt,cost,ac,can,ARM_CLOAK,metal,c}
#define SHIELD(name,desc,kn,mgc,blk,power,prob,delay,wt,cost,ac,can,metal,c) \
	{name,desc,kn,mgc,blk,prob,delay,wt,cost,ac,can,ARM_SHIELD,metal,c}
#define GLOVES(name,desc,kn,mgc,power,prob,delay,wt,cost,ac,can,metal,c) \
	{name,desc,kn,mgc,0,prob,delay,wt,cost,ac,can,ARM_GLOVES,metal,c}
#define BOOTS(name,desc,kn,mgc,power,prob,delay,wt,cost,ac,can,metal,c) \
	{name,desc,kn,mgc,0,prob,delay,wt,cost,ac,can,ARM_BOOTS,metal,c}

/* armor ... */
/* IRON denotes ferrous metals, including steel.
 * Only IRON weapons and armor can rust.
 * Only COPPER (including brass) corrodes.
 * Some creatures are vulnerable to SILVER.
 */

/* helmets */
HELM("elven leather helm", "leather hat",
		0, 0,  0,	6, 1,  3,   8, 9, 0, LEATHER, HI_LEATHER),
HELM("orcish helm", "iron skull cap",
		0, 0,  0,	6, 1, 30,  10, 9, 0, IRON, CLR_BLACK),
HELM("dwarvish iron helm", "hard hat",
		0, 0,  0,	6, 1, 40,  20, 8, 0, IRON, HI_METAL),
HELM("fedora", (char *)0,
		1, 0,  0,	0, 0,  3,   1,10, 0, CLOTH, CLR_BROWN),
HELM("cornuthaum", "conical hat",
		0, 1,  CLAIRVOYANT,
				3, 1,  4,  80,10, 2, CLOTH, CLR_BLUE),
HELM("dunce cap", "conical hat",
		0, 1,  0,	3, 1,  4,   1,10, 0, CLOTH, CLR_BLUE),
HELM("dented pot", (char *)0,
		1, 0,  0,	2, 0, 10,   8, 9, 0, IRON, CLR_BLACK),
/* With shuffled appearances... */
HELM("helmet", "plumed helmet",
		0, 0,  0,      10, 1, 30,  10, 9, 0, IRON, HI_METAL),
HELM("helm of brilliance", "etched helmet",
		0, 1,  0,	6, 1, 50,  50, 9, 0, IRON, CLR_GREEN),
HELM("helm of opposite alignment", "crested helmet",
		0, 1,  0,	6, 1, 50,  50, 9, 0, IRON, HI_METAL),
HELM("helm of telepathy", "visored helmet",
		0, 1,  TELEPAT, 2, 1, 50,  50, 9, 0, IRON, HI_METAL),
#define HELMDESC	11

/* suits of armor */
/*
 * There is code in polyself.c that assumes (1) and (2).
 * There is code in objnam.c, mon.c, read.c that assumes (2).
 *
 *	(1) The dragon scale mails and the dragon scales are together.
 *	(2) That the order of the dragon scale mail and dragon scales is the
 *	    the same defined in monst.c.
 */
#define DRGN_ARMR(name,power,cost,ac,color) \
	ARMOR(name,(char *)0,1,0,1,power,0,5,40,cost,ac,0,ARM_SUIT,DRAGON_HIDE,color)
DRGN_ARMR("gray dragon scale mail",   ANTIMAGIC,  1200, 1, CLR_GRAY),
DRGN_ARMR("silver dragon scale mail", REFLECTING, 1200, 1, SILVER),
DRGN_ARMR("shimmering dragon scale mail", DISPLACED, 1200, 1, CLR_CYAN),
DRGN_ARMR("red dragon scale mail",    FIRE_RES,    900, 1, CLR_RED),
DRGN_ARMR("white dragon scale mail",  COLD_RES,    900, 1, CLR_WHITE),
DRGN_ARMR("orange dragon scale mail", SLEEP_RES,   900, 1, CLR_ORANGE),
DRGN_ARMR("black dragon scale mail",  DISINT_RES, 1200, 1, CLR_BLACK),
DRGN_ARMR("blue dragon scale mail",   SHOCK_RES,   900, 1, CLR_BLUE),
DRGN_ARMR("green dragon scale mail",  POISON_RES,  900, 1, CLR_GREEN),
DRGN_ARMR("yellow dragon scale mail", 0,           900, 1, CLR_YELLOW),

/* For now, only dragons leave these. */
DRGN_ARMR("gray dragon scales",   ANTIMAGIC,  700, 7, CLR_GRAY),
DRGN_ARMR("silver dragon scales", REFLECTING, 700, 7, SILVER),
DRGN_ARMR("shimmering dragon scales", DISPLACED,  700, 7, CLR_CYAN),
DRGN_ARMR("red dragon scales",    FIRE_RES,   500, 7, CLR_RED),
DRGN_ARMR("white dragon scales",  COLD_RES,   500, 7, CLR_WHITE),
DRGN_ARMR("orange dragon scales", SLEEP_RES,  500, 7, CLR_ORANGE),
DRGN_ARMR("black dragon scales",  DISINT_RES, 700, 7, CLR_BLACK),
DRGN_ARMR("blue dragon scales",   SHOCK_RES,  500, 7, CLR_BLUE),
DRGN_ARMR("green dragon scales",  POISON_RES, 500, 7, CLR_GREEN),
DRGN_ARMR("yellow dragon scales", 0,          500, 7, CLR_YELLOW),
#undef DRGN_ARMR
#define DRGNDESC	20

ARMOR("plate mail", (char *)0,
	1, 0, 1, 0,	44, 5, 450, 600,  3, 2, ARM_SUIT, IRON, HI_METAL),
ARMOR("crystal plate mail", (char *)0,
	1, 0, 1, 0,	10, 5, 450, 820,  3, 2, ARM_SUIT, GLASS, CLR_WHITE),
ARMOR("bronze plate mail", (char *)0,
	1, 0, 1, 0,	25, 5, 450, 400,  4, 0, ARM_SUIT, COPPER, HI_COPPER),
ARMOR("splint mail", (char *)0,
	1, 0, 1, 0,	62, 5, 400,  80,  4, 1, ARM_SUIT, IRON, HI_METAL),
ARMOR("banded mail", (char *)0,
	1, 0, 1, 0,	72, 5, 350,  90,  4, 0, ARM_SUIT, IRON, HI_METAL),
ARMOR("dwarvish mithril-coat", (char *)0,
	1, 0, 0, 0,	10, 1, 150, 240,  4, 3, ARM_SUIT, MITHRIL, HI_METAL),
ARMOR("elven mithril-coat", (char *)0,
	1, 0, 0, 0,	15, 1, 150, 240,  5, 3, ARM_SUIT, MITHRIL, HI_METAL),
ARMOR("chain mail", (char *)0,
	1, 0, 0, 0,	72, 5, 300,  75,  5, 1, ARM_SUIT, IRON, HI_METAL),
ARMOR("orcish chain mail", "crude chain mail",
	0, 0, 0, 0,	20, 5, 300,  75,  6, 1, ARM_SUIT, IRON, CLR_BLACK),
ARMOR("scale mail", (char *)0,
	1, 0, 0, 0,	72, 5, 250,  45,  6, 0, ARM_SUIT, IRON, HI_METAL),
ARMOR("studded leather armor", (char *)0,
	1, 0, 0, 0,	72, 3, 200,  15,  7, 1, ARM_SUIT, LEATHER, HI_LEATHER),
ARMOR("ring mail", (char *)0,
	1, 0, 0, 0,	72, 5, 250, 100,  7, 0, ARM_SUIT, IRON, HI_METAL),
ARMOR("orcish ring mail", "crude ring mail",
	0, 0, 0, 0,	20, 5, 250,  80,  8, 1, ARM_SUIT, IRON, CLR_BLACK),
ARMOR("leather armor", (char *)0,
	1, 0, 0, 0,	82, 3, 150,   5,  8, 0, ARM_SUIT, LEATHER, HI_LEATHER),
ARMOR("leather jacket", (char *)0,
	1, 0, 0, 0,	12, 0,	30,  10,  9, 0, ARM_SUIT, LEATHER, CLR_BLACK),
#define ARMDESC		15

/* shirts */
SHIRT("Hawaiian shirt", (char *)0,
	1, 0, 0, 0,	 8, 0,	 5,   3, 10, 0, ARM_SHIRT, CLOTH, CLR_MAGENTA),
SHIRT("T-shirt", (char *)0,
	1, 0, 0, 0,	 2, 0,	 5,   2, 10, 0, ARM_SHIRT, CLOTH, CLR_WHITE),
#define SHIRTDESC	2

/* cloaks */
/*  'cope' is not a spelling mistake... leave it be */
CLOAK("mummy wrapping", (char *)0,
		1, 0,	0,	    0, 0,  3,  2, 10, 1, CLOTH, CLR_GRAY),
CLOAK("elven cloak", "faded pall",
		0, 1,	STEALTH,   10, 0, 10, 60,  9, 3, CLOTH, CLR_BLACK),
CLOAK("orcish cloak", "coarse mantelet",
		0, 0,	0,	   10, 0, 10, 40, 10, 2, CLOTH, CLR_BLACK),
CLOAK("dwarvish cloak", "hooded cloak",
		0, 0,	0,	   10, 0, 10, 50, 10, 2, CLOTH, HI_CLOTH),
CLOAK("oilskin cloak", "slippery cloak",
		0, 0,	0,	   10, 0, 10, 50,  9, 3, CLOTH, HI_CLOTH),
CLOAK("robe", (char *)0,
		1, 1,	0,	    0, 0, 15, 50,  8, 3, CLOTH, CLR_RED),
CLOAK("alchemy smock", "apron",
		0, 1,	POISON_RES, 9, 0, 10, 50,  9, 1, CLOTH, CLR_WHITE),
/* With shuffled appearances... */
CLOAK("cloak of protection", "tattered cape",
		0, 1,	PROTECTION,10, 0, 10, 50,  7, 3, CLOTH, HI_CLOTH),
CLOAK("cloak of invisibility", "opera cloak",
		0, 1,	INVIS,	   11, 0, 10, 60,  9, 2, CLOTH, CLR_BRIGHT_MAGENTA),
CLOAK("cloak of magic resistance", "ornamental cope",
		0, 1,	ANTIMAGIC,  2, 0, 10, 60,  9, 3, CLOTH, CLR_WHITE),
CLOAK("cloak of displacement", "piece of cloth",
		0, 1,	DISPLACED, 11, 0, 10, 50,  9, 2, CLOTH, HI_CLOTH),
#define CLOAKDESC	11

/* shields */
SHIELD("small shield", (char *)0,
		1, 0, 0, 0,	     6, 0, 30,	3,  9, 0, WOOD, HI_WOOD),
SHIELD("elven shield", "blue and green shield",
		0, 0, 0, 0,	     2, 0, 50,	7,  8, 0, IRON, CLR_GREEN),
SHIELD("Uruk-hai shield", "white-handed shield",
		0, 0, 0, 0,	     2, 0, 50,	7,  9, 0, IRON, HI_METAL),
SHIELD("orcish shield", "red-eyed shield",
		0, 0, 0, 0,	     2, 0, 50,	7,  9, 0, IRON, CLR_RED),
SHIELD("large shield", (char *)0,
		1, 0, 1, 0,	     7, 0,100, 10,  8, 0, IRON, HI_METAL),
SHIELD("dwarvish roundshield", "large round shield",
		0, 0, 0, 0,	     4, 0,100, 10,  8, 0, IRON, HI_METAL),
SHIELD("shield of reflection", "polished silver shield",
		0, 1, 0, REFLECTING, 3, 0, 50, 50,  8, 0, SILVER, HI_SILVER),
#define SHIELDDESC	7

/* gloves */
/* these have their color but not material shuffled, so the IRON must stay
 * CLR_BROWN (== HI_LEATHER)
 */
GLOVES("leather gloves", "old gloves",
		0, 0,  0,	  16, 1, 10,  8,  9, 0, LEATHER, HI_LEATHER),
GLOVES("gauntlets of fumbling", "padded gloves",
		0, 1,  FUMBLING,   8, 1, 10, 50,  9, 0, LEATHER, HI_LEATHER),
GLOVES("gauntlets of power", "riding gloves",
		0, 1,  0,	   8, 1, 30, 50,  9, 0, IRON, CLR_BROWN),
GLOVES("gauntlets of dexterity", "fencing gloves",
		0, 1,  0,	   8, 1, 10, 50,  9, 0, LEATHER, HI_LEATHER),
#define GLOVEDESC	4

/* boots */
BOOTS("low boots", "walking shoes",
		0, 0,  0,	  25, 2, 10,  8,  9, 0, LEATHER, HI_LEATHER),
BOOTS("iron shoes", "hard shoes",
		0, 0,  0,	   7, 2, 50, 16,  8, 0, IRON, HI_METAL),
BOOTS("high boots", "jackboots",
		0, 0,  0,	  15, 2, 20, 12,  8, 0, LEATHER, HI_LEATHER),
BOOTS("speed boots", "combat boots",
		0, 1,  FAST,	  12, 2, 20, 50,  9, 0, LEATHER, HI_LEATHER),
/* With shuffled appearances... */
BOOTS("water walking boots", "jungle boots",
		0, 1,  WWALKING,  12, 2, 20, 50,  9, 0, LEATHER, HI_LEATHER),
BOOTS("jumping boots", "hiking boots",
		0, 1,  JUMPING,   12, 2, 20, 50,  9, 0, LEATHER, HI_LEATHER),
BOOTS("elven boots", "mud boots",
		0, 1,  STEALTH,   12, 2, 15,  8,  9, 0, LEATHER, HI_LEATHER),
BOOTS("kicking boots", "buckled boots",
		0, 1,  0,         12, 2, 15,  8,  9, 0, IRON, HI_METAL),
BOOTS("fumble boots", "riding boots",
		0, 1,  FUMBLING,  12, 2, 20, 30,  9, 0, LEATHER, HI_LEATHER),
BOOTS("levitation boots", "snow boots",
		0, 1,  LEVITATION,12, 2, 15, 30,  9, 0, LEATHER, HI_LEATHER)
#define BOOTSDESC	10
#undef HELM
#undef CLOAK
#undef SHIELD
#undef GLOVES
#undef BOOTS
#undef ARMOR
};

#define ARMORDESCS	\
	HELMDESC+CLOAKDESC+SHIELDDESC+GLOVEDESC+BOOTSDESC+ARMDESC+DRGNDESC

static bool armorKnown[ARMORDESCS][MAXPLAYERS];
static char armorRandDesc[ARMORDESCS][MAXPLAYERS];
static int armorProbTotal = 0;

void initArmor() {
    /* Setup total for random generation of armor */
    armorProbTotal = 0;
    for (int x = 0; x < ARMORDESCS; x++)
        armorProbTotal += SArmor[x].prob;
    /* Setup descriptions of objects */
    for (int x = 0; x < MAXPLAYERS; x++)
        quitArmor(x);
}

void loadArmor(ID user) {
}

void saveArmor(ID user) {
    quitArmor(user);
}

void quitArmor(ID user) {
    /* Setup descriptions of objects */
    for (int y = 0; y < ARMORDESCS; y++) {
        armorKnown[y][user] = false;
        armorRandDesc[y][user] = y;
    }
    /* Randomize some of the descriptions */
    for (int y = HelmHelmet; y <= HelmTelepathy; y++) {
        int r = (rn2(HelmTelepathy-HelmHelmet+1) + HelmHelmet);
        char temp = armorRandDesc[y][user];
        armorRandDesc[y][user] = armorRandDesc[r][user];
        armorRandDesc[r][user] = temp;
    }
    for (int y = CloakProtection; y <= CloakDisplacement; y++) {
        int r = (rn2(CloakDisplacement-CloakProtection+1) + CloakProtection);
        char temp = armorRandDesc[y][user];
        armorRandDesc[y][user] = armorRandDesc[r][user];
        armorRandDesc[r][user] = temp;
    }
    for (int y = GloveLeather; y <= GloveDexterity; y++) {
        int r = (rn2(GloveDexterity-GloveLeather+1) + GloveLeather);
        char temp = armorRandDesc[y][user];
        armorRandDesc[y][user] = armorRandDesc[r][user];
        armorRandDesc[r][user] = temp;
    }
    for (int y = BootsWaterWalking; y <= BootsLevitation; y++) {
        int r = (rn2(BootsLevitation-BootsWaterWalking+1) + BootsWaterWalking);
        char temp = armorRandDesc[y][user];
        armorRandDesc[y][user] = armorRandDesc[r][user];
        armorRandDesc[r][user] = temp;
    }
}

CArmor::CArmor() : CObject(Armor, '[', 50) {
    polymorph();
    enchantment = 0;
}

CArmor::CArmor(EArmorType t) : CObject(Armor, '[', 50) {
    armorType = t;
    int r = rn2(34);
    if (r < 1) curseIt();
    if (r > 32) blessIt();
    enchantment = 0;
}

CArmor::CArmor(EArmorType t, int n, int e) : CObject(Armor, '[', 50) {
    armorType = t;
    int r = rn2(34);
    if (r < 1) curseIt();
    if (r > 32) blessIt();
    number = n;
    enchantment = e;
}

void CArmor::polymorph() {
    armorType = (EArmorType)0;
    for(int tprob = rnd(armorProbTotal);
        (tprob -= SArmor[armorType].prob) > 0;
        armorType = (EArmorType)(armorType+1));
    int r = rn2(34);
    if (r < 1) curseIt();
    if (r > 32) blessIt();
    setName("");
    enchantment = 0;
}

bool CArmor::canEat() {
    return false;
}

string CArmor::getDescription(ID user) {
    string s;
    if (getNumber() > 1)
        s = string(number);
    else {
        if (isBlessed() || isCursed() || !isKnown(user))
            s = "a ";
        else s = "an ";
    }
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
    }
    if (isKnown(user)) {
        if (enchantment >= 0) s += '+';
        s += enchantment;
        s += ' ';
        s += SArmor[armorType].name;
    } else {
        char * tstr = SArmor[armorRandDesc[armorType][user]].desc;
        if (tstr == NULL)
            s += SArmor[armorType].name;
        else
            s += tstr;
    }
    return s;
}

/* Returns the protective value of the armor */
int CArmor::getAC() {
    return (10 - SArmor[armorType].ac + enchantment);
}

/* All armor remains the same color */
int CArmor::getColor(ID user) {
    return SArmor[armorType].color;
}

bool CArmor::isKnown(ID user) {
    return armorKnown[armorType][user];
}

void CArmor::identify(ID user) {
    armorKnown[armorType][user] = true;
    BCidentify(user);
}

/* Keep armor isolated */
bool CArmor::operator== (CArmor *o) {
    return false;
}

