/* Interhack - Multi player roguelike game, based on Nethack 3.2.2
   Copyright (C)  1997-1999 by Damian Bentley

   See dat/license for details. If you have not received a copy of the
   license, please email dbentley@mcs.une.edu.au

   Version information can be found in version.h */

/* A fair part of this code has been copied and modified from Nethack
   3.2.2, attrib.c and attrib.h files. */

#include <iostream.h>
#include <stdlib.h>
#include <ctype.h>

#include "attrib.h"
#include "align.h"
#include "log.h"
#include "pm.h"
#include "monflag.h"
#include "monsym.h"
#include "object.h"

char *hu_stat[] = {
        "Satiated",
        "        ",
        "Hungry  ",
        "Weak    ",
        "Fainting",
        "Fainted ",
        "Starved "
};

/*** Table of all roles ***/
/* According to AD&D, HD for some classes (ex. Wizard) should be smaller
 * (4-sided for wizards).  But this is not AD&D, and using the AD&D
 * rule here produces an unplayable character.  Thus I have used a minimum
 * of an 10-sided hit die for everything.  Another AD&D change: wizards get
 * a minimum strength of 4 since without one you can't teleport or cast
 * spells. --KAA
 *
 * As the wizard has been updated (wizard patch 5 jun '96) their HD can be
 * brought closer into line with AD&D. This forces wizards to use magic more
 * and distance themselves from their attackers. --LSZ
 *
 * With the introduction of races, some hit points and energy
 * has been reallocated for each race.  The values assigned
 * to the roles has been reduced by the amount allocated to
 * humans.  --KMH
 *
 * God names use a leading underscore to flag goddesses.
 */
const struct Role roles[] = {
{	{"Archeologist", 0}, {
	{"Digger",      0},
	{"Field Worker",0},
	{"Investigator",0},
	{"Exhumer",     0},
	{"Excavator",   0},
	{"Spelunker",   0},
	{"Speleologist",0},
	{"Collector",   0},
	{"Curator",     0} },
	"Quetzalcoatl", "Camaxtli", "Huhetotl", /* Central American */
	"Arc", "the College of Archeology", "the Tomb of the Toltec Kings",
	PM_ARCHEOLOGIST, NON_PM, NON_PM,
	PM_LORD_CARNARVON, PM_STUDENT, PM_MINION_OF_HUHETOTL,
	NON_PM, PM_HUMAN_MUMMY, S_SNAKE, S_MUMMY,
	OrbOfDetection,
	MH_HUMAN|MH_DWARF|MH_GNOME | ROLE_MALE|ROLE_FEMALE |
	  ROLE_LAWFUL|ROLE_NEUTRAL,
	/* Str Int Wis Dex Con Cha */
	{   7, 10, 10,  7,  7,  7 },
	{  20, 20, 20, 10, 20, 10 },
	/* Init   Lower  Higher */
	{ 11, 0,  0, 8,  1, 0 },	/* Hit points */
	{  1, 0,  0, 1,  0, 1 },14,	/* Energy */
	10, 5, 0, 2, 10, A_INT, SpellMagicMapping,   -4
},
{	{"Barbarian", 0}, {
	{"Plunderer",   "Plunderess"},
	{"Pillager",    0},
	{"Bandit",      0},
	{"Brigand",     0},
	{"Raider",      0},
	{"Reaver",      0},
	{"Slayer",      0},
	{"Chieftain",   "Chieftainess"},
	{"Conqueror",   "Conqueress"} },
	"Mitra", "Crom", "Set", /* Hyborian */
	"Bar", "the Camp of the Duali Tribe", "the Duali Oasis",
	PM_BARBARIAN, NON_PM, NON_PM,
	PM_PELIAS, PM_CHIEFTAIN, PM_THOTH_AMON,
	PM_OGRE, PM_TROLL, S_OGRE, S_TROLL,
	HeartOfAhriman,
	MH_HUMAN|MH_ORC | ROLE_MALE|ROLE_FEMALE |
	  ROLE_NEUTRAL|ROLE_CHAOTIC,
	/* Str Int Wis Dex Con Cha */
	{  16,  7,  7, 15, 16,  6 },
	{  30,  6,  7, 20, 30,  7 },
	/* Init   Lower  Higher */
	{ 14, 0,  0,10,  2, 0 },	/* Hit points */
	{  1, 0,  0, 1,  0, 1 },10,	/* Energy */
	10, 14, 0, 0,  8, A_INT, SpellHasteSelf,      -4
},
{	{"Caveman", "Cavewoman"}, {
	{"Troglodyte",  0},
	{"Aborigine",   0},
	{"Wanderer",    0},
	{"Vagrant",     0},
	{"Wayfarer",    0},
	{"Roamer",      0},
	{"Nomad",       0},
	{"Rover",       0},
	{"Pioneer",     0} },
	"Anu", "_Ishtar", "Anshar", /* Babylonian */
	"Cav", "the Caves of the Ancestors", "the Dragon's Lair",
	PM_CAVEMAN, PM_CAVEWOMAN, PM_LITTLE_DOG,
	PM_SHAMAN_KARNOV, PM_NEANDERTHAL, PM_CHROMATIC_DRAGON,
	PM_BUGBEAR, PM_HILL_GIANT, S_HUMANOID, S_GIANT,
	SceptreOfMight,
	MH_HUMAN|MH_DWARF|MH_GNOME | ROLE_MALE|ROLE_FEMALE |
	  ROLE_LAWFUL|ROLE_NEUTRAL,
	/* Str Int Wis Dex Con Cha */
	{  10,  7,  7,  7,  8,  6 },
	{  30,  6,  7, 20, 30,  7 },
	/* Init   Lower  Higher */
	{ 14, 0,  0, 8,  2, 0 },	/* Hit points */
	{  1, 0,  0, 1,  0, 1 },10,	/* Energy */
	0, 12, 0, 1,  8, A_INT, SpellDig,             -4
},
{	{"Dragon:TODO", 0}, {
	{"TODO:Dragon",     0},
	{"TODO:Dragon",     0},
	{"TODO:Dragon",     0},
	{"TODO:Dragon",     0},
	{"TODO:Dragon",     0},
	{"TODO:Dragon",     0},
	{"TODO:Dragon",     0},
	{"TODO:Dragon",     0},
	{"TODO:Dragon",     0} },
	"Anu", "_Ishtar", "Anshar", /* Babylonian */
	"Cav", "the Caves of the Ancestors", "the Dragon's Lair",
	PM_CAVEMAN, PM_CAVEWOMAN, PM_LITTLE_DOG,
	PM_SHAMAN_KARNOV, PM_NEANDERTHAL, PM_CHROMATIC_DRAGON,
	PM_BUGBEAR, PM_HILL_GIANT, S_HUMANOID, S_GIANT,
	SceptreOfMight,
	MH_HUMAN | ROLE_MALE|ROLE_FEMALE |    /* MH_HUMAN is a hack */
	  ROLE_LAWFUL|ROLE_NEUTRAL|ROLE_CHAOTIC,
	/* Str Int Wis Dex Con Cha */
	{  10,  7,  7,  7,  8,  6 },
	{  30,  6,  7, 20, 30,  7 },
	/* Init   Lower  Higher */
	{ 14, 0,  0, 8,  2, 0 },	/* Hit points */
	{  1, 0,  0, 1,  0, 1 },10,	/* Energy */
	0, 12, 0, 1,  8, A_INT, SpellDig,             -4
},
{	{"Healer", 0}, {
	{"Rhizotomist",    0},
	{"Empiric",        0},
	{"Embalmer",       0},
	{"Dresser",        0},
	{"Medici ossium",  0},
	{"Herbalist",      0},
	{"Magister",       0},
	{"Physician",      0},
	{"Chirurgeon",     0} },
	"_Athena", "Hermes", "Poseidon", /* Greek */
	"Hea", "the Temple of Epidaurus", "the Temple of Coeus",
	PM_HEALER, NON_PM, NON_PM,
	PM_HIPPOCRATES, PM_ATTENDANT, PM_CYCLOPS,
	PM_GIANT_RAT, PM_SNAKE, S_RODENT, S_YETI,
	StaffOfAesculapius,
	MH_HUMAN|MH_GNOME | ROLE_MALE|ROLE_FEMALE | ROLE_NEUTRAL,
	/* Str Int Wis Dex Con Cha */
	{   7,  7, 13,  7, 11, 16 },
	{  15, 20, 20, 15, 25, 10 },
	/* Init   Lower  Higher */
	{ 11, 0,  0, 8,  1, 0 },	/* Hit points */
	{  1, 4,  0, 1,  0, 2 },20,	/* Energy */
	10, 3,-3, 2, 10, A_WIS, SpellCureSickness,   -4
},
{	{"Knight", 0}, {
	{"Gallant",     0},
	{"Esquire",     0},
	{"Bachelor",    0},
	{"Sergeant",    0},
	{"Knight",      0},
	{"Banneret",    0},
	{"Chevalier",   0},
	{"Seignieur",   0},
	{"Paladin",     0} },
	"Lugh", "_Brigit", "Manannan Mac Lir", /* Celtic */
	"Kni", "Camelot Castle", "the Isle of Glass",
	PM_KNIGHT, NON_PM, PM_PONY,
	PM_KING_ARTHUR, PM_PAGE, PM_IXOTH,
	PM_QUASIT, PM_OCHRE_JELLY, S_IMP, S_JELLY,
	MagicMirrorOfMerlin,
	MH_HUMAN | ROLE_MALE|ROLE_FEMALE | ROLE_LAWFUL,
	/* Str Int Wis Dex Con Cha */
	{  13,  7, 14,  8, 10, 17 },
	{  20, 15, 15, 10, 20, 10 },
	/* Init   Lower  Higher */
	{ 14, 0,  0, 8,  2, 0 },	/* Hit points */
	{  1, 4,  0, 1,  0, 2 },10,	/* Energy */
	10, 8,-2, 0,  9, A_WIS, SpellTurnUndead,     -4
},
{	{"Monk", 0}, {
	{"Candidate",                 0},
	{"Novice",                    0},
	{"Initiate",                  0},
	{"Student of the Stone path", 0},
	{"Student of the Waters",     0},
	{"Student of Metals",         0},
	{"Student of the Winds",      0},
	{"Student of Fire",           0},
	{"Master",                    0} },
	"Shan Lai Ching", "Chih Sung-tzu", "Huan Ti", /* Chinese */
	"Mon", "the Monastery of Chan-Sune",
	  "the Monastery of the Earth-Lord",
	PM_MONK, NON_PM, NON_PM,
	PM_GRAND_MASTER, PM_ABBOT, PM_MASTER_KAEN,
	PM_EARTH_ELEMENTAL, PM_XORN, S_ELEMENTAL, S_XORN,
	EyesOfTheOverworld,
	MH_HUMAN | ROLE_MALE|ROLE_FEMALE |
	  ROLE_LAWFUL|ROLE_NEUTRAL|ROLE_CHAOTIC,
	/* Str Int Wis Dex Con Cha */
	{  10,  7,  8,  8,  7,  7 },
	{  25, 10, 20, 20, 15, 10 },
	/* Init   Lower  Higher */
	{ 12, 0,  0, 8,  1, 0 },	/* Hit points */
	{  2, 2,  0, 2,  0, 2 },10,	/* Energy */
	10, 8,-2, 2, 20, A_WIS, SpellRestoreAbility, -4
},
{	{"Priest", "Priestess"}, {
	{"Aspirant",    0},
	{"Acolyte",     0},
	{"Adept",       0},
	{"Priest",      "Priestess"},
	{"Curate",      0},
	{"Canon",       "Canoness"},
	{"Lama",        0},
	{"Patriarch",   "Matriarch"},
	{"High Priest", "High Priestess"} },
	0, 0, 0,	/* chosen randomly from among the other roles */
	"Pri", "the Great Temple", "the Temple of Nalzok",
	PM_PRIEST, PM_PRIESTESS, NON_PM,
	PM_ARCH_PRIEST, PM_ACOLYTE, PM_NALZOK,
	PM_HUMAN_ZOMBIE, PM_WRAITH, S_ZOMBIE, S_WRAITH,
	MitreOfHoliness,
	MH_HUMAN|MH_ELF | ROLE_MALE|ROLE_FEMALE |
	  ROLE_LAWFUL|ROLE_NEUTRAL|ROLE_CHAOTIC,
	/* Str Int Wis Dex Con Cha */
	{   7,  7, 10,  7,  7,  7 },
	{  15, 10, 30, 15, 20, 10 },
	/* Init   Lower  Higher */
	{ 12, 0,  0, 8,  1, 0 },	/* Hit points */
	{  4, 3,  0, 2,  0, 2 },10,	/* Energy */
	0, 3,-2, 2, 10, A_WIS, SpellRemoveCurse,    -4
},
  /* Note:  Rogue precedes Ranger so that use of `-R' on the command line
     retains its traditional meaning. */
{	{"Rogue", 0}, {
	{"Footpad",     0},
	{"Cutpurse",    0},
	{"Rogue",       0},
	{"Pilferer",    0},
	{"Robber",      0},
	{"Burglar",     0},
	{"Filcher",     0},
	{"Magsman",     "Magswoman"},
	{"Thief",       0} },
	"Issek", "Mog", "Kos", /* Nehwon */
	"Rog", "the Thieves' Guild Hall", "the Assassins' Guild Hall",
	PM_ROGUE, NON_PM, NON_PM,
	PM_MASTER_OF_THIEVES, PM_THUG, PM_MASTER_ASSASSIN,
	PM_LEPRECHAUN, PM_GUARDIAN_NAGA, S_NYMPH, S_NAGA,
	MasterKeyOfThievery,
	MH_HUMAN|MH_ORC | ROLE_MALE|ROLE_FEMALE |
	  ROLE_CHAOTIC,
	/* Str Int Wis Dex Con Cha */
	{   7,  7,  7, 10,  7,  6 },
	{  20, 10, 10, 30, 20, 10 },
	/* Init   Lower  Higher */
	{ 10, 0,  0, 8,  1, 0 },	/* Hit points */
	{  1, 0,  0, 1,  0, 1 },11,	/* Energy */
	10, 8, 0, 1,  9, A_INT, SpellDetectTreasure, -4
},
{	{"Ranger", 0}, {
	{"Tenderfoot",    0},
	{"Lookout",       0},
	{"Trailblazer",   0},
	{"Reconnoiterer", "Reconnoiteress"},
	{"Scout",         0},
	{"Arbalester",    0},	/* One skilled at crossbows */
	{"Archer",        0},
	{"Sharpshooter",  0},
	{"Marksman",      "Markswoman"} },
	"Mercury", "_Venus", "Mars", /* Roman/planets */
	"Ran", "Orion's camp", "the cave of the wumpus",
	PM_RANGER, NON_PM, PM_LITTLE_DOG /* Orion & canis major */,
	PM_ORION, PM_HUNTER, PM_SCORPIUS,
	PM_FOREST_CENTAUR, PM_SCORPION, S_CENTAUR, S_SPIDER,
	LongbowOfArtemis,
	MH_HUMAN|MH_ELF|MH_GNOME|MH_ORC | ROLE_MALE|ROLE_FEMALE |
	  ROLE_NEUTRAL|ROLE_CHAOTIC,
	/* Str Int Wis Dex Con Cha */
	{  13, 13, 13,  9, 13,  7 },
	{  30, 10, 10, 20, 20, 10 },
	/* Init   Lower  Higher */
	{ 13, 0,  0, 6,  1, 0 },	/* Hit points */
	{  1, 0,  0, 1,  0, 1 },12,	/* Energy */
	10, 9, 2, 1, 10, A_INT, SpellInvisibility,   -4
},
{	{"Samurai", 0}, {
	{"Hatamoto",    0},  /* Banner Knight */
	{"Ronin",       0},  /* no allegiance */
	{"Ninja",       0},  /* secret society */
	{"Joshu",       0},  /* heads a castle */
	{"Ryoshu",      0},  /* has a territory */
	{"Kokushu",     0},  /* heads a province */
	{"Daimyo",      0},  /* a samurai lord */
	{"Kuge",        0},  /* Noble of the Court */
	{"Shogun",      0} },/* supreme commander, warlord */
	"_Amaterasu Omikami", "Raijin", "Susanowo", /* Japanese */
	"Sam", "the Castle of the Taro Clan", "the Shogun's Castle",
	PM_SAMURAI, NON_PM, PM_LITTLE_DOG,
	PM_LORD_SATO, PM_ROSHI, PM_ASHIKAGA_TAKAUJI,
	PM_WOLF, PM_STALKER, S_DOG, S_ELEMENTAL,
	TsurugiOfMuramasa,
	MH_HUMAN | ROLE_MALE|ROLE_FEMALE | ROLE_LAWFUL,
	/* Str Int Wis Dex Con Cha */
	{  10,  8,  7, 10, 17,  6 },
	{  30, 10, 10, 30, 14, 10 },
	/* Init   Lower  Higher */
	{ 13, 0,  0, 8,  1, 0 },	/* Hit points */
	{  1, 0,  0, 1,  0, 1 },11,	/* Energy */
	10, 10, 0, 0,  8, A_INT, SpellClairvoyance,    -4
},
{	{"Tourist", 0}, {
	{"Rambler",     0},
	{"Sightseer",   0},
	{"Excursionist",0},
	{"Peregrinator","Peregrinatrix"},
	{"Traveler",    0},
	{"Journeyer",   0},
	{"Voyager",     0},
	{"Explorer",    0},
	{"Adventurer",  0} },
	"Blind Io", "_The Lady", "Offler", /* Discworld */
	"Tou", "Ankh-Morpork", "the Thieves' Guild Hall",
	PM_TOURIST, NON_PM, NON_PM,
	PM_TWOFLOWER, PM_GUIDE, PM_MASTER_OF_THIEVES,
	PM_GIANT_SPIDER, PM_FOREST_CENTAUR, S_SPIDER, S_CENTAUR,
	YendorianExpressCard,
	MH_HUMAN | ROLE_MALE|ROLE_FEMALE | ROLE_NEUTRAL,
	/* Str Int Wis Dex Con Cha */
	{   7, 10,  6,  7,  7, 10 },
	{  15, 10, 10, 15, 30, 20 },
	/* Init   Lower  Higher */
	{  8, 0,  0, 8,  0, 0 },	/* Hit points */
	{  1, 0,  0, 1,  0, 1 },14,	/* Energy */
	0, 5, 1, 2, 10, A_INT, SpellCharmMonster,   -4
},
{	{"Valkyrie", 0}, {
	{"Stripling",   0},
	{"Skirmisher",  0},
	{"Fighter",     0},
	{"Man-at-arms", "Woman-at-arms"},
	{"Warrior",     0},
	{"Swashbuckler",0},
	{"Hero",        "Heroine"},
	{"Champion",    0},
	{"Lord",        "Lady"} },
	"Tyr", "Odin", "Loki", /* Norse */
	"Val", "the Shrine of Destiny", "the cave of Surtur",
	PM_VALKYRIE, NON_PM, NON_PM /*PM_WINTER_WOLF_CUB*/,
	PM_NORN, PM_WARRIOR, PM_LORD_SURTUR,
	PM_FIRE_ANT, PM_FIRE_GIANT, S_ANT, S_GIANT,
	OrbOfFate,
	MH_HUMAN|MH_DWARF | ROLE_FEMALE | ROLE_LAWFUL|ROLE_NEUTRAL,
	/* Str Int Wis Dex Con Cha */
	{  10,  7,  7,  7, 10,  7 },
	{  30,  6,  7, 20, 30,  7 },
	/* Init   Lower  Higher */
	{ 14, 0,  0, 8,  2, 0 },	/* Hit points */
	{  1, 0,  0, 1,  0, 1 },10,	/* Energy */
	0, 10,-2, 0,  9, A_WIS, SpellConeOfCold,    -4
},
{	{"Wizard", 0}, {
	{"Evoker",      0},
	{"Conjurer",    0},
	{"Thaumaturge", 0},
	{"Magician",    0},
	{"Enchanter",   "Enchantress"},
	{"Sorcerer",    "Sorceress"},
	{"Necromancer", 0},
	{"Wizard",      0},
	{"Mage",        0} },
	"Ptah", "Thoth", "Anhur", /* Egyptian */
	"Wiz", "the Tower of the Balance", "the Tower of Darkness",
	PM_WIZARD, NON_PM, PM_KITTEN,
	PM_WIZARD_OF_BALANCE, PM_APPRENTICE, PM_DARK_ONE,
	PM_VAMPIRE_BAT, PM_XORN, S_BAT, S_WRAITH,
	EyeOfTheAethiopica,
	MH_HUMAN|MH_ELF|MH_GNOME|MH_ORC | ROLE_MALE|ROLE_FEMALE |
	  ROLE_NEUTRAL|ROLE_CHAOTIC,
	/* Str Int Wis Dex Con Cha */
	{   7, 10,  7,  7,  7,  7 },
	{  10, 30, 10, 20, 20, 10 },
	/* Init   Lower  Higher */
	{ 10, 0,  0, 8,  1, 0 },	/* Hit points */
	{  4, 3,  0, 2,  0, 3 },12,	/* Energy */
	0, 1, 0, 3, 10, A_INT, SpellMagicMissile,   -4
},
/* Array terminator */
{{0, 0}}
};


/* The player's role, created at runtime from initial
 * choices.  This may be munged in role_init().
 */
struct Role urole =
{	{"Undefined", 0}, { {0, 0}, {0, 0}, {0, 0},
	{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0} },
	"L", "N", "C", "Xxx", "home", "locate",
	NON_PM, NON_PM, NON_PM, NON_PM, NON_PM, NON_PM,
	NON_PM, NON_PM, 0, 0, 0, 0,
	/* Str Int Wis Dex Con Cha */
	{   7,  7,  7,  7,  7,  7 },
	{  20, 15, 15, 20, 20, 10 },
	/* Init   Lower  Higher */
	{ 10, 0,  0, 8,  1, 0 },	/* Hit points */
	{  2, 0,  0, 2,  0, 3 },14,	/* Energy */
	0, 10, 0, 0,  4, A_INT, 0, -3
};



/* Table of all races */
const struct Race races[] = {
{	"human", "human", "humanity", "Hum",
	{"man", "woman"},
	PM_HUMAN, NON_PM, PM_HUMAN_MUMMY, PM_HUMAN_ZOMBIE,
	MH_HUMAN | ROLE_MALE|ROLE_FEMALE |
	  ROLE_LAWFUL|ROLE_NEUTRAL|ROLE_CHAOTIC,
	MH_HUMAN, 0, MH_GNOME|MH_ORC,
	/*    Str     Int Wis Dex Con Cha */
	{      3,      3,  3,  3,  3,  3 },
	{ STR18(100), 18, 18, 18, 18, 18 },
	/* Init   Lower  Higher */
	{  2, 0,  0, 2,  1, 0 },	/* Hit points */
	{  1, 0,  2, 0,  2, 0 }		/* Energy */
},
{	"elf", "elven", "elvenkind", "Elf",
	{0, 0},
	PM_ELF, NON_PM, PM_ELF_MUMMY, PM_ELF_ZOMBIE,
	MH_ELF | ROLE_MALE|ROLE_FEMALE | ROLE_CHAOTIC,
	MH_ELF, MH_ELF, MH_ORC,
	/*  Str    Int Wis Dex Con Cha */
	{    3,     3,  3,  3,  3,  3 },
	{   18,    20, 20, 18, 16, 18 },
	/* Init   Lower  Higher */
	{  1, 0,  0, 1,  1, 0 },	/* Hit points */
	{  2, 0,  3, 0,  3, 0 }		/* Energy */
},
{	"dwarf", "dwarven", "dwarvenkind", "Dwa",
	{0, 0},
	PM_DWARF, NON_PM, PM_DWARF_MUMMY, PM_DWARF_ZOMBIE,
	MH_DWARF | ROLE_MALE|ROLE_FEMALE | ROLE_LAWFUL,
	MH_DWARF, MH_DWARF|MH_GNOME, MH_ORC,
	/*    Str     Int Wis Dex Con Cha */
	{      3,      3,  3,  3,  3,  3 },
	{ STR18(100), 16, 16, 20, 20, 16 },
	/* Init   Lower  Higher */
	{  4, 0,  0, 3,  2, 0 },	/* Hit points */
	{  0, 0,  0, 0,  0, 0 }		/* Energy */
},
{	"gnome", "gnomish", "gnomehood", "Gno",
	{0, 0},
	PM_GNOME, NON_PM, PM_GNOME_MUMMY, PM_GNOME_ZOMBIE,
	MH_GNOME | ROLE_MALE|ROLE_FEMALE | ROLE_NEUTRAL,
	MH_GNOME, MH_DWARF|MH_GNOME, MH_HUMAN,
	/*  Str    Int Wis Dex Con Cha */
	{    3,     3,  3,  3,  3,  3 },
	{STR18(50),19, 18, 18, 18, 18 },
	/* Init   Lower  Higher */
	{  1, 0,  0, 1,  0, 0 },	/* Hit points */
	{  2, 0,  2, 0,  2, 0 }		/* Energy */
},
{	"orc", "orcish", "orcdom", "Orc",
	{0, 0},
	PM_ORC, NON_PM, PM_ORC_MUMMY, PM_ORC_ZOMBIE,
	MH_ORC | ROLE_MALE|ROLE_FEMALE | ROLE_CHAOTIC,
	MH_ORC, 0, MH_HUMAN|MH_ELF|MH_DWARF,
	/*  Str    Int Wis Dex Con Cha */
	{   3,      3,  3,  3,  3,  3 },
	{STR18(50),16, 16, 18, 18, 16 },
	/* Init   Lower  Higher */
	{  1, 0,  0, 1,  0, 0 },	/* Hit points */
	{  1, 0,  1, 0,  1, 0 }		/* Energy */
},
/* Array terminator */
{ 0, 0, 0, 0 }};


/* The player's race, created at runtime from initial
 * choices.  This may be munged in role_init().
 */
struct Race urace =
{	"something", "undefined", "something", "Xxx",
	{0, 0},
	NON_PM, NON_PM, NON_PM, NON_PM,
	0, 0, 0, 0,
	/*    Str     Int Wis Dex Con Cha */
	{      3,      3,  3,  3,  3,  3 },
	{ STR18(100), 18, 18, 18, 18, 18 },
	/* Init   Lower  Higher */
	{  2, 0,  0, 2,  1, 0 },	/* Hit points */
	{  1, 0,  2, 0,  2, 0 }		/* Energy */
};


/* Table of all genders */
const struct Gender genders[] = {
	{"male",	"he",	"him",	"his",	"Mal",	ROLE_MALE},
	{"female",	"she",	"her",	"her",	"Fem",	ROLE_FEMALE},
	{"neuter",	"it",	"it",	"its",	"Ntr",	ROLE_NEUTER}
};


/* Table of all alignments */
const struct Align aligns[] = {
	{"law",		"lawful",	"Law",	ROLE_LAWFUL,	A_LAWFUL},
	{"balance",	"neutral",	"Neu",	ROLE_NEUTRAL,	A_NEUTRAL},
	{"chaos",	"chaotic",	"Cha",	ROLE_CHAOTIC,	A_CHAOTIC},
	{"evil",	"unaligned",	"Una",	0,		A_NONE}
};


bool CAttributes::valid() {
    return (validrole(flags.initrole) &&
            validrace(flags.initrole, flags.initrace) &&
            validgend(flags.initrole, flags.initrace, flags.initgend) &&
            validalign(flags.initrole, flags.initrace, flags.initalign));
}


bool CAttributes::validrole(int rolenum) {
	return (rolenum >= 0 && rolenum < SIZE(roles)-1);
}


int CAttributes::randrole() {
	return (rn2(SIZE(roles)-1));
}


bool CAttributes::validrace(int rolenum, int racenum) {
	/* Assumes validrole */
	return (racenum >= 0 && racenum < SIZE(races)-1 &&
		(roles[rolenum].allow & races[racenum].allow & ROLE_RACEMASK));
}


int CAttributes::randrace(int rolenum) {
	int i, n = 0;

	/* Count the number of valid races */
	for (i = 0; races[i].noun; i++)
	    if (roles[rolenum].allow & races[i].allow & ROLE_RACEMASK)
	    	n++;

	/* Pick a random race */
	/* Use a factor of 100 in case of bad random number generators */
	if (n) n = rn2(n*100)/100;
	for (i = 0; races[i].noun; i++)
	    if (roles[rolenum].allow & races[i].allow & ROLE_RACEMASK) {
	    	if (n) n--;
	    	else return (i);
	    }

	/* This role has no permitted races? */
	return (rn2(SIZE(races)-1));
}


bool CAttributes::validgend(int rolenum, int racenum, int gendnum) {
	/* Assumes validrole and validrace */
	return (gendnum >= 0 && gendnum < ROLE_GENDERS &&
		(roles[rolenum].allow & races[racenum].allow &
		 genders[gendnum].allow & ROLE_GENDMASK));
}


int CAttributes::randgend(int rolenum, int racenum) {
	int i, n = 0;

	/* Count the number of valid genders */
	for (i = 0; i < ROLE_GENDERS; i++)
	    if (roles[rolenum].allow & races[racenum].allow &
	    		genders[i].allow & ROLE_GENDMASK)
	    	n++;

	/* Pick a random gender */
	if (n) n = rn2(n);
	for (i = 0; i < ROLE_GENDERS; i++)
	    if (roles[rolenum].allow & races[racenum].allow &
	    		genders[i].allow & ROLE_GENDMASK) {
	    	if (n) n--;
	    	else return (i);
	    }

	/* This role/race has no permitted genders? */
	return (rn2(ROLE_GENDERS));
}


bool CAttributes::validalign(int rolenum, int racenum, int alignnum) {
	/* Assumes validrole and validrace */
	return (alignnum >= 0 && alignnum < ROLE_ALIGNS &&
		(roles[rolenum].allow & races[racenum].allow &
		 aligns[alignnum].allow & ROLE_ALIGNMASK));
}


int CAttributes::randalign(int rolenum, int racenum) {
	int i, n = 0;

	/* Count the number of valid alignments */
	for (i = 0; i < ROLE_ALIGNS; i++)
	    if (roles[rolenum].allow & races[racenum].allow &
	    		aligns[i].allow & ROLE_ALIGNMASK)
	    	n++;

	/* Pick a random alignment */
	if (n) n = rn2(n);
	for (i = 0; i < ROLE_ALIGNS; i++)
	    if (roles[rolenum].allow & races[racenum].allow &
	    		aligns[i].allow & ROLE_ALIGNMASK) {
	    	if (n) n--;
	    	else return (i);
	    }

	/* This role/race has no permitted alignments? */
	return (rn2(ROLE_ALIGNS));
}


const char * CAttributes::Hello() {
	switch (Role_switch) {
	case PM_KNIGHT:
	    return ("Salutations"); /* Olde English */
	case PM_SAMURAI:
	    return ("Konnichi wa"); /* Japanese */
	case PM_TOURIST:
	    return ("Aloha");       /* Hawaiian */
	case PM_VALKYRIE:
	    return ("Velkommen");   /* Norse */
	default:
	    return ("Hello");
	}
}

const char * CAttributes::Goodbye() {
	switch (Role_switch) {
	case PM_KNIGHT:
	    return ("Fare thee well");  /* Olde English */
	case PM_SAMURAI:
	    return ("Sayonara");        /* Japanese */
	case PM_TOURIST:
	    return ("Aloha");           /* Hawaiian */
	case PM_VALKYRIE:
	    return ("Farvel");          /* Norse */
	default:
	    return ("Goodbye");
	}
}

/******************************************************************************/

CAttributes::CAttributes() {
}

/* Init all stats of a player. */
void CAttributes::sconstruct() {
    ptype = (enum role_types)flags.initrole;
    switch(ptype) {
        case Healer:  gold = gold0 = rand() % 1000; break;
        case Rogue:   gold = gold0 = 0;             break;
        case Tourist: gold = gold0 = rand() % 1000; break;
        default: ;
    }

    create(75);
    hp = maxhp = newhp();
    maxpw = roles[ptype].enadv.infix + races[flags.initrace].enadv.infix;
    if (roles[ptype].enadv.inrnd > 0)
        maxpw += rnd(roles[ptype].enadv.inrnd);
    if (races[flags.initrace].enadv.inrnd > 0)
        maxpw += rnd(races[flags.initrace].enadv.inrnd);
    pw = maxpw;
    ac = 10;

    moves = 0;
    hunger = 0;
    hs = satiated;		// Safe values set here...
    exp = 0;
    ulevel = 0;
    gold = gold0 = 0;

    dirtyflag = FALSE;
}

/* Create the attributes for a particular character. */
void CAttributes::create(int np) {
	register int	i, x, tryct;


	for(i = 0; i < A_MAX; i++) {
	    ABASE(i) = AMAX(i) = urole.attrbase[i];
	    ATEMP(i) = ATIME(i) = 0;
	    np -= urole.attrbase[i];
	}

	tryct = 0;
	while(np > 0 && tryct < 100) {

	    x = rn2(100);
	    for (i = 0; (i < A_MAX) && ((x -= urole.attrdist[i]) > 0); i++) ;
	    if(i >= A_MAX) continue; /* impossible */

	    if(ABASE(i) >= ATTRMAX(i)) {

		tryct++;
		continue;
	    }
	    tryct = 0;
	    ABASE(i)++;
	    AMAX(i)++;
	    np--;
	}

	tryct = 0;
	while(np < 0 && tryct < 100) {		/* for redistribution */

	    x = rn2(100);
	    for (i = 0; (i < A_MAX) && ((x -= urole.attrdist[i]) > 0); i++) ;
	    if(i >= A_MAX) continue; /* impossible */

	    if(ABASE(i) <= ATTRMIN(i)) {

		tryct++;
		continue;
	    }
	    tryct = 0;
	    ABASE(i)--;
	    AMAX(i)--;
	    np++;
	}
}

/* Set a players hit points */
int CAttributes::newhp() {
    struct Role urole = roles[ptype];
    struct Race urace = races[rtype];
    int hp, conplus = 0;

    if (ulevel == 0) {
        hp = urole.hpadv.infix + races[flags.initrace].hpadv.infix;
        if (urole.hpadv.inrnd > 0) hp += rnd(urole.hpadv.inrnd);
        if (urace.hpadv.inrnd > 0) hp += rnd(urace.hpadv.inrnd);

        /* Initialize alignment stuff */
        ualign.type = aligns[flags.initalign].value;
        ualign.record = urole.initrecord;

        return hp;
    } else {
        if (ulevel < urole.xlev) {
            hp = urole.hpadv.lofix + urace.hpadv.lofix;
            if (urole.hpadv.lornd > 0) hp += rnd(urole.hpadv.lornd);
            if (urace.hpadv.lornd > 0) hp += rnd(urace.hpadv.lornd);
        } else {
            hp = urole.hpadv.hifix + urace.hpadv.hifix;
            if (urole.hpadv.hirnd > 0) hp += rnd(urole.hpadv.hirnd);
            if (urace.hpadv.hirnd > 0) hp += rnd(urace.hpadv.hirnd);
        }
    }

    switch (ACURR(A_CON)) {
        case    3:      conplus = -2; break;
        case    4:
        case    5:
        case    6:      conplus = -1; break;
        case    15:
        case    16:     conplus = 1; break;
        case    17:     conplus = 2; break;
        case    18:     conplus = 3; break;
        default:        conplus = 0;
    }
    hp += conplus;
    return((hp <= 0) ? 1 : hp);
}

schar CAttributes::acurrf(int x) {
    register int tmp = (abon.a[x] + atemp.a[x] + acurr.a[x]);
    return tmp;
}

/* Generate a player */
int CAttributes::cconstruct() {
    int index, count, kin, i;
    string prompt;
    string temp[20];
    char * menuOpts[22];
    char * blankOpt = "";
    char * finalOpts[] = { "* - random", "q - Quit", "(end)", (char *)NULL };

    /* Select player role here */
    index = 0;
    prompt = "Pick a role";
    menuOpts[index++] = prompt.getStr();
    menuOpts[index++] = (char *)blankOpt;
    for (i = 0; i < SIZE(roles)-1; i++) {
        if (i != Ranger) /* Hack for rangers */
            temp[i] = (char)tolower(roles[i].name.m[0]);
        else
            temp[i] = (char)toupper(roles[i].name.m[0]);
        temp[i] += " - ";
        temp[i] += (char *)roles[i].name.m;
        menuOpts[index++] = temp[i].getStr();
    }
    for (i = 0; i < 4; i++)
        menuOpts[index++] = (char *)finalOpts[i];
    while (1) {
        kin = vision.menu(menuOpts);
        if (kin == 'q') return -1;
        if (kin == '*') { flags.initrole = randrole(); break; };
        for (i = 0; i < SIZE(roles)-1; i++)
            if (i != Ranger) {
                if ((char)tolower(roles[i].name.m[0]) == (char)kin)
                    break;
            } else
                if ((char)roles[i].name.m[0] == (char)kin)
                    break;
        flags.initrole = i;
        break;
    }


    /* Select player race here */
    index = count = 0;
    prompt = "Pick the race of your ";
    prompt += (char *)roles[flags.initrole].name.m;
    menuOpts[index++] = prompt.getStr();
    menuOpts[index++] = (char *)blankOpt;
    for (i = 0; i < SIZE(races)-1; i++)
        if (validrace(flags.initrole, i)) {
            count++;
            temp[i] = (char)(races[i].noun[0]);
            temp[i] += " - ";
            temp[i] += (char *)races[i].noun;
            menuOpts[index++] = temp[i].getStr();
        }
    if (count > 1) {
        for (i = 0; i < 4; i++)
            menuOpts[index++] = (char *)finalOpts[i];
        while (1) {
            kin = vision.menu(menuOpts);
            if (kin == 'q') return -1;
            if (kin == '*') {
                flags.initrace = randrace(flags.initrole); break; };
            for (i = 0; i < SIZE(races)-1; i++)
                if ((char)races[i].noun[0] == (char)kin)
                    break;
            flags.initrace = i;
            break;
        }
    } else
        flags.initrace = randrace(flags.initrole);


    /* Select player gender here */
    index = count = 0;
    prompt = "Pick the gender of your ";
    if (flags.initrole != Dragon) {
        prompt += (char *)races[flags.initrace].adj;
        prompt += ' ';
    }
    prompt += (char *)roles[flags.initrole].name.m;
    menuOpts[index++] = prompt.getStr();
    menuOpts[index++] = (char *)blankOpt;
    for (i = 0; i < SIZE(genders); i++)
        if (validgend(flags.initrole, flags.initrace, i)) {
            count++;
            temp[i] = (char)(genders[i].adj[0]);
            temp[i] += " - ";
            temp[i] += (char *)genders[i].adj;
            menuOpts[index++] = temp[i].getStr();
        }
    if (count > 1) {
        for (i = 0; i < 4; i++)
            menuOpts[index++] = (char *)finalOpts[i];
        while (1) {
            kin = vision.menu(menuOpts);
            if (kin == 'q') return -1;
            if (kin == '*') {
                flags.initgend = randgend(flags.initrole, flags.initrace);
                break; };
            for (i = 0; i < SIZE(genders); i++)
                if ((char)genders[i].adj[0] == (char)kin)
                    break;
            flags.initgend = i;
            break;
        }
    } else
        flags.initgend = randgend(flags.initrole, flags.initrace);


    /* Select player alignment here */
    index = count = 0;
    prompt = "Pick the alignment of your ";
    prompt += (char *)genders[flags.initgend].adj;
    prompt += ' ';
    if (flags.initrole != Dragon) {
        prompt += (char *)races[flags.initrace].adj;
        prompt += ' ';
    }
    prompt += (char *)roles[flags.initrole].name.m;
    menuOpts[index++] = prompt.getStr();
    menuOpts[index++] = (char *)blankOpt;
    for (i = 0; i < SIZE(aligns); i++)
        if (validalign(flags.initrole, flags.initrace, i)) {
            count++;
            temp[i] = (char)(aligns[i].adj[0]);
            temp[i] += " - ";
            temp[i] += (char *)aligns[i].adj;
            menuOpts[index++] = temp[i].getStr();
        }
    if (count > 1) {
        for (i = 0; i < 4; i++)
            menuOpts[index++] = (char *)finalOpts[i];
        while (1) {
            kin = vision.menu(menuOpts);
            if (kin == 'q') return -1;
            if (kin == '*') {
                flags.initalign = randalign(flags.initrole, flags.initrace);
                break; };
            for (i = 0; i < SIZE(aligns); i++)
                if ((char)aligns[i].adj[0] == (char)kin)
                    break;
            flags.initalign = i;
            break;
        }
    } else
        flags.initalign = randalign(flags.initrole, flags.initrace);

    return 0;
}

/* Send attributes to a socket */
CSock& operator<< (CSock& s, CAttributes a) {
    for (int i = 0; i < A_MAX; i++)
        s << (int)a.acurr.a[i] << (int)a.amax.a[i];
    s << (int)a.ptype << a.hp << a.maxhp << a.pw << a.maxpw
      << a.ulevel << a.exp << a.hunger << a.moves
      << a.gold << a.gold0 << a.ac;
    s << (int)a.hs;
    s << a.levelName;
    s << a.name;

    /* These two still have to be sent
    struct Align align; */

    return s;
}

/* Read attributes from a socket */
CSock& operator>> (CSock& s, CAttributes &a) {
    for (int i = 0; i < A_MAX; i++)
        s >> (int &)a.acurr.a[i] >> (int &)a.amax.a[i];
    s >> (int &)a.ptype >> a.hp >> a.maxhp >> a.pw >> a.maxpw
      >> a.ulevel >> a.exp >> a.hunger >> a.moves
      >> a.gold >> a.gold0 >> a.ac;
    s >> (int &)a.hs;
    s >> a.levelName;
    s >> a.name;

    /* These two still have to be sent
    struct Align align; */

    a.dirtyflag = TRUE;

    return s;
}

/* this is used to display the stats to a string on the bottom lines */
void CAttributes::show() {
    string s;

    /* Construct line 1 */
    s += name; s += " the "; s += obtain_rank();
    s += " St:"; s += ABASE(A_STR);
    s += " Dx:"; s += ABASE(A_DEX);
    s += " Co:"; s += ABASE(A_CON);
    s += " In:"; s += ABASE(A_INT);
    s += " Wi:"; s += ABASE(A_WIS);
    s += " Ch:"; s += ABASE(A_CHA);
    switch (ualign.type) {
        case A_CHAOTIC: s += " Chaotic";   break;
        case A_NEUTRAL: s += " Neutral";   break;
        case A_LAWFUL:  s += " Lawful";    break;
        case A_NONE:    s += " Unaligned"; break;
        default:        s += " Unknown";   break;
    }
    vision.statusline1(s);

    /* Construct the second status line */
    s = "Dlvl:"; s += levelName; s += " $:"; s += gold;
    s += " HP:"; s += hp; s += '('; s += maxhp; s += ')';
    s += " Pw:"; s += pw; s += '('; s += maxpw; s += ')';
    s += " AC:"; s += ac; s += " Exp:"; s += ulevel; s += '/';
    s += exp; s += " T:"; s += moves;
    s += ' '; s += hu_stat[hs];

    /* These still have to be done
    ptype */

    vision.statusline2(s);
}

/* Obtain the rank of the player as a string */
char * CAttributes::obtain_rank() {
    return (char *)roles[ptype].rank[0].m;
}

/* Save a players attributes to a file */
ostream& operator<< (ostream& os, CAttributes a) {
    return os;
}

/* Load a players attributes from a file */
istream& operator>> (istream& is, CAttributes &a) {
    return is;
}
