/* Interhack - Multi player roguelike game, based on Nethack 3.2.2
   Copyright (C)  1997-1999 by Damian Bentley

   See dat/license for details. If you have not received a copy of the
   license, please email dbentley@mcs.une.edu.au

   Version information can be found in version.h */

#ifndef _attrib_h_
#define _attrib_h_

#include "align.h"
#include "sock.h"
#include "vision.h"
#include "system.h"

#define A_STR	0
#define A_INT	1
#define A_WIS	2
#define A_DEX	3
#define A_CON	4
#define A_CHA	5

#define A_MAX	6	/* used in rn2() selection of attrib */

#define ABASE(x)	(acurr.a[x])
#define ABON(x)		(abon.a[x])
#define AEXE(x)		(aexe.a[x])
#define ACURR(x)	(acurrf(x))
#define ACURRSTR	(acurrstr())
/* should be: */
/* #define ACURR(x) (ABON(x) + ATEMP(x) + (Upolyd  ? MBASE(x) : ABASE(x)) */
#define MCURR(x)	(macurr.a[x])
#define AMAX(x)		(amax.a[x])
#define MMAX(x)		(mamax.a[x])

#define ATEMP(x)	(atemp.a[x])
#define ATIME(x)	(atime.a[x])

/* KMH -- Conveniences when dealing with strength constants */
#define STR18(x)	(18+(x))	/* 18/xx */
#define STR19(x)	(100+(x))	/* For 19 and above */

struct	attribs {
	schar	a[A_MAX];
};

#define ATTRMAX(x) (urace.attrmax[x])
#define ATTRMIN(x) (urace.attrmin[x])

/*** Substructures ***/

struct RoleName {
	const char	*m;	/* name when character is male */
	const char	*f;	/* when female; null if same as male */
};

struct RoleAdvance {
	/* "fix" is the fixed amount, "rnd" is the random amount */
	xchar infix, inrnd;	/* at character initialization */
	xchar lofix, lornd;	/* gained per level <  urole.xlev */
	xchar hifix, hirnd;	/* gained per level >= urole.xlev */
};

struct u_have {
	Bitfield(amulet,1);	/* carrying Amulet	*/
	Bitfield(bell,1);	/* carrying Bell	*/
	Bitfield(book,1);	/* carrying Book	*/
	Bitfield(menorah,1);	/* carrying Candelabrum */
	Bitfield(questart,1);	/* carrying the Quest Artifact */
	Bitfield(unused,3);
};

struct u_event {
	Bitfield(minor_oracle,1);	/* received at least 1 cheap oracle */
	Bitfield(major_oracle,1);	/*  "  expensive oracle */
	Bitfield(qcalled,1);		/* called by Quest leader to do task */
	Bitfield(qexpelled,1);		/* expelled from the Quest dungeon */
	Bitfield(qcompleted,1);		/* successfully completed Quest task */
	Bitfield(uheard_tune,2);	/* 1=know about, 2=heard passtune */
	Bitfield(uopened_dbridge,1);	/* opened the drawbridge */

	Bitfield(invoked,1);		/* invoked Gate to the Sanctum level */
	Bitfield(gehennom_entered,1);	/* entered Gehennom via Valley */
	Bitfield(uhand_of_elbereth,2);	/* became Hand of Elbereth */
	Bitfield(udemigod,1);		/* killed the wiz */
	Bitfield(ascended,1);		/* has offered the Amulet */
};

/* KMH, conduct --
 * These are voluntary challenges.  Each field denotes the number of
 * times a challenge has been violated.
 */
struct u_conduct {		/* number of times... */
	long	meat;		/* eaten corpse, tin, egg, tripe, body part */
	long	flesh;		/* ... or anything "fleshy" */
	long	food;		/* ... or any comestible */
	long	gnostic;	/* used prayer, priest, or altar */
	long	weaphit;	/* hit a monster with a weapon */
	long	killer;		/* killed a monster yourself */
	long	literate;	/* read something (other than BotD) */
	long	polypiles;	/* polymorphed an object */
	long	polyselfs;	/* transformed yourself */
	long	wishes;		/* used a wish */
	long	wisharti;	/* wished for an artifact */
				/* genocides already listed at end of game */
};

/*** Unified structure containing role information ***/
struct Role {
	/*** Strings that name various things ***/
	struct RoleName name;	/* the role's name (from u_init.c) */
	struct RoleName rank[9]; /* names for experience levels (from botl.c) */
	const char *lgod, *ngod, *cgod; /* god names (from pray.c) */
	const char *filecode;	/* abbreviation for use in file names */
	const char *homebase;	/* quest leader's location (from questpgr.c) */
	const char *intermed;	/* quest intermediate goal (from questpgr.c) */

	/*** Indices of important monsters and objects ***/
	short malenum,		/* index (PM_) as a male (botl.c) */
	      femalenum,	/* ...or as a female (NON_PM == same) */
	      petnum,		/* PM_ of preferred pet (NON_PM == random) */
	      ldrnum,		/* PM_ of quest leader (questpgr.c) */
	      guardnum,		/* PM_ of quest guardians (questpgr.c) */
	      neminum,		/* PM_ of quest nemesis (questpgr.c) */
	      enemy1num,	/* specific quest enemies (NON_PM == random) */
	      enemy2num;
	char  enemy1sym,	/* quest enemies by class (S_) */
	      enemy2sym;
	short questarti;	/* index (ART_) of quest artifact (questpgr.c) */

	/*** Bitmasks ***/
	short allow;		/* bit mask of allowed variations */
#define ROLE_RACEMASK	0x0ff8		/* allowable races */
#define ROLE_GENDMASK	0xf000		/* allowable genders */
#define ROLE_MALE	0x1000
#define ROLE_FEMALE	0x2000
#define ROLE_NEUTER	0x4000
#define ROLE_ALIGNMASK	AM_MASK		/* allowable alignments */
#define ROLE_LAWFUL	AM_LAWFUL
#define ROLE_NEUTRAL	AM_NEUTRAL
#define ROLE_CHAOTIC	AM_CHAOTIC

	/*** Attributes (from attrib.c and exper.c) ***/
	xchar attrbase[A_MAX];	/* lowest initial attributes */
	xchar attrdist[A_MAX];	/* distribution of initial attributes */
	struct RoleAdvance hpadv; /* hit point advancement */
	struct RoleAdvance enadv; /* energy advancement */
	xchar xlev;		/* cutoff experience level */
	xchar initrecord;	/* initial alignment record */

	/*** Spell statistics (from spell.c) ***/
	int spelbase;		/* base spellcasting penalty */
	int spelheal;		/* penalty (-bonus) for healing spells */
	int spelshld;		/* penalty for wearing any shield */
	int spelarmr;		/* penalty for wearing metal armour */
	int spelstat;		/* which stat (A_) is used */
	int spelspec;		/* spell (SPE_) the class excels at */
	int spelsbon;		/* penalty (-bonus) for that spell */

	/*** Properties in variable-length arrays ***/
	/* intrinsics (see attrib.c) */
	/* initial inventory (see u_init.c) */
	/* skills (see u_init.c) */

	/*** Don't forget to add... ***/
	/* quest leader, guardians, nemesis (monst.c) */
	/* quest artifact (artilist.h) */
	/* quest dungeon definition (dat/Xyz.dat) */
	/* quest text (dat/quest.txt) */
	/* dictionary entries (dat/data.bas) */
};

extern const struct Role roles[];	/* table of available roles */
extern struct Role urole;
#define Role_if(X)	(urole.malenum == (X))
#define Role_switch	(urole.malenum)


/*** Unified structure specifying race information ***/

struct Race {
	/*** Strings that name various things ***/
	const char *noun;	/* noun ("human", "elf") */
	const char *adj;	/* adjective ("human", "elven") */
	const char *coll;	/* collective ("humanity", "elvenkind") */
	const char *filecode;	/* code for filenames */
	struct RoleName individual; /* individual as a noun ("man", "elf") */

	/*** Indices of important monsters and objects ***/
	short malenum,		/* PM_ as a male monster */
	      femalenum,	/* ...or as a female (NON_PM == same) */
	      mummynum,		/* PM_ as a mummy */
	      zombienum;	/* PM_ as a zombie */

	/*** Bitmasks ***/
	short allow;		/* bit mask of allowed variations */
	short selfmask,		/* your own race's bit mask */
	      lovemask,		/* bit mask of always peaceful */
	      hatemask;		/* bit mask of always hostile */

	/*** Attributes ***/
	xchar attrmin[A_MAX];	/* minimum allowable attribute */
	xchar attrmax[A_MAX];	/* maximum allowable attribute */
	struct RoleAdvance hpadv; /* hit point advancement */
	struct RoleAdvance enadv; /* energy advancement */
#if 0	/* DEFERRED */
	int   nv_range;		/* night vision range */
	int   xray_range;	/* X-ray vision range */
#endif

	/*** Properties in variable-length arrays ***/
	/* intrinsics (see attrib.c) */

	/*** Don't forget to add... ***/
	/* quest leader, guardians, nemesis (monst.c) */
	/* quest dungeon definition (dat/Xyz.dat) */
	/* quest text (dat/quest.txt) */
	/* dictionary entries (dat/data.bas) */
};

extern const struct Race races[];	/* Table of available races */
extern struct Race urace;
#define Race_if(X)	(urace.malenum == (X))
#define Race_switch	(urace.malenum)


/*** Unified structure specifying gender information ***/
struct Gender {
	const char *adj;	/* male/female/neuter */
	const char *he;		/* he/she/it */
	const char *him;	/* him/her/it */
	const char *his;	/* his/her/its */
	const char *filecode;	/* file code */
	short allow;		/* equivalent ROLE_ mask */
};
#define ROLE_GENDERS	2	/* number of permitted player genders */
				/* increment to 3 if you allow neuter roles */

extern const struct Gender genders[];	/* table of available genders */


/*** Unified structure specifying alignment information ***/
struct Align {
	const char *noun;	/* law/balance/chaos */
	const char *adj;	/* lawful/neutral/chaotic */
	const char *filecode;	/* file code */
	short allow;		/* equivalent ROLE_ mask */
	aligntyp value;		/* equivalent A_ value */
};
#define ROLE_ALIGNS	3	/* number of permitted player alignments */

extern const struct Align aligns[];	/* table of available alignments */

/* These are the character definitions */
enum role_types{Archeologist, Barbarian, Caveman, Dragon, Healer, 
    Knight, Monk, Priest, Rogue, Ranger, Samurai,
    Tourist, Valkyrie, Wizard};

enum hunger_status {satiated, not_hungry, hungry, weak, fainting, fainted,
    starved};

struct Flags {
    int initrole, initrace, initgend, initalign;
};


class CAttributes {
    /* These are used to send / receive player stats from server to client */
    friend CSock& operator<< (CSock&, CAttributes);
    friend CSock& operator>> (CSock&, CAttributes&);
    /* This is used to save and load the player file */
    friend ostream& operator<< (ostream&, CAttributes);
    friend istream& operator>> (istream&, CAttributes&);
public:
    CAttributes();         /* Init all a players attributes. */
    void generate();       /* Construct the player stats */
    int cconstruct();      /* Client Side construction */
    void sconstruct();     /* Server side construction */
    bool dirty() { return dirtyflag; };
    void clear_dirty() { dirtyflag = FALSE; };
    void set_dirty() { dirtyflag = TRUE; };
    void show();
    void setLevel(string s) { levelName = s; };
    void addAC(int i) { ac += i; };
    void subAC(int i) { ac -= i; };
    void setac(int i) { ac = i; };
    int getac() { return ac; };
    void sethp(int i) { ac = i; };
    int gethp() { return ac; };
    void setName(string s) { name = s; };

    const char * Hello();
    const char * Goodbye();
    bool valid();

    enum role_types ptype;   /* Players current role type */
    struct Flags flags;      /* Initial flags / settings */
private:
    struct attribs acurr,    /* your current attributes (eg. str)*/
                   aexe,     /* for gain/loss via "exercise" */
                   abon,     /* your bonus attributes (eg. str) */
                   amax,     /* your max attributes (eg. str) */
                   atemp,    /* used for temporary loss/gain */
                   atime;    /* used for loss/gain countdown */
    int rtype;               /* Players current race type */
    int hp, maxhp;           /* Players hit points and maximum hp */
    int ac;                  /* Armor class, always starts at 10 */
    int pw, maxpw;           /* Players magical power */
    int ulevel, exp;         /* Players exp level and experience */
    align ualign;            /* Players alignment */
    int hunger;              /* How hungry a player is */
    enum hunger_status hs;   /* What a players hunger status is */
    int moves;               /* Number of moves a player has made */
    int gold, gold0;         /* Gold and something else? */
    bool dirtyflag;          /* true if attributes have changed */
    string levelName;        /* Name of level player is on */
    string name;             /* Players name */

    int newhp();             /* Sets the players hit points. */
    void create(int);        /* Init all the attributes */
    char * obtain_rank();    /* Obtain the rank of a player as a string */
    schar acurrf(int);

    int randalign(int, int);
    int randgend(int, int);
    int randrace(int);
    int randrole();

    bool validalign(int, int, int);
    bool validgend(int, int, int);
    bool validrace(int, int);
    bool validrole(int);
};

#endif
