/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "clevel.h"
#include "log.h"
#include "vision.h"
#include "monsym.h"

/* This is the vision magic. A fair part of this code was taken from
   Nethack 3.2.2 until such time I get around to re-writting something */

/* Default monster class symbols.  See monsym.h. */
const char def_monsyms[MAXMCLASSES] = {
        '\0',           /* holder */
        DEF_ANT,
        DEF_BLOB,
        DEF_COCKATRICE,
        DEF_DOG,
        DEF_EYE,
        DEF_FELINE,
        DEF_GREMLIN,
        DEF_HUMANOID,
        DEF_IMP,
        DEF_JELLY,              /* 10 */
        DEF_KOBOLD,
        DEF_LEPRECHAUN,
        DEF_MIMIC,
        DEF_NYMPH,
        DEF_ORC,
        DEF_PIERCER,
        DEF_QUADRUPED,
        DEF_RODENT,
        DEF_SPIDER,
        DEF_TRAPPER,            /* 20 */
        DEF_UNICORN,
        DEF_VORTEX,
        DEF_WORM,
        DEF_XAN,
        DEF_LIGHT,
        DEF_ZRUTY,
        DEF_ANGEL,
        DEF_BAT,
        DEF_CENTAUR,
        DEF_DRAGON,             /* 30 */
        DEF_ELEMENTAL,
        DEF_FUNGUS,
        DEF_GNOME,
        DEF_GIANT,
        '\0',
        DEF_JABBERWOCK,
        DEF_KOP,
        DEF_LICH,
        DEF_MUMMY,
        DEF_NAGA,               /* 40 */
        DEF_OGRE,
        DEF_PUDDING,
        DEF_QUANTMECH,
        DEF_RUSTMONST,
        DEF_SNAKE,
        DEF_TROLL,
        DEF_UMBER,
        DEF_VAMPIRE,
        DEF_WRAITH,
        DEF_XORN,               /* 50 */
        DEF_YETI,
        DEF_ZOMBIE,
        DEF_HUMAN,
        DEF_GHOST,
        DEF_GOLEM,
        DEF_DEMON,
        DEF_EEL,
        DEF_LIZARD,
        DEF_WORM_TAIL,
        DEF_MIMIC_DEF,          /* 60 */
};

CSock& operator>> (CSock& s, CClientLevel& l) {
    char temp;
    vision.clean();
    for (int y = 0; y < HEIGHT; y++)
        for (int x = 0; x < WIDTH; x++) {
            s >> (char &)temp;
            l.map[y][x] = (EFloorType)temp;
            l.update[x][y] = false;
            l.refresh[x][y] = false;
            l.seen[x][y] = EFTEmpty;
        }
    l.objects.removeAll();   /* Remove all objects */
    l.cmonsters.removeAll(); /* Remove all old monsters */
    s >> l.objects;          /* Read new objects on the level */
    s >> l.cmonsters;        /* Read new monsters on the level */
    return s;
}

CClientLevel::CClientLevel() {
    for (int y = 0; y < HEIGHT; y++)
        for (int x = 0; x < WIDTH; x++) {
            map[y][x] = EFTEmpty;
            update[x][y] = false;
            refresh[x][y] = false;
            seen[x][y] = EFTEmpty;
        }
    objects.removeAll();   /* Remove all objects */
    cmonsters.removeAll(); /* Remove all monsters */
}

chtype floorSym[] = {
    ' ', ' ', '.', ACS_BOARD,
    ACS_HLINE, ACS_VLINE, ACS_ULCORNER, ACS_URCORNER, ACS_LLCORNER, ACS_LRCORNER,
    ACS_PLUS, ACS_BTEE, ACS_TTEE, ACS_RTEE, ACS_LTEE,
    '~', '~', '<', '>', '.'};

char OSymbols[] = {
    'X', '"', '[', '(', '%', '*', '!', '=',
    '?', '+', ')', '/', '(', '$', '^'};

#define ihrefresh(x, y) {						\
    CLocation l; l.set(x,y);						\
    update[x][y] = false;						\
    EFloorType t = seen[x][y];						\
    CClientObject * o = objects.get(l);					\
    CClientMonster * m = cmonsters.find(l); 				\
    if (o == NULL && m == NULL) {					\
        if (t == EFTHWall) vision.mapxy(x, y, ACS_HLINE);		\
        else if (t == EFTVWall) vision.mapxy(x, y, ACS_VLINE);		\
        else if (t == EFTTLWall) vision.mapxy(x, y, ACS_ULCORNER);	\
        else if (t == EFTTRWall) vision.mapxy(x, y, ACS_URCORNER);	\
        else if (t == EFTBLWall) vision.mapxy(x, y, ACS_LLCORNER);	\
        else if (t == EFTBRWall) vision.mapxy(x, y, ACS_LRCORNER);	\
        else if (t == EFTCorridor) vision.mapxy(x, y, ACS_BOARD);	\
        else vision.mapxy(x, y, floorSym[t]);				\
    } else if (m != NULL) {						\
        vision.color(m->getColor());					\
        vision.mapxy(x, y, def_monsyms[m->getSpecies()]);		\
        vision.color(CLR_WHITE);					\
    } else if (o != NULL) {						\
        vision.color(o->color);						\
        vision.mapxy(x, y, o->getSymbol());				\
        vision.color(CLR_WHITE);					\
    }									\
}

#define ihrefresh2(x, y) {						\
    refresh[x][y] = false;						\
    EFloorType t = seen[x][y];						\
    if (t == EFTHWall) vision.mapxy(x, y, ACS_HLINE);			\
    else if (t == EFTVWall) vision.mapxy(x, y, ACS_VLINE);		\
    else if (t == EFTTLWall) vision.mapxy(x, y, ACS_ULCORNER);		\
    else if (t == EFTTRWall) vision.mapxy(x, y, ACS_URCORNER);		\
    else if (t == EFTBLWall) vision.mapxy(x, y, ACS_LLCORNER);		\
    else if (t == EFTBRWall) vision.mapxy(x, y, ACS_LRCORNER);		\
    else if (t == EFTCorridor) vision.mapxy(x, y, ACS_BOARD);		\
    else vision.mapxy(x, y, floorSym[t]);				\
}

#define check						\
	{ last_lit = false;				\
	if (!blocked_path(sx, sy, x, y)) {		\
	    all_blocked = false;			\
	    if (map[y][x] != seen[x][y]) {		\
                bool blk = blocked(x, y);		\
	        if (blk && last_lit)			\
	            update[x][y] = true;		\
	        if (!blk && lit(x, y))			\
	            update[x][y] = true;		\
                if (map[y][x] == EFTUpStair ||		\
                    map[y][x] == EFTDownStair)		\
                    update[x][y] = true;		\
	    }						\
	} }

/* This does all the player vision stuff.
    Not fantastic, but it does what is needed for now. */
void CClientLevel::dovision(CLocation p) {
for (int s = 0; s < 2; s++) {
    int sx = p.x;
    int sy = p.y;
    register int x;
    register int y;
    register int r;

    vision.color(CLR_WHITE);
    for (x = sx - 1; x <= sx + 1; x++)
        for (y = sy - 1; y <= sy + 1; y++)
            if (!blocked(x, y) && !lit(x, y) && seen[x][y] != map[y][x])
                update[x][y] = true;

    for (r = 1; r < WIDTH - sx; r++) {
        all_blocked = true;
        x = sx + r;
        for (y = sy - r + 1; y <= sy + 1; y++)
            check;
        if (all_blocked)
            break;
    }
    for (r = 1; r < WIDTH - sx; r++) {
        all_blocked = true;
        x = sx + r;
        for (y = sy + 1; y <= sy + r + 1; y++)
            check;
        if (all_blocked)
            break;
    }
    /* ============================== */
    for (r = 1; r < HEIGHT - sy; r++) {
        all_blocked = true;
        y = sy + r;
        for (x = sx + r - 1; x >= sx - 1; x--)
            check;
        if (all_blocked)
            break;
    }
    for (r = 1; r < HEIGHT - sy; r++) {
        all_blocked = true;
        y = sy + r;
        for (x = sx - 1; x >= sx - r - 1; x--)
            check;
        if (all_blocked)
            break;
    }
    /* ============================== */
    for (r = 1; r < sx + 1; r++) {
        all_blocked = true;
        x = sx - r;
        for (y = sy + r - 1; y >= sy - 1; y--)
            check;
        if (all_blocked)
            break;
    }
    for (r = 1; r < sx + 1; r++) {
        all_blocked = true;
        x = sx - r;
        for (y = sy - 1; y >= sy - r - 1; y--)
            check;
        if (all_blocked)
            break;
    }
    /* ============================== */
    for (r = 1; r < sy + 1; r++) {
        all_blocked = true;
        y = sy - r;
        for (x = sx - r + 1; x <= sx + 1; x++)
            check;
        if (all_blocked)
            break;
    }
    for (r = 1; r < sy + 1; r++) {
        all_blocked = true;
        y = sy - r;
        for (x = sx + 1; x <= sx + r + 1; x++)
            check;
        if (all_blocked)
            break;
    }
    doupdate();
}
}

#define diagonal(floortype)	\
	(seen[x-1][y-1] == floortype || seen[x+1][y-1] == floortype || \
	 seen[x-1][y+1] == floortype || seen[x+1][y+1] == floortype)

/* Screen update. We only flush at the end of the update, and we only send
   co-ords at first character in the line to be displayed. The less we
   write to the screen the better. */
void CClientLevel::doupdate() {
    register int x, y;

    /* Do the updating of the level */
    for (y = 1; y < HEIGHT; y++) {
        for (x = 1; x < WIDTH; x++) {
            if (update[x][y])
                seen[x][y] = map[y][x];
            if (blocked(x,y) && seen[x][y] != map[y][x])
                if (diagonal(EFTFloor) || diagonal(EFTUpStair) ||
                    diagonal(EFTDownStair)) {
                    update[x][y] = true;
                    seen[x][y] = map[y][x];
                }
            if (refresh[x][y])
                ihrefresh2(x, y);
            if (update[x][y])
                ihrefresh(x, y);
        }
    }
}

/* Destroy the level */
void CClientLevel::destroy() {
    register int x, y;
    for (y = 0; y < HEIGHT; y++)
        for (x = 0; x < WIDTH; x++)
            update[x][y] = true;
    doupdate();
}

/* Quadrant I - Checks the path from sx,sy to fx, fy,
   returns 1 if path is blocked, 0 otherwise. It does not check fx, fy; */
#define blocked1(sx, sy, fx, fy) {				\
    int dx, dy;							\
    register int k, err, x, y, dxs, dys;			\
								\
    x = sx;		y = sy;					\
    dx = fx - x;	dy = y - fy;				\
								\
    dxs = dx << 1;						\
    dys = dy << 1;						\
    if (dy > dx) {						\
        err = dxs - dy;						\
        for (k = dy - 1; k; k--) {				\
            if (err >= 0) {					\
                x++;						\
                err -= dys;					\
            }							\
            y--;						\
            err += dxs;						\
            last_lit = lit(x, y);				\
            if (blocked(x,y))					\
                return 1;					\
        }							\
    } else {							\
        err = dys - dx;						\
        for (k = dx - 1; k; k--) {				\
            if (err >= 0) {					\
                y--;						\
                err -= dxs;					\
            }							\
            x++;						\
            err += dys;						\
            last_lit = lit(x, y);				\
            if (blocked(x, y))					\
                return 1;					\
        }							\
    }								\
    return 0;							\
}								\

/* Quadrant IV - Checks the path from sx,sy to fx, fy,
   returns 1 if path is blocked, 0 otherwise. It does not check fx, fy; */
#define blocked4(sx, sy, fx, fy) {				\
    int dx, dy;							\
    register int k, err, x, y, dxs, dys;			\
								\
    x  = sx;		y  = sy;				\
    dx = fx - x;	dy = fy - y;				\
								\
    dxs = dx << 1;						\
    dys = dy << 1;						\
    if (dy > dx) {						\
        err = dxs - dy;						\
        for (k = dy-1; k; k--) {				\
            if (err >= 0) {					\
                x++;						\
                err -= dys;					\
            }							\
            y++;						\
            err += dxs;						\
            last_lit = lit(x, y);				\
            if (blocked(x,y))					\
                return 1;					\
        }							\
    } else {							\
        err = dys - dx;						\
								\
        for (k = dx-1; k; k--) {				\
            if (err >= 0) {					\
                y++;						\
                err -= dxs;					\
            }							\
            x++;						\
            err += dys;						\
            last_lit = lit(x, y);				\
            if (blocked(x,y))					\
                return 1;					\
        }							\
    }								\
    return 0;							\
}								\

/* Quadrant II - Checks the path from sx,sy to fx, fy,
   returns 1 if path is blocked, 0 otherwise. It does not check fx, fy; */
#define blocked2(sx, sy, fx, fy) {				\
    int dx, dy;							\
    register int k, err, x, y, dxs, dys;			\
								\
    x  = sx;		y  = sy;				\
    dx = x - fx;	dy = y - fy;				\
								\
    dxs = dx << 1;						\
    dys = dy << 1;						\
    if (dy > dx) {						\
        err = dxs - dy;						\
        for (k = dy-1; k; k--) {				\
            if (err >= 0) {					\
                x--;						\
                err -= dys;					\
            }							\
            y--;						\
            err += dxs;						\
            last_lit = lit(x, y);				\
            if (blocked(x, y))					\
                return 1;					\
        }							\
    } else {							\
        err = dys - dx;						\
        for (k = dx-1; k; k--) {				\
            if (err >= 0) {					\
                y--;						\
                err -= dxs;					\
            }							\
            x--;						\
            err += dys;						\
            last_lit = lit(x, y);				\
            if (blocked(x, y))					\
                return 1;					\
        }							\
    }								\
    return 0;							\
}								\

/* Quadrant III - Checks the path from sx,sy to fx, fy,
   returns 1 if path is blocked, 0 otherwise. It does not check fx, fy; */
#define blocked3(sx, sy, fx, fy) {				\
    int dx, dy;							\
    register int k, err, x, y, dxs, dys;			\
								\
    x  = sx;		y  = sy;				\
    dx = x - fx;	dy = fy - y;				\
								\
    dxs = dx << 1;						\
    dys = dy << 1;						\
    if (dy > dx) {						\
        err = dxs - dy;						\
        for (k = dy-1; k; k--) {				\
            if (err >= 0) {					\
                x--;						\
                err -= dys;					\
            }							\
            y++;						\
            err += dxs;						\
            last_lit = lit(x, y);				\
            if (blocked(x, y))					\
                return 1;					\
        }							\
    } else {							\
        err = dys - dx;						\
        for (k = dx-1; k; k--) {				\
            if (err >= 0) {					\
                y++;						\
                err -= dxs;					\
            }							\
            x--;						\
            err += dys;						\
            last_lit = lit(x, y);				\
            if (blocked(x, y))					\
                return 1;					\
        }							\
    }								\
    return 0;							\
}								\

/* This decides which quadrant to use,
   returns 1 if path is blocked, 0 otherwise */
int CClientLevel::blocked_path(int sx, int sy, int fx, int fy) {
    if (sx < fx) {
        if (sy > fy) {
            blocked1(sx, sy, fx, fy);
        } else {
            blocked4(sx, sy, fx, fy);
        }
    } else {
        if (sy > fy) {
            blocked2(sx, sy, fx, fy);
        } else if (sx == fx && sy == fy) {
            return 1;
        } else {
            blocked3(sx, sy, fx, fy);
        }
    }
}
