/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifndef _clevel_h_
#define _clevel_h_

#include "cobjlist.h"
#include "cmonlist.h"

class CClientLevel {
    friend CSock& operator>> (CSock&, CClientLevel&);
public:
    CClientLevel();
    void dovision(CLocation);
    inline bool lit(int, int);
    inline bool blocked(int, int);
    int blocked_path(int, int, int, int);
    void update_it(int x, int y) {
        update[x][y] = true; };
    void refresh_it(int sx, int sy, int x, int y) {
        if (!blocked_path(sx, sy, x, y)) update[x][y] = true; };
    void refresh_it(int x, int y) { refresh[x][y] = true; };
    void destroy();
    void doupdate();
    EFloorType map[HEIGHT][WIDTH];
    CClientObjectList objects;
    CClientMonsterList cmonsters;
private:
    EFloorType seen[WIDTH+1][HEIGHT+1];
    bool update[WIDTH+1][HEIGHT+1];
    bool refresh[WIDTH+1][HEIGHT+1];
    bool last_lit;
    bool all_blocked;
    string name;
protected:
};

/* Returns 1 if x,y is lit, 0 otherwise */
bool CClientLevel::lit(int x, int y) {
    if (x < 1 || x >= WIDTH || y < 1 || y > HEIGHT)
        return false;
    /* For the time being this is not done on a proper basis... */
    /* slog << "FIXME: Floor lighting in CClientLevel::lit\n"; */
    if (map[y][x] == EFTFloor)
        return true;
    return false;
}

/* Returns 1 if there is a blocking object in location (x, y),
   0 otherwise */
bool CClientLevel::blocked(int x, int y) {
    if (x < 1 || x >= WIDTH || y < 1 || y >= HEIGHT)
        return true;
    EFloorType t = map[y][x];
    if ((t >= EFTHWall && t <= EFTWTWall) || t == EFTEmpty)
        return true;
    return false;
}

#endif
