/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifndef _cobjlist_h_
#define _cobjlist_h_

#include "string.h"
#include "location.h"
#include "system.h"
#include "color.h"
#include "log.h"
#include "object.h"
#include "vision.h"

extern char OSymbols[];

class CClientObject {
public:
    CClientObject(CClientObject *n, string s, ID i, int w, char l,
                  EObjectType t, CLocation loc, int col) {
        letter = l; des = l; des += " - "; otype = t; xy = loc;
        next = n; des += s; id = i; weight = w; color = col;
        cursed = false; };
    CClientObject(CClientObject *n, string s, ID i, int w, char l,
                  EObjectType t, CLocation loc, int col, EArmorType at) {
        letter = l; des = l; des += " - "; otype = t; xy = loc;
        next = n; des += s; id = i; weight = w; color = col;
        cursed = false; atype = at; };
    ~CClientObject() { if (next != NULL) delete next; next = NULL; };
    void remove(ID i) {
        if (next != NULL) {
            if (next->id == i) {
                CClientObject * temp = next;
                next = next->next;
                temp->next = NULL;
                delete temp;
            } else
                next->remove(i);
        } };
    void update(string s, int w, CLocation loc) {
        des = letter; des += " - "; des += s; weight = w; xy = loc;
        if (s == "null") des = ""; };
    void update(string s) {
        des = letter; des += " - "; des += s; if (s == "null") des = ""; };
    void updateObjects() {
        if (next == NULL) return;
        CClientObject * temp;
        if (mystrlen(next->menu()) < 1) {
            temp = next;
            next = temp->next;
            temp->next = NULL;
            delete temp; }
        if (next != NULL) next->updateObjects(); };

    int getWeight() {
        if (next != NULL) return weight + next->getWeight();
        return weight; };
    char * menu() { return des.getStr(); };
    string letterList(EObjectType t) {
        if (next != NULL && otype == t) {
            string temp = next->letterList(t); temp += letter;
            return temp; }
        if (next != NULL) return next->letterList(t);
        if (next == NULL && otype == t) return letter;
        return ""; };
    string letterList() {
        if (next != NULL) {
            string temp = next->letterList(); temp += letter;
            return temp;
        }
        return letter; };
    CClientObject * find(char let) {
        if (let == letter) return this;
        if (next != NULL) return next->find(let);
        return NULL; };
    CClientObject * get(ID i) {
        if (id == i) return this;
        if (next != NULL) return next->get(i);
        return NULL; };
    CClientObject * get(CClientObject *o) {
        if (o == this) return this;
        if (next != NULL) return next->get(o);
        return NULL; };
    CClientObject * get(CLocation l) {
        if (l == xy) return this;
        if (next != NULL) return next->get(l);
        return NULL; };
    CClientObject * remove(CClientObject *o) {
        if (o == next) {
            CClientObject * temp = next;
            next = temp->next;
            temp->next = NULL;
            return temp;
        } else if (next != NULL) return next->remove(o);
        return NULL; };

    char * getDescription() { return des.getStr(); };
    char * mangleDescription() {
        if (des.size() < 5) return des.getStr();
        return &(des[4]); };
    char getSymbol() { return OSymbols[otype]; };
    void setLetter(char l) { letter = l; string tmp = l;
        tmp += " - "; tmp += mangleDescription(); des = tmp; };

    CClientObject& operator= (CLocation& l) { xy = l; return *this; };

    CClientObject * next;
    bool cursed;
    ID id;
    char letter;
    EObjectType otype;
    CLocation xy;
    int color;
    EArmorType atype;
private:
    int weight;
    string des;
protected:
};

static const char * endptr = "(end)\0";

static const char * otypeHeaders[] = {
    "Null Objects",
    "Amulets", "Armor", "Artifacts", "Food", "Gems", "Potions", "Rings",
    "Scrolls", "Spellbooks", "Tools", "Wands", "Weapons", "Gold",
    "Level Objects"};

class CClientObjectList {
    /* Move an object o from src to tgt */
    friend bool moveObject(CClientObject *o, CClientObjectList& src,
        CClientObjectList& tgt) {
        if (o == NULL) return false;
        CClientObject *it = src.move_remove(o);
        if (it == NULL) return false;
        tgt.move_add(it);
        return true; };
    /* Read an object list from a socket connection */
    friend CSock& operator>> (CSock& s, CClientObjectList& o) {
        ID id; string str; int w; int c;
        char t, at;
        CLocation xy;
        int count;
        s >> count;
        for (int i = 0 ; i < count; i++) {
            /* Doing this normally resulted in seg fault when compiled
               with -O2 optimisation. Works fine like this. */
            s >> str; s >> id; s >> w; s >> t; s >> xy; s >> c;
            if ((EObjectType)t == Armor) {
                s >> at;
                o.head = new CClientObject(o.head, str, id, w, o.letter++,
                                           (EObjectType)t, xy, c, (EArmorType)at);
            } else
                o.head = new CClientObject(o.head, str, id, w, o.letter++,
                                           (EObjectType)t, xy, c);
            o.size++;
        }
        o.changed = true;
        return s; };
public:
    CClientObjectList() { letter = 'a'; head = NULL; changed = false;
        pack = false; };
    ~CClientObjectList() { if (head != NULL) delete head; head = NULL; };
    void add(string s, ID i, int w, EObjectType t, CLocation xy, int c) {
        head = new CClientObject(head, s, i, w, 'X', t, xy, c); size++;
        changed = true; };
    void removeAll() { if (head != NULL) delete head; head = NULL;
                       size = 0; letter = 'a'; changed = true; };
    void remove(ID i) {
        if (head != NULL) {
            if (head->id == i) {
                CClientObject * temp = head;
                head = head->next;
                temp->next = NULL;
                delete temp;
            } else
                head->remove(i);
        size--; changed = true; } };
    int getSize() { return size; };
    void isPack() { pack = true; };
    void sortPack() {
        if (!pack || !changed) return;
log << "Sorting pack" << '\n';
        CClientObject * newhead = NULL, * tobj = NULL;
        EObjectType ot = NullObject;
        for (ot = NullObject; ot <= LevelObject; ot = (EObjectType)(ot+1))
            for (tobj = head; tobj != NULL; tobj = tobj->next)
                if (tobj->next != NULL && tobj->next->otype == ot) {
                    CClientObject * n = newhead, *n2 = tobj->next->next;
                    newhead = tobj->next;
                    newhead->next = n;
                    tobj->next = n2;
                }
        changed = false; };
    char ** menu() {
        CClientObject * temp;
        EObjectType lastotype = NullObject;
        int index = 0;
        for (temp = head; temp != NULL; temp = temp->next) {
            if (temp->otype != lastotype) {
                lines[index++] = (char *)otypeHeaders[temp->otype];
                lastotype = temp->otype; };
            lines[index++] = temp->menu();
        }
        lines[index++] = (char *)endptr;
        lines[index++] = NULL;
        return lines; };
    char ** menu(EObjectType t1, EObjectType t2) {
        CClientObject * temp;
        EObjectType lastotype = NullObject;
        int index = 0;
        for (temp = head; temp != NULL; temp = temp->next)
            if ((temp->otype == t1) || (temp->otype == t2 && t2 != NullObject)) {
                if (temp->otype != lastotype) {
                    lines[index++] = (char *)otypeHeaders[temp->otype];
                    lastotype = temp->otype; };
                lines[index++] = temp->menu();
            }
        lines[index++] = (char *)endptr;
        lines[index++] = NULL;
        return lines; };
    char ** menu(EObjectType t) {
        return menu(t, NullObject); };
    CClientObject * select(string prompt, EObjectType obtype1,
        EObjectType obtype2) {
        int kin;
        CClientObject * tempobj = NULL;
        string tempstr = "What do you want to ";
        tempstr += prompt; tempstr += "? [";
        string tempstr2;
        if (obtype1 != NullObject) tempstr2 = letterList(obtype1);
        else                       tempstr2 = letterList();
        if (obtype2 != NullObject) tempstr2 += letterList(obtype2);
        if (tempstr2.size() < 1) {
            tempstr = "You don't have anything to "; tempstr += prompt;
            tempstr += '.';
            vision.topline(tempstr);
            return NULL;
        }
        tempstr += tempstr2.sort();
        tempstr += " or ?*] ";
        /* Get the players input */
        do {
            kin = vision.inputch(tempstr);
            if (kin == '\r') {
                vision.topline("Never mind."); return NULL; };
            if (kin == '*') {
                kin = vision.menu(menu());
                if (kin == '\r') continue;
            } else if (kin == '?') {
                if (obtype1 != NullObject)
                    kin = vision.menu(menu(obtype1, obtype2));
                else
                    kin = vision.menu(menu());
                if (kin == '\r') continue;
            }
            tempobj = find(kin);
            if (tempobj == NULL)
                vision.inputch("You don't have that object.--More--");
        } while (tempobj == NULL);
        if (tempobj->otype != obtype1 &&
            tempobj->otype != obtype2 &&
            tempobj->otype != NullObject &&
            obtype1 != NullObject) {
            tempstr = "That is a silly thing to "; tempstr += prompt;
            tempstr += '.';
            vision.topline(tempstr);
            return NULL; };
        return tempobj; };
    CClientObject * select(string prompt, EObjectType obtype1) {
        return select(prompt, obtype1, NullObject); };
    CClientObject * select(string prompt) {
        return select(prompt, NullObject, NullObject); };
    string letterList(EObjectType t) {
        if (head != NULL) return head->letterList(t);
        return ""; };
    string letterList() {
        if (head != NULL) return head->letterList();
        return ""; };
    CClientObject * find(char let) {
        if (head != NULL) return head->find(let);
        return NULL; };
    CClientObject * get(ID id) {
        if (head != NULL) return head->get(id);
        return NULL; };
    CClientObject * get(CLocation l) {
        if (head != NULL) return (current = head->get(l));
        return (current = NULL); };
    CClientObject * getNext(CLocation l) {
        if (current == NULL) return NULL;
        if (current->next != NULL) return (current = current->next->get(l));
        return (current = NULL); };
    void updateObjects() {
        if (head == NULL) return;
        CClientObject * temp;
        if (mystrlen(head->menu()) < 1) {
            temp = head;
            head = temp->next;
            temp->next = NULL;
            delete temp; }
        if (head != NULL) head->updateObjects();
        changed = true; };
private:
    int size;
    CClientObject * head, *current;
    char *lines[512];
    bool pack;
    char letter;
    bool changed;

    /* these are used by the function moveObject(...) */
    void move_add(CClientObject *o) {
        if (find(o->letter) != NULL) {
            char c;
            for (c = 'a'; c <= 'z'; c++)
                if (find(c) == NULL) break;
            if (c > 'z') for (c = 'A'; c <= 'Z'; c++)
                if (find(c) == NULL) break;
            if (c == ('Z'+1)) {
                log << "WARNING: could not assign letter" << '\n';
                exit(1);
            }
            o->setLetter(c);
        }
        o->next = head; head = o; size++; changed = true; };
    CClientObject * move_remove(CClientObject *o) {
        CClientObject * temp = NULL;
        if (head != NULL && head == o) {
            temp = head;
            head = temp->next;
            temp->next = NULL;
            size--;
        } else if (head != NULL) {
            temp = head->remove(o);
            if (temp != NULL) size--; }
        changed = true;
        return temp; };
protected:
};

#endif
