/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifndef _cplist_h_
#define _cplist_h_

#include "log.h"
#include "location.h"

class CClientPlayer {
public:
    CClientPlayer(string n, int t, int i, string dgn,
                  int lvl, string flg, char sym) {
        name = n; type = t; id = i; dungeon = dgn; level = lvl;
        flags = flg; symbol = sym; phantom = false; };
    ~CClientPlayer() { };
    char * menu() { return line; };
    void constructLine() {
        string temp = name;
        temp += '('; temp += type; temp += ") ";
        temp += level; temp += " "; temp += flags;
        mystrcpy(line, temp.getStr()); };
    void setLocation(CLocation l) { xy = l; };
    CLocation getLocation() { return xy; };
    char getSym() { return symbol; };
    string getName() { return name; };
    int getLevel() { return level; };
    void setPhantom() { phantom = true; };
    bool getPhantom() { return phantom; };
    void unsetPhantom() { phantom = false; };
private:
    CLocation xy;
    string name, dungeon, flags;
    int id, level;
    int type;
    char line[80];
    char symbol;
    bool phantom;
protected:
};

extern CClientPlayer * cplist[MAXPLAYERS];

class CClientPlayerList {
    friend CSock& operator>> (CSock&, CClientPlayerList&);
public:
    CClientPlayerList() {
        for (int i = 0; i < MAXPLAYERS; i++)
            cplist[i] = NULL;
        copy = false; size = 0; };
    CClientPlayerList(const CClientPlayerList &rhs) {
        copy = true; size = rhs.size; };
    ~CClientPlayerList() {
        if (copy == false)
            for (int i = 0; i < MAXPLAYERS; i++)
                if (cplist[i] != NULL) { delete cplist[i]; cplist[i] = NULL; }; };
    void removeAll() {
        for (int i = 0; i < MAXPLAYERS; i++)
            if (cplist[i] != NULL) { delete cplist[i]; cplist[i] = NULL; }; };
    int getSize() { return size; };
    char ** menu();
    void moved(int id, CLocation l) {
        if (cplist[id] == NULL)
            log << "ERROR: No such player to update location\n";
        else {
            cplist[id]->setLocation(l);
            cplist[id]->constructLine(); } };
    int getId(string s) {
        for (int i = 0; i < MAXPLAYERS; i++)
            if (cplist[i] != NULL &&
                strcmp(cplist[i]->getName().getStr(), s.getStr()) == 0)
                return i;
        return -1; };
    bool onLocation(int id, CLocation l) {
        int lev = cplist[id]->getLevel();
        for (int i = 0; i < MAXPLAYERS; i++)
            if (i != id && cplist[i] != NULL && cplist[i]->getLevel() == lev &&
                l == cplist[i]->getLocation())
                return true;
        return false; };
private:
    int size;
    bool copy;
    char *lines[MAXPLAYERS+1];
protected:
};

#endif
