/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "deffile.h"

Deffile deffile;

Deffile::Deffile() {
}

Deffile::~Deffile() {
    /* Close the file if it is still open */
}

/* Use 'filename' as the file to read from
   return -1 on error, 0 on success. */
int Deffile::start(char *filename) {
    infile.open(filename, ios::in);
    if (infile.fail())
        return -1;
    return 0;
}

/* Close the file if we are finished,
   returns -1 on error, 0 on success */
int Deffile::finish() {
    infile.clear();
    infile.close();
    if (infile.fail())
        return -1;
    return 0;
}

/* Returns a pointer to the next line read in, NULL on error
   Any lines that are comments '#' are skipped over */
char * Deffile::readline() {
    char *firstchar;

    /* Loop until a line with no comments is found */
    do {
        infile.getline(inbuffer, LINESIZE);
        if (infile.fail())
            return NULL;

        /* Strip out any leading spaces */
        firstchar = inbuffer;
        while (*firstchar == SPACE && *firstchar == TAB &&
               (firstchar - inbuffer) < LINESIZE)
            firstchar++;
        if ((firstchar - inbuffer) >= LINESIZE)
            continue;
        mymemmove(inbuffer, firstchar, LINESIZE - (firstchar - inbuffer));
    } while (*firstchar == COMMENTCHAR || *firstchar == ENDLINE);

    infile.clear();

    /*first_field();
    linetype();*/

    return inbuffer;
}

/* Returns a pointer to the line type (the part proceeding the ':')
   returns NULL on error. */
char * Deffile::linetype() {
    char *temp;

    /* Find the end of the linetype */
    temp = inbuffer;
    while (*temp != TYPECHAR && (temp - inbuffer) < LINESIZE &&
           *temp != ENDLINE)
        temp++;
    if ((temp - inbuffer) >= LINESIZE || *temp == ENDLINE)
        return NULL;

    /* Copy the information into its resting place */
    memcpy(thelinetype, inbuffer, (temp - inbuffer));
    thelinetype[(temp - inbuffer)] = ENDLINE;

    return thelinetype;
}

/* Returns a pointer to the first field on the line
   returns NULL on error */
char * Deffile::first_field() {
    /* Find start of first field */
    fieldptr = inbuffer;
    while (*fieldptr != TYPECHAR && *fieldptr != ENDLINE &&
           (fieldptr - inbuffer) < LINESIZE)
        fieldptr++;
    if ((fieldptr - inbuffer) >= LINESIZE)
        return NULL;
    fieldptr++;

    /* Find end of first field - next field does all this stuff */
    return next_field();
}

/* Returns a pointer to the next field on the line
   returns NULL on error */
char * Deffile::next_field() {
    char endchar;
    char * endoffield;
    int hasbracket = 0;

    /* Strip out leading spaces and tabs */
    while ((*fieldptr == SPACE || *fieldptr == TAB) &&
           (fieldptr - inbuffer) < LINESIZE && *fieldptr != ENDLINE)
        fieldptr++;
    if ((fieldptr - inbuffer) >= LINESIZE || *fieldptr == ENDLINE)
        return NULL;

    /* Determine the ending character */
    endchar = ',';

    /* Find the end of the field */
    endoffield = fieldptr;
    while ((*endoffield != endchar && *endoffield != ENDLINE &&
           (endoffield - inbuffer) < LINESIZE) ||
           (hasbracket > 0 && *endoffield != ENDLINE &&
           (endoffield - inbuffer) < LINESIZE)) {
        if (*endoffield == '(') hasbracket++;
        if (*endoffield == ')') hasbracket--;
        endoffield++;
    }

    /* Copy the field information to its final resting place */
    if (*fieldptr == '\"' && *(endoffield-1) == '\"') {
        memcpy(field, fieldptr+1, (endoffield - fieldptr) - 2);
        field[(endoffield - fieldptr) - 2] = ENDLINE;
    } else {
        memcpy(field, fieldptr, (endoffield - fieldptr) + 1);
        field[(endoffield - fieldptr) + 1] = ENDLINE;
        if (field[mystrlen(field)-1] == ',')
            field[mystrlen(field)-1] = '\0';
    }
    if (*endoffield == ENDLINE)
        fieldptr = endoffield;
    else
        fieldptr = endoffield + 1;

    return field;
}

/* Return the numerical value of the field */
int Deffile::read_number() {
    return atoi(field);
}

/* Return a random number - the field contains "(x, y)" somewhere. Find
   it, extract x and y, convert to integers, return x + rn2(y) */
int Deffile::read_random() {
    int x, y;
    char * temp = field;
    char * endt;
    char number[4];

    /* Find start of ( section */
    while (*temp != '-' && (*temp < '0' || *temp > '9') && *temp != ENDLINE)
        temp++;
    if (*temp == ENDLINE || ((*temp < '0' || *temp > '9') && *temp != '-'))
        return -333;

    /* Find end of first number */
    endt = temp;
    while ((*endt >= '0' && *endt <= '9') || *endt == '-')
        endt++;
    if (*endt == ENDLINE || *endt == ')') return -333;

    /* Find the first number!!! */
    memcpy(number, temp, (endt - temp) + 1);
    number[(endt - temp)] = '\0';
    x = atoi(number);

    /* Find start of next number */
    temp = endt;
    while (*temp != ENDLINE && (*temp < '0' || *temp > '9'))
        temp++;
    if (*temp < '0' || *temp > '9') return -333;

    /* Find end of second number */
    endt = temp;
    while (*endt >= '0' && *endt <= '9')
        endt++;
    if (*endt == ENDLINE) return -333;

    /* Find the second number!!! */
    memcpy(number, temp, (endt - temp) + 1);
    number[(endt - temp)] = '\0';
    y = atoi(number);

    if (y == 0) return x;

    return (x + rn2(y));
}

/* Read the base string of current field (ie the string from
   "string" + (X, Y)   ) */
char * Deffile::read_base() {
    char * firstq = field, * lastq;

    /* Find the first " */
    while (*firstq != '\"' && *firstq != ENDLINE && *firstq != ',')
        firstq++;
    if (*firstq == ENDLINE || *firstq == ',') return NULL;

    /* Find the last " */
    lastq = firstq+1;
    while (*lastq != '\"' && *lastq != ENDLINE && *lastq != ',')
        lastq++;
    if (*lastq == ENDLINE || *lastq == ',') return NULL;

    /* Copy the string into the buffer */
    memcpy(base, firstq+1, (lastq - firstq)-1);
    base[(lastq - firstq)-1] = '\0';

    return base;
}

/* Returns field n of the current line
   returns NULL on error */
char * Deffile::n_field(int n) {
    char * temp;
    int x;

    temp = first_field();
    if (temp == NULL)
        return NULL;
    for (x = 1; x < n; x++)
        if ((temp = next_field()) == NULL)
            return NULL;
    return temp;
}

/* Read a location from a field with "(X, Y)". */
CLocation Deffile::read_location() {
    int x, y;
    char * temp = field;
    char * endt;
    char number[4];
    CLocation loc;

    /* Find start of ( section */
    while (*temp != '-' && (*temp < '0' || *temp > '9') && *temp != ENDLINE)
        temp++;
    if (*temp == ENDLINE || ((*temp < '0' || *temp > '9') && *temp != '-'))
        return loc;

    /* Find end of first number */
    endt = temp;
    while ((*endt >= '0' && *endt <= '9') || *endt == '-')
        endt++;
    if (*endt == ENDLINE || *endt == ')') return loc;

    /* Find the first number!!! */
    memcpy(number, temp, (endt - temp) + 1);
    number[(endt - temp)] = '\0';
    if (strcmp(number, "center") == 0)
        x = rn2(80);
    else
        x = atoi(number);

    /* Find start of next number */
    temp = endt;
    while (*temp != ENDLINE && (*temp < '0' || *temp > '9'))
        temp++;
    if (*temp < '0' || *temp > '9') return loc;

    /* Find end of second number */
    endt = temp;
    while (*endt >= '0' && *endt <= '9')
        endt++;
    if (*endt == ENDLINE) return loc;

    /* Find the second number!!! */
    memcpy(number, temp, (endt - temp) + 1);
    number[(endt - temp)] = '\0';
    if (strcmp(number, "center") == 0)
        y = rn2(20);
    else
        y = atoi(number);

    loc.set(x, y);

    return loc;
}
