/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "dungeon.h"
#include "log.h"

CDungeon * dungeon = NULL;

void initDungeon() {
    dungeon = new CDungeon();
    dungeon->create();
}

void quitDungeon() {
    if (dungeon != NULL) delete dungeon;
    dungeon = NULL;
}

CDungeon::CDungeon() {
    main = NULL;
}

CDungeon::~CDungeon() {
    if (main != NULL)
        delete main;
    main = NULL;
}

void CDungeon::create() {
    /* open the definition file */
    if (deffile.start(DEFFILE) == -1) {
        log << "ERROR: could not open " << DEFFILE << '\n';
        exit(-1);
    }

    /* create the main branch */
    if (deffile.readline() == NULL) {
        log << "ERROR: could not read MAIN in " << DEFFILE << '\n';
        exit(-1);
    }
    if (strcmp(deffile.linetype(), "MAIN") != 0) {
        log << "ERROR: expected MAIN, found " << deffile.linetype()
             << " in file " << DEFFILE << '\n';
        exit(-1);
    }
    main = new CBranch(NULL, NULL, deffile.first_field(), 1);
    if (main == NULL) {
        log << "ERROR: failed to create main branch\n";
        exit(-1);
    }

    // send control to the main branch
    main->create();

    // close the definition file
    if (deffile.finish() == -1)
        log << "ERROR: could not close " << DEFFILE << '\n';
}

/* Returns a common object with the level in it */
CLevel * CDungeon::getFirst() {
    return main->getLevel(1);
}

/* Returns the starting location on the first level */
CLocation CDungeon::getStart() {
    return main->getLevel(1)->getBranch();
}

/* Find idstr and return the number of that level / branch */
int CDungeon::getNum(char *strid) {
    return main->getNum(strid);
}

/* Returns a level based on its string id */
CLevel * CDungeon::get(string s) {
    return main->get(s);
}
