/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

/* Updated to nethack 3.3.0
   NOTE: foods do not need random descriptions, as we can tell what
         it is by looking at it. */

struct S_Food {
    char * name;
    int prob;
    int delay;
    int weight;
    int unk;
    int tin;
    int nutrition;
    int color;
    int cost;
} SFood[] = {

#define FOOD(name,prob,delay,wt,unk,tin,nutrition,color) \
	{name,prob,delay,wt,unk,tin,nutrition,color,nutrition/20 + 5}

#define VEGGY	1
#define FLESH	2
#define METAL	3

/* all types of food (except tins & corpses) must have a delay of at least 1. */
/* delay on corpses is computed and is weight dependant */
/* dog eats foods 0-4 but prefers tripe rations above all others */
/* fortune cookies can be read */
/* carrots improve your vision */
/* +0 tins contain monster meat */
/* +1 tins (of spinach) make you stronger (like Popeye) */
/* food CORPSE is a cadaver of some type */
/* meatballs/sticks/rings are only created from objects via stone to flesh */

/* meat */
FOOD("tripe ration",       140, 2, 10, 0, FLESH, 200, CLR_BROWN),
FOOD("corpse",               0, 1,  0, 0, FLESH,   0, CLR_BROWN),
FOOD("egg",                 85, 1,  1, 1, FLESH,  80, CLR_WHITE),
FOOD("meatball",             0, 1,  1, 0, FLESH,   5, CLR_BROWN),
FOOD("meat stick",           0, 1,  1, 0, FLESH,   5, CLR_BROWN),
FOOD("huge chunk of meat",   0,20,400, 0, FLESH,2000, CLR_BROWN),
FOOD("meat ring",            0, 1,  5, 0, FLESH,   5, CLR_BROWN),

/* fruits & veggies */
FOOD("kelp frond",           0, 1,  1, 0, VEGGY,  30, CLR_GREEN),
FOOD("eucalyptus leaf",      3, 1,  1, 0, VEGGY,  30, CLR_GREEN),
FOOD("apple",               15, 1,  2, 0, VEGGY,  50, CLR_RED),
FOOD("orange",              10, 1,  2, 0, VEGGY,  80, CLR_ORANGE),
FOOD("pear",                10, 1,  2, 0, VEGGY,  50, CLR_BRIGHT_GREEN),
FOOD("melon",               10, 1,  5, 0, VEGGY, 100, CLR_BRIGHT_GREEN),
FOOD("banana",              10, 1,  2, 0, VEGGY,  80, CLR_YELLOW),
FOOD("carrot",              15, 1,  2, 0, VEGGY,  50, CLR_ORANGE),
FOOD("sprig of wolfsbane",   7, 1,  1, 0, VEGGY,  40, CLR_GREEN),
FOOD("clove of garlic",      7, 1,  1, 0, VEGGY,  40, CLR_WHITE),
FOOD("slime mold",          75, 1,  5, 0, VEGGY, 250, HI_ORGANIC),

/* people food */
FOOD("lump of royal jelly",  0, 1,  2, 0, VEGGY, 200, CLR_YELLOW),
FOOD("cream pie",           25, 1, 10, 0, VEGGY, 100, CLR_WHITE),
FOOD("candy bar",           13, 1,  2, 0, VEGGY, 100, CLR_BROWN),
FOOD("fortune cookie",      55, 1,  1, 0, VEGGY,  40, CLR_YELLOW),
FOOD("pancake",             25, 2,  2, 0, VEGGY, 200, CLR_YELLOW),
FOOD("lembas wafer",        20, 2,  5, 0, VEGGY, 800, CLR_WHITE),
FOOD("cram ration",         20, 3, 15, 0, VEGGY, 600, HI_ORGANIC),
FOOD("food ration",        380, 5, 20, 0, VEGGY, 800, HI_ORGANIC),
FOOD("K-ration",             0, 1, 10, 0, VEGGY, 400, HI_ORGANIC),
FOOD("C-ration",             0, 1, 10, 0, VEGGY, 300, HI_ORGANIC),
FOOD("tin",                 75, 0, 10, 1, METAL,   0, HI_METAL)
#undef FOOD
};

#define FOODDESCS	29

void initFoods() {
}

void loadFoods(ID user) {
}

void saveFoods(ID user) {
}

void quitFoods(ID user) {
}

CFood::CFood() : CObject(Food, '%') {
    polymorph();
}

CFood::CFood(int n) : CObject(Food, '%') {
    polymorph();
    number = n;
}

CFood::CFood(EFoodType t) : CObject(Food, '%') {
    foodType = t;
    setCost(SFood[foodType].cost);
    setWeight(SFood[foodType].weight);
    int r = rn2(100);
    if (foodType == FoodTin)
    {
        if (r > 94) blessIt();
        if (r < 5) curseIt();
    }
    ammountEaten = 0;
    openAttempt = 0;
}

CFood::CFood(EFoodType t, int n) : CObject(Food, '%') {
    foodType = t;
    setCost(SFood[foodType].cost);
    setWeight(SFood[foodType].weight);
    int r = rn2(100);
    if (foodType == FoodTin)
    {
        if (r > 94) blessIt();
        if (r < 5) curseIt();
    }
    ammountEaten = 0;
    openAttempt = 0;
    number = n;
}

void CFood::polymorph() {
    foodType = (EFoodType)0;
    for(int tprob = rnd(1000);
        (tprob -= SFood[foodType].prob) > 0;
        foodType = (EFoodType)(foodType+1));
    setCost(SFood[foodType].cost);
    setWeight(SFood[foodType].weight);
    int r = rn2(100);
    if (foodType == FoodTin)
    {
        if (r > 94) blessIt();
        if (r < 5) curseIt();
    }
    setName("");
    ammountEaten = 0;
    openAttempt = 0;
}

string CFood::getDescription(ID user) {
    string s;

    // WE have special cases for plural on sprig of wolfsbane, clove of
    // garlic, and lump of royal jelly

    if (getNumber() > 1) {
        s = string(number);
        s += ' ';
    } else {
        if (isBlessed() || isCursed() || !isKnown(user))
            s = "a ";
        else s = "an ";
    }
    if (ammountEaten > 0 && ammountEaten < SFood[foodType].delay)
        s += "partialy eaten ";
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
    }
    s += SFood[foodType].name;
    if (getNumber() > 1) s += "s";
    return s;
}

int CFood::getColor(ID user) {
    return SFood[foodType].color;
}

bool CFood::isKnown(ID user) {
    return true;
}

void CFood::identify(ID user) {
    BCidentify(user);
}

// Eat some food, returns the nutrition obtained
int CFood::eat() {
    // Tins are a special case
    ammountEaten++;
    return (SFood[foodType].nutrition / SFood[foodType].delay);
}

bool CFood::operator== (CFood *f) {
    if (f->foodType != foodType)
        return false;
    if (f->isBlessed() != isBlessed())
        return false;
    if (f->isCursed() != isCursed())
        return false;
    return true;
}
