/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

#define GEMSTONE	1
#define GLASS		2
#define MINERAL		3

struct S_Gems {
    char * name;
    char * desc;
    int prob;
    int weight;
    int cost;
    int nutrition;
    int mohs;
    int glass;
    int color;
    /* Extras for rocks... none right now */
} SGems[] = {

#define GEM(name,desc,prob,wt,gval,nutr,mohs,glass,color) \
	{name,desc,prob,wt,gval,nutr,mohs,glass,color}

#define ROCK(name,desc,kn,prob,wt,gval,sdam,ldam,mgc,nutr,mohs,glass,color) \
	{name,desc,prob,wt,gval,nutr,mohs,glass,color}

GEM("dilithium crystal", "white",      2,  1, 4500, 15,  5, GEMSTONE, CLR_WHITE),
GEM("diamond", "white",                3,  1, 4000, 15, 10, GEMSTONE, CLR_WHITE),
GEM("ruby", "red",                     4,  1, 3500, 15,  9, GEMSTONE, CLR_RED),
GEM("jacinth", "orange",               3,  1, 3250, 15,  9, GEMSTONE, CLR_ORANGE),
GEM("sapphire", "blue",                4,  1, 3000, 15,  9, GEMSTONE, CLR_BLUE),
GEM("black opal", "black",             3,  1, 2500, 15,  8, GEMSTONE, CLR_BLACK),
GEM("emerald", "green",                5,  1, 2500, 15,  8, GEMSTONE, CLR_GREEN),
GEM("turquoise", "green",              6,  1, 2000, 15,  6, GEMSTONE, CLR_GREEN),
GEM("citrine", "yellow",               4,  1, 1500, 15,  6, GEMSTONE, CLR_YELLOW),
GEM("aquamarine", "green",             6,  1, 1500, 15,  8, GEMSTONE, CLR_GREEN),
GEM("amber", "yellowish brown",        8,  1, 1000, 15,  2, GEMSTONE, CLR_BROWN),
GEM("topaz", "yellowish brown",       10,  1,  900, 15,  8, GEMSTONE, CLR_BROWN),
GEM("jet", "black",                    6,  1,  850, 15,  7, GEMSTONE, CLR_BLACK),
GEM("opal", "white",                  12,  1,  800, 15,  6, GEMSTONE, CLR_WHITE),
GEM("chrysoberyl", "yellow",           8,  1,  700, 15,  5, GEMSTONE, CLR_YELLOW),
GEM("garnet", "red",                  12,  1,  700, 15,  7, GEMSTONE, CLR_RED),
GEM("amethyst", "violet",             14,  1,  600, 15,  7, GEMSTONE, CLR_MAGENTA),
GEM("jasper", "red",                  15,  1,  500, 15,  7, GEMSTONE, CLR_RED),
GEM("fluorite", "violet",             15,  1,  400, 15,  4, GEMSTONE, CLR_MAGENTA),
GEM("obsidian", "black",               9,  1,  200, 15,  6, GEMSTONE, CLR_BLACK),
GEM("agate", "orange",                12,  1,  200, 15,  6, GEMSTONE, CLR_ORANGE),
GEM("jade", "green",                  10,  1,  300, 15,  6, GEMSTONE, CLR_GREEN),
GEM("worthless piece of white glass", "white",   77, 1, 0, 6, 5, GLASS, CLR_WHITE),
GEM("worthless piece of blue glass", "blue",     77, 1, 0, 6, 5, GLASS, CLR_BLUE),
GEM("worthless piece of red glass", "red",       77, 1, 0, 6, 5, GLASS, CLR_RED),
GEM("worthless piece of yellowish brown glass", "yellowish brown",
						 77, 1, 0, 6, 5, GLASS, CLR_BROWN),
GEM("worthless piece of orange glass", "orange", 76, 1, 0, 6, 5, GLASS, CLR_ORANGE),
GEM("worthless piece of yellow glass", "yellow", 77, 1, 0, 6, 5, GLASS, CLR_YELLOW),
GEM("worthless piece of black glass",  "black",  76, 1, 0, 6, 5, GLASS, CLR_BLACK),
GEM("worthless piece of green glass", "green",   77, 1, 0, 6, 5, GLASS, CLR_GREEN),
GEM("worthless piece of violet glass", "violet", 77, 1, 0, 6, 5, GLASS, CLR_MAGENTA),

ROCK("luckstone", "gray",	0, 10,  10, 60, 3, 3, 1, 10, 7, MINERAL, CLR_GRAY),
ROCK("loadstone", "gray",	0, 10, 500,  1, 3, 3, 1, 10, 6, MINERAL, CLR_GRAY),
ROCK("flint", "gray",		0, 18,  10,  1, 6, 6, 0, 10, 7, MINERAL, CLR_GRAY),
ROCK("rock", (char *)0,		1,100,  10,  0, 3, 3, 0, 10, 7, MINERAL, CLR_GRAY)
#undef GEM
#undef ROCK
};

#define GEMDESCS	35

static bool gemKnown[GEMDESCS][MAXPLAYERS];

void initGems() {
    for (int x = 0; x < MAXPLAYERS; x++)
        for (int y = 0; y < GEMDESCS; y++)
            gemKnown[y][x] = 0;
}

void loadGems(ID user) {
}

void saveGems(ID user) {
}

void quitGems(ID user) {
}

CGem::CGem() : CObject(Gem, '*', 1) {
    polymorph();
}

CGem::CGem(EGemType t) : CObject(Gem, '*', 1) {
    gemType = t;
    setCost(SGems[gemType].cost);
    setWeight(SGems[gemType].weight);
    if (gemType == GemLoadstone) curseIt();
    setName("");
}

CGem::CGem(EGemType t, int n) : CObject(Gem, '*', 1) {
    gemType = t;
    setCost(SGems[gemType].cost);
    setWeight(SGems[gemType].weight);
    if (gemType == GemLoadstone) curseIt();
    setName("");
    number = n;
}

void CGem::polymorph() {
    gemType = (EGemType)0;
    for(int tprob = rnd(1000);   
        (tprob -= SGems[gemType].prob) > 0;
        gemType = (EGemType)(gemType+1));
    setCost(SGems[gemType].cost);
    setWeight(SGems[gemType].weight);
    if (gemType == GemLoadstone) curseIt();
    setName("");
}

string CGem::getDescription(ID user) {
    string s;
    if (getNumber() > 1) {
        s = string(number);
        s += ' ';
    } else {
        if (isBlessed() || isCursed() || !isKnown(user))
            s = "a ";
        else s = "an ";
    }
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
    }
    if (isKnown(user)) {
        if (gemType >= GemBlueGlass && gemType <= GemYellBrownGlass) {
            if (getNumber() > 1)
                s += "worthless pieces";
            else
                s += "worthless piece";
            s += " of ";
        }
        s += SGems[gemType].name;
        if ((gemType < GemBlueGlass || gemType > GemYellBrownGlass) &&
            getNumber() > 1) s += "s";
    } else {
        s += SGems[gemType].desc;
        if (gemType <= GemYellBrownGlass)
            s += " gem";
        if (getNumber() > 1) s += "s";
    }
    return s;
}

int CGem::getColor(ID user) {
    return SGems[gemType].color;
}

bool CGem::isKnown(ID user) {
    return gemKnown[gemType][user];
}

void CGem::identify(ID user) {
    gemKnown[gemType][user] = true;
    BCidentify(user);
}

bool CGem::operator== (CGem *g) {
    if (g->gemType != gemType)
        return false;
    if (g->isBlessed() != isBlessed())
        return false;
    if (g->isCursed() != isCursed())
        return false;
    return true;
}
