/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifdef GETOPTH
#include <getopt.h>
#endif

#include "hack.h"
#include "cplist.h"
#include "cobjlist.h"
#include "log.h"
#include "version.h"
#include "tcpip.h"
#include "vision.h"
#include "options.h"

CHack hack;

#ifndef SERVER
    CConnection connection(NULL,NULL,NULL);
#endif

int incomming_data() {
    return hack.incomming_data();
}

//[991220 OC start
char* server_name = SERVER_NAME;
int   server_port = SERVER_PORT;

char server_useage[] = "ihack [-s :port]\n";
char client_useage[] = "hack [-s system]\n"
                       "hack [-s [system]:port]\n";

// command line reader is shared between client and server.
// client running within server must not parse a second time.
//
// EXPORTS server_name, server_port
// EXIT: 1 on system error, 2 on syntax error.
//
void read_command_line(int argc, char**argv, bool amServer) {
    char* useage = amServer ? server_useage : client_useage;
    int cc;
    while(EOF != (cc= getopt(argc,argv,"s:"))){
        switch(cc){
            case 's':{
                bool here = (*optarg == ':');
                char* str = here ? server_name : (char *)mystrdup(optarg);
                if (!str) { cerr << "Out of memory\n"; exit(1); }

                char junk; 
                int count, port;
				
                if( here ) {
                    count = sscanf(optarg,":%d%c",&port,&junk);
		    if( 0<count ) ++count;
                }else
                    count = sscanf(optarg,"%[^:]:%d%c", str, &port,&junk);

                switch( count ){
                    case 2:
                        server_port = port;
                        /* two implies one, fall through */
                    case 1:
                        if( amServer & ! here ){
                            cerr << "Cannot start server remotely!\n"
                                 << useage;
                            exit(2);
                        }
                        server_name = str;
                        break;
                    default:
                        cerr << "Syntax error\n" << useage;
                        exit(2);
                }//switch count
                break; }
            default: // cc
                cerr << useage;
                exit(2);
        }//switch cc
    }//while cc
    if( *(argv+optind) ){
        cerr << useage;
        exit(2);
    }
}
//[991220 OC end

int CHack::client_main(bool wizMode) {
    int p;

    inWizMode = wizMode;

    vision.on();

    vision.outputstr(LICENSE);

    cpack.removeAll();
    helmet = gloves = boots = shirt = armor = cloak = shield = NULL;
    pri_weapon = sec_weapon = left_ring = right_ring = NULL;
    blindfold = amulet = NULL;

    string temp = "\rConnecting to ";
    temp += server_name; temp += ':'; temp += server_port; temp += "...";
    vision.outputstr(temp);
    refresh();
/* This is probably the worst hack of the Interhack code. This
   trick is required so the client can be compiled and work as a
   part of the server, and as an isolated program as well. */

    if ((p = connection.c_connect(server_name, server_port)) != -1) {
        sock = new CSock(p);
        vision.outputstr("done.\r\r");

/* Turn on wizard mode if server requested it */
#ifdef SERVER
extern void wizard_mode(int, int);
        if (wizMode == true) {
            int r = rn2(65536);
            wizard_mode(p, r);
            *sock << p_wizard << p << r << p_end;
            inWizMode = true;
        }
#endif

        login();

        /* Main loop */
        int kin;
        while (1) {
            kin = vision.inputch();
            if (kin < 0) break;
            if (keypress(kin) < 0) break;
        }

    } else {
        vision.outputstr("error.\r\r");
        return 0;
    }

    vision.outputstr("Closing connection...");
    delete sock; sock = NULL;
    connection.c_close();
    vision.outputstr("done.\r");
    cpack.removeAll();
    helmet = gloves = boots = shirt = armor = cloak = shield = NULL;
    pri_weapon = sec_weapon = left_ring = right_ring = NULL;
    blindfold = amulet = NULL;
    level.objects.removeAll();
    level.cmonsters.removeAll();
    cplayerlist.removeAll();
    return 0;
}

/* Login to the server */
void CHack::login() {
    string name, passwd;

    do
        name = vision.inputstr("Who are you? ");
    while (name.size() < 4 || name.size() > 8);
    vision.outputstr("\r\r");
    do
        passwd = vision.inputpasswd("What is your password? ");
    while (passwd.size() < 4 || passwd.size() > 8);
    *sock << p_name << name << passwd << p_end;
    myname = name;
}

/* Send player type information to server */
int CHack::character_type() {
    if (attr.cconstruct() < 0)
        return -1;
    *sock << p_type
          << attr.flags.initrole
          << attr.flags.initrace
          << attr.flags.initgend
          << attr.flags.initalign
          << p_end;
    return 0;
}

int CHack::incomming_data() {
    Protocol prot;
    int id, tint; ID tempid;
    CLocation l, coxy;
    string tempstr;
    int r;
    char alive;
    CClientMonster * mtmp;
    CClientObject * tobj;
    int location;
    do {
        r = sock->fill_inbuf();
        if (r < 0) {
            log << "Connection terminated\n";
            vision.topline("Connection terminated.");
            return -1;
        }
        if (r < 1) return 0; /* Ignore empty messages */
        *sock >> prot;
        switch(prot) {

            case p_name:
                login();
                break;

            case p_type:
                if (character_type() < 0) return -1;
                break;

            case p_stats:
                *sock >> attr;
                attr.show();
                break;
            case p_ac:
                *sock >> tint;
                attr.setac(tint);
                attr.show();
                break;
            case p_hp:
                *sock >> tint;
                attr.sethp(tint);
                attr.show();
                break;

            case p_level:
                oxy.set(-1,-1);
                *sock >> level;
                *sock >> xy;
                break;

            case p_yell:
            case p_talk:
                *sock >> tempstr;
                vision.msgline(tempstr);
                break;

            case p_topline:
                *sock >> tempstr;
                vision.topline(tempstr);
                break;

            case p_list:
                players_refresh();
                *sock >> cplayerlist;
                myid = cplayerlist.getId(myname);
                players_update(); /* Display players entering / leaving */
                break;

            case p_move: /* Movement of either this player or another */
                *sock >> id;
                if (id != myid) {
                    coxy = cplist[id]->getLocation();
                    level.refresh_it(coxy.x, coxy.y);
                    *sock >> l;
                    cplayerlist.moved(id, l);
#ifndef OTHERS_MOVEMENT_CHEAT
                    if (!level.blocked_path(xy.x, xy.y, l.x, l.y))
#endif
                        vision.mapxy(l.x, l.y, cplist[id]->getSym());
                } else {
                    *sock >> xy;
                    /* Display a message if they stepped in something :) */
                    tobj = level.objects.get(xy);
                    if (tobj != NULL) {
                        string tstr;
                        tstr = "You see here ";
                        tstr += tobj->mangleDescription();
                        tstr += '.';
                        if (level.objects.getNext(xy) != NULL)
                            tstr = "There are many objects here.";
                        vision.topline(tstr);
                    }
                }
                waiting = false;
                break;

            case p_pack: /* Players pack is arriving */
                *sock >> cpack;
                cpack.isPack();
                break;

            case p_object: /* Update an object */
                updateObject();
                break;

            case p_drop: /* Other player dropped an object */
                dropedObject();
                break;

            case p_pickup: /* Other player picked up an object */
                *sock >> tempid;
                level.objects.remove(tempid);
                break;

            case p_monster: /* Update a monster */
                *sock >> tempid;
                mtmp = level.cmonsters.find(tempid);
                if (mtmp == NULL) {
                    log << "ERROR: server wants client to update non-existant monster"
                        << '\n';
                    break;
                }
                level.refresh_it(mtmp->xy.x, mtmp->xy.y);
                *sock >> mtmp->xy;
#ifndef OTHERS_MOVEMENT_CHEAT
                if (!level.blocked_path(xy.x, xy.y, mtmp->xy.x, mtmp->xy.y))
#endif
                    level.update_it(mtmp->xy.x, mtmp->xy.y);
                break;

            case p_hit: /* A player hit something */
                *sock >> tempstr;
                vision.topline(tempstr);
                *sock >> tempid; *sock >> (char &)alive;
                if (!alive) {
                    CClientMonster *m = level.cmonsters.find(tempid);
                    if (m != NULL) {
                        CLocation l = m->xy;
                        level.refresh_it(xy.x, xy.y, l.x, l.y);
                    }
                    level.cmonsters.remove(tempid);
                }
                break;

            case p_puton: /* Server is telling player to put on something */
                *sock >> tempid; *sock >> location; *sock >> tempstr;
                tobj = cpack.get(tempid);
                if (tobj == NULL) {
                    log << "ERROR: server telling us puton non-existing object" << '\n';
                    break;
                }
                tobj->update(tempstr);
                switch (location) {
                    case BODY_AMULET:     amulet = tobj;     break;
                    case BODY_RIGHT_RING: right_ring = tobj; break;
                    case BODY_LEFT_RING:  left_ring = tobj;  break;
                    default:
                        log << "ERROR: server telling us puton invalid location" << '\n';
                        update();
                        return 0;
                }
                vision.topline(tobj->getDescription());
                break;

            case p_takeoff: /* Server telling player remove something */
                *sock >> location; *sock >> tempstr;
                switch (location) {
                    case BODY_AMULET: tobj = amulet; amulet = NULL; break;
                    case BODY_RIGHT_RING: tobj = right_ring; right_ring = NULL; break;
                    case BODY_LEFT_RING: tobj = left_ring; left_ring = NULL; break;
                    default:
                        log << "ERROR: server telling us takeoff invalid location" << '\n';
                        update();
                        return 0;
                }
                tobj->update(tempstr);
                tempstr = "You were wearing ";
                tempstr += tobj->mangleDescription();
                tempstr += '.';
                vision.topline(tempstr);
                break;

            case p_wield: /* Server says we can wield a weapon */
                *sock >> tempid;
                tobj = cpack.get(tempid);
                if (tobj == NULL) {
                    log << "ERROR: server telling us wield non-existing object" << '\n';
                    break;
                }
                *sock >> tempstr; /* Read new desc. for old weapon */
                if (pri_weapon != NULL) pri_weapon->update(tempstr);
                pri_weapon = tobj;
                *sock >> tempstr; /* Read new desc. for new weapon */
                tobj->update(tempstr);
                vision.topline(tobj->getDescription());
                break;

            case p_exchange: /* Exchange primary and secondary weapons */
                *sock >> tint;
                /* Read in the new primary weapon */
                if (tint && 1) {
                    *sock >> tempid;
                    tobj = cpack.get(tempid);
                    if (tobj == NULL) {
                        log << "WARNING: server exchange with invalid object" << '\n';
                        break;
                    }
                    *sock >> tempstr; /* New desc. for weapon */
                    pri_weapon = tobj;
                    tobj->update(tempstr);
                    vision.topline(tobj->getDescription());
                }
                /* Read in the new secondary weapon */
                if (tint && 2) {
                    *sock >> tempid;
                    tobj = cpack.get(tempid);
                    if (tobj == NULL) {
                        log << "WARNING: server exchange with invalid object" << '\n';
                        break;
                    }
                    *sock >> tempstr; /* New desc. for weapon */
                    sec_weapon = tobj;
                    tobj->update(tempstr);
                    vision.topline(tobj->getDescription());
                }
                break;

            case p_wear: /* Server is telling player to put on something */
                *sock >> tempid; *sock >> location; *sock >> tempstr;
                tobj = cpack.get(tempid);
                if (tobj == NULL) {
                    log << "ERROR: server telling us wear non-existing object" << '\n';
                    break;
                }
                tobj->update(tempstr);
                switch (location) {
                    case BODY_BOOTS:  boots = tobj;  break;
                    case BODY_GLOVES: gloves = tobj; break;
                    case BODY_HELMET: helmet = tobj; break;
                    case BODY_SHIELD: shield = tobj; break;
                    case BODY_SHIRT:  shirt = tobj;  break;
                    case BODY_ARMOR:  armor = tobj;  break;
                    case BODY_CLOAK:  cloak = tobj;  break;
                    default:
                        log << "ERROR: server telling us wear invalid location" << '\n';
                        update();
                        return 0;
                }
                vision.topline(tobj->getDescription());
                break;

            case p_remove: /* Server telling player remove something */
                *sock >> location; *sock >> tempstr;
                switch (location) {
                    case BODY_BOOTS:  tobj = boots;  boots = NULL;  break;
                    case BODY_GLOVES: tobj = gloves; gloves = NULL; break;
                    case BODY_HELMET: tobj = helmet; helmet = NULL; break;
                    case BODY_SHIELD: tobj = shield; shield = NULL; break;
                    case BODY_SHIRT:  tobj = shirt;  shirt = NULL;  break;
                    case BODY_ARMOR:  tobj = armor;  armor = NULL;  break;
                    case BODY_CLOAK:  tobj = cloak;  cloak = NULL;  break;
                    default:
                        log << "ERROR: server telling us takeoff invalid location" << '\n';
                        update();
                        return 0;
                }
                if (tobj == NULL) {
                    log << "WARNING: server telling us takeoff non-existing object" << '\n';
                    break;
                }
                tobj->update(tempstr);
                tempstr = "You were wearing ";
                tempstr += tobj->mangleDescription();
                tempstr += '.';
                vision.topline(tempstr);
                break;

            case p_cursed: /* Player has just found cursed object */
                *sock >> tempid; *sock >> tempstr;
                tobj = cpack.get(tempid);
                if (tobj == NULL) {
                    log << "WARNING: server telling us non-existing obj is cursed" << '\n';
                    break;
                }
                tobj->update(tempstr);
                tobj->cursed = true;
                vision.topline("You can't.  It seems to be cursed.");
                break;

            default:
                log << "TODO: complete client incomming data (prot="
                     << (int)prot << ")" << '\n';
        }
    } while (r > 1);
    update();
    return 0;
}

int CHack::keypress(int key) {
    int kin, dir;
    CClientObject * tobj;
    string tempstr, letters;
    if (key < 0) return -1;
    string temp;
    switch (key) {
#ifdef SERVER
        case 6: /* ^F - magic mapping in wizard mode */
            if (inWizMode == true) {
                level.destroy();
                update();
            } else
                vision.topline("You must be in wizard mode to do magic mapping.");
            break;
#endif /*SERVER*/

        case 8: /* ^H - list players in the game */
            vision.menu(cplayerlist.menu());
            update(); /* Force vision update */
            break;

        case 13: /* ^M - last message line message */
            vision.lastMsgline();
            break;

        case 16: /* ^P - last top line message */
            vision.lastTopline();
            break;

#ifdef SERVER
extern int wizard_wish(string);
        case 23: /* ^W - wizard mode wish for object */
            if (inWizMode) {
                string tstr = vision.inputtopstr("What do you wish for? ");
                wizard_wish(tstr);
            } else
                vision.topline("You must start in wizard mode for this command.");
            break;
#endif /*SERVER*/

        case '"': /* Yell to all players on same dungeon level */
            temp = vision.inputtopstr("Yell? ");
            *sock << p_yell << temp << p_end;
            break;

        case '\'': /* Talk to players within 16 units */
            temp = vision.inputtopstr("Talk? ");
            *sock << p_talk << temp << p_end;
            break;

        case '?': /* Help */
            vision.clienthelp(xy);
            level.doupdate();
            break;

        case '/': /* What is something on the map */
            vision.info_on_char(xy);
            break;

        case '\\': /* List of discoveries */
            log << "TODO: List of discoveries keypress\n";
            break;

        case '!': /* Shell not allowed! */
            vision.topline("Sorry, Interhack does not allow you to use a shell.");
            break;

        case '#': /* Extended command */
            if (extendedCommand() < 0) {
                kin = vision.inputch("Are you sure you want to quit [y/n] ");
                cout << (char)kin;
                if (vision.uppercase(kin) == 'Y') {
                    *sock << p_quit << p_end;
                    return -1;
                }
            }
            break;

        case ',': /* Pick up an object */
            tobj = level.objects.get(xy);
            if (tobj == NULL) {
                vision.topline("There is nothing here to pick up.");
                break;
            }
            moveObject(tobj, level.objects, cpack);
            tempstr = tobj->getDescription(); tempstr += '.';
            vision.topline(tempstr);
            *sock << p_pickup << tobj->id << p_end;
            break;

        case ':': /* Look at current location */
            log << "TODO: Look at current location keypress\n";
            break;

        case ';': /* Look distantly... */
            log << "TODO: Look distantly keypress\n";
            break;

        case '^': /* Id a trap */
            log << "TODO: ID a trap keypress\n";
            break;

        case '$': /* Tell how much gold you have */
            log << "TODO: Gold count keypress\n";
            break;

        case '+': /* List known spells */
            log << "TODO: List spells keypress\n";
            break;

        case '@': /* Toggle auto-pickup */
            log << "TODO: Toggle auto-pickup keypress\n";
            break;

        /* Movement */
        case KEY_UP: case KEY_DOWN: case KEY_LEFT: case KEY_RIGHT:
        case KEY_HOME: case KEY_END: case KEY_NPAGE: case KEY_PPAGE:
        case KEY_B2:
            switch (key) {
                case KEY_UP:    key = '8'; break;
                case KEY_DOWN:  key = '2'; break;
                case KEY_RIGHT: key = '6'; break;
                case KEY_LEFT:  key = '4'; break;
                case KEY_HOME:  key = '7'; break;
                case KEY_PPAGE: key = '9'; break;
                case KEY_B2:    key = '5'; break;
                case KEY_END:   key = '1'; break;
                case KEY_NPAGE: key = '3'; break;
            }
        case '1': case '2': case '3': case '4': case '5':
        case '6': case '7': case '8': case '9': case '.': case ' ':
            if (waiting) break;
            dir = key - '0';
            if (key == '5' || key == '.' || key == ' ') {
                log << "TODO: Deal with 'wait' movements\n";
                break;
            }
            if (xy.x > 1 && (dir-1)%3 == 0) xy.x--;
            if (xy.x < WIDTH && dir%3 == 0) xy.x++;
            if (xy.y > 1 && dir > 6) xy.y--;
            if (xy.y < HEIGHT && dir < 4) xy.y++;

            /* If the new point is blocked, or player already there */
            if (level.blocked(xy.x, xy.y) ||
                cplayerlist.onLocation(myid, xy) == true) {
                vision.topline("TODO: check for players attacking each other.");
                xy = oxy;
                break;
            }

            /* If the new point is blocked by a monster, attack it */
            if (level.cmonsters.find(xy) != NULL) {
                vision.topline("TODO: check for player attacking friendly monster.");
                *sock << p_hit << xy << p_end;
                xy = oxy;
                break;
            }

            /* Inform the server of the movement */
            if (key >= '1' && key <= '9')
                *sock << p_move << (int)(key - '1') << p_end;
            else
                *sock << p_move << (int)d_none << p_end;
            xy = oxy; /* the movement will be sent from the server when
                         sureal time says it is a valid movement */
            waiting = true;
            break;

        case '<': case '>': /* Up or down stairs */
            if (level.map[xy.y][xy.x] == EFTUpStair)
                *sock << p_move << (int)d_up << p_end;
            else if (level.map[xy.y][xy.x] == EFTDownStair)
                *sock << p_move << (int)d_down << p_end;
            else
                vision.topline("I see no stairs here.");
            break;

        case 'A': /* Remove all armor */
            log << "TODO: Remove all armor keypress\n";
            break;

        case 'a': /* Apply an object */
            tobj = cpack.select("apply", Tool);
            if (tobj != NULL)
                *sock << p_apply << tobj->id << p_end;
            break;

        case 'C': /* Call a friendly monster something */
            log << "TODO: Call a monster keypress\n";
            break;

        case 'c': /* Close a door */
            log << "TODO: Close door keypress\n";
            getDirection();
            break;

        case 'D': /* Drop many objects */
            if (cpack.getSize() < 1) {
                vision.topline("You don't have anything to drop.");
                break;
            }
            kin = vision.inputch("What do you want to drop [....] ? ");
            tobj = cpack.find(kin);
            if (tobj == NULL) {
                vision.topline("You do not have that object <- FIXME!");
                break;
            }
            //vision << "You drop " << tobj->getDescription() << end1;
            *sock << p_drop << tobj->id << p_end;
            break;

        case 'd': /* Drop a single object */
            if (cpack.getSize() < 1) {
                vision.topline("You don't have anything to drop.");
                break;
            }
            letters = "What do you want to drop? [";
            letters += cpack.letterList();
            letters += " or ?*] ";
            while (1) {
                kin = vision.inputch(letters);
                if (kin == '*' || kin == '?') {
                    kin = vision.menu(cpack.menu());
                    if (kin == '\r') continue; }
                tobj = cpack.find(kin);
                if (kin == '\r') break;
                if (tobj == NULL)
                    vision.inputch("You don't have that object.--More--");
                else break;
            }
            if (tobj == NULL) {
                vision.topline("Never mind.");
                break;
            }
            if (tobj == amulet || tobj == left_ring || tobj == right_ring ||
                tobj == blindfold || tobj == boots || tobj == gloves ||
                tobj == helmet || tobj == shirt || tobj == armor ||
                tobj == cloak) {
                vision.topline("You cannot drop something you are wearing.");
                break;
            }
            *tobj = xy;
            moveObject(tobj, cpack, level.objects);
            tempstr = "You drop "; tempstr += tobj->mangleDescription();
            tempstr += '.';
            vision.topline(tempstr);
            *sock << p_drop << tobj->id << p_end;
            break;

        case 'E': /* Engrave a message */
            log << "TODO: Engrave a message keypress\n";
            break;

        case 'e': /* Eat food */
            tobj = cpack.select("eat", Food);
            if (tobj != NULL)
                *sock << p_eat << tobj->id << p_end;
            break;

        case 'I': /* Inventory groups */
            log << "TODO: Display inventory groups keypress\n";
            break;

        case 'i': /* Display inventory */
        case '0':    /* Zero key on keypad */
            if (cpack.getSize() < 1)
                vision.topline("Not carrying anything.");
            else
                vision.menu(cpack.menu());
            break;

        case 'k': /* Kick */
            log << "TODO: Kick keypress\n";
            break;

        case 'l': /* Loot */
            log << "TODO: Loot keypress\n";
            break;

        case 'N': /* Name an item or object */
            log << "TODO: Name item keypress\n";
            break;

        case 'O': /* Options */
            options.change();
            break;

        case 'o': /* Open */
            log << "TODO: Open keypress\n";
            getDirection();
            break;

        case 'P': /* Put on ring, amulet... */
            puton_accessory();
            break;

        case 'p': /* Pay your bill */
            log << "TODO: Pay your bill keypress\n";
            break;

        case 'Q': /* Quit */
            log << "TODO: Q\n";
            break;

        case 'q': /* Drink a potion */
            tobj = cpack.select("drink", Potion);
            if (tobj != NULL)
                *sock << p_quaff << tobj->id << p_end;
            break;

        case 'R': /* Remove an accessory */
            remove_accessory();
            break;

        case 'r': /* Read scroll or book */
            tobj = cpack.select("read", Spellbook, Scroll);
            if (tobj != NULL)
                *sock << p_read << tobj->id << p_end;
            break;

        case 'S': /* Save */
            kin = vision.inputch("Are you sure you want to save [y/n] ");
            cout << (char)kin;
            if (vision.uppercase(kin) == 'Y')
                *sock << p_save << p_end;
            break;

        case 's': /* Search current location */
            log << "TODO: Search current location keypress\n";
            break;

        case 'T': /* Remove some armor */
            remove_armor();
            break;

        case 't': /* Throw */
            tobj = cpack.select("throw", Weapon);
            if (tobj != NULL) {
                getDirection();
                *sock << p_throw << tobj->id << p_end;
            }
            break;

        case 'u': /* Untrap a trap */
            log << "TODO: Untrap a trap keypress\n";
            break;

        case 'V': /* History information */
            log << "TODO: Display game history\n";
            break;

        case 'v': /* Version information */
            vision.topline(VERSION_CLIENT);
            break;

        case 'W': /* Wear some armor */
            wear_armor();
            break;

        case 'w': /* Wield a weapon */
            wield();
            break;

        case 'X': /* Explore mode */
            vision.topline("Sorry, Explore mode is not available in Interhack.");
            break;

        case 'x': /* Exchange weapon */
            if (pri_weapon != NULL || sec_weapon != NULL)
                *sock << p_exchange << p_end;
            break;

        case 'Z': /* Case a spell */
            log << "TODO: Cast a spell keypress\n";
            break;

        case 'z': /* Zap a wand */
            tobj = cpack.select("zap", Wand);
            if (tobj != NULL)
                *sock << p_zap << tobj->id << p_end;
            break;

        //default:
            //vision << "I do not understand that command. (key="
            //       << (int)key << ")" << end1;
    }
    /* Do not need to update vision here - server _sends_ all changes
       in position, so vision only needs to be done there. */
    if (myid >= 0) /* Just update the cursor */
        vision.playerxy(xy.x, xy.y, cplist[myid]->getSym());
    return 0;
}

/* Obtain an input direction from the player */
Direction CHack::getDirection() {
    vision.topline("In what direction? ");
    int kin;
        kin = vision.inputch();
    if (kin >= '1' && kin <= '9')
        return (Direction)(kin - '1');
    else if (kin == '>')
        return d_down;
    else if (kin == '<')
        return d_up;
    else if (kin == '.' || kin == ' ')
        return d_none;
    vision.topline("I do not know that direction.");
    return d_invalid;
}

/* Update the players position on the screen */
void CHack::update() {
    if (myid >= 0 && cplist[myid] != NULL) {
        players_update();
        level.update_it(oxy.x, oxy.y);
        level.refresh_it(oxy.x, oxy.y);
        level.dovision(xy);
        vision.playerxy(xy.x, xy.y, cplist[myid]->getSym());
        oxy = xy;
   }
}

/* Update the display of players on the level (for when the local player
   moves. */
/* This is really horrid code, but it achieves its task */
void CHack::players_update() {
    /* Go through each player. If they exist, same level, and in los, draw */
    if (myid < 0 || cplist[myid] == NULL) return;
    CClientPlayer * pl = NULL;
    int mylev = cplist[myid]->getLevel();
    for (int i = 0; i < MAXPLAYERS; i++) {
        if ((pl = cplist[i]) != NULL) {
            if (myid != i && pl->getLevel() == mylev) {
                CLocation ploc = pl->getLocation();
                if (!level.blocked_path(xy.x, xy.y, ploc.x, ploc.y))
                    vision.mapxy(ploc.x, ploc.y, pl->getSym());
            }
        }
    }
}
/* Refresh player locations in case one has been removed -yuck*/
void CHack::players_refresh() {
    /* Go through each player. If they exist, same level, and in los, refresh */
    if (myid < 0 || cplist[myid] == NULL) return;
    CClientPlayer * pl = NULL;
    int mylev = cplist[myid]->getLevel();
    for (int i = 0; i < MAXPLAYERS; i++) {
        if ((pl = cplist[i]) != NULL) {
            if (myid != i && pl->getLevel() == mylev) {
                CLocation ploc = pl->getLocation();
                level.refresh_it(xy.x, xy.y, ploc.x, ploc.y);
            }
        }
    }
}

/* Update an object in players pack, or objects on the level */
void CHack::updateObject() {
    string des;
    ID tempid;
    int w;
    char otype;
    CLocation oxy;

    /* Read in the new object information */
    *sock >> des; *sock >> tempid; *sock >> w; *sock >> otype;
    *sock >> oxy;

    /* Find the object */
    CClientObject * obj = cpack.get(tempid);
    if (obj == NULL)
        ; // FIXME: check objects on level for tempid
    if (obj == NULL) {
        log << "ERROR: could not find object to update it!\n";
        return;
    }

    /* Update the object and both lists */
    obj->update(des, w, oxy);
    cpack.updateObjects();
    // FIXME: update lists of objects on the level
}

/* Another player dropped an object onto the level */
void CHack::dropedObject() {
    ID id; string str; int w; int c;
    char t;
    CLocation xy;
    /* Doing this normally resulted in seg fault when compiled
       with -O2 optimisation. Works fine like this. */
    *sock >> str; *sock >> id; *sock >> w; *sock >> t;
    *sock >> xy; *sock >> c;
    level.objects.add(str, id, w, (EObjectType)t, xy, c);
}
