/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifndef _level_h_
#define _level_h_

#include "location.h"
#include "objlist.h"
#include "monlist.h"
#include "deffile.h"

#define WIDTH	80
#define HEIGHT	20

#define MAXROOMWIDTH	16
#define MINROOMWIDTH	5
#define MAXROOMHEIGHT	10
#define MINROOMHEIGHT	4

/* These are just temp storage to create coridors on normal levels */
class CRoom {
public:
    CRoom(CRoom *n, int x, int y, int w, int h) {
        next = n; xloc = x; yloc = y; width = w; height = h; };
    ~CRoom() {
        if (next != NULL) delete next;
        next = NULL; };
    CRoom * getNext() { return next; };
    CLocation * getADoor() {
        int tempx = xloc-1, tempy = yloc-1;
        int side;
        if (xloc < 3) tempx = xloc + width;
        if (yloc < 3) tempy = yloc + height;
        do {
            side = rn2(4);
            switch (side) {
                case 0: tempy = yloc + height;
                case 1: tempx = xloc + (rn2(width-2)) + 1; break;
                case 2: tempx = xloc + width;
                case 3: tempy = yloc + (rn2(height-2)) + 1; break;
            }
        } while (tempx >= WIDTH || tempy >= HEIGHT);
        // Make sure the doors are centralised and not on the edges
//        if (tempx-1 < 3) tempx = xloc + width;
//        if (tempy-1 < 3) tempy = yloc + height;
        if (tempx > WIDTH-3) tempx = xloc - 1;
        if (tempy > HEIGHT-3) tempy = yloc - 1;
        tempdoor.set(tempx,tempy);

        if (tempx == xloc - 1)
            thedoor.set(xloc,tempy);
        else if (tempx == xloc + width)
            thedoor.set(xloc+width-1,tempy);
        else if (tempy == yloc - 1)
            thedoor.set(tempx,yloc);
        else /* if (tempy == yloc + height) */
            thedoor.set(tempx,yloc+height-1);

        return &tempdoor; };
    CLocation * getAFloor() {
        int tempx = xloc + 1 + (rn2(width-2));
        int tempy = yloc + 1 + (rn2(height-2));
        tempfloor.set(tempx,tempy);
        return &tempfloor; };
    CLocation * getTheDoor() {
        return &thedoor; };
    int getX() { return xloc; };
    int getY() { return yloc; };
    bool overlapping(int x, int y, int xs, int ys) {
        if (x < xloc + width && x + xs > xloc &&
            y < yloc + height && y + ys > yloc)
            return true;
        if (next) return next->overlapping(x, y, xs, ys);
        return false; };
private:
    CRoom * next;
    int xloc, yloc, width, height;
    CLocation tempdoor, tempfloor, thedoor;
protected:
};

class CBranch;

class CLevel {
public:
    void send(CSock *, ID);
    CLevel(CLevel& l);
    virtual ~CLevel();
    int load();
    int save();
    void display();
    void populate();
    virtual void regenerate() = 0;
    CLevel * get(int);
    CLevel * get(CLocation);
    CLocation getUp() { return upStair; };
    CLocation getDown() { return downStair; };
    CLocation getBranch() { return branch; };
    int getNumber() { return number; };
    virtual int getNum(char *nam) {
        if (next != NULL) return next->getNum(nam);
        return -333; };
    int blocked_path(int, int, int, int);
    string getName();

    CLocation * addUp();
    CLocation * addDown();
    CLocation * addBranch(EEntryType);
    CBranch * getParent() { return parent; };
    virtual bool isTemplate() { return false; };
    virtual bool isNormal()   { return false; };
    virtual bool isMaze()     { return false; };
    virtual bool isSewer()    { return false; };
    virtual bool isMine()     { return false; };
    CLocation getStart(CLocation);
    CLocation getRandomFloor();
    CLocation getRandomCorridor();
    CLocation getRandomAny();

    EFloorType ** map;
    CMonsterList monsters;
    CObjectList objects;
private:
    int isTempCopy;
    int last_lit;
    int lit(int, int);
    int blocked(int, int);
protected:
    CLevel(CLevel *, CBranch *, int);
    void addRoom(uchar, uchar, uchar, uchar);
    void addRoomWithin(uchar, uchar, uchar, uchar);
    void addSubRoom(uchar, uchar, uchar, uchar);
    void ellipse(int, int, int, int);
    void solidline(int, int, int, int);
    void clean();

    void addCorridors();
    void doCorridor(CLocation, CLocation);
    void around(int *, int *, int, int, int, int);

    CLevel * next;
    CBranch * parent;
    int number;
    CRoom * rooms;
    CLocation upStair;
    CLocation downStair;
    CLocation branch;
};

class CNormalLevel : public CLevel {
public:
    CNormalLevel(CLevel *, CBranch *, int);
    virtual void regenerate();
    virtual bool isNormal() { return true; };
private:
protected:
};

class CMineLevel : public CLevel {
public:
    CMineLevel(CLevel *, CBranch *, int);
    virtual void regenerate();
    virtual bool isMine() { return true; };
private:
protected:
};

// These are some definitions for the maze generation
#define cell_empty(a) (!(a)->up && !(a)->right && !(a)->down && !(a)->left)
typedef struct {
    unsigned int up      : 1;
    unsigned int right   : 1;
    unsigned int down    : 1;
    unsigned int left    : 1;
} cell_t;
typedef cell_t *maze_t;
#define UP              0
#define RIGHT           1
#define DOWN            2
#define LEFT            3

class CMazeLevel : public CLevel {
public:
    CMazeLevel(CLevel *, CBranch *, int);
    CMazeLevel(CLevel *, CBranch *, int, uchar, uchar);
    virtual void regenerate();
    virtual bool isMaze() { return true; };
private:
    uchar width, height;
    maze_t maz;
    void create(uchar, uchar);
    void convert(uchar, uchar);
protected:
};

class CSewerLevel : public CLevel {
public:
    CSewerLevel(CLevel *, CBranch *, int);
    virtual void regenerate();
    virtual bool isSewer() { return true; };
private:
protected:
};

enum EGeometry {
    EGeomCenter, EGeomLeft, EGeomRight, EGeomTop, EGeomBottom,
    EGeomHalfLeft, EGeomHalfRight, EGeomRandom};

class CTemplateLevel : public CLevel {
public:
    CTemplateLevel(CLevel *, CBranch *, int, string, string, string, int);
    virtual void regenerate();
    virtual int getNum(char *nam) {
        if (name == nam) return number;
        if (next != NULL) return next->getNum(nam);
        return -333; };
    void taddroom();
    virtual bool isTemplate() { return true; };
private:
    string name, desfile, id;
    int choices;
    Deffile dfile;
    EGeometry xgeom, ygeom;
    void constructMap();
    void setGeometry();
    void addMonster();
    void addObject();
    void addObject2();
    void addTrap();
    void addAltar();
    void addDoor();
    void addStair();
    void addRegion();
    void addFlags();
    void addMessage();
    void addTeleportRegion();
    void addPortal();
    void addNonDiggable();
    void addNonPasswall();
    void addLadder();
    void addTBranch();
    void addDrawBridge();
    void addFountain();
    void addInitMap();
    void addRandomPlaces();
    void addRandomMonsters();
    void addRandomObjects();
    void addContainer();
    void addEngraving();
    void taddSubRoom();
    void addName();
protected:
};

#endif
