/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include <sys/time.h>
#include <stdio.h>

#include "log.h"
#include "location.h"

Log log("ihack.log");

/* Open the log file s */
Log::Log(char *s) {
    file.open(s, ios::out | ios::app);
    if (file.fail()) {
        cerr << "Unable to open log file: " << s << "\n";
        exit(1);
    }
    file << ddmmyy() << "-" << hhmmss()
         << "Log opened: " << s << "\n";
    first = 1;
}

/* Close the log file. */
Log::~Log() {
    file << ddmmyy() << "-" << hhmmss() << "Log closed\n";
    file.close();
    if (file.fail())
        cerr << "Unable to close log file\n";
}

/* Obtain the local time */
struct tm * Log::getlt() {
    time_t date;

    (void) time(&date);
    return(localtime(&date));
}

/* Returns the time in hours, minutes and seconds. */
char * Log::hhmmss() {
    static char timestr[8];
    struct tm *lt;

    lt = getlt();

    sprintf(timestr, "%02d%02d%02d ",
        lt->tm_hour, lt->tm_min, lt->tm_sec);
    return(timestr);
}

/* Returns the time in days, months, and years. */
char * Log::ddmmyy() {
    static char datestr[7];
    struct tm *lt;

    lt = getlt();

    /* Y2K bug! yay... the % 100 is needed on some systems to handle a bug
       where lt->tm_year would result in ddmm1yy. */
    sprintf(datestr, "%02d%02d%02d",
        lt->tm_mday, lt->tm_mon + 1, lt->tm_year % 100);
    return(datestr);
}

/* Send a location to the log file */
Log& operator<< (Log& logstream, class CLocation l) {
    if (logstream.first == 1) {
        logstream.first = 0;
        logstream.file << logstream.ddmmyy() << "-" << logstream.hhmmss();
    }
    logstream.file << '(' << l.x << ',' << l.y << ')';
    if (logstream.file.fail())
        cerr << "*** CLocation output to log file failed\n";
    logstream.file.flush();
    return logstream;
}

/* Send a string to the log file */
Log& operator<< (Log& logstream, char *s) {
    if (logstream.first == 1) {
        logstream.first = 0;
        logstream.file << logstream.ddmmyy() << "-" << logstream.hhmmss();
    }
    logstream.file << s;
    if (logstream.file.fail())
        cerr << "*** char* output to log file failed\n";
    if (s[mystrlen(s)-1] == '\n')
        logstream.first = 1;
    logstream.file.flush();
    return logstream;
}

Log& operator<< (Log& logstream, string s) {
    if (logstream.first == 1) {
        logstream.first = 0;
        logstream.file << logstream.ddmmyy() << "-" << logstream.hhmmss();
    }
    logstream.file << s.getStr();
    if (logstream.file.fail())
        cerr << "*** String output to log file failed\n";
    if (s[s.size()-1] == '\n')
        logstream.first = 1;
    logstream.file.flush();
    return logstream;
}

/* Send an integer to the log file. */
Log& operator<< (Log& logstream, int i) {
    if (logstream.first == 1) {
        logstream.first = 0;
        logstream.file << logstream.ddmmyy() << "-" << logstream.hhmmss();
    }
    logstream.file << i;
    if (logstream.file.fail())
        cerr << "*** Integer output to log file failed\n";
    logstream.file.flush();
    return logstream;
}

/* Send a single character to the log file. */
Log& operator<< (Log& logstream, char c) {
    if (logstream.first == 1) {
        logstream.first = 0;
        logstream.file << logstream.ddmmyy() << "-" << logstream.hhmmss();
    }
    logstream.file << c;
    if (logstream.file.fail())
        cerr << "*** Character output to log file failed\n";
    if (c == '\n')
        logstream.first = 1;
    logstream.file.flush();
    return logstream;
}
