/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "level.h"
#include "log.h"

// Create a new maze
CMazeLevel::CMazeLevel(CLevel *newNext, CBranch *newParent, int newNumber) :
    CLevel(newNext, newParent, newNumber) {
#ifdef DEBUG_LEVEL
    log << "LEVEL: constructing maze " << newNumber << '\n';
#endif
    width = WIDTH;
    height = HEIGHT;
    regenerate();
}

// Create a maze with a set size
CMazeLevel::CMazeLevel(CLevel *newNext, CBranch *newParent, int newNumber,
    uchar newWidth, uchar newHeight) :
    CLevel(newNext, newParent, newNumber) {
    width = newWidth;
    height = newHeight;
    regenerate();
}

// Regenerate the maze
void CMazeLevel::regenerate() {
    clean();

    // Check the sizes of the maze and adjust if too big or small
    if (width <= 0 || height <= 0 || width > WIDTH || height > HEIGHT) {
        log << "ERROR: CMazeLevel::regenerate- Invalid size of maze\n";
        width = WIDTH;
        height = HEIGHT;
    }
    int xsize = (width / 2) - 2;
    int ysize = (height / 2) - 1;

    maz = NULL;
    if ((maz = (maze_t)calloc (xsize * ysize, sizeof(cell_t))) == NULL)
        log << "ERROR: CMazeLevel::regenerate- could not allocate memory\n";
    create(xsize, ysize);
    convert(xsize, ysize);
    if (maz != NULL)
        free(maz);
    maz = NULL;
}

// Now we just convert the maze into the floor plan
void CMazeLevel::convert(uchar xsize, uchar ysize) {
    int ix, iy, a, b;

    b = 1;
    for (iy = 0; iy < ysize; iy++) {
        a = 1;
        map[b][a++] = EFTHWall;
        for (ix = 0; ix < xsize; ix++) {
            if ((maz + ix + (iy * xsize))->up)
                map[b][a++] = EFTFloor;
            else
                map[b][a++] = EFTHWall;
            map[b][a++] = EFTHWall;
        }
        b++;
        a = 1;
        for (ix = 0; ix < xsize; ix++) {
            if ((maz + ix + (iy * xsize))->left)
                map[b][a++] = EFTFloor;
            else
                map[b][a++] = EFTHWall;
            map[b][a++] = EFTFloor;
        }
        map[b++][a++] = EFTHWall;
    }
    a = 1;
    map[b][a++] = EFTHWall;
    for (ix = 0; ix < xsize; ix++) {
        if ((maz + ix + ((ysize-1) * xsize))->down)
            map[b][a++] = EFTFloor;
        else
            map[b][a++] = EFTHWall;
        map[b][a++] = EFTHWall;
    }
}

// Create the maze itself
void CMazeLevel::create(uchar xsize, uchar ysize) {
    maze_t mp, maze_top;
    char paths [4];
    int visits, directions;

    visits = xsize * ysize - 1;
    mp = maz;
    maze_top = mp + (xsize * ysize) - 1;

    while (visits) {
        directions = 0;

        if ((mp - xsize) >= maz && cell_empty (mp - xsize))
            paths [directions++] = UP;
        if (mp < maze_top && ((mp - maz + 1) % xsize) && cell_empty (mp + 1))
            paths [directions++] = RIGHT;
        if ((mp + xsize) <= maze_top && cell_empty (mp + xsize))
            paths [directions++] = DOWN;
        if (mp > maz && ((mp - maz) % xsize) && cell_empty (mp - 1))
            paths [directions++] = LEFT;

        if (directions) {
            visits--;
            directions = ((unsigned) rn2(directions));

            switch (paths [directions]) {
                case UP:
                    mp->up = 1;
                    (mp -= xsize)->down = 1;
                    break;
                case RIGHT:
                    mp->right = 1;
                    (++mp)->left = 1;
                    break;
                case DOWN:
                    mp->down = 1;
                    (mp += xsize)->up = 1;
                    break;
                case LEFT:
                    mp->left = 1;
                    (--mp)->right = 1;
                    break;
                default:
                    break;
            }
        } else {
            do {
                if (++mp > maze_top)
                    mp = maz;
            } while (cell_empty (mp));
        }
    }
}
