/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "level.h"
#include "log.h"

// Create a normal level with rooms in it
// Rather than just randomly dump rooms onto the map, we split the screen
// up into similaar sized segments, and place a room in each part.
// In other words:
//     Select a random number for the rooms (3 to 9 inclusive)
//     If the number is 3, divide the level in 3 horizontally
//     If the number is 4, choose randomly:
//         Divide the level in 2 horizontally, and 2 vertically
//         Divide the level in 4 horizontally
//     For all higher number of rooms:
//         Divide vertically by two
//         Allocate half rooms to top, half to bottom
//         If it is an odd number, allocate one room randomly top or bottom
//     Create the rooms in each of these sections
// At first it looks complicated, but it generally creates an even spread
// of rooms. Things like gold rooms can just be added randomly, as they
// are small enough
CNormalLevel::CNormalLevel(CLevel *newNext, CBranch *newParent, int newNumber) :
    CLevel(newNext, newParent, newNumber) {
#ifdef DEBUG_LEVEL
    log << "LEVEL: constructing normal level " << newNumber << '\n';
#endif
    regenerate();
    if (rooms != NULL)
        delete rooms;
    rooms = NULL;
}

void CNormalLevel::regenerate() {
    // Clean the level up so there is no junk on it before regenerating.
    clean();

    // Determin the number of rooms
    int nRooms = (rn2(7)) + 3;

    // Determin the number of vertical divisions
    int vDivisions = 1;
    if (nRooms > 4)
        vDivisions = 2;
    else if (nRooms == 4 && (rn2(10)) >= 5)
        vDivisions = 2;

    // Determin the number of rooms in the top and bottom if
    // vDivisions == 2 and the number of rooms is odd
    int nTopRooms = nRooms, nBottomRooms = nRooms;
    if (vDivisions == 2)
        if ((nRooms % 2) == 1) {
            if ((rn2(10)) >= 5) {
                nTopRooms = nRooms / 2;
                nBottomRooms = nRooms - nTopRooms;
            } else {
                nBottomRooms = nRooms / 2;
                nTopRooms = nRooms - nBottomRooms;
            }
        } else
            nTopRooms = nBottomRooms = nRooms / 2;

    /* Create each room. Specifically done Top, then bottom - varies the 
       coridors more than just something like a semi-circle */
    log << "TODO: Change room generation to top,bot,top,bot..." << '\n';
    for (int j = 0; j < vDivisions; j++) {
        int hDivisions = 0;
        if (j == 0) hDivisions = nTopRooms; else hDivisions = nBottomRooms;
        for (int i = 0; i < hDivisions; i++)
            addRoomWithin(i * (WIDTH / hDivisions),
                          j * (HEIGHT / vDivisions),
                          WIDTH / hDivisions,
                          HEIGHT / vDivisions);
    }

    addCorridors();
}
