/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "level.h"
#include "log.h"

CTemplateLevel::CTemplateLevel(CLevel *newNext, CBranch *newParent, int newNumber,
    string nam, string desfil, string idletter, int choicenum) :
    CLevel(newNext, newParent, newNumber) {
#ifdef DEBUG_LEVEL
    log << "LEVEL: constructing template " << nam << '-' << desfil << '\n';
#endif
    name = nam;
    desfile = "./dat/";
    desfile += desfil;
    id = idletter;
    choices = choicenum;
    regenerate();
#ifdef DEBUG_LEVEL
    cout << this;
#endif
}

void CTemplateLevel::regenerate() {
    string section;
    /* Determin the section name to search the file for */
    section = name;
    if (choices > 1) {
        section += '-';
        section += (int)(rn2(choices)+1);
    }

    /* Open the template file */
    if (dfile.start(desfile.getStr()) < 0) {
        log << "ERROR: unable to open " << desfile << "\n";
        exit(0);
    }

    /* Find the template entry in the file */
    while (1) {
        if (dfile.readline() == NULL) {
            log << "ERROR: unable to find " << section << " in "
                 << desfile << "\n";
            exit(1);
        }
        char * templt = dfile.linetype();
        if (templt != NULL && (mystrcmp(templt, "MAZE") == 0 ||
            mystrcmp(templt, "LEVEL") == 0)) {
            char * tempname = dfile.first_field();
            if (tempname == NULL) {
                log << "ERROR: Expected template name in "
                     << desfile << "\n";
                exit(0);
            }
            if (mystrcmp(tempname, section.getStr()) == 0)
                break;
        }
    }

    /* Read each line in and process it */
    char * line, * linetype;
    while (1) {
        line = dfile.readline();
        if (line == NULL) break;
        if (mystrncmp(line, "MAZE", 4) == 0 ||
            mystrncmp(line, "LEVEL", 5) == 0)
            break;
        linetype = dfile.linetype();

        if (linetype == NULL) {
            if (mystrcmp(line, "MAP") == 0)
                constructMap();
            else if (mystrcmp(line, "RANDOM_CORRIDORS") == 0)
                addCorridors();
        } else if (mystrcmp(linetype, "ROOM") == 0)
            taddroom();
        else if (mystrcmp(linetype, "GEOMETRY") == 0)
            setGeometry();
        else if (mystrcmp(linetype, "MONSTER") == 0)
            addMonster();
        else if (mystrcmp(linetype, "OBJECT") == 0)
            addObject();
        else if (mystrncmp(linetype, "OBJECT", 6) == 0)
            addObject2();
        else if (mystrcmp(linetype, "TRAP") == 0)
            addTrap();
        else if (mystrcmp(linetype, "ALTAR") == 0)
            addAltar();
        else if (mystrcmp(linetype, "DOOR") == 0)
            addDoor();
        else if (mystrcmp(linetype, "STAIR") == 0)
            addStair();
        else if (mystrcmp(linetype, "REGION") == 0)
            addRegion();
        else if (mystrcmp(linetype, "FLAGS") == 0)
            addFlags();
        else if (mystrcmp(linetype, "MESSAGE") == 0)
            addMessage();
        else if (mystrcmp(linetype, "TELEPORT_REGION") == 0)
            addTeleportRegion();
        else if (mystrcmp(linetype, "PORTAL") == 0)
            addPortal();
        else if (mystrcmp(linetype, "NON_DIGGABLE") == 0)
            addNonDiggable();
        else if (mystrcmp(linetype, "NON_PASSWALL") == 0)
            addNonPasswall();
        else if (mystrcmp(linetype, "LADDER") == 0)
            addLadder();
        else if (mystrcmp(linetype, "BRANCH") == 0)
            addTBranch();
        else if (mystrcmp(linetype, "DRAWBRIDGE") == 0)
            addDrawBridge();
        else if (mystrcmp(linetype, "FOUNTAIN") == 0)
            addFountain();
        else if (mystrcmp(linetype, "INIT_MAP") == 0)
            addInitMap();
        else if (mystrcmp(linetype, "RANDOM_PLACES") == 0)
            addRandomPlaces();
        else if (mystrcmp(linetype, "RANDOM_MONSTERS") == 0)
            addRandomMonsters();
        else if (mystrcmp(linetype, "RANDOM_OBJECTS") == 0)
            addRandomObjects();
        else if (mystrcmp(linetype, "CONTAINER") == 0)
            addContainer();
        else if (mystrcmp(linetype, "ENGRAVING") == 0)
            addEngraving();
        else if (mystrcmp(linetype, "SUBROOM") == 0)
            taddSubRoom();
        else if (mystrcmp(linetype, "NAME") == 0)
            addName();
        else
            log << "WARNING: unknown template line: " << line << "\n";
    }

    /* Close the template file */
    if (dfile.finish() < 0)
        log << "ERROR: unable to close " << desfile << "\n";
}

uchar lastx, lasty;

/* Format of a room line is:
   ROOM: "name", [un]lit, (XPos, YPos), (XAlign, YAlign), (Width, Height)
   random can be specified for any of these except name */
void CTemplateLevel::taddroom() {
    char * name = dfile.n_field(1);
    char * lit = dfile.n_field(2);
    CLocation pos, size;
    char * tempPos = dfile.n_field(3);
    if (tempPos != NULL) pos = dfile.read_location();
    char * tempSize = dfile.n_field(5);
    if (tempSize != NULL) size = dfile.read_location();
    bool isLit;

    if (pos.x < 0 || pos.y < 0 || size.x < 0 || size.y < 0) {
        int a = pos.x, b = pos.y, c = size.x, d = size.y;
        if (pos.x < 0 || pos.y < 0) a = b = 0;
        if (size.x < 0 || size.y < 0) { c = WIDTH-a; d = HEIGHT-b; };
        addRoomWithin(a, b, c, d);
        lastx = rooms->getX(); lasty = rooms->getY();
        return;
    }

    if (name == NULL || lit == NULL || tempPos == NULL ||
        tempSize == NULL) {
        log << "ERROR: expected:"
             << "ROOM: \"name\", [un]lit, (XPos, YPos), (XAlign, YAlign), (Width, Height)\n";
        exit(-1);
    }

    if (mystrcmp(lit, "random") == 0) {
        if (rn2(100) >= 50) isLit = true;
        else isLit = false;
    } else if (mystrcmp(lit, "lit") == 0)
        isLit = true;
    else
        isLit = false;

    EGeometry rxgeom, rygeom;
    rxgeom = rygeom = EGeomCenter;
    /* Get the x geometry */
    char * entry = dfile.n_field(4);
    if (entry == NULL) return;
    if (mystrcmp(entry, "center") == 0)
        rxgeom = EGeomCenter;
    else if (mystrcmp(entry, "left") == 0)
        rxgeom = EGeomLeft;
    else if (mystrcmp(entry, "right") == 0)
        rxgeom = EGeomRight;
    else if (mystrcmp(entry, "half-left") == 0)
        rxgeom = EGeomHalfLeft;
    else if (mystrcmp(entry, "half-right") == 0)
        rxgeom = EGeomHalfRight;
    else if (mystrcmp(entry, "random") == 0)
        rxgeom = EGeomRandom;
    else
        log << "WARNING: Unknown xgeometry type: " << entry << '\n';

    /* Get the y geometry */
    for ( ; *entry != '\0' && *entry != ',' && *entry != ')'; entry++) ;
    if (entry == NULL) return;
    if (mystrcmp(entry, "center") == 0)
        rygeom = EGeomCenter;
    else if (mystrcmp(entry, "top") == 0)
        rygeom = EGeomTop;
    else if (mystrcmp(entry, "bottom") == 0)
        rygeom = EGeomBottom;
    else if (mystrcmp(entry, "random") == 0)
        rygeom = EGeomRandom;
    else
        log << "WARNING: Unknown ygeometry type: " << entry << '\n';

    size.x += 2;
    size.y += 2;

    /* Now align the room */
    switch (rxgeom) {
        case EGeomCenter:    pos.x = (WIDTH - size.x) / 2; break;
        case EGeomLeft:      pos.x = 0;                    break;
        case EGeomRight:     pos.x = (WIDTH - size.x);     break;
        case EGeomHalfLeft:  pos.x = (WIDTH - size.x) / 4; break;
        case EGeomHalfRight: pos.x = (WIDTH + size.x) / 4; break;
        case EGeomRandom:    pos.x = rn2(WIDTH-size.x);    break;
        default: pos.x = 0;
    };
    switch (rygeom) {
        case EGeomCenter:    pos.y = (HEIGHT - size.y) / 2; break;
        case EGeomTop:       pos.y = 0;                     break;
        case EGeomBottom:    pos.y = (HEIGHT - size.y);     break;
        case EGeomRandom:    pos.y = rn2(HEIGHT-size.y);    break;
        default: pos.y = 0;
    };

    addRoom((lastx = pos.x), (lasty = pos.y), size.x, size.y);
}

/* Construct the map based on the MAP .... ENDMAP entry in the des file */
void CTemplateLevel::constructMap() {
    int lvlWidth = 0, lvlHeight = 0;
    char * line;
    char tempMap[HEIGHT][WIDTH];

    /* Read the map into termporary storage */
    while (1) {
        line = dfile.readline();
        if (line == NULL) break;
        if (mystrncmp(line, "MAZE", 4) == 0 ||
            mystrncmp(line, "LEVEL", 5) == 0) {
            log << "ERROR: ENDMAP expected, " << line << "found\n";
            exit(-1);
        }
        if (mystrcmp(line, "ENDMAP") == 0)
            break;

        lvlWidth = mystrlen(line);
        mystrncpy(tempMap[lvlHeight], line, lvlWidth);
        lvlHeight++;
    }
    if (lvlWidth > WIDTH || lvlHeight > HEIGHT) {
        log << "ERROR: map height or width too large\n";
        exit(-1);
    }

    /* Now add the map to the dungeon (depending on geometry) */
    int xoff = 0, yoff = 0;
    switch (xgeom) {
        case EGeomCenter:    xoff = (WIDTH - lvlWidth) / 2; break;
        case EGeomLeft:      xoff = 0; break;
        case EGeomRight:     xoff = WIDTH - lvlWidth; break;
        case EGeomHalfLeft:  xoff = (WIDTH - lvlWidth) / 4; break; //???
        case EGeomHalfRight: xoff = (WIDTH - lvlWidth) / 4; break; //???
        default: xoff = 0;
    }
    switch (ygeom) {
        case EGeomCenter: yoff = (HEIGHT - lvlHeight) / 2; break;
        case EGeomTop:    yoff = 0; break;
        case EGeomBottom: yoff = HEIGHT - lvlHeight; break;
        default: yoff = 0;
    }
    for (int i = 0; i < lvlWidth; i++)
        for (int j = 0; j < lvlHeight; j++)
            switch (tempMap[j][i]) {
                case '.': map[j+yoff][i+xoff] = EFTFloor; break;
                case ' ': map[j+yoff][i+xoff] = EFTEmpty; break;
                case '|': map[j+yoff][i+xoff] = EFTVWall; break;
                case '-': map[j+yoff][i+xoff] = EFTHWall; break;
                case '+': map[j+yoff][i+xoff] = EFTFloor; break;
                case '}':
                case 'P': map[j+yoff][i+xoff] = EFTWater; break;
                case 'S': map[j+yoff][i+xoff] = EFTFloor; break;
                case 'B': map[j+yoff][i+xoff] = EFTFloor; break;
                case 'C': map[j+yoff][i+xoff] = EFTFloor; break;
                case 'W': map[j+yoff][i+xoff] = EFTFloor; break;
                case 'A': map[j+yoff][i+xoff] = EFTFloor; break;
                case 'L': map[j+yoff][i+xoff] = EFTFloor; break;
                case 'I': map[j+yoff][i+xoff] = EFTFloor; break;
                case '\\': break; // Ignore these two: rest of template
                case '{': break;  // file usually contains these entries
                default: log << "WARNING: Unknown map type: "
                              << tempMap[j][i] << '\n';
            }
}

/* Set the geometry for the template map */
void CTemplateLevel::setGeometry() {
    xgeom = ygeom = EGeomCenter;
    /* Get the x geometry */
    char * entry = dfile.first_field();
    if (entry == NULL) return;
    if (mystrcmp(entry, "center") == 0)
        xgeom = EGeomCenter;
    else if (mystrcmp(entry, "left") == 0)
        xgeom = EGeomLeft;
    else if (mystrcmp(entry, "right") == 0)
        xgeom = EGeomRight;
    else if (mystrcmp(entry, "half-left") == 0)
        xgeom = EGeomHalfLeft;
    else if (mystrcmp(entry, "half-right") == 0)
        xgeom = EGeomHalfRight;
    else
        log << "WARNING: Unknown xgeometry type: " << entry << '\n';

    /* Get the y geometry */
    entry = dfile.next_field();
    if (entry == NULL) return;
    if (mystrcmp(entry, "center") == 0)
        ygeom = EGeomCenter;
    else if (mystrcmp(entry, "left") == 0)
        ygeom = EGeomLeft;
    else if (mystrcmp(entry, "right") == 0)
        ygeom = EGeomRight;
    else if (mystrcmp(entry, "top") == 0)
        ygeom = EGeomTop;
    else if (mystrcmp(entry, "bottom") == 0)
        ygeom = EGeomBottom;
    else
        log << "WARNING: Unknown ygeometry type: " << entry << '\n';
}

void CTemplateLevel::addMonster() {
}

void CTemplateLevel::addObject() {
}

void CTemplateLevel::addObject2() {
}

void CTemplateLevel::addTrap() {
}

void CTemplateLevel::addAltar() {
}

void CTemplateLevel::addDoor() {
}

void CTemplateLevel::addStair() {
}

void CTemplateLevel::addRegion() {
}

void CTemplateLevel::addFlags() {
}

void CTemplateLevel::addMessage() {
}

void CTemplateLevel::addTeleportRegion() {
}

void CTemplateLevel::addPortal() {
}

void CTemplateLevel::addNonDiggable() {
}

void CTemplateLevel::addNonPasswall() {
}

void CTemplateLevel::addTBranch() {
}

void CTemplateLevel::addLadder() {
}

void CTemplateLevel::addDrawBridge() {
}

void CTemplateLevel::addFountain() {
}

void CTemplateLevel::addInitMap() {
}

void CTemplateLevel::addRandomPlaces() {
}

void CTemplateLevel::addRandomMonsters() {
}

void CTemplateLevel::addRandomObjects() {
}

void CTemplateLevel::addContainer() {
}

void CTemplateLevel::addEngraving() {
}

/* Add a subroom to the level based on last room added
   SUBROOM: "name/type", [[un]lit/random], (offset loc), (size), "part of"
   ... possible: MONSTER, CHANCE, ALTAR
   DOOR: bool=hidden, [open/closed], [north/south/east/west/random], random? */
void CTemplateLevel::taddSubRoom() {
    char * name = dfile.n_field(1);
    char * lit = dfile.n_field(2);
    CLocation pos, size;
    char * tempPos = dfile.n_field(3);
    if (tempPos != NULL) pos = dfile.read_location();
    char * tempSize = dfile.n_field(4);
    if (tempSize != NULL) size = dfile.read_location();
    addSubRoom((pos.x+lastx), (pos.y+lasty), size.x+2, size.y+2);

    /* Read in rest of the subroom definition */
    char * line, * linetype;
    while (1) {
        line = dfile.readline();
        if (line == NULL) break;
        if (mystrncmp(line, "MAZE", 4) == 0 ||
            mystrncmp(line, "LEVEL", 5) == 0) {
            log << "ERROR: expected DOOR to end subroom, found next definition" << '\n';
            exit(-1);
        }
        linetype = dfile.linetype();

        if (linetype == NULL) {
            log << "ERROR: expected DOOR to end SUBROOM" << '\n';
            exit(-1);
        } else if (mystrcmp(linetype, "CHANCE") == 0) {
            int n;
            if (dfile.n_field(1) != NULL && (n = dfile.read_number()) < 100 &&
                n > 0) {
                if (rn2(100) < n)
                    log << "CREATE SHOP!!!" << '\n';
            } else {
                log << "ERROR: CHANCE must have 0 < i < 100" << '\n';
                exit(-1);
            }
        } else if (mystrcmp(linetype, "ALTAR") == 0) {
            objects.add((CObject *)new CLevelObject(ELOTAltar));
        } else if (mystrcmp(linetype, "FOUNTAIN") == 0) {
            objects.add((CObject *)new CLevelObject(ELOTFountain));
        } else if (mystrcmp(linetype, "MONSTER") == 0) {
        } else if (mystrcmp(linetype, "DOOR") == 0) {
            objects.add((CObject *)new CLevelObject(ELOTDoor));
            break; /* Now we can exit the subroom definition */
        } else {
            log << "ERROR: expected DOOR, found " << linetype << '\n';
            exit(-1);
        }
    }
}

void CTemplateLevel::addName() {
}
