/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include <stdlib.h>  //for rand()

#include "monlist.h"
#include "level.h"
#include "log.h"
#include "player.h"
#include "plist.h"
#include "vision.h"
#include "monattk.h"
#include "monflag.h"
#include "monsym.h"
#include "mondata.h"
#include "string.h"
#include "pm.h"
#include "moon.h"


CMonsterNode::CMonsterNode(int monst, CMonsterNode * n, ID currentID)
{
    next = n;
    data = new MonsterType(Monsters.DefaultMon[monst]);
    data->IID = currentID;

    /* These need to be changed depending on the monster generated */
    data->mhp = data->mhpmax = 10;
    data->movement = NORMAL_SPEED;
}

CMonsterNode::CMonsterNode(CMonsterNode * n, MonsterType *m) {
    next = n;
    data = m;
}

CMonsterNode::~CMonsterNode() {
    if (next != NULL) delete next; next = NULL;

}

CMonsterList::CMonsterList() {
    head = NULL;
    size = 0;
    currentID = 0;
    current = NULL;
}

CMonsterList::~CMonsterList() {
    if (head != NULL) delete head;
    head = NULL;
    size = 0;
}

//add new monster
int CMonsterList::add(int monst) {
    currentID++;
    head = new CMonsterNode(monst, head, currentID);
    size++;
    return 1;
}


/* Move a monster into the list */
int CMonsterList::add(MonsterType *m) {
    head = new CMonsterNode(head, m);
    size++;
    return 1;
}


//remove a monster
int CMonsterList::remove(ID monst) {
    //modifed find() function
    CMonsterNode *temp = NULL;
    for (current = head; current != NULL;
        last = current, current = current->next)
        if (current->data->IID == monst)
        {
            temp = current;
            break;
        }

    if (temp != NULL)
    {
        if (current == head)
        {
            head = current->next;
            current->next = NULL;
            delete current;
        }
        else
        {
            last->next = current->next;
            current->next = NULL;
            delete current;
        }
        size--;
        return 1;
    }
    return 0;
}

//find the ID of a monster in a specific location
MonsterType * CMonsterList::findatloc(int ycoord, int xcoord) {
    for (current = head; current != NULL; current = current->next)
        if (current->data->xy.y == ycoord &&
            current->data->xy.x == xcoord)
                return current->data;
    return NULL;
}

/* Find a minster of type ID */
MonsterType * CMonsterList::find(ID monst) {
    for (current = head; current != NULL; current = current->next)
        if (current->data->IID == monst)
            return current->data;
    return NULL;
}

/* Create a set of monsters based on level n */
void CMonsterList::createLevel(CLevel *lev) {
	int r = 0;

	for (r = 0; r < (10 + (lev->getNumber() * 2)); r++)
	{
   	float tlevel = (float(lev->getNumber())/55)*30;
		if (tlevel > 15) tlevel = 50;
		int tflag = 0;
      do
		{
			int tmonst = (rand() % 345) + 1;
			if(Monsters.DefaultMon[tmonst].ILevel <= int(tlevel))
			{
				tflag = 1;
				add(tmonst);
				head->data->xy = lev->getRandomAny();
			}
		}while(tflag == 0);
   }
}

/* Remove any monsters that are no longer valid (ie hp < 1) */
void CMonsterList::validMonsters()
{
	if (size < 1) return;
   CMonsterNode * temp;
   CMonsterNode * tempm;
   for (temp = head; temp != NULL; temp = temp->next)
	{
   	if (temp->next && temp->next->data->mhp < 1)
      {
      	tempm = temp->next;
         temp->next = tempm->next;
         tempm->next = NULL;
         delete tempm;
      }

      if (head->data->mhp < 1)
    	{
      	tempm = head;
        	head = tempm->next;
        	tempm->next = NULL;
        	delete tempm;
    	}
   }
}

/* Move a monster with id m from src to dest */
void moveMonster(ID m, CMonsterList src, CMonsterList dest) {
    MonsterType * it = src.find(m);
    if (it == NULL) return;
    dest.add(it);
    src.remove(m);
}

/* Move monsters on a level. This should be called every time a player
   moves for the monsters on the same level as the player.
   For each monster {
       If their player has moved,
           move the monster.
       If monster is closer to another player,
           change who they are assigned to.
   }
   This could be changed so the re-assignment to other players is done
   every X moves a player the monster is assigned to is moved.

   Monster movement is tricky to implement - you have to take into
   account multiple monsters, players, all with different speeds, added
   to that you must not forget about the sureal time arrangements. */
void CMonsterList::movemon()
{
	CMonsterNode *temp;
   MonsterType *mon;
   /* Go through each monster */
   for (temp = head; temp != NULL; temp = temp->next)
   {
   	mon = temp->data;
      /* If the player the monster is assigned to has moved */
      if (mon->player != NULL &&
      	mon->pMoveCount < mon->player->getMoveCount())
      {
      	/* log << "Player has moved\n"; */
         mon->movement += mon->ISpeed;   /* Add the monsters speed to
                                               how much it can move */
         while (mon->movement - mon->player->getSpeed() >= 0)
         {
         	domovemon(mon);
            mon->movement -= mon->player->getSpeed();
         }

         mon->pMoveCount = mon->player->getMoveCount();
      }
      /* Assign monster to closest player... needs to be more efficient */
      if (mon->player != NULL)
      	mon->player = players.assignClosest(mon->player->dlevel, mon->xy);
      else
      	mon->player = players.assignClosest(NULL, mon->xy);
   }
	log << "TURN DONE!!\n";
}

/* Assign each monster to their players */
void CMonsterList::assignToPlayers(CLevel *lev)
{
	CMonsterNode *temp;
   MonsterType *mon;
   /* Go through each monster */
   for (temp = head; temp != NULL; temp = temp->next)
   {
   	mon = temp->data;
      mon->player = players.assignClosest(lev, mon->xy);
   }
}

/* Actually move the monster
   mtmp    pointer to the monster
   ptmp    pointer to the player
   this    pointer to the list the monster is in
   ltmp    pointer to the level the monster (and player) is on

   NOTE:  See Clock algorithm doc for more info on this function
   (That is when I get around to making it)

	7  0  1           (-1)<-X->(1)    (-1)
	 \ | /                              |
	6-----2                             Y
	 / | \                              |
	5  4  3                            (1)
 */
int CMonsterList::domovemon(MonsterType *mtmp)
{
	CPlayer * ptmp = mtmp->player;
   CLevel * ltmp = ptmp->dlevel;
   CLocation oxy = mtmp->xy;

	//For calculating the direction of player below
	int tempvert = 0, temphorz = 0;

	//Previous movement and direction
	int prevx = mtmp->xy.x;
	int prevy = mtmp->xy.y;

	int prevdir = mtmp->DefaultDir;

	//Calculate Default Direction to player
   if (mtmp->xy.x > ptmp->xy.x) temphorz = -1;
   else if (mtmp->xy.x < ptmp->xy.x) temphorz = 1;

   if (mtmp->xy.y > ptmp->xy.y) tempvert = -1;
   else if (mtmp->xy.y < ptmp->xy.y) tempvert = 1;

	if(temphorz == 0 && tempvert == -1) mtmp->DefaultDir = 0;
	if(temphorz == 1 && tempvert == -1) mtmp->DefaultDir = 1;
	if(temphorz == 1 && tempvert == 0) mtmp->DefaultDir = 2;
	if(temphorz == 1 && tempvert == 1) mtmp->DefaultDir = 3;
	if(temphorz == 0 && tempvert == 1) mtmp->DefaultDir = 4;
	if(temphorz == -1 && tempvert == 1) mtmp->DefaultDir = 5;
	if(temphorz == -1 && tempvert == 0) mtmp->DefaultDir = 6;
	if(temphorz == -1 && tempvert == -1) mtmp->DefaultDir = 7;

	//If rotation isnt set then randonly choose a rotation (CW or CCW)
	if(mtmp->rotation == 0)
	{
        //Randomly choose a rotation pattern
		int temp = (rand() % 2) + 1;

		if(temp == 1) mtmp->rotation = -1;
		else mtmp->rotation = 1;


		//Set Queue
		//Take intial direction and fan out until a legal move is possible
		mtmp->DirQueue[0] = mtmp->DefaultDir;
		for(int tloop=0;tloop<3;tloop++)
		{
			if(mtmp->rotation == -1)
			{
				mtmp->DirQueue[(tloop*2+1)] = mtmp->DefaultDir -1-tloop;
				if(mtmp->DirQueue[(tloop*2+1)] < 0) mtmp->DirQueue[(tloop*2+1)] += 8;

				mtmp->DirQueue[(tloop*2+1)] = mtmp->DefaultDir +1+tloop;
				if(mtmp->DirQueue[(tloop*2+2)] > 7) mtmp->DirQueue[(tloop*2+2)] -= 8;
			}
			else
			{
				mtmp->DirQueue[(tloop*2+1)] = mtmp->DefaultDir +1+tloop;
				if(mtmp->DirQueue[(tloop*2+1)] > 7) mtmp->DirQueue[(tloop*2+1)] -= 8;

				mtmp->DirQueue[(tloop*2+2)] = mtmp->DefaultDir -1-tloop;
				if(mtmp->DirQueue[(tloop*2+2)] < 0) mtmp->DirQueue[(tloop*2+2)] += 8;
			}
		}
		mtmp->DirQueue[7] = mtmp->DefaultDir - 4;
		if(mtmp->DirQueue[7] < 0) mtmp->DirQueue[7] += 8;

	}
	//If a rotation was already decided then just use clock algorithm
	else if(mtmp->rotation != 0 && mtmp->DefaultDir != prevdir)
	{
		for(int tloop=0;tloop<8;tloop++)
		{
			if(mtmp->rotation == 1)
			{
				mtmp->DirQueue[tloop] = mtmp->DefaultDir + tloop;
				if(mtmp->DirQueue[tloop] > 7) mtmp->DirQueue[tloop] -=8;
			}
			else
			{
				mtmp->DirQueue[tloop] = mtmp->DefaultDir - tloop;
				if(mtmp->DirQueue[tloop] < 0) mtmp->DirQueue[tloop] +=8;
			}
		}
	}

   int flag;
	int z;
	for(z = 0; z<8;z++)
	{
		//flag = 2 is no valid move
		flag = 2;
		int tempx = 0;
		int tempy = 0;
		switch(mtmp->DirQueue[z])
		{
			case 0:
				tempx = 0;
				tempy = -1;
				break;
			case 1:
				tempx = 1;
				tempy = -1;
				break;
			case 2:
				tempx = 1;
				tempy = 0;
				break;
			case 3:
				tempx = 1;
				tempy = 1;
				break;
			case 4:
				tempx = 0;
				tempy = 1;
				break;
			case 5:
				tempx = -1;
				tempy = 1;
				break;
			case 6:
				tempx = -1;
				tempy = 0;
				break;
			case 7:
				tempx = -1;
				tempy = -1;
				break;
		}
		//4 main checks: 1. On a floor 2. Not a previous movement 3. Not another Monster
        //4. Check for player/check to see if a player is going to be hit\

        //Check 1
      if((ltmp->map[(mtmp->xy.y + tempy)][(mtmp->xy.x + tempx)] == EFTFloor ||
			ltmp->map[(mtmp->xy.y + tempy)][(mtmp->xy.x + tempx)] == EFTCorridor))
		{
			flag = 0;
			//Check 2
			for(int loop=0;loop<4;loop++)
			{
				if(mtmp->MonPrevMove[loop].x == (mtmp->xy.x + tempx) && mtmp->MonPrevMove[loop].y == (mtmp->xy.y + tempy))
				{
					flag = 2;
               break;
				}
			}
			//Check 3

			MonsterType *mon;
			/* Go through each monster */

			for (current = head; current != NULL; current = current->next)
			{
         	mon = current->data;
           
        		if (mon->xy.x == mtmp->xy.x + tempx && mon->xy.y == mtmp->xy.y + tempy &&
         		mon != mtmp)
            {
            	flag = 2;
               break;
				}
			}

			//Check 4
			for(int ploop = 0;ploop < MAXPLAYERS; ploop++)
			{
         	if(plist[ploop] != NULL)
            {
            	if(plist[ploop]->dlevel == ptmp->dlevel)
					{
            		for (current = head; current != NULL; current = current->next)
						{
                  	mon = current->data;
                     if (mon->xy.x + tempx == plist[ploop]->xy.x && mon->xy.y + tempy == plist[ploop]->xy.y)
                     {
                     	flag = ploop + 4;
                        break;
                     }
                  }
					}
				}
			}


			if(flag == 0)
			{
				mtmp->xy.x += tempx;
				mtmp->xy.y += tempy;
				flag = 1;
			}

		}
		if (flag == 1 || flag > 3) break;
	}

	//Straight Movement.  If monster is moving straight at the player
	//for a while, then reset rotation
	if(z == 0) mtmp->StraightMove += 2;
	else if(z == 1 || z == 2) mtmp->StraightMove++;
	else mtmp->StraightMove--;

	if(mtmp->StraightMove >= 8 || mtmp->StraightMove <= -8)
	{
		mtmp->StraightMove = 0;
		mtmp->rotation = 0;
	}

	//If you are moving completely backwards then the monster is stuck, or if it is close
	int tmpdist = (abs(mtmp->xy.x - ptmp->xy.x) + abs(mtmp->xy.y - ptmp->xy.y));

	if((z >= 8) || tmpdist < 5)
	{
		mtmp->StraightMove = 0;
		mtmp->rotation = 0;
		mtmp->MonPrevMove[0].x = -1;
		mtmp->MonPrevMove[1].x = -1;
		mtmp->MonPrevMove[2].x = -1;
		mtmp->MonPrevMove[3].x = -1;
		mtmp->MoveCycle = 0;
	}

   if(flag == 1)
	{
		//Log previous move
		mtmp->MonPrevMove[mtmp->MoveCycle].x = prevx;
		mtmp->MonPrevMove[mtmp->MoveCycle].y = prevy;

		mtmp->MoveCycle++;
		if(mtmp->MoveCycle > 3) mtmp->MoveCycle = 0;

		/* Update the monster if they have moved */
		players.updateMonster(ltmp, mtmp);
	}
	else if(flag > 3)
   {
   	/* Hit the player! Yay, the battles start :) */

		/*This is to tell which player the monster actually hit.
		  It could be the fact that the monster "ran" into another player other
		  than its target.*/
		ptmp = plist[flag - 4];

      /*Its the monster's turn to kick butt*/
      for(int attcycle = 0; attcycle < 6; attcycle++)
      {
      	int didhit[attcycle];
         int tempdamage;
         if(mtmp->IAttarea[attcycle] != 0 || mtmp->IAtttype[attcycle] != 0)
         {
      		//int * mTHAC0;
            

         	bool ranged = (mondist(mtmp->xy, ptmp->xy) > 3);
            /* Is it near you?  Affects your actions */
            bool range2 = (mondist(mtmp->xy, ptmp->xy) < 3);
            /* Does it think it's near you?  Affects its actions */
            bool foundyou = (mtmp->xy.x == ptmp->xy.x && mtmp->xy.y == ptmp->xy.y);
            /* Is it attacking you or your image? */
            bool youseeit = canseemon(mtmp, ptmp, ltmp);
            /* Might be attacking your image around the corner, or
            * invisible, or you might be blind....
            */

            //If already dead or drowning
            if(mtmp->mhp <= 0 || (mtmp->underwater && !mtmp->is_swimmer))
            	return 1;

            /* If swallowed, can only be affected by u.ustuck */
            if(ptmp->swallow)
            {
            	if(mtmp != ptmp->ptmpstuck.stuckmon)
               	return 1;
               ptmp->ptmpstuck.xy.x = ptmp->xy.x;
               ptmp->ptmpstuck.xy.y = ptmp->xy.y;
               range2 = 0;
               foundyou = 1;
               if(ptmp->invulnerable)
               	return (1); /* stomachs can't hurt you! */
            }


            //If monster is your steed it doesnt attack
            if (!rn2(is_orc(mtmp) ? 2 : 4) &&
            mondist(mtmp->xy, ptmp->xy) <= 2)
            {
            	if(mtmp->IID == ptmp->steed->IID)
            		return 1;
            }

            //Implement later - if an orc then attack your steed instead
            if(ptmp->steed && is_orc(mtmp) && mondist(mtmp->xy, ptmp->xy) <=2)
            {	/*mattackm(mtmp, ptmp->steed)*/}


//TODO: POLYMORPHING

/*	#ifdef STEED
	else if (u.usteed) {
		if (mtmp == u.usteed)
			/* Your steed won't attack you
			return (0);
		/* Orcs like to steal and eat horses and the like
		if (!rn2(is_orc(mtmp->data) ? 2 : 4) &&
				mondist(mtmp->xy, ptmp->xy) <=2 {
			/* Attack your steed instead
			i = mattackm(mtmp, u.usteed);
			if ((i & MM_AGR_DIED))
				return (1);
			if (i & MM_DEF_DIED)
				return (0);
			/* Let your steed retaliate
			return (!!(mattackm(u.usteed, mtmp) & MM_DEF_DIED));
		}
	}
#endif

	if (u.uundetected && !range2 && foundyou && !u.uswallow) {
		u.uundetected = 0;
		if (is_hider(youmonst.data)) {
		    coord cc; /* maybe we need a unexto() function?

		    You("fall from the %s!", ceiling(u.ux,u.uy));
		    if (enexto(&cc, u.ux, u.uy, youmonst.data)) {
			remove_monster(mtmp->mx, mtmp->my);
			newsym(mtmp->mx,mtmp->my);
			place_monster(mtmp, u.ux, u.uy);
			if(mtmp->wormno) worm_move(mtmp);
			teleds(cc.x, cc.y);
			set_apparxy(mtmp);
			newsym(u.ux,u.uy);
		    } else {
			pline("%s is killed by a falling %s (you)!",
						Monnam(mtmp), youmonst.data->mname);
			killed(mtmp);
			newsym(u.ux,u.uy);
			if (mtmp->mhp > 0) return 0;
			else return 1;
		    }
		    if (youmonst.data->mlet != S_PIERCER)
			return(0);	/* trappers don't attack

		    if (which_armor(mtmp, WORN_HELMET)) {
			Your("blow glances off %s helmet.",
			               s_suffix(mon_nam(mtmp)));
		    } else {
			if (3 + find_mac(mtmp) <= rnd(20)) {
			    pline("%s is hit by a falling piercer (you)!",
								Monnam(mtmp));
			    if ((mtmp->mhp -= d(3,6)) < 1)
				killed(mtmp);
			} else
			  pline("%s is almost hit by a falling piercer (you)!",
								Monnam(mtmp));
		    }
		} else {
		    if (!youseeit)
			pline("It tries to move where you are hiding.");
		    else {
			/* Ugly kludge for eggs.  The message is phrased so as
			 * to be directed at the monster, not the player,
			 * which makes "laid by you" wrong.  For the
			 * parallelism to work, we can't rephrase it, so we
			 * zap the "laid by you" momentarily instead.

			struct obj *obj = level.objects[u.ux][u.uy];

			if (obj ||
			      (youmonst.data->mlet == S_EEL && is_pool(u.ux, u.uy))) {
			    int save_spe = 0; /* suppress warning
			    if (obj) {
				save_spe = obj->spe;
				if (obj->otyp == EGG) obj->spe = 0;
			    }
			    if (youmonst.data->mlet == S_EEL)
		pline("Wait, %s!  Theres a hidden %s named %s there!",
				m_monnam(mtmp), youmonst.data->mname, plname);
			    else
	     pline("Wait, %s!  Theres a %s named %s hiding under %s!",
				m_monnam(mtmp), youmonst.data->mname, plname,
				doname(level.objects[u.ux][u.uy]));
			    if (obj) obj->spe = save_spe;
			} else
			    impossible("hiding under nothing?");
		    }
		    newsym(u.ux,u.uy);
		}
		return(0);
	}
	if (youmonst.data->mlet == S_MIMIC && youmonst.m_ap_type && !range2 && foundyou && !u.uswallow) {
		if (!youseeit) pline("It gets stuck on you.");
		else pline("Wait, %s!  Thats a %s named %s!",
			   m_monnam(mtmp), youmonst.data->mname, plname);
		u.ustuck = mtmp;
		youmonst.m_ap_type = M_AP_NOTHING;
		youmonst.mappearance = 0;
		newsym(u.ux,u.uy);
		return(0);
	}

	/* player might be mimicking gold
	if (youmonst.m_ap_type == M_AP_OBJECT
		&& youmonst.mappearance == GOLD_PIECE
		&& !range2 && foundyou && !u.uswallow) {
	    if (!youseeit)
		 pline("%s %s!", Something, likes_gold(mtmp->data) ?
			"tries to pick you up" : "disturbs you");
	    else pline("Wait, %s!  That gold is really %s named %s!",
			m_monnam(mtmp),
			an(mons[u.umonnum].mname),
			plname);
	    if (multi < 0) {	/* this should always be the case
		char buf[BUFSIZ];
		Sprintf(buf, "You appear to be %s again.",
			Upolyd ? (const char *) an(youmonst.data->mname) :
			    (const char *) "yourself");
		unmul(buf);	/* immediately stop mimicking gold
	    }
	    return 0;
	}*/

//END POLYMORPHING
            
				int mTHAC0 = ptmp->attr.getac() + 10;
            mTHAC0 += mtmp->ILevel;
            if(mtmp->multi < 0) mTHAC0 += 4;
            if((mtmp->invis && !percieves(mtmp)) || !mtmp->cansee)
            	mTHAC0 -= 2;
            if(mtmp->trapped)
            	mTHAC0 -= 2;
            if(mTHAC0 <= 0)
            	mTHAC0 = 1;

            /* make eels visible the moment they hit/miss us */
            if(mtmp->CSpecies == S_EEL && mtmp->invis && cansee(mtmp, ptmp, ltmp))
            {
            	mtmp->invis = 0;
            }

            //TODO: fix PM_ so that they become the INumber
            /*	Special demon handling code */
            if(!mtmp->cham && is_demon(mtmp) && !range2
            	&& mtmp->INumber != PM_BALROG
               && mtmp->INumber != PM_SUCCUBUS
               && mtmp->INumber != PM_INCUBUS)
            {
            	if(!mtmp->mcan && !rn2(13))
	       				msummon(mtmp);
            }

				/*	Special lycanthrope handling code */
				if(!mtmp->cham && is_were(mtmp) && !range2)
				{
	    			if(is_human(mtmp))
					{
						if(!rn2(5 /*- (moon->night() * 2)*/) && !mtmp->mcan)
               		new_were(mtmp, ptmp);
      			}
					else if(!rn2(30) && !mtmp->mcan)
               	new_were(mtmp, ptmp);

					if(!rn2(10) && !mtmp->mcan)
					{
						if(ptmp->youseeit)
						{
							string S = mtmp->SName += "summons help!";
							vision.topline(S);
						}
						else
						{
							string S = "You feel hemmed in.";
                  	vision.topline(S);
						}

						/* Technically wrong; we really should check if you can see the
						* help, but close enough...
						*/
               	if (!were_summon(mtmp,FALSE) && ptmp->youseeit)
               	{
               		string S = "But none comes.";
							vision.topline(S);
						}
					}
				}

				if(ptmp->invulnerable)
            {
	    			/* monster's won't attack you */
	    			if(mtmp == ptmp->ptmpstuck.stuckmon)
					{
						string S = mtmp->SName += "loosens its grip slightly.";
						vision.topline(S);
					}
	    			else if(!range2)
					{
						if(ptmp->youseeit)
						{
							string S = mtmp->SName += " starts to attack you, but pulls back.";
							vision.topline(S);
						}
						else
						{
							string S = "You feel something move nearby.";
		    				vision.topline(S);
	    				}
               }
               return (1);
            }

				/* Unlike defensive stuff, don't let them use item _and_ attack. */
   			//TODO: Weapon
				/*if(find_offensive(mtmp))
				{
					int foo = use_offensive(mtmp);
					if (foo != 0) return(1);
				}*/
           
          	switch(mtmp->IAtttype[attcycle])
          	{

					case AT_TENT:
					{
						if(!range2 && (/*!MON_WEP(mtmp) || */mtmp->mconf || mtmp->conflict /*||
							//TODO: polymorph(youmonst)
							/*!touch_petrifies(youmonst.data)*/))
						{
               		int j;
							if (mtmp->foundyou)
							{
								if(mTHAC0 > ( j = (rand() % (20+attcycle))))
								{
									if (/*!thick_skinned(youmonst.data)*/0) {}
										//TODO: Fix
										//sum[i] = hitmu(mtmp, mattk);
								}
								else
									missmu(ptmp, mtmp, (mTHAC0 == j), mtmp->IAtttype[attcycle]);
							}
							//else
							//wildmiss(ptmp, mtmp, ltmp, mtmp->IAtttype[attcycle]);
						}
						break;
            	}

					case AT_HUGS:
					{
						/* automatic if prev two attacks succeed */
						/* Note: if displaced, prev attacks never succeeded */
						if((!range2 && attcycle>=2 && didhit[attcycle-1] && didhit[attcycle-2])
							|| mtmp == ptmp->ptmpstuck.stuckmon)
							didhit[attcycle]= /*hitmu(mtmp, mattk)*/0;
						break;
					}

					case AT_GAZE:
					{
						/* can affect you either ranged or not */
						/* Medusa gaze already operated through m_respond in
						*dochug(); don't gaze more than once per round.
						*/
						if (mtmp->INumber != PM_MEDUSA)
							gazemu(mtmp, mtmp->IAtttype[attcycle]);
						break;
					}

					case AT_EXPL:
					{
						/* automatic hit if next to, and aimed at you */
						if(!range2) /*explmu(mtmp, mattk, foundyou)*/{};
						break;
					}

					case AT_ENGL:
					{
						if (!range2)
						{
               		int j;
							if(foundyou)
							{
								if(ptmp->swallow || mTHAC0 > (j = (rand() % (20+attcycle))))
                     	{
									/* Force swallowing monster to be
									* displayed even when player is
									* moving away */
									//TODO
									//flush_screen(1);
									gulpmu(mtmp, mtmp->IAtttype[attcycle]);
								}
								else
								{
									missmu(ptmp, mtmp, (mTHAC0 == j), mtmp->IAtttype[attcycle]);
								}
							}
							else if (is_animal(mtmp))
							{
								//TODO: id nivis change SName to "It"
                  		string S2 = mtmp->invis ? "It" : mtmp->SName;
								string S = S2 += " gulps some air!";
								vision.topline(S);
							}
							else if (ptmp->youseeit)
							{
								string S = mtmp->SName += " lunges forward and recoils!";
								vision.topline(S);
							}
							else
							{
                  		//TODO
                  		//string S2 = is_whirly(mtmp) ? "rushing noise" : "splat";
                     	string S2 = "rushing noise";
								string S = "You hear a ";
                     	string S3 = " nearby.";
                     	S = S += S2 += S3;
								vision.topline(S);
							}
                  }
						break;
					}

					case AT_BREA:
					{
						if(range2) {}//breamu(mtmp, mattk);
						/* Note: breamu takes care of displacement */
						break;
					}

					case AT_SPIT:
					{
						if(range2){}
						/*spitmu(mtmp, mtmp->IAtttype[attcycle]);*/
						/* Note: spitmu takes care of displacement */
						break;
					}

					case AT_WEAP:
					{
						//TODO: Later
					}

					case AT_MAGC:
					{
						//TODO: Later
					}
            }


            //TODO: If successful attack
            if (ptmp->sleep && ptmp->sleep < mtmp->movement && !rn2(10))
            {
					ptmp->multi = -1;
               string S = "The combat suddenly awakens you.";
               vision.topline(S);
            }
          
            int mRoll = (rand() % 20) + 1;
            vision.flushtop();
            if (mRoll > mTHAC0)
            {
            	log << "Hitting!";
            	//HIT
               //didhit[attcycle] = 1;
               tempdamage = 0;
               tempdamage = hitmu(mtmp, ptmp, mtmp->IAtttype[attcycle]);
               /*for(int tempattknum = 0; tempattknum < mtmp->IAttnum[attcycle]; tempattknum++)
               {
               	tempdamage = tempdamage + ((rand() % mtmp->IAttdam[attcycle]) +1);
               }
               */
               string SAttarea = AttackType(mtmp->IAttarea[attcycle]);

               string SAttfinal = " ";
               SAttfinal = string("A ") += string(mtmp->SName) += string(SAttarea) += string("you for ") += string(tempdamage) += string(" points of damage.");
               //players.yellMsg(SAttfinal,SAttfinal,SAttfinal,int(ptmp->id));
               vision.topline(SAttfinal);

            }
            else
            {
               log << "missing";
            	//MISS
               //didhit[attcycle] = 0;
               missmu(ptmp, mtmp, (mTHAC0 == (mRoll - 1)), mtmp->IAtttype[attcycle]);
              
            }
			}
		}
	}
}


//TODO: ALIGN EVERYTHING

int CMonsterList::mondist(CLocation mxy, CLocation pxy)
{
	return (abs(mxy.x - pxy.x) + abs(mxy.y + pxy.y));
}

//TODO: find the difference between cansee() and canseemon()
int CMonsterList::cansee(MonsterType *mtmp, CPlayer *ptmp, CLevel *ltmp)
{
	if (ltmp->blocked_path(mtmp->xy.x, mtmp->xy.y, ptmp->xy.x, ptmp->xy.y))
   	return 0;
   else
   	return 1;
}

int CMonsterList::canseemon(MonsterType *mtmp, CPlayer *ptmp, CLevel *ltmp)
{
	if (ltmp->blocked_path(mtmp->xy.x, mtmp->xy.y, ptmp->xy.x, ptmp->xy.y))
   	return 0;
   else
   	return 1;
}

//Make global
/*int CMonsterList::rn2(int temprand)
{
	int temp = (rand() % tamprand);
   if(temp == 0) return 1;
   else return 0;
}*/

int CMonsterList::missmu(CPlayer * ptmp, MonsterType *mtmp, int nearmiss, int mattk)		/* monster missed you */
{
	/*if (!canspotmon(mtmp))
	    map_invisible(mtmp->mx, mtmp->my);

	if(could_seduce(mtmp, &youmonst, mattk) && !mtmp->mcan)
	    pline("%s pretends to be friendly.", Monnam(mtmp));
	else {
	    if (!flags.verbose || !nearmiss)
		pline("%s misses.", Monnam(mtmp));
	    else
		pline("%s just misses!", Monnam(mtmp));
	}*/
   string S;
	if (nearmiss)
		S = mtmp->SName += " just misses!";
	else
		S = mtmp->SName += " misses.";

	vision.topline(S);
	return 1;
}


int wildmiss(CPlayer *ptmp, MonsterType *mtmp, CLevel *ltmp, int mattk)		/* monster attacked your displaced image */
{
	int compat;

	/* no map_invisible() -- no way to tell where _this_ is coming from */

	//if (!flags.verbose) return;
	//if (!cansee(mtmp, ptmp, ltmp)) return 0;  //TODO:Check this, this was removed due to implicit dec
		/* maybe it's attacking an image around the corner? */

	//compat = (mattk->adtype == AD_SEDU || mattk->adtyp == AD_SSEX) &&
	//	 could_seduce(mtmp, &youmonst, (struct attack *)0);

	if (!mtmp->cansee || (mtmp->invis && !perceives(mtmp)))
	{
	    string swings =
		mattk == AT_BITE ? "snaps" :
		mattk == AT_KICK ? "kicks" :
		(mattk == AT_STNG ||
		 mattk == AT_BUTT ||
		 nolimbs(mtmp)) ? "lunges" : "swings";

	    //if (compat)
		//pline("%s tries to touch you and misses!", Monnam(mtmp));
	    //else
      string S;
		switch(rn2(3))
		{

			case 0:
         	S = mtmp->SName += swings += " wildly and misses!";
				break;
			case 1:
            S = mtmp->SName += " attacks a spot beside you.";
				break;
			case 2:
         	S = mtmp->SName += " strikes at thin air!";
				//TODO: fix to thin air or empty water
				/*pline("%s strikes at %s!", Monnam(mtmp),
				levl[mtmp->mux][mtmp->muy].typ == WATER
				    ? "empty water" : "thin air");*/
				break;
			default:
         	S = mtmp->SName += swings += " wildly!";
				break;
		}
		vision.topline(S);

	}
	else if (ptmp->displaced)
	{
	    //if (compat)
		//pline("%s smiles %s at your %sdisplaced image...",
		//	Monnam(mtmp),
		//	compat == 2 ? "engagingly" : "seductively",
		//	Invis ? "invisible " : "");
	    //else
		string S = mtmp->SName += " strikes at your displaced image and misses you!";
		//pline("%s strikes at your %sdisplaced image and misses you!",
			/* Note: if you're both invisible and displaced,
			 * only monsters which see invisible will attack your
			 * displaced image, since the displaced image is also
			 * invisible.
			 */
			//Monnam(mtmp),
		//TODO: implement line below
			//Invis ? "invisible " : "");
		vision.topline(S);

	}
	else if (ptmp->underwater)
	{
	    /* monsters may miss especially on water level where
	       bubbles shake the player here and there */
	    //if (compat)
		//pline("%s reaches towards your distorted image.",Monnam(mtmp));
	    //else
		string S = mtmp->SName += "is fooled by water reflections and misses!";
		vision.topline(S);
	}
	else
	{
		string S = mtmp->SName += "attacks you without knowing your location?";
		vision.topline(S);
	}
}

int CMonsterList::msummon(MonsterType *mtmp)		/* ptr summons a monster */
{
	int dtype = 0, cnt = 0;
	int atyp = mtmp->IAlignment;
   MonsterType * tmon;

	if (is_dprince(mtmp) || (mtmp->INumber == PM_WIZARD_OF_YENDOR))
   {
	    dtype = (!rn2(20)) ? dprince(atyp) :
				 (!rn2(4)) ? dlord(atyp) : ndemon(atyp);
       tmon = &Monsters.DefaultMon[dtype];
	    cnt = (!rn2(4) && !is_ndemon(tmon)) ? 2 : 1; //is_ndemon(dtype) is that right?
	}
   else if (is_dlord(mtmp))
   {
	    dtype = (!rn2(50)) ? dprince(atyp) :
				 (!rn2(20)) ? dlord(atyp) : ndemon(atyp);
       tmon = &Monsters.DefaultMon[dtype];
	    cnt = (!rn2(4) && is_ndemon(tmon)) ? 2 : 1;
	}
   else if (is_ndemon(mtmp))
   {
	    dtype = (!rn2(20)) ? dlord(atyp) :
				 (!rn2(6)) ? ndemon(atyp) : monsndx(mtmp);
       tmon = &Monsters.DefaultMon[dtype];
	    cnt = 1;
	}
   else if (is_lminion(mtmp))
   {
	    dtype = (is_lord(mtmp) && !rn2(20)) ? llord() :
		     (is_lord(mtmp) || !rn2(6)) ? lminion() : monsndx(mtmp);
       tmon = &Monsters.DefaultMon[dtype];
	    cnt = (!rn2(4) && !is_lord(tmon)) ? 2 : 1;
	}

	if (!dtype) return 0;

	/* sanity checks */
	//if (cnt > 1 && (mons[dtype].geno & G_UNIQ)) cnt = 1;
	/*
	 * If this daemon is unique and being re-summoned (the only way we
	 * could get this far with an extinct dtype), try another.
	 */
	//if (mvitals[dtype].mvflags & G_GONE) {
	//    dtype = ndemon(atyp);
	//    if (!dtype) return;
	//}

	while (cnt > 0) {
	    add(dtype); //TODO: make this into addsummon() to place it near the summoner
	    cnt--;
	}
	return 1;
}

int CMonsterList::set_mon_data(MonsterType *mtmp, MonsterType *mtmp2, int flag)
{
    mtmp = mtmp2;
    if (flag == -1) return 0;		//dont care

    if (flag == 1)
	mtmp->IIntrinsics |= (mtmp2->IResist & 0x00FF);
    else
	mtmp->IIntrinsics = (mtmp2->IResist & 0x00FF);
    return 1;
}

//TODO: reorganize
#include "were.cc"

string CMonsterList::AttackType(int attk)
{
    string SAttarea = " doesnt really hit ";
	switch(attk)
	{
		case AT_CLAW: SAttarea = " claws "; break;
		case AT_BITE: SAttarea = " bites ";break;
		case AT_KICK: SAttarea = " kicks ";break;
		case AT_BUTT: SAttarea = " butts ";break;
		case AT_TUCH: SAttarea = " touches ";break;
		case AT_STNG: SAttarea = " stings ";break;
		case AT_HUGS: SAttarea = " crushes ";break;
		case AT_SPIT: SAttarea = " spits at ";break;
		case AT_ENGL: SAttarea = " engulfs ";break;
		case AT_BREA: SAttarea = " breathes on ";break;
		case AT_EXPL: SAttarea = " explodes on ";break;
		case AT_BOOM: SAttarea = " kamikazied ";break;
		case AT_GAZE: SAttarea = " looks at ";break;
		case AT_TENT: SAttarea = " tentacled ";break;
		case AT_WEAP: SAttarea = " weapon-hits ";break;
		case AT_MAGC: SAttarea = " casts on ";break;
	}
	return SAttarea;
}

/*
 * hitmu: monster hits you
 *	  returns 2 if monster dies (e.g. "yellow light"), 1 otherwise
 *	  3 if the monster lives but teleported/paralyzed, so it can't keep
 *	       attacking you
 */

int CMonsterList::hitmu(MonsterType * mtmp, CPlayer * ptmp, int mattk)
{
	CLevel * ltmp = ptmp->dlevel;
	int dmg, armpro, ptemp;
   int attcycle = mattk;

//	struct permonst *olduasmon = youmonst.data;
	int res;

//	if (!canspotmon(mtmp))
//	    map_invisible(mtmp->mx, mtmp->my);

/*	If the monster is undetected & hits you, you should know where
 *	the attack came from.
 */
	if(mtmp->mundetected && (hides_under(mtmp) || mtmp->CSpecies == S_EEL))
	{
	    mtmp->mundetected = 0;
	    /*if (!(ptmp->blind ? ptmp->blind_telepat : ptmp->unblind_telepat))
		{
			struct obj *obj;
			const char *what;

		if ((obj = level.objects[mtmp->mx][mtmp->my]) != 0) {
		    if (Blind && !obj->dknown)
			what = something;
		    else if (is_pool(mtmp->mx, mtmp->my) && !Underwater)
			what = "the water";
		    else
			what = doname(obj);

		    pline("%s was hidden under %s!", Amonnam(mtmp), what);
		}
		newsym(mtmp->mx, mtmp->my);
	    }*/
	}

/*	First determine the base damage done */
	dmg = 0;
	for(int tempattknum = 0; tempattknum < mtmp->IAttnum[attcycle]; tempattknum++)
	{
   		dmg = ((rand() % mtmp->IAttdam[attcycle]) +1);
		if(is_undead(mtmp) /*&& moon->midnight()*/)
      	dmg += ((rand() % mtmp->IAttdam[attcycle]) +1);
	}

/*	Next a cancellation factor	*/
/*	Use uncancelled when the cancellation factor takes into account certain
 *	armor's special magic protection.  Otherwise just use !mtmp->mcan.
 */
/*	armpro = 0;
	if (uarm && armpro < objects[uarm->otyp].a_can)
		armpro = objects[uarm->otyp].a_can;
	if (uarmc && armpro < objects[uarmc->otyp].a_can)
		armpro = objects[uarmc->otyp].a_can;
	if (uarmh && armpro < objects[uarmh->otyp].a_can)
		armpro = objects[uarmh->otyp].a_can;
	uncancelled = !mtmp->mcan && ((rn2(3) >= armpro) || !rn2(50));
*/
/*	Now, adjust damages via resistances or specific attacks */
	switch(mtmp->IAtttype[attcycle])
	{
   	case AD_PHYS:
			if (mtmp->IAttdam[attcycle] == AT_HUGS /*&& !sticks(youmonst.data)*/)
			{
				if(!ptmp->ptmpstuck.stuckmon && rn2(2))
				{
            	//OBJECT
					/*if (u_slip_free(mtmp, mattk))
					{
						dmg = 0;
					}
					else
					{*/
						ptmp->ptmpstuck.stuckmon = mtmp;
						string S = mtmp->SName += " grabs you!";
          				vision.topline(S);
					//}
				}
				else if(ptmp->ptmpstuck.stuckmon == mtmp)
				{
					exercise(A_STR, FALSE);
					string S = "You are being choked.";
          			vision.topline(S);
				//TODO
				/*	You("are being %s.",
			      (mtmp->data == &mons[PM_ROPE_GOLEM])
			      ? "choked" : "crushed");*/
				}
			}
			/*else
			{			  /* hand to hand weapon
				if(mtmp->IAttdam[attcycle] == AT_WEAP && otmp)
				{
					if (otmp->otyp == CORPSE
					&& touch_petrifies(&mons[otmp->corpsenm]))
					{
						dmg = 1;
						pline("%s hits you with the %s corpse.",
						Monnam(mtmp), mons[otmp->corpsenm].mname);
						if (!Stoned)
						goto do_stone;
					}
					dmg += dmgval(otmp, &youmonst);
					if (dmg <= 0) dmg = 1;
					if (!(otmp->oartifact &&
					artifact_hit(mtmp, &youmonst, otmp, &dmg,dieroll)))
					hitmsg(mtmp, mattk);
					if (!dmg) break;
					if (u.mh > 1 && u.mh > ((u.uac>0) ? dmg : dmg+u.uac) &&
						(u.umonnum==PM_BLACK_PUDDING
						|| u.umonnum==PM_BROWN_PUDDING))
					{
						/* This redundancy necessary because you have to
						* take the damage _before_ being cloned.

						if (u.uac < 0) dmg += u.uac;
						if (dmg < 1) dmg = 1;
						if (dmg > 1) exercise(A_STR, FALSE);
						u.mh -= dmg;
						flags.botl = 1;
						dmg = 0;
						if(cloneu())
						You("divide as %s hits you!",mon_nam(mtmp));
					}
					urustm(mtmp, otmp);
				}
				*/
       	else if (mtmp->IAttdam[attcycle] != AT_TUCH || dmg != 0 ||
					mtmp != ptmp->ptmpstuck.stuckmon)
					hitmsg(mtmp, mattk);
			break;

		case AD_DISE:
			hitmsg(mtmp, mattk);
			if (!diseasemu(mtmp, ptmp)) dmg = 0;
			break;

      case AD_FIRE:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled)
			{
				string S = "Youre being roasted!";

            vision.topline(S);
				//TODO
				/*pline("Youre %s!",
				mattk->aatyp == AT_HUGS ? "being roasted" :
				"on fire");*/
				/*if (youmonst.data == &mons[PM_STRAW_GOLEM] ||
		        youmonst.data == &mons[PM_PAPER_GOLEM])
				{
					You("roast!");
					/* KMH -- this is okay with unchanging
					rehumanize();
					break;
				}
				else*/
				if (ptmp->fire_resistance)
				{
					string S = "The fire doesnt feel hot!";
          			vision.topline(S);
					dmg = 0;
				}

				/*if((int) mtmp->m_lev > rn2(20))
				destroy_item(SCROLL_CLASS, AD_FIRE);
				if((int) mtmp->m_lev > rn2(20))
				destroy_item(POTION_CLASS, AD_FIRE);
				if((int) mtmp->m_lev > rn2(25))
				destroy_item(SPBOOK_CLASS, AD_FIRE);*/
				burn_away_slime(mtmp);
			}
			else dmg = 0;
			break;

			//TODO: Check format of this case and next case
      case AD_COLD:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled)
			{
         	string S = "Youre covered in frost!";
            vision.topline(S);

				if (ptmp->cold_resistance)
				{
					string S = "The frost doesnt seem cold!";
            	vision.topline(S);
					dmg = 0;
         	}
         /*if((int) mtmp->m_lev > rn2(20))
			destroy_item(POTION_CLASS, AD_COLD);*/
			}
         else dmg = 0;
			break;

   	case AD_ELEC:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled)
			{
      		string S = "You get zapped!";
         	vision.topline(S);
         	if (ptmp->shock_resistance)
				{
         		string S = "The zap doesnt shock you!";
            	vision.topline(S);
					dmg = 0;
         	}
		    /*if((int) mtmp->m_lev > rn2(20))
			destroy_item(WAND_CLASS, AD_ELEC);
		    if((int) mtmp->m_lev > rn2(20))
			destroy_item(RING_CLASS, AD_ELEC);*/
			}
         else dmg = 0;
			break;

		case AD_SLEE:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled && ptmp->multi >= 0 && !rn2(5))
			{
         	if (ptmp->sleep_resistance) break;
         	fall_asleep(mtmp, ptmp, -rnd(10), TRUE);
		    	if (ptmp->blind)
				{
					string S = "You are put to sleep!";
          		vision.topline(S);

				}
		    	else
				{
					string S = "You are put to sleep by";
            	string S2 = mtmp->SName;
            	string S3 = "!";
            	S = S += S2 += S3;
            	vision.topline(S);
				}
			}
			break;

		case AD_BLND:
		/*if (!resists_blnd(&youmonst) && !Blind) {
		    pline("%s blinds you!", Monnam(mtmp));
		    make_blinded((long)dmg,FALSE);
		}*/
			dmg = 0;
			break;

      case AD_DRST:
			ptemp = A_STR;
			goto dopois;

		case AD_DRDX:
			ptemp = A_DEX;
			goto dopois;

		case AD_DRCO:
			ptemp = A_CON;

	dopois:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled && !rn2(8))
			{
				string S = "TODO: something to do with hallu or poison";
          	vision.topline(S);
			/*Sprintf(buf, "%s %s",
				!canspotmon(mtmp) ? "Its" :
				Hallucination ? s_suffix(rndmonnam()) :
				                s_suffix(mdat->mname),
				mpoisons_subj(mtmp, mattk));
			poisoned(buf, petmp, mdat->mname, 30);*/
			}
			break;

		case AD_DRIN:
			hitmsg(mtmp, mattk);
		/*if (defends(AD_DRIN, uwep) || !has_head(youmonst.data)) {
		    You("dont seem harmed.");
		    break;
		}*/
		//if (u_slip_free(mtmp, mattk)) break;

		/*if (uarmh && rn2(8)) {
		    Your("helmet blocks the attack to your head.");
		    break;
		}*/
			if (ptmp->half_physical_damage) dmg = (dmg+1) / 2;
		//losehp(dmg, mon_nam(mtmp), KILLED_BY_AN);

		/*if (!uarmh || uarmh->otyp != DUNCE_CAP) {
		    Your("brain is eaten!");
		    /* No such thing as mindless players...
		    if (ABASE(A_INT) <= ATTRMIN(A_INT)) {
			int lifesaved = 0;
			struct obj *wore_amulet = uamul;

			while(1) {
			    /* avoid looping on "die(y/n)?"
			    if (lifesaved && (discover || wizard)) {
				if (wore_amulet && !uamul) {
				    /* used up AMULET_OF_LIFE_SAVING; still
				       subject to dying from brainlessness
				    wore_amulet = 0;
				} else {
				    /* explicitly chose not to die;
				       arbitrarily boost intelligence
				    ABASE(A_INT) = ATTRMIN(A_INT) + 2;
				    You_feel("like a scarecrow.");
				    break;
				}
			    }

			    if (lifesaved)
				pline("Unfortunately your brain is still gone.");
			    else
				Your("last thought fades away.");
			    killer = "brainlessness";
			    killer_format = KILLED_BY;
			    done(DIED);
			    lifesaved++;
			}
		    }
		}

		/* adjattrib gives dunce cap message when appropriate
		(void) adjattrib(A_INT, -rnd(2), FALSE);
		forget_levels(25);	/* lose memory of 25% of levels
		forget_objects(25);	/* lose memory of 25% of objects
		exercise(A_WIS, FALSE);
		*/
			break;

		case AD_PLYS:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled && ptmp->multi >= 0 && !rn2(3))
			{
	    		if (ptmp->free_action)
				{
					string S = "You momentarily stiffen.";
          		vision.topline(S);
				}
	    		else
				{
            	if (ptmp->blind)
					{
						string S = "You are frozen!";
          			vision.topline(S);
					}
	    	    	else
					{
						string S = "You are frozen by ";
               	string S2 = mtmp->SName;
               	S = S += S2;
          			vision.topline(S);
					}
            	ptmp->nomovemsg = 0;	/* default: "you can move again" */
            	nomul(ptmp, -rnd(10));
            	exercise(A_DEX, FALSE);
	    		}
			}
			break;

		case AD_DRLI:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled && !rn2(3) && !ptmp->drain_resistance)
			{}//losexp();
			break;

		case AD_LEGS:
//		{ register long side = rn2(2) ? RIGHT_SIDE : LEFT_SIDE;
//		  const char *sidestr = (side == RIGHT_SIDE) ? "right" : "left";

		/* This case is too obvious to ignore, but Nethack is not in
		 * general very good at considering height--most short monsters
		 * still _can_ attack you when you're flying or mounted.
		 */
/*		  if (
#ifdef STEED
			u.usteed ||
#endif
				    Levitation || Flying) {
		    pline("%s tries to reach your %s %s!", Monnam(mtmp),
			  sidestr, body_part(LEG));
		  } else if (mtmp->mcan) {
		    pline("%s nuzzles against your %s %s!", Monnam(mtmp),
			  sidestr, body_part(LEG));
		  } else {
		    if (uarmf) {
			if (rn2(2) && (uarmf->otyp == LOW_BOOTS ||
					     uarmf->otyp == IRON_SHOES))
			    pline("%s pricks the exposed part of your %s %s!",
				Monnam(mtmp), sidestr, body_part(LEG));
			else if (!rn2(5))
			    pline("%s pricks through your %s boot!",
				Monnam(mtmp), sidestr);
			else {
			    pline("%s scratches your %s boot!", Monnam(mtmp),
				sidestr);
			    break;
			}
		    } else pline("%s pricks your %s %s!", Monnam(mtmp),
			  sidestr, body_part(LEG));
		    set_wounded_legs(side, rnd(60-ACURR(A_DEX)));
		    exercise(A_STR, FALSE);
		    exercise(A_DEX, FALSE);
		  }*/

			break;
		//}
		case AD_STON:	/* cockatrice */
			hitmsg(mtmp, mattk);
			if(!rn2(3) && !ptmp->stoned)
			{
         	if (mtmp->mcan)
				{
			//if (flags.soundok)
					string S = "You hear a cough from ";
            	string S2 = mtmp->SName;
            	string S3 = "!";
            	S = S += S2 += S3;
         		vision.topline(S);
		    	}
				else
				{
			//if (flags.soundok)
					string S = "You hear ";
            	string S2 = mtmp->SName;
            	string S3 = " hissing!";
            	S = S += S2 += S3;
          		vision.topline(S);
					if(!rn2(10) /*||
			    (flags.moonphase == NEW_MOON && !have_lizard())*/)
					{
do_stone:
			    		if (!ptmp->stone_resistance)
				    //&& !(poly_when_stoned(youmonst.data) &&
					//polymon(PM_STONE_GOLEM)))
						{
							ptmp->stoned = 5;
							return(1);
			    		}
					}
			//TODO: ???
		    //}
				}
         }
			break;

		case AD_STCK:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled && !ptmp->ptmpstuck.stuckmon /*&& !sticks(youmonst.data)*/)
			ptmp->ptmpstuck.stuckmon = mtmp;
			break;

		case AD_WRAP:
			if ((!mtmp->mcan || ptmp->ptmpstuck.stuckmon == mtmp) /*&& !sticks(youmonst.data)*/)
			{
      		if (!ptmp->ptmpstuck.stuckmon && !rn2(10))
				{
				/*if (u_slip_free(mtmp, mattk))
				{
					dmg = 0;
				}
				else
				{*/
					string S = mtmp->SName += "Swings itself around you!";
               vision.topline(S);
					ptmp->ptmpstuck.stuckmon = mtmp;
				//}
         	}
         }
			else if(ptmp->ptmpstuck.stuckmon == mtmp)
			{
         	if (ltmp->map[mtmp->xy.y][mtmp->xy.x] == EFTWater/*is_pool(mtmp->xy.x,mtmp->xy.y)*/ && !ptmp->swimming
			    && !ptmp->amphibious)
				{
					int moat = 0;/*(levl[u.ux][u.uy].typ != POOL) &&
					(levl[u.ux][u.uy].typ != WATER) &&
					!Is_medusa_level(&u.uz) &&
					!Is_waterlevel(&u.uz);*/

					string S = mtmp->SName += " drowns you...!";
          			vision.topline(S);

					//killer_format = KILLED_BY_AN;
					//TODO: Death
			   /* Sprintf(buf, "%s by %s",
				    moat ? "moat" : "pool of water",
				    a_monnam(mtmp));
			    killer = buf;
			    done(DROWNING);*/
				}
				else if(mtmp->IAttdam[attcycle] == AT_HUGS)
				{
					string S = mtmp->SName += "You are being crushed!";
          			vision.topline(S);
				}
         }
			else
			{
				dmg = 0;
			//if(flags.verbose)
				//TODO:  Fic left right leg
				string S = mtmp->SName += "brushed against your leg.";
          		vision.topline(S);
         }
		//}
		//else dmg = 0;
			break;

		case AD_WERE:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled && !rn2(4) && ptmp->lycn == NON_PM &&
			!ptmp->protection_from_shape_changers /*&&
			!ptmp->defends(AD_WERE,uwep)*/)
			{
      		string S = "You feel feverish.";
         	vision.topline(S);
         	exercise(A_CON, FALSE);
         	ptmp->lycn = monsndx(mtmp);
			}
			break;

		case AD_SGLD:
/*		hitmsg(mtmp, mattk);
		if (youmonst.data->mlet == mdat->mlet) break;
		if(!mtmp->mcan) stealgold(mtmp);
		break;

	    case AD_SITM:	/* for now these are the same
	    case AD_SEDU:
		if (is_animal(mtmp->data)) {
			hitmsg(mtmp, mattk);
			if (mtmp->mcan) break;
			/* Continue below
		} else if (dmgtype(youmonst.data, AD_SEDU)
#ifdef SEDUCE
			|| dmgtype(youmonst.data, AD_SSEX)
#endif
						) {
			pline("%s %s.", Monnam(mtmp), mtmp->minvent ?
		    "brags about the goods some dungeon explorer provided" :
		    "makes some remarks about how difficult theft is lately");
			if (!tele_restrict(mtmp)) rloc(mtmp);
			return 3;
		} else if (mtmp->mcan) {
		    if (!Blind) {
			pline("%s tries to %s you, but you seem %s.",
			    Adjmonnam(mtmp, "plain"),
			    flags.female ? "charm" : "seduce",
			    flags.female ? "unaffected" : "uninterested");
		    }
		    if(rn2(3)) {
			if (!tele_restrict(mtmp)) rloc(mtmp);
			return 3;
		    }
		    break;
		}
		switch (steal(mtmp)) {
		  case -1:
			return 2;
		  case 0:
			break;
		  default:
			if (!is_animal(mtmp->data) && !tele_restrict(mtmp))
			    rloc(mtmp);
			mtmp->mflee = 1;
			return 3;
		}
		break;
#ifdef SEDUCE
	    case AD_SSEX:
		if(could_seduce(mtmp, &youmonst, mattk) == 1
			&& !mtmp->mcan)
		    if (doseduce(mtmp))
			return 3;
			break;
#endif
		case AD_SAMU:
			hitmsg(mtmp, mattk);
		/* when the Wiz hits, 1/20 steals the amulet
		if (u.uhave.amulet ||
		     u.uhave.bell || u.uhave.book || u.uhave.menorah
		     || u.uhave.questart) /* carrying the Quest Artifact
		    if (!rn2(20)) stealamulet(mtmp);
			*/
			break;

      case AD_TLPT:
			hitmsg(mtmp, mattk);
			if (ptmp->uncancelled)
			{
				//if(flags.verbose)
				string S = "Your position suddenly seems very uncertain!";
          		vision.topline(S);

            //TODO:
				//tele();
			}
			break;

	    /*case AD_RUST:
			hitmsg(mtmp, mattk);
			if (mtmp->mcan) break;
			if (ptmp->INumber == PM_IRON_GOLEM)
			{
				string S = "You rust!";
          		vision.topline(S);
				/* KMH -- this is okay with unchanging
				//rehumanize();
				break;
			}
			//hurtarmor(mdat, AD_RUST);
			break;*/
	    /*case AD_DCAY:
			hitmsg(mtmp, mattk);
			if (mtmp->mcan) break;
			if (ptmp->monnum == PM_WOOD_GOLEM ||
		    u.umonnum == PM_LEATHER_GOLEM) {
			You("rot!");
			/* KMH -- this is okay with unchanging
			rehumanize();
			break;
		}
		hurtarmor(mdat, AD_DCAY);
		break;*/
	    /*case AD_HEAL:
		if(!uwep
#ifdef TOURIST
		   && !uarmu
#endif
		   && !uarm && !uarmh && !uarms && !uarmg && !uarmc && !uarmf) {
		    boolean goaway = FALSE;
		    pline("%s hits!  (I hope you dont mind.)", Monnam(mtmp));
		    if (Upolyd) {
			u.mh += rnd(7);
			if (!rn2(7)) {
			    /* no upper limit necessary; effect is temporary
			    u.mhmax++;
			    if (!rn2(13)) goaway = TRUE;
			}
			if (u.mh > u.mhmax) u.mh = u.mhmax;
		    } else {
			u.uhp += rnd(7);
			if (!rn2(7)) {
			    /* hard upper limit via nurse care: 25 * ulevel
			    if (u.uhpmax < 5 * u.ulevel + d(2 * u.ulevel, 10))
				u.uhpmax++;
			    if (!rn2(13)) goaway = TRUE;
			}
			if (u.uhp > u.uhpmax) u.uhp = u.uhpmax;
		    }
		    if (!rn2(3)) exercise(A_STR, TRUE);
		    if (!rn2(3)) exercise(A_CON, TRUE);
		    if (Sick) make_sick(0L, (char *) 0, FALSE, SICK_ALL);
		    flags.botl = 1;
		    if (goaway) {
			mongone(mtmp);
			return 2;
		    } else if (!rn2(33)) {
			if (!tele_restrict(mtmp)) rloc(mtmp);
			if (!mtmp->mflee) {
			    mtmp->mflee = 1;
			    mtmp->mfleetim = d(3,6);
			}
			return 3;
		    }
		    dmg = 0;
		} else {
		    if (Role_if(PM_HEALER)) {
			if (flags.soundok && !(moves % 5))
		      verbalize("Doc, I cant help you unless you cooperate.");
			dmg = 0;
		    } else hitmsg(mtmp, mattk);
		}
		break;
	    case AD_CURS:
		hitmsg(mtmp, mattk);
		if(!night() && mdat == &mons[PM_GREMLIN]) break;
		if(!mtmp->mcan && !rn2(10)) {
		    if (flags.soundok)
			if (Blind) You_hear("laughter.");
			else       pline("%s chuckles.", Monnam(mtmp));
		    if (u.umonnum == PM_CLAY_GOLEM) {
			pline("Some writing vanishes from your head!");
			/* KMH -- this is okay with unchanging
			rehumanize();
			break;
		    }
		    attrcurse();
		}
		break;*/

      case AD_STUN:
			hitmsg(mtmp, mattk);
			if(!mtmp->mcan && !rn2(4))
			{
				//make_stunned(ptmp->stun + dmg, TRUE);
				dmg /= 2;
			}
			break;

      case AD_ACID:
       	hitmsg(mtmp, mattk);
			if (!mtmp->mcan && !rn2(3))
         {
         	if (ptmp->acid_resistance)
				{
					string S = "Youre covered in acid, but it seems harmless.";
          		vision.topline(S);
					dmg = 0;
		    	}
				else
				{
					string S = "Youre covered in acid!  It burns!";
          		vision.topline(S);
					exercise(A_STR, FALSE);
		    	}
         }
			else	dmg = 0;
			break;

      case AD_SLOW:
       	hitmsg(mtmp, mattk);
			//if (ptmp->uncancelled && ptmp->fast &&
					//!defends(AD_SLOW, uwep) && !rn2(4)){}
				//u_slow_down();
			break;

      case AD_DREN:
			hitmsg(mtmp, mattk);
		//if (ptmp->uncancelled && !rn2(4)) {}
		    //drain_en(dmg);
			dmg = 0;
			break;

      case AD_CONF:
			hitmsg(mtmp, mattk);
			if(!mtmp->mcan && !rn2(4) && !mtmp->mspec_used)
			{
				mtmp->mspec_used = mtmp->mspec_used + (dmg + rn2(6));
				if(ptmp->confusion)
				{
					string S = "You are getting even more confused!";
               vision.topline(S);
				}
				else
				{
					string S = "You are getting confused!";
               vision.topline(S);
					//make_confused(ptmp->confusion + dmg, FALSE);
				}
			}
		/* fall through to next case */
      case AD_DETH:
      	if(1)
      	{
      		string S = mtmp->SName += "reached out with its deadly touch.";
         	vision.topline(S);
      	}
				/*if (is_undead(youmonst.data)) {
		    /* Still does normal damage
		    pline("Was that the touch of death?");
          break;
		}*/
			if(!ptmp->antimagic && rn2(20) > 16)
			{
		    //killer_format = KILLED_BY_AN;
		    //killer = "touch of death";
		    //done(DIED);
			}
			else
			{
      		if(!rn2(5))
				{
				//if(ptmp->antimagic) shieldeff(u.ux, u.uy);
					string S = "Lucky for you, it didnt work!";
            	vision.topline(S);
					dmg = 0;
         	}
				else
				{
					string S = "You feel your life force draining away!";
            	vision.topline(S);
				}

			}

			break;

      case AD_PEST:
      	if(1)
       	{
				string S = mtmp->SName += "reahes out, and you feel fever and chills.";
         	vision.topline(S);
       	}
			diseasemu(mtmp, ptmp); /* plus the normal damage */
			break;

      case AD_FAMN:
			if(1)
       	{
       		string S = mtmp->SName += " reached out, and your body shrivels.";
         	vision.topline(S);
       	}
			exercise(A_CON, FALSE);
		//if (!is_fainted()) morehungry(rn1(40,40));
		/* plus the normal damage */
			break;

      case AD_SLIM:
	    	hitmsg(mtmp, mattk);
	    	if (!ptmp->uncancelled) break;
	    	/*if (youmonst.data == &mons[PM_FIRE_VORTEX] ||
	    			youmonst.data == &mons[PM_FIRE_ELEMENTAL]) {
	    	    pline_The("slime burns away!");
	    	    dmg = 0;
	    	} else if (ptmp->unchanging ||
	    			youmonst.data == &mons[PM_GREEN_SLIME]) {
	    	    You("are unaffected.");
	    	    dmg = 0;
	    	} else*/
         if (!ptmp->slimed)
			{
       		string S = "You dont feel very well.";
            vision.topline(S);
            ptmp->slimed = 10L;
	    	}
			else
			{
				string S = "Yuck!";
          		vision.topline(S);
			}
	    	break;

	    /*case AD_ENCH:	/* KMH -- remove enchantment (disenchanter)
	    	hitmsg(mtmp, mattk);
	    	/* uncancelled is sufficient enough; please
	    	   don't make this attack less frequent
	    	if (uncancelled) {
	    		struct obj *obj = some_armor(&youmonst);

	    		if (drain_item(obj)) {
	    			Your("%s less effective.", aobjnam(obj, "seem"));
	    		}
	    	}
	    	break;*/
	    	default:
			dmg = 0;
         if(1)
         {
				string S = "NOTE: Default damage reaced.";
          	vision.topline(S);
         }
			break;
	}
	//if(ptmp->hp < 1) done_in_by(mtmp);

/*	Negative armor class reduces damage done instead of fully protecting
 *	against hits.
 */
 	if (dmg && ptmp->ac < 0)
   {
   	dmg -= rnd(-ptmp->ac);
      if (dmg < 1) dmg = 1;
   }

	/*if(dmg)
	{
	    if (ptmp->half_physical_damage
					/* Mitre of Holiness
		|| (Role_if(PM_PRIEST) && uarmh && is_quest_artifact(uarmh) &&
		    (is_undead(mtmp->data) || is_demon(mtmp->data))))
		dmg = (dmg+1) / 2;
	    mdamageu(mtmp, dmg);
	}*/

	/*if (dmg) {
	    res = passiveum(olduasmon, mtmp, mattk);
	    stop_occupation();
	    return res;
	} else*/
   return 1;
}


int CMonsterList::gulpmu( MonsterType *mtmp, int attcycle)	/* monster swallows you, or damage if u.uswallow */
{
	CPlayer * ptmp = mtmp->player;
	//struct trap *t = t_at(u.ux, u.uy);

	int tmp = 0;
	for(int tempattknum = 0; tempattknum < mtmp->IAttnum[attcycle]; tempattknum++)
	{
   		tmp = ((rand() % mtmp->IAttdam[attcycle]) +1);
		if(is_undead(mtmp) /*&& moon->midnight()*/)
      	tmp += ((rand() % mtmp->IAttdam[attcycle]) +1);
	}

	int	tim_tmp;
	//register struct obj *otmp2;
	int	i;

	if (!ptmp->swallow)
	{	/* swallows you */
		//if (youmonst.data->msize >= MZ_HUGE) return(0);
		//TODO: Traps
		/*if ((t && ((t->ttyp == PIT) || (t->ttyp == SPIKED_PIT))) &&
		    sobj_at(BOULDER, u.ux, u.uy))
			return(0);
		*/
		//if (ptmp->punished) unplacebc();/* ball&chain go away */
		//TODO: Why this?
		//remove_monster(mtmp->mx, mtmp->my);
		//place_monster(mtmp, u.ux, u.uy);
		ptmp->ptmpstuck.stuckmon = mtmp;
		//newsym(mtmp->mx,mtmp->my);
//#ifdef STEED
		if (is_animal(mtmp) && ptmp->steed)
		{
			//char buf[BUFSZ];
			/* Too many quirks presently if hero and steed
			 * are swallowed. Pretend purple worms don't
			 * like horses for now :-)
			 */
          if(1)
          {
          	string S = mtmp->SName += " lunges forward and plucks you off (your mount).";
            vision.topline(S);
          }
				//TODO:
				//dismount_steed(DISMOUNT_ENGULFED);
		}
		else
//#endif
		{
			string S = mtmp->SName += " engulfs you!";
          	vision.topline(S);

			//stop_occupation();
			//reset_occupations();	/* behave as if you had moved */

		if (ptmp->trap)
		{
			string S = "You are released from the trap";
          	vision.topline(S);
			//TODO
			//You("are released from the %s!",
			//	u.utraptype==TT_WEB ? "web" : "trap");
			ptmp->trap = 0;
		}

		/*i = number_leashed();
		if (i > 0)
		{
			string S = "The leash(es) snaps loose";
          	vision.topline(S);
			//TODO: pluralization
			pline_The("leash%s snap%s loose.",
					(i > 1) ? "es" : "",
					(i > 1) ? "" : "s");
			unleash_all();
		} */

		/*if (touch_petrifies(youmonst.data) && !resists_ston(mtmp)) {
			minstapetrify(mtmp, TRUE);
			if (mtmp->mhp > 0) return 0;
			else return 2;
		}*/

		//TODO: cant see anything
		//display_nhwindow(WIN_MESSAGE, FALSE);
		//vision_recalc(2);	/* hero can't see anything */
		ptmp->swallow = 1;
		/* assume that u.uswldtim always set >= 0 */
		ptmp->swldtim = (tim_tmp =
			(-ptmp->ac + 10 + rnd(25 - (int)mtmp->ILevel)) >> 1) > 0 ?
			    tim_tmp : 0;
	  //	swallowed(1);
		//for(otmp2 = invent; otmp2; otmp2 = otmp2->nobj) {
		//	(void) snuff_lit(otmp2);
		//}
	}

	if (mtmp != ptmp->ptmpstuck.stuckmon) return(0);

	switch(mtmp->IAtttype[attcycle])
	{

		case AD_DGST:
		    if (ptmp->slow_digestion) 
			{
		    	/* Messages are handled below */
		    	ptmp->swldtim = 0;
		    	tmp = 0;
		    }
			else if(ptmp->swldtim <= 1) 
			{	/* a3 *//*no cf unsigned <=0*/
				string S = mtmp->SName += " totally digests you!";
          		vision.topline(S);
				tmp = ptmp->hp;
				if (ptmp->half_physical_damage) tmp *= 2; /* sorry */
		    } 
			else 
			{
				string S = mtmp->SName += " digests you!";
          		vision.topline(S);
		        exercise(A_STR, FALSE);
		    }
		    break;

		case AD_PHYS:
      if(1)
      {
			string S = "You are pummeled with debris!";
          	vision.topline(S);
      }
			exercise(A_STR, FALSE);
		    break;

		case AD_ACID:
		    if (ptmp->acid_resistance)
			{
				string S = "You are covered in a seemingly harmless goo.";
          		vision.topline(S);
				tmp = 0;
		    }
			else
			{
				if (ptmp->hallucination)
				{
					string S = "Ouch!  Youve been slimed!";
          			vision.topline(S);
				}
				else
				{
					string S = "You are covered in slime!  It burns!";
          			vision.topline(S);
				}
				exercise(A_STR, FALSE);
		    }
		    break;

		case AD_BLND:
		    //if (!resists_blnd(&youmonst)) {
			if(!ptmp->blind)
			{
				string S = "You cant see in here!";
          		vision.topline(S);
			    make_blinded(mtmp, (long)tmp,FALSE);
			} else
			    /* keep him blind until disgorged */
			    make_blinded(mtmp, ptmp->blinded+1,FALSE);
		    //}
		    tmp = 0;
		    break;

		case AD_ELEC:
		    if(!mtmp->mcan && rn2(2))
			{
				string S = "The air around you crackles with electricity.";
          		vision.topline(S);
				if (ptmp->shock_resistance)
				{
					//shieldeff(u.ux, u.uy);
					string S = "You seem unhurt";
					vision.topline(S);
					//ugolemeffects(AD_ELEC,tmp);
					tmp = 0;
				}
		    }
			else tmp = 0;
		    break;

		case AD_COLD:
		    if(!mtmp->mcan && rn2(2))
			{
				if (ptmp->cold_resistance)
				{
					//shieldeff(u.ux, u.uy);
						string S = "You feel mildly chilly.";
          			vision.topline(S);
					//ugolemeffects(AD_COLD,tmp);
					tmp = 0;
				}
				else
				{
					string S = "You are freezing to death!";
          			vision.topline(S);
				}
		    } else tmp = 0;
		    break;

		case AD_FIRE:
		    if(!mtmp->mcan && rn2(2))
			{
				if (ptmp->fire_resistance)
				{
					//shieldeff(u.ux, u.uy);
					string S = "you feel mildly hot.";
          			vision.topline(S);
					//ugolemeffects(AD_FIRE,tmp);
					tmp = 0;
				}
				else
				{
					string S = "You are burning to a crisp!";
          			vision.topline(S);
					burn_away_slime(mtmp);
            }
		    }
			else tmp = 0;
		    break;

		case AD_DISE:
		    if (!diseasemu(mtmp, ptmp)) tmp = 0;
		    break;
		default:
		    tmp = 0;
		    break;
	}

	if (ptmp->half_physical_damage) tmp = (tmp+1) / 2;

	mdamageu(mtmp, tmp);
	//if (tmp) stop_occupation();
	if (ptmp->swldtim) --ptmp->swldtim;

	/*if (touch_petrifies(youmonst.data) && !resists_ston(mtmp)) {
		pline("%s very hurriedly %s you!", Monnam(mtmp),
		       is_animal(mtmp->data)? "regurgitates" : "expels");
		expels(mtmp, mtmp->data, FALSE);
	} else if (!u.uswldtim || youmonst.data->msize >= MZ_HUGE) {
	    You("get %s!", is_animal(mtmp->data)? "regurgitated" : "expelled");
	    if (flags.verbose && (is_animal(mtmp->data) || Slow_digestion))
		   pline("Obviously %s doesnt like your taste.",
			   mon_nam(mtmp));
	    expels(mtmp, mtmp->data, FALSE);
	}*/
	return(1);
}
}

int CMonsterList::explmu(MonsterType *mtmp, int attcycle, int ufound)	/* monster explodes in your face */
{
	CPlayer * ptmp = mtmp->player;
    if (mtmp->mcan) return(0);

    if (!ptmp->ufound)
	{
		string S = mtmp->SName += "Explodes at a spot in thin air.";
         vision.topline(S);
		 //TODO
		/*pline("%s explodes at a spot in %s!",
	    canseemon(mtmp) ? Monnam(mtmp) : "It",
	    levl[mtmp->mux][mtmp->muy].typ == WATER
		? "empty water" : "thin air");*/
	}
    else
	{
	int tmp = 0;
	for(int tempattknum = 0; tempattknum < mtmp->IAttnum[attcycle]; tempattknum++)
	{
   		tmp = ((rand() % mtmp->IAttdam[attcycle]) +1);
		if(is_undead(mtmp) /*&& moon->midnight()*/)
      	tmp += ((rand() % mtmp->IAttdam[attcycle]) +1);
	}
	//int not_affected = defends((int)mattk->adtyp, uwep);
	//TODO: fix this
	//temporary
	int not_affected = 0;

	hitmsg(mtmp, attcycle);

	switch (mtmp->IAtttype[attcycle])
	{
	    case AD_COLD:
		not_affected |= ptmp->cold_resistance;
		goto common;
	    case AD_FIRE:
		not_affected |= ptmp->fire_resistance;
		goto common;
	    case AD_ELEC:
		not_affected |= ptmp->shock_resistance;
common:

		if (!not_affected)
		{
		   /* if (ACURR(A_DEX) > rnd(20))
			{
				string S = "You duck some of the blast.";
          		vision.topline(S);
				tmp = (tmp+1) / 2;
			}
			else
			{ */
		        //if (flags.verbose)
				string S = "You get blasted!";
          		vision.topline(S);
		  //  }
		    if (mtmp->IAtttype[attcycle] == AD_FIRE) burn_away_slime(mtmp);
		    if (ptmp->half_physical_damage) tmp = (tmp+1) / 2;
		    mdamageu(mtmp, tmp);
		}
		break;

	    case AD_BLND:
		//not_affected = resists_blnd(&youmonst);
		if (!not_affected)
		{
		    /* sometimes you're affected even if it's invisible */
		    if (mtmp->visible || (rnd(tmp /= 2) > ptmp->dlevel->getNumber()))
			{
				string S = "you are blinded by a blast of light!";
          		vision.topline(S);
				make_blinded(mtmp, (long)tmp, FALSE);
		    }
			else
			{
				//if (flags.verbose)
				string S = "You get the impression it was not terribly bright.";
          		vision.topline(S);
			}
		}
		break;

	    case AD_HALU:
		//not_affected |= ptmp->blind ||
		//	(ptmp->monnum == PM_BLACK_LIGHT ||
		//	 u.umonnum == PM_VIOLET_FUNGUS ||
		//	 dmgtype(youmonst.data, AD_STUN));
		if (!not_affected)
		{
		    if (!ptmp->hallucination)
			{
				string S = "You are freaked by a blast of kaleidoscopic light!";
          		vision.topline(S);
			    //make_hallucinated(HHallucination + (long)tmp,FALSE,0L);
			}
		}
		break;

	    default:
		break;
	}
	if (not_affected)
	{
		string S = "You seem unaffected by it.";
        vision.topline(S);

	    //ugolemeffects((int)mattk->adtyp, tmp);
	}
    }
    //mondead(mtmp);
    if (mtmp->mhp > 0) return(0);
    return(2);	/* it dies */
}

int CMonsterList::gazemu( MonsterType *mtmp, int attcycle)	/* monster gazes at you */
{
	CPlayer * ptmp = mtmp->player;
   CLevel * ltmp = ptmp->dlevel;
	switch(mtmp->IAtttype[attcycle])
	{
	    case AD_STON:
		if (mtmp->mcan)
		{
		    if (/*mtmp->data == */mtmp->INumber == PM_MEDUSA && canseemon(mtmp,ptmp,ltmp))
			{
				string S = mtmp->SName += " doesnt look all that ugly.";
				vision.topline(S);
			    break;
			}
		}
		if(ptmp->reflecting && /*m_canseeu(mtmp) &&*/
		   !mtmp->mcan && mtmp->INumber == PM_MEDUSA)
		{
		    if(!ptmp->blind)
			{
				string S = mtmp->SName += " gaze is reflected by your (something)";
				vision.topline(S);
				//TODO
		    	/*(void) ureflects("%s gaze is reflected by your %s.",
		    			s_suffix(Monnam(mtmp)));
		    	if (mon_reflects(mtmp,
		    			"The gaze is reflected away by %s %s!"))*/
            break;
		    }
          string S = mtmp->SName += " is turned to stone!";
          vision.topline(S);
		    ptmp->stoned = TRUE;
		    //killed(mtmp);

		    if (mtmp->mhp > 0) break;
		    return 2;
		}
		if (canseemon(mtmp,ptmp,ltmp) && !ptmp->stone_resistance)
		{
			string S = "You look upon ";
         string S2 = mtmp->SName;
         string S3 = ".";
         S = S += S2 += S3;
			vision.topline(S);
			//if(poly_when_stoned(youmonst.data) && polymon(PM_STONE_GOLEM))
			    //break;
			//string S = "You turn to stone.";
			//vision.topline(S);

			//killer_format = KILLED_BY;
			//killer = mons[PM_MEDUSA].mname;
			//done(STONING);
		}
		break;

	    case AD_CONF:
		if(!mtmp->mcan && canseemon(mtmp,ptmp,ltmp) && mtmp->cansee &&
					!mtmp->mspec_used && rn2(5))
		{
			//TODO: implement function to calculate specific damage
		    //int conf = d(3,4);
			int conf = 0; //temp

		    mtmp->mspec_used = mtmp->mspec_used + (conf + rn2(6));
		    if(!ptmp->confusion)
			{
				string S = mtmp->SName += " gaze confuses you!";
				vision.topline(S);
			}
		    else
			{
				string S = "You are getting more and more confused.";
				vision.topline(S);
			    //make_confused(ptmp->confusion + conf, FALSE);
			}
		}
		break;

	    case AD_STUN:
		if(!mtmp->mcan && canseemon(mtmp,ptmp,ltmp) && mtmp->cansee &&
					!mtmp->mspec_used && rn2(5))
		{
		    //int stun = d(2,6);
			int stun = 0;

		    mtmp->mspec_used = mtmp->mspec_used + (stun + rn2(6));
		    //make_stunned(HStun + stun, TRUE);
			string S = mtmp->SName += " stares piercingly at you!";
			vision.topline(S);
		}
		break;

	    case AD_BLND:
		if (!mtmp->mcan && canseemon(mtmp,ptmp,ltmp) /*&& !resists_blnd(&youmonst)*/
			&& mondist(mtmp->xy,ptmp->xy) <= 0/*BOLT_LIM*BOLT_LIM*/) //fix
		{
		    int blnd = 0;
			for(int tempattknum = 0; tempattknum < mtmp->IAttnum[attcycle]; tempattknum++)
			{
   				blnd = ((rand() % mtmp->IAttdam[attcycle]) +1);
				if(is_undead(mtmp) /*&& moon->midnight()*/)
      			blnd += ((rand() % mtmp->IAttdam[attcycle]) +1);
			};
		    make_blinded(mtmp, (long)blnd,FALSE);
		    //make_stunned((long)d(1,3),TRUE);
			string S = "You are blinded by ";
         string S2 = mtmp->SName;
         string S3 = "s radiance!";
         S = S += S2 += S3;
			vision.topline(S);
		}
		break;

	    case AD_FIRE:
	    	if(!mtmp->mcan && canseemon(mtmp,ptmp,ltmp) && mtmp->cansee &&
	    			!mtmp->mspec_used && rn2(5))
			{
	    	    //int dmg = d(2,6);
				int dmg = 0;
				string S = mtmp->SName += " attacks you with its fiery gaze!";
				vision.topline(S);
	    	    if (ptmp->fire_resistance)
				{
					string S = "The fire doesnt feel hot!";
					vision.topline(S);
	    	    	dmg = 0;
	    	    }
	    	    burn_away_slime(mtmp);
	    	    /*if((int) mtmp->ILevel > rn2(20))
	    	    	destroy_item(SCROLL_CLASS, AD_FIRE);
	    	    if((int) mtmp->ILevel > rn2(20))
	    	    	destroy_item(POTION_CLASS, AD_FIRE);
	    	    if((int) mtmp->ILevel > rn2(25))
	    	    	destroy_item(SPBOOK_CLASS, AD_FIRE);*/
	    	    if (dmg) mdamageu(mtmp, dmg);
	    	}
        break;

//#ifdef PM_BEHOLDER /* work in progress */
	    case AD_SLEE:
		if(ptmp->multi >= 0 && !rn2(5) && !ptmp->sleep_resistance)
		{
		    //fall_asleep(mtmp, -rnd(10), TRUE);
			string S = mtmp->SName += " gaze makes you very sleepy...";
			vision.topline(S);
		}
		break;

	    case AD_SLOW:
		if((ptmp->fast /*& (INTRINSIC|TIMEOUT))*/ &&
					/*!defends(AD_SLOW, uwep) &&*/ !rn2(4))){}
		    //u_slow_down();
		break;
//#endif
	    default:
			string S = "Something is fucked up in the gaze code";
			vision.topline(S);
		break;
	}
	return(0);
}

void CMonsterList::mdamageu(MonsterType *mtmp, int n)	/* mtmp hits you for n points damage */
{
	CPlayer * ptmp = mtmp->player;
	//flags.botl = 1;
	if /*(Upolyd)*/ (1) //fix
	{
		//ptmp->mh -= n;
		//if (ptmp->mh < 1) rehumanize();
	} else {
		ptmp->hp -= n;
		if(ptmp->hp < 1) {}//done_in_by(mtmp);
	}
}

int CMonsterList::percieves(MonsterType *mtmp)
{
	CPlayer * ptmp = mtmp->player;
	CLevel* ltmp = ptmp->dlevel;
	if(/*ptmp->see_invisible &&*/ canseemon(mtmp,ptmp,ltmp))
   	return 1;
   else
   	return 0;
}

int CMonsterList::dprince(int atyp)
{
	int tryct, pm;

	for (tryct = 0; tryct < 20; tryct++) {
	    pm = rn1(PM_DEMOGORGON + 1 - PM_ORCUS, PM_ORCUS);
	    if (/*!(mvitals[pm].mvflags & G_GONE) &&
		    (*/atyp == A_NONE /*|| &Monsters.DefaultMon[pm]->IAlignment == &Monsters.DefaultMon[atyp]->IAlignment*/)
		return(pm);
	}
	return(dlord(atyp));	/* approximate */
}

int CMonsterList::dlord(int atyp)
{
	int tryct, pm;

	for (tryct = 0; tryct < 20; tryct++)
   {
	    pm = rn1(PM_YEENOGHU + 1 - PM_JUIBLEX, PM_JUIBLEX);
	    if (/*!(mvitals[pm].mvflags & G_GONE) &&
       //TODO:find out what sgn() is
		    (*/atyp == A_NONE /*|| Monsters.DefaultMon[pm]->IAlignment == Monsters.DefaultMon[atyp]->IAlignment*/)
		return(pm);
	}
	return(ndemon(atyp));	/* approximate */
}

/* create lawful (good) lord */
//TODO:Make sure that maligntype is the same as mtmp->IAlignment, if not then redo all IAlignments
int CMonsterList::llord()
{
	//if (!(mvitals[PM_ARCHON].mvflags & G_GONE))
	//	return(PM_ARCHON);

	return(lminion());	/* approximate */
}

int CMonsterList::lminion()
{
	/*MonsterType * tmon;
	for (int tryct = 0; tryct < 20; tryct++)
   {
	    tmon = mkclass(S_ANGEL,0);
	    if (mtmp && !is_lord(tmon))
		return(monsndx(tmon));
	}*/
  	return NON_PM;
}

int CMonsterList::ndemon(int atyp)
{
	/*MonsterType * ptr;
	for (int tryct = 0; tryct < 20; tryct++)
   {
	    ptr = mkclass(S_DEMON, 0);
	    if (is_ndemon(ptr) &&
		    (atyp == A_NONE || Monsters.DefaultMon[pm]->IAlignment == Monsters.DefaultMon[atyp]->IAlignment)
		return(monsndx(ptr));
	} */

	return NON_PM;
}

int CMonsterList::monsndx(MonsterType * mtmp)		/* return an index into the mons array */
{
	CPlayer * ptmp = mtmp->player;
	int i;

   //TODO: What the FUCK is this function doing?
	i = (int)(mtmp - &Monsters.DefaultMon[0]);
	if (/*i < LOW_PM ||*/ i >= NUMMONS)
   {
		/* ought to switch this to use `fmt_ptr' */
      string S = "monsndx - could not index monster";
      vision.topline(S);
	    return FALSE;		/* will not get here */
	}
	return i;
}

//attrib.c
void CMonsterList::exercise(int i, int inc_or_dec)
{
	if (i == A_INT || i == A_CHA) return;	/* can't exercise these */

	/* no physical exercise while polymorphed; the body's temporary */
	//if (Upolyd && i != A_WIS) return;

	//if(abs(AEXE(i)) < AVAL)
   //{
		/*
		 *	Law of diminishing returns (Part I):
		 *
		 *	Gain is harder at higher attribute values.
		 *	79% at "3" --> 0% at "18"
		 *	Loss is even at all levels (50%).
		 *
		 *	Note: *YES* ACURR is the right one to use.
		 */
	//	AEXE(i) += (inc_or_dec) ? (rn2(19) > ACURR(i)) : -rn2(2);
	//	pline("%s, %s AEXE = %d",
	//		(i == A_STR) ? "Str" : (i == A_WIS) ? "Wis" :
	//		(i == A_DEX) ? "Dex" : "Con",
	//		(inc_or_dec) ? "inc" : "dec", AEXE(i));
	//}
	if (/*ptmp->moves > 0 && */(i == A_STR || i == A_CON)); //(void)encumber_msg();
}

void CMonsterList::hitmsg(MonsterType * mtmp, int mattk)
{
	CPlayer * ptmp = mtmp->player;
	int compat;

	/* Note: if opposite gender, "seductively" */
	/* If same gender, "engagingly" for nymph, normal msg for others */
	/*if((compat = could_seduce(mtmp, &youmonst, mattk))
			&& !mtmp->mcan && !mtmp->mspec_used) {
		pline("%s %s you %s.", Monnam(mtmp),
			Blind ? "talks to" : "smiles at",
			compat == 2 ? "engagingly" : "seductively");
	} else*/
   string S;
	switch (mtmp->IAtttype[mattk])
   {

   	case AT_BITE:
      	S = mtmp->SName += " bites!";
      	vision.topline(S);
			break;
		case AT_KICK:
      	S = mtmp->SName += " kicks!";
      	vision.topline(S);
         //TODO
			//pline("%s kicks%c", Monnam(mtmp),
			//	    thick_skinned(youmonst.data) ? '.' : '!');
			break;
		case AT_STNG:
      	S = mtmp->SName += " stings!";
      	vision.topline(S);
			break;
		case AT_BUTT:
         S = mtmp->SName += " butts!";
      	vision.topline(S);
			break;
		case AT_TUCH:
      	S = mtmp->SName += " touches you!";
      	vision.topline(S);
			break;
		case AT_TENT:
      	S = mtmp->SName += " tentacles suck you!";
      	vision.topline(S);
			break;
		case AT_EXPL:
		case AT_BOOM:
      	S = mtmp->SName += " explodes!";
      	vision.topline(S);
			break;
		default:
      	S = mtmp->SName += " hits!";
      	vision.topline(S);
	}
}

int CMonsterList::diseasemu(MonsterType * mtmp, CPlayer * ptmp)
{
	if (ptmp->sick_resistance)
   {
      string S = "You feel a slight illness.";
      vision.topline(S);
		return FALSE;
	}
   else
   {
	 //	make_sick(ptmp->sick ? ptmp->sick/3L + 1L : (long)rn1(ACURR(A_CON), 20),
	 //		mtmp->SName, TRUE, SICK_NONVOMITABLE);
		return TRUE;
	}
}

void CMonsterList::fall_asleep(MonsterType * mtmp, CPlayer * ptmp, int how_long, int wakeup_msg)
{
	//stop_occupation();
	//nomul(how_long);
	/* early wakeup from combat won't be possible until next monster turn */
	ptmp->sleep = mtmp->movement;
   //int nomovemsg;
	//nomovemsg = wakeup_msg ? "You wake up." : You_can_move_again;
}

void CMonsterList::burn_away_slime(MonsterType * mtmp)
{
	CPlayer * ptmp = mtmp->player;
	if (ptmp->slimed)
   {
   	string S = "The slmie that covers you is burned away!";
      vision.topline(S);
      ptmp->slimed = 0L;
	}
	return;
}

void CMonsterList::nomul(CPlayer * ptmp, int nval)
{
	if(ptmp->multi < nval) return;	/* This is a bug fix by ab@unido */
	ptmp->invulnerable = FALSE;	/* Kludge to avoid ctrl-C bug -dlc */
	ptmp->sleep = 0;
	ptmp->multi = nval;
	//flags.mv = flags.run = 0;
}

void CMonsterList::make_blinded(MonsterType * mtmp, long xtime, int talk)
{
	CPlayer * ptmp = mtmp->player;
	long old = ptmp->blinded;
	int changed = FALSE;

	if (ptmp->sleep) talk = FALSE;

	if (!xtime && old && !ptmp->blindfolded /*&& haseyes(youmonst.data)*/)
   {
   	if (talk)
      {
			if (ptmp->hallucination)
         {
         	string S = "Far out!  Evwerything is all cosmic again!.";
      		vision.topline(S);
         }
			else
         {
         	string S = "You can see again.";
      		vision.topline(S);
         }

      }
      changed = TRUE;
	}
	if (xtime && !old && !ptmp->blindfolded /*&& haseyes(youmonst.data)*/)
   {
   	if (talk)
      {
			if (ptmp->hallucination)
         {
         	string S = "Oh bummer! Everything is dark! Help!";
      		vision.topline(S);
			}
			else
         {
         	string S = "A cloud of darkness falls upon you.";
      		vision.topline(S);
			}
      }
      changed = TRUE;

	    /* Before the hero goes blind, set the ball&chain variables. */
	    //if (ptmp->punished) set_bc(0);
	}
	//set_itimeout(&Blinded, xtime);
	/*if (changed) {
	    flags.botl = 1;
	    vision_full_recalc = 1;
	    if (Blind_telepat || Infravision) see_monsters();
	}*/
}











