/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

/*monlist.h - header file for linked list of monsters.

	This linked list is farily straight forward.  A singly linked list class
   holds a dynamic number of node classes that contain monster data.

   There are currently two ways to access a monsters information.
   1.  By monster ID you can find a specific number.

   2.  By monster location.  Use the findatloc() function instead of
     inserting an integer for a get or set instruction.

   You can also pull out an entire MonsterType class if desired for testing
     purposes.

*/

#ifndef _monlist_h_
#define _monlist_h_

#include "monster.h"
#include "sock.h"

class CLevel;

//Each monster on the current level is a seperate CmonsterNode
class CMonsterNode {
public:
    CMonsterNode(int monst, CMonsterNode * n, ID currentID);
    CMonsterNode(CMonsterNode * n, MonsterType *);
    ~CMonsterNode();

    MonsterType * data;  //all the monster information including location
    CMonsterNode * next;
private:
};

class CMonsterList {
    friend void moveMonster(ID, CMonsterList, CMonsterList);
public:
    CMonsterList();
    ~CMonsterList();

    int add(int monst);		//add a new monster to the list
    int remove(ID monst);	//remove a monster from the list
    int getlength() { return size; };
    MonsterType * find(ID monst);
    MonsterType * findatloc(int ycoord, int xcoord);
    MonsterType * find(CLocation l) { return findatloc(l.y, l.x); };
    void createLevel(CLevel *);      //create monsters for a certain dungeon level


    friend CSock& operator<< (CSock& s, CMonsterList& l) {
        s << l.getlength();
        CMonsterNode * temp;
        for (temp = l.head; temp != NULL; temp = temp->next)
            l.sendMonster(&s, temp->data);
        return s; };
    void sendMonster(CSock * s, MonsterType * m) {
        if (m->IID < 1)
            *s << "null" << m->IID
               << m->SName << m->CSpecies
               << m->xy << m->IColor;
        else
            *s << "notnull" << m->IID
               << m->SName << m->CSpecies
               << m->xy << m->IColor;
    };
    void sendToPlayer(CSock *s) {
         *s << *this; };

    /* Validate the list of monsters, remove any with < 1 hp */
    void validMonsters();

    /* Move the monsters on the level in accordance with sureal time.
       This should be called every time a player moves */
    void movemon();
    int domovemon(MonsterType *mon);

    /* Assign each monster to its player */
    void assignToPlayers(CLevel *);

	 string AttackType(int attk);


    

    //int rn2(int temprand);

    int canseemon(MonsterType *mtmp, CPlayer *ptmp, CLevel *ltmp);  //fix
    int cansee(MonsterType *mtmp, CPlayer *ptmp, CLevel *ltmp);     //fix
    int missmu(CPlayer *ptmp, MonsterType *mtmp, int nearmiss, int mattk);
    int gazemu(MonsterType * mtmp, int attcycle);
    int gulpmu(MonsterType * mtmp, int attcycle);
    int spitmu(MonsterType * mtmp, int attcycle);
    int explmu(MonsterType * mtmp, int attcycle, int ufound);

    int hitmu(MonsterType * mtmp, CPlayer * ptmp, int mattk);
    int wildmiss(CPlayer *ptmp, MonsterType *mtmp, CLevel *ltmp, int mattk);
    int msummon(MonsterType *mtmp);
    int mondist(CLocation mxy, CLocation pxy);
    void were_change(MonsterType *mtmp, CPlayer *ptmp);
    int counter_were(int pm);
    void new_were(MonsterType *mtmp, CPlayer *ptmp);
    int were_summon(MonsterType *mtmp, int yours);
    int percieves(MonsterType *mtmp);                    //fix

    int dprince(int atyp);
    int ndemon(int atyp);
    int dlord(int atyp);
    int lminion(void);
    int llord(void);

    int monsndx(MonsterType * mtmp);
    int set_mon_data(MonsterType *mtmp, MonsterType *mtmp2, int flag);
    void exercise(int i, int inc_or_dec);
    void hitmsg(MonsterType * mtmp, int mattk);
    int diseasemu(MonsterType * mtmp, CPlayer * ptmp);
    void fall_asleep(MonsterType * mtmp, CPlayer * ptmp, int how_long, int wakeup_msg);
    void burn_away_slime(MonsterType * mtmp);
    void nomul(CPlayer * ptmp, int nval);
    void make_blinded(MonsterType * mtmp, long xtime, int talk);
    void mdamageu(MonsterType *mtmp, int n);

    int tempfunc(MonsterType * mtmp, int tempx, int tempy);



private:
    int add(MonsterType *);

    
    CMonsterNode * head;
    CMonsterNode * current;
    CMonsterNode * last;  //for remove
    int size;
    ID currentID;
};

#endif
