/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifndef _monster_h_
#define _monster_h_

#include "system.h"
#include "string.h"
#include "location.h"

/*    This file deals with all the initial loading of the monster before the
game begins.  It has various small functions to read the old nethack code.
This may be time consuming but it doesnt affect the program in any way since all
of this is done before the game even starts up.  I am attempting to work on it
anyhow to make it faster*/


//Some defined constants from Nethack code
#define NO_ATTK Monsters.ATTK(0,0,0,0)

#define WT_ELF	   800
#define WT_DRAGON 4500
#define WT_HUMAN	1450

//NOT CORRECT!!  I have included these just to fill in these numbers, I
//couldnt find the location of these constants
#define HI_LORD 17
#define HI_DOMESTIC 18
#define MS_ARREST 99
#define PL_NSIZ 99
#define MS_FERRY 1
//#define SEDUCTION_ATTACKS 99


//This defines the number of monsters that are in the game
//  so to create an array
#define MAXMONSTERS 400

#define VERY_SLOW	3
#define SLOW_SPEED	9
#define NORMAL_SPEED	12 /* movement rates */
#define FAST_SPEED	15
#define VERY_FAST	24

void initMonsters(void);

//The Definition of the class is the last line of the code

class CPlayer;

class MonsterType {
public:
    MonsterType();
    bool isAlive() { return mhp > 0; };

    CPlayer * player;   /* Player the monster is assigned to */
    int movement;       /* Number of moves player has made compared to mon */
    int pMoveCount;	/* Number of movers player has made */

    ID IID;
    int INumber;
    char CSpecies;
    string SName;
    int mhp, mhpmax;
    int ILevel;
    int ISpeed;
    int IAC;
    int IMR;
    int IAlignment;
    int IAttarea[6];
    int IAtttype[6];
    int IAttnum[6];
    int IAttdam[6];
    int IWeight;
    int INutrition;
    int IPixel;
    int ISound;
    int ISize;
    int IResist;
    int IIntrinsics;
    int IConvey;
    int IM1;
    int IM2;
    int IM3;
    int IClass;
    int IColor;
    int IGenerate;
    CLocation xy;

	int rotation;  //1=clockwise -1=counter-clockwise
    CLocation MonPrevMove[4];  //to track movement
	int StraightMove;
	int DefaultDir;
	int DirQueue[8];
	int MoveCycle;

   int underwater;
   int is_swimmer;
   int invis;
   int cham;
   int mcan;
   int foundyou;
   int cansee;
   int trapped;
   int sleeping;
   int canmove;
   int frozen;

   int multi;
   int mconf;
   int conflict;
   //TODO: player only?
   int protection_from_shape_changers;
   int hallucination;
   int mundetected;
   int mspec_used;
	int visible;

};

//This loads in all of the monsters
class DefMonster {
public:
    DefMonster();

    int AddMonster(string, char, int, int, int, int, int, int, int, int, int, int);  //How each monster is inserted
    int GetRMonster(char, int, int);  //A function that will pick a random monster
    int GetMonster(string);
    int RMonster(void);  //Pick a completely random monster

    //these are all for the nethack monster defitions
    int LVL(int,int,int,int,int); //Monster level information

    int ATTK(int,int,int,int);

    int A(int,int,int,int,int,int);

    int SIZ(int,int,int,int,int);

    MonsterType DefaultMon[MAXMONSTERS];  //The array

private:
    int currentnum;  //For loading in monsters
    int attorder;  //For listing the 6 attacks
};

//The adding of monsters
extern DefMonster Monsters;

#endif
