/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include <sys/time.h>
#include <stdio.h>

#include "moon.h"

struct tm * Moon::getlt() {
    time_t date;

    (void) time(&date);
    return(localtime(&date));
}

int Moon::getyear() {
    return(1900 + getlt()->tm_year);
}

char * Moon::hhmmss() {
    static char timestr[7];
    struct tm *lt;

    lt = getlt();

    sprintf(timestr, "%02d%02d%02d",
        lt->tm_hour, lt->tm_min, lt->tm_sec);
    return(timestr);
}

char * Moon::ddmmyy() {
    static char datestr[7];
    struct tm *lt;

    lt = getlt();

    sprintf(datestr, "%02d%02d%02d",
        lt->tm_mday, lt->tm_mon + 1, lt->tm_year);
    return(datestr);
}

/* moon period = 29.53058 days ~= 30, year = 365.2422 days
   days moon phase advances on first day of year compared to preceding year
        = 365.2422 - 12*29.53058 ~= 11
   years in Metonic cycle (time until same phases fall on the same days of
        the month) = 18.6 ~= 19
   moon phase on first day of year (epact) ~= (11*(year%19) + 29) % 30
        (29 as initial condition)
   current phase in days = first day phase + days elapsed in year
   6 moons ~= 177 days
   177 ~= 8 reported phases * 22
   + 11/22 for rounding
*/

int Moon::moon_phase() {        /* 0-7, with 0: new, 4: full */
    register struct tm *lt = getlt();
    register int epact, diy, goldn;

    diy = lt->tm_yday;
    goldn = (lt->tm_year % 19) + 1;
    epact = (11 * goldn + 18) % 30;
    if ((epact == 25 && goldn > 11) || epact == 24)
        epact++;

    return( (((((diy + epact) * 6) + 11) % 177) / 22) & 7 );
}

int Moon::friday_13th() {
    register struct tm *lt = getlt();

    return((lt->tm_wday == 5 /* friday */ && lt->tm_mday == 13));
}

int Moon::night() {
    register int hour = getlt()->tm_hour;

    return(hour < 6 || hour > 17);
}

int Moon::midnight() {
    return(getlt()->tm_hour == 0);
}

/* This returns the distance between a,b and c,d */
int distance(int a, int b, int c, int d) {
    register int l, t;

    /* first find the square of the distance */
    l = ((c-a)*(c-a) + (d-b)*(d-b));

    /* Then find the closest square root. */
    for (t = 1; t < l; t++)
        if ((t+1)*(t+1) > l && t*t <= l)
            break;
    if (l - t*t < (t+1)*(t+1) - l)  /* Find closest root. */
        return t;
    else
        return (t+1);
}
