/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "objlist.h"
#include "log.h"
#include "level.h"
#include "attrib.h"

/* This stuff can be found in u_init.c in Nethack 3.3.0
   skills are done in other parts of the code. Here is
   only the stuff we place in a players pack */

/* Add an object to the list */
void CObjectList::add(CObject *o) {
    if (o == NULL) log << "ERROR: adding null object to objectlist\n";
    if (o->inList()) return; /* Ignore if object already int list */

    /* If an object of similar qualities is already in the list, add
       it to that object */
    CObject * similar = findFirst(o->getObjectType());
    if (similar != NULL)
        if (*similar == o) {
            similar->setNumber(similar->getNumber() + o->getNumber());
            changed = true;
            delete o;
            return;
        }

    head = new CObjectNode(head, o);
    o->isInList();
    size++;
    changed = true;
};

void CObjectList::packadd(CObject *o, ID player) {
    o->identify(player);
    add(o);
}

/* Add objects in revers order so letters are correct */

void CObjectList::createPack(int type, ID pid) {
    pack = true;
    switch (type) {
        case Archeologist:
            if (!rn2(10)) packadd(new CTool(TinOpener),pid);
            else if (!rn2(4)) packadd(new CTool(OilLamp),pid);
            else if (!rn2(10)) packadd(new CTool(MagicMarker),pid);
            packadd(new CTool(Sack),pid);
            packadd(new CTool(TinningKit),pid);
            packadd(new CTool(PickAxe),pid);
            packadd(new CFood(FoodRation, 3),pid);
            packadd(new CArmor(HelmFedora),pid);
            packadd(new CWeapon(WeBullwhip, 1, 2),pid);
            packadd(new CArmor(ArmLeatherJacket),pid);
            break;

        case Barbarian:
            if (!rn2(6)) packadd(new CTool(OilLamp),pid);
            packadd(new CFood(FoodRation),pid);
            packadd(new CArmor(ArmRing),pid);
            if (rn2(100) >= 50) {
                packadd(new CWeapon(WeShortSword),pid);
                packadd(new CWeapon(WeBattleAxe),pid);
            } else {
                packadd(new CWeapon(WeAxe),pid);
                packadd(new CWeapon(WeTwohandedSword),pid);
            }
            break;

        case Caveman:
            packadd(new CArmor(ArmLeatherArmor),pid);
            packadd(new CGem(GemRock, 3),pid);
            packadd(new CGem(GemFlintstone, rn1(11, 10)),pid);
            packadd(new CWeapon(WeSling, 1, 2),pid);
            packadd(new CWeapon(WeClub, 1, 1),pid);
            break;

        case Dragon:
            log << "TODO: A dragons pack\n";
            packadd(new CPotion(PoSickness, 1), pid);
            packadd(new CWeapon(WeClub, 1, 1),pid);
            break;

        case Healer:
            if (!rn2(25)) packadd(new CTool(OilLamp),pid);
            packadd(new CFood(FoodApple, 5),pid);
            packadd(new CSpell(SpellStoneToFlesh),pid);
            packadd(new CSpell(SpellExtraHealing),pid);
            packadd(new CSpell(SpellHealing),pid);
            packadd(new CWand(WandSleep),pid);
            packadd(new CPotion(PoExtraHealing, 4),pid);
            packadd(new CPotion(PoHealing, 4),pid);
            packadd(new CTool(Stethoscope),pid);
            packadd(new CArmor(GloveLeather, 1, 1),pid);
            packadd(new CWeapon(WeScalpel),pid);
            packadd(new CGold(rn1(1000, 1001)),pid);
            break;

        case Knight:
            packadd(new CFood(FoodCarrot, 10),pid);
            packadd(new CFood(FoodApple, 10),pid);
            packadd(new CArmor(GloveLeather),pid);
            packadd(new CArmor(ShieldSmall),pid);
            packadd(new CArmor(ArmRing, 1, 1),pid);
            packadd(new CWeapon(WeSpear, 1, 2),pid);
            packadd(new CWeapon(WeLongSword),pid);
            break;

        case Monk:
            if (!rn2(5)) packadd(new CTool(MagicMarker),pid);
            else if (!rn2(10)) packadd(new CTool(OilLamp),pid);
            packadd(new CFood(FoodFortuneCookie, 3),pid);
            packadd(new CFood(FoodOrange, 5),pid);
            packadd(new CFood(FoodApple, 5),pid);
            packadd(new CFood(FoodRation),pid);
            packadd(new CPotion(PoHealing),pid);
            packadd(new CSpell(),pid);
            switch (rn2(90) / 30) {
                case 0: packadd(new CSpell(SpellHealing),pid); break;
                case 1: packadd(new CSpell(SpellProtection),pid); break;
                case 2: packadd(new CSpell(SpellSleep),pid); break;
            }
            packadd(new CArmor(CloakRobe, 1, 1),pid);
            packadd(new CArmor(GloveLeather, 1, 2),pid);
            break;

        case Priest:
            if (!rn2(10)) packadd(new CTool(MagicMarker),pid);
            else if (!rn2(10)) packadd(new CTool(OilLamp),pid);
            packadd(new CSpell(),pid);
            packadd(new CSpell(),pid);
            packadd(new CFood(FoodWolfsbane),pid);
            packadd(new CFood(FoodGarlic),pid);
            packadd(new CPotion(PoWater, 4),pid);
            packadd(new CArmor(ShieldSmall),pid);
            packadd(new CArmor(CloakRobe),pid);
            packadd(new CWeapon(WeMace, 1, 1),pid);
            break;

        case Ranger:
            packadd(new CFood(FoodCramRation, 4),pid);
            packadd(new CArmor(CloakDisplacement, 1, 2),pid);
            packadd(new CWeapon(WeArrow, rn1(10, 30)),pid);
            packadd(new CWeapon(WeArrow, rn1(10, 50), 2),pid);
            packadd(new CWeapon(WeBow, 1, 1),pid);
            packadd(new CWeapon(WeDagger, 1, 1),pid);
            break;

        case Rogue:
            if (!rn2(5)) packadd(new CTool(Blindfold),pid);
            packadd(new CTool(Sack),pid);
            packadd(new CTool(LockPick, 1, 9),pid);
            packadd(new CPotion(PoSickness),pid);
            packadd(new CArmor(ArmLeatherArmor, 1, 1),pid);
            packadd(new CWeapon(WeDagger, rn1(10, 6)),pid);
            packadd(new CWeapon(WeShortSword),pid);
            break;

        case Samurai:
            if (!rn2(5)) packadd(new CTool(Blindfold),pid);
            packadd(new CArmor(ArmSplint),pid);
            packadd(new CWeapon(WeYa, rn1(20, 26)),pid);
            packadd(new CWeapon(WeYumi),pid);
            packadd(new CWeapon(WeShortSword),pid);
            packadd(new CWeapon(WeKatana),pid);
            break;

        case Tourist:
            if (!rn2(25)) packadd(new CTool(TinOpener),pid);
            else if (!rn2(25)) packadd(new CTool(Leash),pid);
            else if (!rn2(25)) packadd(new CTool(Towel),pid);
            else if (!rn2(25)) packadd(new CTool(MagicMarker),pid);
            packadd(new CTool(CreditCard),pid);
            packadd(new CTool(ExpensiveCamera),pid);
            packadd(new CArmor(ShirtHawaiian),pid);
            packadd(new CScroll(ScrMagicMapping, 4),pid);
            packadd(new CPotion(PoExtraHealing, 2),pid);
            packadd(new CFood(10),pid);
            packadd(new CGold(rnd(1000)),pid);
            packadd(new CWeapon(WeDart, rn1(20, 21), 2),pid);
            break;

        case Valkyrie:
            if (!rn2(5)) packadd(new CTool(OilLamp),pid);
            packadd(new CFood(FoodRation),pid);
            packadd(new CArmor(ShieldSmall, 1, 3),pid);
            packadd(new CWeapon(WeDagger),pid);
            packadd(new CWeapon(WeLongSword, 1, 1),pid);
            break;

        case Wizard:
            if (!rn2(5)) packadd(new CTool(Blindfold),pid);
            if (!rn2(5)) packadd(new CTool(MagicMarker),pid);
            packadd(new CSpell(),pid);
            packadd(new CSpell(SpellForceBolt),pid);
            packadd(new CScroll(),pid);
            packadd(new CScroll(),pid);
            packadd(new CScroll(),pid);
            packadd(new CPotion(),pid);
            packadd(new CPotion(),pid);
            packadd(new CPotion(),pid);
            packadd(new CRing(),pid);
            packadd(new CRing(),pid);
            packadd(new CWand(),pid);
            packadd(new CArmor(CloakMagicResistance),pid);
            packadd(new CWeapon(WeQuaterstaff, 1, 1),pid);
            break;

        default:
            log << "ERROR: Cannot create pack for type " << (int)type << "\n";
            exit(1);
    }
}

/******************************************************************************/
/* These are the object probabilities for different parts of the dungeon */

struct icp {
    int         iprob;         /* probability of an item type */
    EObjectType iclass;        /* item class */
};

const struct icp normalprobs[] = {
{10, Weapon},
{10, Armor},
{20, Food},
{ 8, Tool},
{ 8, Gem},
{16, Potion},
{16, Scroll},
{ 4, Spellbook},
{ 4, Wand},
{ 3, Ring},
{ 1, Amulet} };

const struct icp boxprobs[] = {
{18, Gem},
{15, Food},
{18, Potion},
{18, Scroll},
{12, Spellbook},
{ 7, Gold},
{ 6, Wand},
{ 5, Ring},
{ 1, Amulet} };

const struct icp rogueprobs[] = {
{12, Weapon},
{12, Armor},
{22, Food},
{22, Potion},
{22, Scroll},
{ 5, Wand},
{ 5, Ring} };

const struct icp mazeprobs[] = {
{20, Weapon},
{20, Armor},
{16, Food},
{12, Tool},
{10, Gem},
{ 1, Potion},
{ 1, Scroll},
{ 8, Wand},
{ 8, Ring},
{ 4, Amulet} };

const struct icp mineprobs[] = {
{20, Weapon},
{20, Armor},
{16, Food},
{12, Tool},
{10, Gem},
{ 1, Potion},
{ 1, Scroll},
{ 8, Wand},
{ 8, Ring},
{ 4, Amulet} };

const struct icp sewerprobs[] = {
{20, Weapon},
{20, Armor},
{16, Food},
{12, Tool},
{10, Gem},
{ 1, Potion},
{ 1, Scroll},
{ 8, Wand},
{ 8, Ring},
{ 4, Amulet} };

const struct icp templateprobs[] = {
{20, Weapon},
{20, Armor},
{16, Food},
{12, Tool},
{10, Gem},
{ 1, Potion},
{ 1, Scroll},
{ 8, Wand},
{ 8, Ring},
{ 4, Amulet} };

/* Create a set of objects for the level (level number lnum)
   fair part of this code adapted from nethack 3.3.0 */
void CObjectList::createLevel(CLevel *lev) {
    const struct icp *iprobs, *tiprobs;
    int tprob, nobjs;
    EObjectType otype;
    CObject * tobj;

    log << "TODO: adjust level object probabilities" << '\n';

    if (lev->isNormal()) iprobs = normalprobs;
    else if (lev->isMine()) iprobs = mineprobs;
    else if (lev->isMaze()) iprobs = mazeprobs;
    else if (lev->isSewer()) iprobs = sewerprobs;
    else if (lev->isTemplate()) return;

    /* The actual generation in nethack 3.3.0 is done in mklev.c:818,
       for our purposes we generate fewer objects the deeper in the
       dungeon */
    nobjs = rn2(30 - lev->getNumber());
    if (lev->getNumber() < 20) nobjs = rn2(7);
    if (lev->getNumber() < 10) nobjs = rn2(10);

    while (nobjs-- > 0) {
        tiprobs = iprobs;
        otype = (EObjectType)0;
        for(tprob = rnd(100);
            (tprob -= tiprobs->iprob) > 0;
            tiprobs++);
            otype = tiprobs->iclass;

        switch (otype) {
            case Amulet:    tobj = new CAmulet();    break;
            case Armor:     tobj = new CArmor();     break;
            case Food:      tobj = new CFood();      break;
            case Gem:       tobj = new CGem();       break;
            case Potion:    tobj = new CPotion();    break;
            case Ring:      tobj = new CRing();      break;
            case Scroll:    tobj = new CScroll();    break;
            case Spellbook: tobj = new CSpell();     break;
            case Tool:      tobj = new CTool();      break;
            case Wand:      tobj = new CWand();      break;
            case Weapon:    tobj = new CWeapon();    break;

            /* Can not randomly create these objects */
            case Gold: case LevelObject: case NullObject: case Artifact:
                log << "ERROR: invalid random object construction on level" << '\n';
                exit(0);
        }
        tobj->xy = lev->getRandomAny();
        add(tobj);
    }
}
