/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifndef _objlist_h_
#define _objlist_h_

#include "object.h"

class CLevel;

class CObjectNode {
public:
    CObjectNode(CObjectNode *n, CObject *d) {
        next = n; data = d; };
    ~CObjectNode() {
        if (data != NULL) delete data;
        if (next != NULL) delete next;
        data = NULL; next = NULL; };
    /* Returns true if object was removed, false otherwise */
    bool remove(CObject *o) {
        if (next != NULL) {
            if (next->data == o) {
                CObjectNode *temp = next;
                next = next->next;
                temp->next = NULL;
                delete temp;
                return true;
            }
            return next->remove(o);
        }
        return false; };
    CObjectNode * move_remove(CObject *o) {
        if (next != NULL) {
            if (next->data == o) {
                CObjectNode *temp = next;
                next = next->next;
                temp->next = NULL;
                return temp;
            }
            return next->move_remove(o);
        }
        return NULL; };

    CObjectNode * find(EObjectType t) {
        if (t == data->getObjectType()) return this;
        if (next) next->find(t);
        return NULL; };

    CObject * find(ID id) {
        if (id == data->id) return data;
        if (next) return next->find(id);
        return NULL; };

    int getWeight() {
        if (next != NULL)
            return (data->getWeight() + next->getWeight());
        return data->getWeight(); };

    void polymorph() {
        data->polymorph();
        if (next != NULL) next->polymorph(); };

    CObjectNode * next;
    CObject * data;
private:
protected:
};

class CObjectList {
    /* Move object o from source to target
       returns true if successful, false otherwise */
    friend bool moveobject(CObject *o, CObjectList& src, CObjectList& tgt) {
        if (o == NULL) return false;
        CObjectNode *it = src.move_remove(o);
        if (it == NULL || it->data != o) return false;
        tgt.move_add(it);
        return true; };
public:
    CObjectList() {
        size = 0; head = NULL; copy = false;
        changed = false; weight = 0; pack = false; };
    CObjectList(const CObjectList& rhs) {
        size = rhs.size;
        head = rhs.head;
        changed = rhs.changed;
        weight = rhs.weight;
        lastFoundNode = rhs.lastFoundNode;
        lastFindType = rhs.lastFindType;
        copy = true; };
    ~CObjectList() {
        if (copy == false && head != NULL) delete head;
        head = NULL; size = 0; copy = false; weight = 0; };

    /* Create a players pack */
    void createPack(int, ID);

    /* Create objects for a level */
    void createLevel(CLevel *);

    CObject * findFirst(EObjectType t) {
        lastFoundNode = head;
        lastFindType = t;
        if (head != NULL)
            if (head->data->getObjectType() == t)
                return head->data;
        return findNext(); };
    CObject * findNext() {
        if (lastFoundNode != NULL) {
            lastFoundNode = lastFoundNode->next;
            if (lastFoundNode == NULL) return NULL;
            lastFoundNode = lastFoundNode->find(lastFindType);
        }
        if (lastFoundNode == NULL) return NULL;
        return lastFoundNode->data; };
    CObject * find(ID id) {
        if (head == NULL) return NULL;
        return head->find(id); };

    void add(CObject *);
    void packadd(CObject *, ID);
    bool remove(CObject *o) { /* Rets true if object removed, false otherwise */
        if (o == NULL) return false;
        if (head != NULL) {
            if (head->data == o) {
                CObjectNode *temp = head;
                head = head->next;
                temp->next = NULL;
                delete temp;
                changed = true;
                size--;
                return true;
            }
            bool tempb = head->remove(o);
            if (tempb == true) { changed = true; size--; }
            return tempb;
        }
        return false; };
    int getSize() { return size; };
    int getWeight() {
        if (changed && head != NULL) {
            weight = head->getWeight(); changed = false; };
        if (changed && head == NULL) { weight = 0; changed = false; };
        return weight; };

    void polymorph() {
        if (head != NULL) { head->polymorph(); changed = true; }; };

    void send(CSock * s, ID id) {
        *s << size;
        CObjectNode * temp;
        for (temp = head; temp != NULL; temp = temp->next)
            sendObject(s, temp->data, id); };
    void sendObject(CSock * s, CObject * o, ID id) {
        if (o->getNumber() < 1) {
            *s << "null"; *s << o->id;
            *s << o->getWeight(); *s << (char)o->getObjectType();
            *s << o->xy; *s << o->getColor(id);
            if (o->getObjectType() == Armor)
                *s << (char)((CArmor *)o)->getArmorType();
        } else {
            *s << o->getDescription(id); *s << o->id;
            *s << o->getWeight(); *s << (char)o->getObjectType();
            *s << o->xy; *s << o->getColor(id);
            if (o->getObjectType() == Armor)
                *s << (char)((CArmor *)o)->getArmorType();
        } };
    void validObjects() {
        if (size <= 0) return;
        CObjectNode * temp;
        CObjectNode * tempo;
        for (temp = head; temp != NULL; temp = temp->next)
            if (temp->next && temp->next->data->getNumber() < 1) {
                tempo = temp->next;
                temp->next = tempo->next;
                tempo->next = NULL;
                delete tempo;
            }
        if (head->data->getNumber() < 1) {
            tempo = head;
            head = tempo->next;
            tempo->next = NULL;
            delete tempo; } };
private:
    int size;
    bool copy;
    bool changed;		/* true if weight needs to be re-calculated */
    bool pack;			/* true if players backpack */
    int weight;
    CObjectNode * head;
    EObjectType lastFindType;
    CObjectNode * lastFoundNode;
protected:
    /* We do not want normal users accessing these */
    void move_add(CObjectNode *o) {
        if (o == NULL) return;
        o->next = head;
        head = o;
        changed = true; size++; };
    CObjectNode * move_remove(CObject *o) {
        if (o == NULL) return NULL;
        if (head != NULL) {
            if (head->data == o) {
                CObjectNode *temp = head;
                head = head->next;
                temp->next = NULL;
                changed = true;
                size--;
                return temp;
            }
            CObjectNode *tempn = head->move_remove(o);
            if (tempn != NULL) { changed = true; size--; }
            return tempn;
        }
        return NULL; };
};

#endif
