/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "player.h"
#include "log.h"
#include "plist.h"
#include "dungeon.h"
#include "vision.h"

int wizardfd;
int wizardr;

CPlayer::CPlayer(int new_id, int sockfd, char * from) {
    /* General player characteristics */
    sock = new CSock(sockfd);
    canplay = false;
    wizard = false;
    machine = from;
    id = new_id;
    name = "noname";
    invalidpasswd = 0;
    dlevel = NULL;
    symbol = '@';
    moveCount = 0;

    swallow = 0;
    invulnerable = 0;

    is_steed = 0;
    youseeit = 0;
    sleep = 0;
    displaced = 0;
    underwater = 0;
    uncancelled = 0;
    //stuck = 0;
    fire_resistance = 0;
    cold_resistance = 0;
    shock_resistance = 0;
    sleep_resistance = 0;
    half_physical_damage = 0;
    free_action = 0;
    drain_resistance = 0;
    stoned = 0;
    stone_resistance = 0;
    swimming = 0;
    amphibious = 0;
    lycn = 0;
    protection_from_shape_changers = 0;
    nomovemsg = 0;
    stun = 0;
    fast = 0;
    confusion = 0;
    antimagic = 0;
    acid_resistance = 0;
    slimed = 0;
    hp = 0;
    ac = 0;
    trap = 0;
    swldtim = 0;
    slow_digestion = 0;
    hallucination = 0;
    blinded = 0;
    ufound = 0;
    reflecting = 0;
    sick_resistance = 0;
    blindfolded = 0;


    string temp;
    temp = "---> id=";
    temp += id; temp += " machine="; temp += machine;
    log << temp << '\n';
    vision.msgline(temp);

    /* Actual properties of the player */
    invisible = aggrivate = blind = hallucinating = see_invisible =
    nutrition = confused = levitating = detect_monsters = detect_objects =
    paralised = sleeping = 0;
    speed = NORMAL_SPEED;

    /* Setup the sureal time stuff */
    STTimer = -1;
    STnForced = 0;
    STForcedRemain = 0;
    log << "FIXME: set players ST timeout correctly" << '\n';
    setTimeout(0, 1000);
    ih_timerclear(&STNext);
    ih_timerclear(&STForced);
    backlogged = 0;

    /* Setup what a player is wearing / carrying */
    helmet = gloves = boots = shirt = armor = cloak = shield = NULL;
    pri_weapon = sec_weapon = NULL;
    left_ring = right_ring = NULL;
    blindfold = NULL;
    amulet = NULL;
}

CPlayer::~CPlayer() {
    if (sock != NULL)
        delete sock;
    sock = NULL;
    dlevel = NULL;
    string temp;
    temp = "<--- ";
    temp += name;
    temp += '(';
    temp += id;
    temp += ") machine=";
    temp += machine;
    temp += " exit";
    log << temp << '\n';
    vision.msgline(temp);
}

/* Process incomming data from the player */
int CPlayer::incomming_data() {
    int r;
    do {
        sock->reset_buf();
        r = sock->fill_inbuf();
        if (r < 0) return -1; /* Check for lost connections*/
        if (r < 1) return 0; /* Ignore if there is no full message in buffer */
        *sock >> prot;
        if (canplay) {
            r = incomming_data_play();
            players.arrangeTimersM();
            dlevel->monsters.movemon();
            return r;
        } else
            return incomming_data_login();
    } while (r > 1); /* In case there is more than one message ready */
    return 0;
}

int CPlayer::incomming_data_backlog() {
    int r;
    sock->reset_buf();
    *sock >> prot;
    if (canplay) {
        r = incomming_data_play();
        players.arrangeTimersM();
        dlevel->monsters.movemon();
        return r;
    } else
        return incomming_data_login();
    return 0;
}

int CPlayer::incomming_data_login() {
    string filename, temppasswd, tempname, temp;
    ifstream *file = NULL;
    char line[256];
    int wizfd, wizr;

    switch (prot) {
        case p_name:
            *sock >> tempname;
            *sock >> passwd;

            /* Check size of name and passwd, disconnect if not */
            if (tempname.size() < 4 || tempname.size() > 8 ||
                passwd.size() < 4 || passwd.size() > 8) {
                log << "WARNING: id=" << id
                     << " invalid name or password size\n";
                return -1;
            }

            /* Check to make sure player is not already logged on */
            if (players.already_on(tempname) != 0) {
                log << "WARNING: " << tempname << " already logged on\n";
                if (++invalidpasswd >= 3) {
                    log << "WARNING: id=" << id
                         << " already logged on (3 unsuccessful tries)\n";
                    return -1;
                }
                *sock << p_name << p_end;
                break;
            }
            name = tempname;

            /* Check for player 'name' (a save file). */
            filename = PLAYERDIR;
            filename += name;
            file = new ifstream(filename.getStr());

            /* If file did not exist, new player */
            if (file == NULL || !(*file)) {
                *sock << p_type << p_end;
                if (file != NULL) delete file; file = NULL;
                break;
            }

            /* Check players password (assumed to be first line) */
            file->get(line, 256);
            temppasswd = line;
            file->close();
            delete file; file = NULL;
            if (passwd != temppasswd) {
                if (++invalidpasswd >= 3) {
                    log << "WARNING: id=" << id
                         << " invalid password (3 unsuccessful tries)\n";
                    return -1;
                }
                *sock << p_name << p_end;
            } else {
                load(); /* Load the players file */
                canplay = true;
            }
            break;

        case p_type:
            /* If the player has not sent their name and passwd, disconnect */
            if (name.size() < 4 || name.size() > 8 ||
                passwd.size() < 4 || passwd.size() > 8) {
                log << "WARNING: id=" << id
                     << " sent character type before valid name and passwd\n";
                return -1;
            }

            /* Check the character type */
            *sock >> attr.flags.initrole;
            attr.ptype = (enum role_types)attr.flags.initrole;
            *sock >> attr.flags.initrace;
            *sock >> attr.flags.initgend;
            *sock >> attr.flags.initalign;
            if (!attr.valid()) {
                log << "WARNING: id " << id
                     << " sent invalid player type\n";
                return -1; /* Brutal-disconnect */
            }
            /* Dragon has different symbol */
            if (attr.ptype == Dragon) symbol = 'D';

            /* Construct the player */
            attr.sconstruct();
            attr.setName(name);

            /* Construct the players pack */
            pack.createPack(attr.ptype, id);

            /* Place the player on the level */
            xy = dungeon->getStart();

            /* Send the player the level */
            dlevel = dungeon->getFirst();
            *sock << p_level;
            dlevel->send(sock, id);
            *sock << xy; *sock << p_end;

            /* Send player their stats. first get level name */
            attr.setLevel(dlevel->getName());
            *sock << p_stats; *sock << attr; *sock << p_end;

            /* Re-arrange monsters assignments on the dungeon level */
            dlevel->monsters.assignToPlayers(dlevel);

            /* Send player their pack
               Can not simply use << pack, as the getDescription function
               requires the players id to get the correct description */
            *sock << p_pack;
            pack.send(sock, id);
            *sock << p_end;

            /* Send updated player list to all players */
            players.sendToAll();

            /* Update the sureal time timers */
            players.arrangeTimers();

            temp = "*** ";
            temp += name; temp += " logged in";
            log << temp << '\n';
            vision.msgline(temp);
            canplay = true;
            break;

        case p_quit:
            temp = "*** ";
            temp += name; temp += " quit";
            log << temp << '\n';
            vision.msgline(temp);
            return -1;
            break;

        case p_wizard:
            *sock >> wizfd;
            *sock >> wizr;
            if (wizardfd != wizfd || wizardr != wizr) {
                log << "WARNING: id=" << id
                     << " ilegal wizard mode login\n";
                return -1;
            }
            temp = "*** id=";
            temp += id; temp += " started wizard mode";
            log << temp << '\n';
            vision.msgline(temp);
            wizard = true;
            break;

        default:
            log << "WARNING: " << name << '(' << id
                 << ") trying to play when not logged in" << '\n';
            return -1; /* Brutal-anything else results in disconnection */
    }
    return 0;
}

int CPlayer::incomming_data_play() {
    int wizfd, wizr;
    switch (prot) {
        /* THESE ARE NOT ALLOWED - DISCONNECT */
        case p_name: /* Player name and password */
            log << "NAME\n";
        case p_type: /* Player type */
            log << "TYPE\n";
        case p_stats: /* Player stats */
            log << "STATS\n";
        case p_level: /* Player can not request a level */
            log << "LEVEL\n";
        case p_list: /* Player is always send list of players */
            log << "WARNING: " << name << " sent non-allowed protocol\n";
            return -1; /* Brutal Disconnect */
            break;

        /* GAME CONTROL COMMANDS */
        case p_quit: /* Player quit */
            log << "*** " << name  << " quit" << '\n';
            players.sendToAllExclude(id);
            quitObjects(id); /* Rand obj. desc. for next player */
            return -1;
        case p_save: /* Player saved */
            log << "*** " << name  << " saved" << '\n';
            players.sendToAllExclude(id);
            save();
            return -1;

        /* COMMUNICATION COMMANDS */
        case p_yell: /* Player yelled */
        case p_talk: /* Player talked */
        case p_chat: /* Player chatted */
            return incomming_communication();
            break;

        /* GENERAL MOVEMENT COMMANDS */
        case p_move: /* Player moved */
        case p_jump: /* Player jumped */
        case p_wait: /* Player wait */
        case p_open: /* Player opened something */
        case p_close: /* Player closed something */
        case p_search: /* Player searched */
        case p_pickup: /* Player picked something up */
        case p_drop: /* Player dropped an item */
            return incomming_movement();
            break;

        /* GENERAL ATTACK COMMANDS */
        case p_hit: /* Hit something */
            return hit();
        case p_throw: /* Throw something */
            return othrow();
        case p_cast: /* Cast a spell */
            return cast();
        case p_zap: /* Zap a wand */
            return zap();
        case p_kick: /* Kick an object */
            return kick();

        /* OTHER ACTION COMMANDS */
        case p_quaff: /* Quaff a potion */
            return quaff();
        case p_wield: /* Wield an object as a weapon */
            return wield();
        case p_exchange: /* Exchange primary and secondary weapon */
            return exchange();
        case p_wear: /* Wear some armor */
            return wear();
        case p_eat: /* Eat something */
            return eat();
        case p_engrave: /* Engrave on the floor */
            return engrave();
        case p_pay: /* Pay a shopkeeper */
            return pay();
        case p_read: /* Read a scroll or spell */
            return read();
        case p_puton: /* Put an accessory on */
            return puton();
        case p_takeoff: /* Remove an accessory */
            return takeoff();
        case p_remove: /* Remove some armor */
            return remove();
        case p_dip: /* Dip a potion into something else */
            return dip();

        /* WIZARD MODE COMMANDS */
        case p_wizard:
            *sock >> wizfd;
            *sock >> wizr;
            if (wizardfd != wizfd || wizardr != wizr) {
                log << "WARNING: id=" << id
                     << " ilegal wizard mode login\n";
                return -1;
            }
            log << "WIZARD: mode started for id=" << id << '\n';
            wizard = true;
            break;

        /* ANY OTHER UNKNOWN COMMANDS */
        default: /* Anything else produces a log warning message and disconnection */
            log << "WARNING: " << name << " invalid protocol "
                 << (int)prot << '\n';
            return -1;
    }
    return 0;
}

int CPlayer::lost_connection() {
    return 0;
}

/* Deal with incommin communication. Three types:
   p_yell (all players on the level),
   p_talk (all players in 16 unit radius on level),
   p_chat (to a specific player / monster). */
int CPlayer::incomming_communication() {
    string msg, tmp;

    /* Convert string to the display format on clients and log the msg */
    *sock >> msg;
    msg.validStr();
    if (prot == p_yell) msg.upper();
    else msg.lower();
    log << name << ": " << msg << '\n';

    /* Check for bad language... log a warning message! */
    tmp = msg;
    tmp.lower();
    log << "TODO: Check for bad lang in communication from players\n";

    /* Send the communication to the appropriate group */
    tmp = name;
    tmp += ": ";
    tmp += msg;
    if (prot == p_yell)
        players.yellMsg(tmp);
    else
        players.talkMsg(tmp, id);

    return 0;
}

int CPlayer::incomming_movement() {
    Direction dir;
    EFloorType ploctype;
    ID oid;
    CObject * otmp = NULL;
    switch (prot) {
        case p_move:
            *sock >> (int &)dir;
            if (dir == d_none || dir == d_special || dir == d_invalid) {
                log << "WARNING: Invalid movement direction "
                     << (int)dir << '\n';
                return -1;
            }

            /* Check for movement up and down stairs */
            if (dir == d_up || dir == d_down) {
                CLevel * temp = dlevel->get(xy);
                if (temp == NULL) {
                    log << "WARNING: Invalid movement up/down\n";
                    return -1;
                }
                xy = dlevel->getStart(xy);
                dlevel = temp;
                *sock << p_level;
                dlevel->send(sock, id);
                *sock << xy << p_end;
                /* Re-arrange monsters assignments on the dungeon level */
                dlevel->monsters.assignToPlayers(dlevel);
                players.sendToAll(); /* Update all players lists */
                break;
            }

            /* Check sureal time! */
            if (players.doMove(id) == false)
                break;

            /* Move the player */
            if (xy.x > 1 && dir%3 == 0) xy.x--;
            if (xy.x < WIDTH && (dir+1)%3 == 0) xy.x++;
            if (xy.y > 1 && dir > 5) xy.y--;
            if (xy.y < HEIGHT && dir < 3) xy.y++;

            /* Check player is within bounds of the map */
            if (xy.x < 1 || xy.x >= WIDTH ||
                xy.y < 1 || xy.y >= HEIGHT) {
                log << "WARNING: Invalid player location "
                     << xy.x << ',' << xy.y << '\n';
                return -1;
            }

            /* Check player can walk where they are standing */
            ploctype = dlevel->map[xy.y][xy.x];
            if (ploctype != EFTFloor && ploctype != EFTCorridor &&
                ploctype != EFTUpStair && ploctype != EFTDownStair) {
                log << "WARNING: Player walking through something ("
                     << (int)ploctype << ')' << xy.x << ','
                     << xy.y << '\n';
                return -1;
            }
            /* Check players are not walking over each other */
            if (players.sameLocation(id, xy) == true) {
                log << "WARNING: Player walking over another player ("
                     << id << ')' << xy.x << ',' << xy.y << '\n';
                return -1;
            }
            /* Check players are not walking on monsters */
            if (dlevel->monsters.find(xy) != NULL) {
                log << "WARNING: Player walking on monster ("
                     << id << ')' << xy.x << ',' << xy.y << '\n';
                return -1;
            }

            moveCount++; /* Incriment players move counter */

            players.sendMovement(id);
            lastDirection = dir;
            break;

        /* Player has dropped an object */
        case p_drop:
            *sock >> oid;
            otmp = pack.find(oid);
            if (otmp == NULL) {
                log << "WARNING: player dropping non-existant object ("
                    << id << ")" << '\n';
                return -1;
            }
            if (otmp == amulet || otmp == left_ring || otmp == right_ring ||
                otmp == blindfold || otmp == boots || otmp == gloves ||
                otmp == helmet || otmp == shirt || otmp == armor ||
                otmp == cloak) {
                log << "WARNING: player dropping object being worn" << '\n';
                return -1;
            }
            if (moveobject(otmp, pack, dlevel->objects) == false) {
                log << "WARNING: player could not drop object ("
                    << id << ")" << '\n';
                return -1;
            }

            /* Check sureal time! */
            if (players.doMove(id) == false)
                break;

            otmp->xy = xy;
            players.drop(this, otmp);
            moveCount++; /* Incriment players move counter */
            break;

        /* Player has picked up an object */
        case p_pickup:
            *sock >> oid;
            otmp = dlevel->objects.find(oid);
            if (otmp == NULL) {
                log << "WARNING: player pickup non-existant object ("
                    << id << ")" << '\n';
                return -1;
            }
            if (moveobject(otmp, dlevel->objects, pack) == false) {
                log << "WARNING: player could not pickup object ("
                    << id << ")" << '\n';
                return -1;
            }

            /* Check sureal time! */
            if (players.doMove(id) == false)
                break;

            otmp->xy.set(-1,-1);
            players.pickup(this, otmp);
            moveCount++; /* Incriment players move counter */
            break;

        default:
            log << "TODO: Complete incoming movement" << '\n';
    }
    return 0;
}

/* Player has been forced to make a move by sureal time
   forced moves should never be illeagal */
void CPlayer::forcedMove() {
    EFloorType ploctype;

    /* this currently a temporary hack! */

    CLocation oxy = xy;

    /* Move the player */
    if (xy.x > 1 && lastDirection%3 == 0) xy.x--;
    if (xy.x < WIDTH && (lastDirection+1)%3 == 0) xy.x++;
    if (xy.y > 1 && lastDirection > 5) xy.y--;
    if (xy.y < HEIGHT && lastDirection < 3) xy.y++;
            
    /* Check player is within bounds of the map */
    if (xy.x < 1 || xy.x >= WIDTH ||
        xy.y < 1 || xy.y >= HEIGHT) {
        xy = oxy;
        return;
    }
            
    /* Check player can walk where they are standing */
    ploctype = dlevel->map[xy.y][xy.x];
    if (ploctype != EFTFloor && ploctype != EFTCorridor &&
        ploctype != EFTUpStair && ploctype != EFTDownStair) {
        xy = oxy;
        return;
    }
    /* Check players are not walking over each other */
    if (players.sameLocation(id, xy) == true) {
        xy = oxy;
        return;
    }
 
    players.arrangeTimersM();
    players.sendMovement(id);
}

/* Player has been forced to quit by sureal time */
void CPlayer::forcedQuit() {
    log << "TODO: forcedQuit\n";
}


