/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifndef _player_h_
#define _player_h_

#include "level.h"
#include "system.h"
#include "wepskill.h"
#include "attrib.h"

class CPlayer {
public:
    CPlayer(int, int, char *);
    ~CPlayer();
    int incomming_data();
    int lost_connection();
    char * getName() { return name.getStr(); };
    char * getFrom() { return machine.getStr(); };
    bool isWizard() { return wizard; };
    CSock * sock;
    char * getDungeon() { return name.getStr(); };
    char * getFlags() { return name.getStr(); };
    int getLevelNum() {
        if (dlevel !=  NULL) return dlevel->getNumber();
        return -1; };
    char getSymbol() { return symbol; };
    int getMoveCount() { return moveCount; };



    int isInvisible() { return invisible; };
    int isAggrivate() { return aggrivate; };
    int isBlind() { return blind; };
    int isHallucinating() { return hallucinating; };
    int canSeeInvisible() { return see_invisible; };
    int getNutrition() { return nutrition; };
    int isConfused() { return confused; };
    int isLevitating() { return levitating; };
    int canDetectMonsters() { return detect_monsters; };
    int canDetectObjects() { return detect_objects; };
    int isParalised() { return paralised; };
    int isSleeping() { return sleeping; };
    int isFast() { return speed > NORMAL_SPEED; };
    int getSpeed() { return speed; };

    /* Sureal time variables */
    int STTimer;        // The timer this player is associated with
    Timeval STTimeout;  // The players timeout value (connection speed)
    Timeval STNext;     // When the next move can be made
    Timeval STForced;   // When the player is forced to move
    int STForcedRemain; // Number of forced moves yet to be made
    int STnForced;      // Number of contiguous forced moves
    int backlogged;     // Number of backlogged messages

    /* Sureal time functions */
    Direction lastDirection;  /* For forced movement */
    int incomming_data_backlog();
    void forcedMove();  // Force the player to move
    void forcedQuit();  // Force the player to quit
    void setTimeout(int a, int b) {
        STTimeout.tv_sec = a; STTimeout.tv_usec = b; };

    /* Direct access to these variables */
    int id;
    CLocation xy;
    CLevel * dlevel;
    CAttributes attr;

    /* This player is holding / wearing */
    CArmor * helmet, * gloves, * boots, * shirt, * armor, * cloak, * shield;
    CObject * pri_weapon, * sec_weapon; /* Anything can be weapon */
    CRing * left_ring, * right_ring;
    CTool * blindfold;  /* Towel or blindfold */
    CAmulet * amulet;

 //TEST VARIABLES
    int swallow;
    int invulnerable;
    struct stuck
    {
    	MonsterType * stuckmon;
    	CLocation xy;
    } ptmpstuck;
    MonsterType *steed;
    //int is_steed;
    int youseeit;
    int sleep;
    int multi;
    int displaced;
    int underwater;
    int uncancelled;
    int fire_resistance;
    int cold_resistance;
    int shock_resistance;
    int sleep_resistance;
    int half_physical_damage;
    int free_action;
    int drain_resistance;
    int stoned;
    int stone_resistance;
    int swimming;
    int amphibious;
    int lycn;
    int protection_from_shape_changers;
    int blind;
    int nomovemsg;
    int stun;
    int fast;
    int confusion;
    int antimagic;
    int acid_resistance;
    int slimed;
    int hp;
    int ac;
    int trap;
    int swldtim;
    int slow_digestion;
    int hallucination;
    int blinded;
    int ufound;
    int reflecting;
    int sick_resistance;
    int blindfolded;
    int is_steed;


private:
    CWeaponSkills weaponSkills;
    string machine, name, passwd;
    bool canplay, wizard;
    int invalidpasswd;
    char symbol;
    CObjectList pack;
	CMonsterList mlist;
    int moveCount;

    /* These are player properties. In general:
           -1 - the property is perminant,
            0 - currently not applied to player,
           >0 - number of terms remaining with this property. */
    int invisible;
    int aggrivate;

    int hallucinating;
    int see_invisible;
    int nutrition;
    int confused;
    int levitating;
    int detect_monsters;
    int detect_objects;
    int paralised;
    int sleeping;
    int speed;			/* 1->36, normal human = 12 */
    int sickness;

    Protocol prot; /* Incoming data protocol */
    int incomming_data_login();
    int incomming_data_play();
    int incomming_communication();
    int incomming_movement();

    void load();
    void save();
    int quaff();
    int read();
    int wield();
    int exchange();
    int eat();
    int engrave();
    int apply();
    int pay();
    int wear();
    int remove();
    int puton();
    int takeoff();
    int hit();
    int othrow();
    int cast();
    int zap();
    int kick();
    int dip();
protected:
};

extern int wizardfd;
extern int wizardr;

#endif
