/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

/* Potions are now Nethack 3.3.0 compliant */

struct S_Potion {
    char *name;
    char *desc;
    char magic;
    char prob;
    int cost;
    char color;
} SPotions[] = {

#define POTION(name,desc,mgc,power,prob,cost,color) \
	{name, desc, mgc, prob, cost, color}

POTION("gain ability", "ruby",          1, 0,          42, 300, CLR_RED),
POTION("restore ability", "pink",       1, 0,          40, 100, CLR_BRIGHT_MAGENTA),
POTION("confusion", "orange",           1, CONFUSION,  42, 100, CLR_ORANGE),
POTION("blindness", "yellow",           1, BLINDED,    40, 150, CLR_YELLOW),
POTION("paralysis", "emerald",          1, 0,          42, 300, CLR_BRIGHT_GREEN),
POTION("speed", "dark green",           1, FAST,       42, 200, CLR_GREEN),
POTION("levitation", "cyan",            1, LEVITATION, 42, 200, CLR_CYAN),
POTION("hallucination", "sky blue",     1, HALLUC,     40, 100, CLR_CYAN),
POTION("invisibility", "brilliant blue",1, INVIS,      40, 150, CLR_BRIGHT_BLUE),
POTION("see invisible", "magenta",      1, SEE_INVIS,  42,  50, CLR_MAGENTA),

POTION("healing", "purple-red",         1, 0,          57, 100, CLR_MAGENTA),
POTION("extra healing", "puce",         1, 0,          47, 100, CLR_RED),
POTION("gain level", "milky",           1, 0,          20, 300, CLR_WHITE),
POTION("enlightenment", "swirly",       1, 0,          20, 200, CLR_BROWN),
POTION("monster detection", "bubbly",   1, 0,          40, 150, CLR_WHITE),
POTION("object detection", "smoky",     1, 0,          42, 150, CLR_GRAY),
POTION("gain energy", "cloudy",         1, 0,          42, 150, CLR_WHITE),
POTION("sleeping",  "effervescent",     1, 0,          42, 100, CLR_GRAY),
POTION("full healing",  "black",        1, 0,          10, 200, CLR_BLACK),
POTION("polymorph", "golden",           1, 0,          10, 200, CLR_YELLOW),

POTION("booze", "brown",                0, 0,          42,  50, CLR_BROWN),
POTION("sickness", "fizzy",             0, 0,          42,  50, CLR_CYAN),
POTION("fruit juice", "dark",           0, 0,          42,  50, CLR_BLACK),
POTION("acid", "white",                 0, 0,          10, 250, CLR_WHITE),
POTION("oil", "murky",                  0, 0,          30, 250, CLR_BROWN),
POTION("water", "clear",                0, 0,          92, 100, CLR_CYAN),
#undef POTION
};

#define POTIONDESCS	26

static bool potionKnown[POTIONDESCS][MAXPLAYERS];
static char SPotionRandDesc[POTIONDESCS][MAXPLAYERS];

void initPotions() {
    for (int x = 0; x < MAXPLAYERS; x++)
        quitPotions(x);
}

void loadPotions(ID user) {
    /* TODO: load users knowledge of potions */
}

void savePotions(ID user) {
    /* TODO: save users knowledge of potions */
    quitPotions(user); /* Just randomize the descriptions for next player */
}

/* Essentially randomize the descriptions for next player */
void quitPotions(ID user) {
    for (int y = 0; y < POTIONDESCS-1; y++) {
        potionKnown[y][user] = false;
        SPotionRandDesc[y][user] = y;
    }
    for (int y = 0; y < POTIONDESCS-1; y++) {
        int r = rn2(POTIONDESCS-1);
        char temp = SPotionRandDesc[y][user];
        SPotionRandDesc[y][user] = SPotionRandDesc[r][user];
        SPotionRandDesc[r][user] = temp;
    }
}

CPotion::CPotion() : CObject(Potion, '!', 20) {
    polymorph();
}

CPotion::CPotion(EPotionType t) : CObject(Potion, '!', 20) {
    potionType = t;
    int r = rn2(128);
    if (r < 16) curseIt();
    else if (r >= 112) blessIt();
    setCost(SPotions[potionType].cost);
    diluted = false;
}

CPotion::CPotion(EPotionType t, int n) : CObject(Potion, '!', 20) {
    potionType = t;
    int r = rn2(128);
    if (r < 16) curseIt();
    else if (r >= 112) blessIt();
    setCost(SPotions[potionType].cost);
    diluted = false;
    number = n;
}

void CPotion::polymorph() {
    potionType = (EPotionType)0;
    for(int tprob = rnd(1000);
        (tprob -= SPotions[potionType].prob) > 0;
        potionType = (EPotionType)(potionType+1));
    int r = rn2(128);
    if (r < 16) curseIt();
    else if (r >= 112) blessIt();
    setCost(SPotions[potionType].cost);
    diluted = false;
}

string CPotion::getDescription(ID user) {
    string s;
    if (getNumber() > 1) {
        s = string(number);
        s += ' ';
    } else
        s = "a ";
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
        if (isDiluted()) s += "diluted ";
    }
    if (isKnown(user)) {
        s += "potion";
        if (getNumber() > 1) s += "s";
        s += " of ";
        s += SPotions[potionType].name;
    } else {
        s += SPotions[SPotionRandDesc[potionType][user]].desc;
        s += " potion";
        if (getNumber() > 1) s += "s";
    }
    if (number < 1) s += "[invalid]";
    return s;
}

int CPotion::getColor(ID user) {
    return SPotions[SPotionRandDesc[potionType][user]].color;
}

bool CPotion::isKnown(ID user) {
    return potionKnown[potionType][user];
}

void CPotion::identify(ID user) {
    potionKnown[potionType][user] = true;
    BCidentify(user);
}

// This is where all the heavy duty dipping code is
// Basically we are dipping 'it' into this
void CPotion::dip(CObject& it) {

    switch (it.getObjectType()) {
        case Potion: dip((CPotion&)it); return;
        case Tool:   dip((CTool&)it);   return;
        case Weapon: dip((CWeapon&)it); return;
        default: break;
    }

    dipFinish(it);
    number--;

    // All other combinations result in this:
    //cout << "Interesting...\n";
}

// This performs all the default dip stuff, like bless / curse status,
// and reduces the number of potions in inventory
void CPotion::dipFinish(CObject& it) {
    // Dipping object into holy water
    if (potionType == PoWater && isBlessed()) {
        if (it.isCursed()) it.neutralIt();
        else it.blessIt();
    }
    // Dipping object into unholy water
    if (potionType == PoWater && isCursed()) {
        if (it.isBlessed()) it.neutralIt();
        else it.curseIt();
    }
}

void CPotion::dip(CPotion& it) {
    // The player tried to dip a potion into itself
    if (this == &it) {
        //cout << "That is a potion bottle, not a Klein bottle!\n";
        return;
    }

    // Dipping a potion into itself
    if (it.potionType == potionType) {
        //cout << "Interesting...\n";
        return;
    }

    // Always explode if it is cursed
    if (it.isCursed()) {
        //cout << "[Explosion]...\n";
        return;
    }

    // Potion into uncursed water
    if (!isBlessed() && !isCursed() && potionType == PoWater) {
        // Diluted potion into uncursed water
        if (it.isDiluted()) {
            it.potionType = PoWater;
            it.diluted = false;
            it.number++;

        // Undiluted potion into uncursed water
        } else
            it.diluted = true;
    }

//    EPotionType oldp = it.potionType;

    switch (it.potionType) {
        case PoBooze:
            if (potionType == PoEnlightenment)
                it.potionType = PoConfusion;
            if (potionType == PoGainEnergy || potionType == PoGainLevel)
                it.potionType = PoHallucination;
            break;
        case PoConfusion:
            if (potionType == PoGainEnergy || potionType == PoGainLevel) {
                int r = rn2(99);
                if (r < 33) it.potionType = PoEnlightenment;
                else it.potionType = PoBooze;
            }
            break;
        case PoEnlightenment:
            if (potionType == PoBooze) it.potionType = PoConfusion;
            break;
        case PoExtraHealing:
            if (potionType == PoGainEnergy || potionType == PoGainLevel)
                it.potionType = PoGainAbility;
            break;
        case PoFruitJuice:
            if (potionType == PoEnlightenment || potionType == PoSpeed)
                it.potionType = PoBooze;
            else if (potionType == PoGainEnergy || potionType == PoGainLevel)
                it.potionType = PoSeeInvisible;
            else if (potionType == PoSickness)
                it.potionType = PoSickness;
            break;
        case PoGainEnergy: case PoGainLevel:
            if (potionType == PoBooze)
                it.potionType = PoHallucination;
            else if (potionType == PoExtraHealing)
                it.potionType = PoGainAbility;
            else if (potionType == PoFruitJuice)
                it.potionType = PoSeeInvisible;
            else if (potionType == PoHealing)
                it.potionType = PoExtraHealing;
            else if (potionType == PoConfusion) {
                int r = rn2(99);
                if (r < 33) it.potionType = PoEnlightenment;
                else it.potionType = PoBooze;
            }
            break;
        case PoHealing:
            if (potionType == PoGainEnergy || potionType == PoGainLevel ||
                potionType == PoSpeed)
                it.potionType = PoExtraHealing;
            break;
        case PoLevitation:
            if (potionType == PoEnlightenment && rn2(99) < 66)
                it.potionType = PoGainLevel;
            break;
        case PoSickness:
            if (potionType == PoFruitJuice) it.potionType = PoSickness;
            break;
        case PoSpeed:
            if (potionType == PoFruitJuice) it.potionType = PoBooze;
            else if (potionType == PoHealing) it.potionType = PoExtraHealing;
            break;
        default:
            break;
    }

    dipFinish(it);
    number--;
}

void CPotion::dip(CTool& it) {
    dipFinish(it);
    number--;
}

void CPotion::dip(CWeapon& it) {
    // Poison the weapon
    if (potionType == PoSickness) {
        int d = isDiluted() ? 2 : 1;
        if (isBlessed())
            it.setPoisoned(rnd(20/d)+5);
        else if (isCursed())
            it.setPoisoned(rnd(20/d)+5);
        else
            it.setPoisoned(rnd(10/d)+5);
    }

    // Unpoison the weapon
    else if (potionType == PoHealing || potionType == PoExtraHealing)
        it.unsetPoisoned();

    else if (potionType == PoOil) {
        // Damage the weapon more
        if (it.isDamaged() == true)
            it.addDamage(1);

        // Grease the hands...
        else
            ;
    }

    dipFinish(it);
    number--;
}

bool CPotion::operator== (CPotion *p) {
    if (p->potionType != potionType)
        return false;
    if (p->isBlessed() != isBlessed())
        return false;
    if (p->isCursed() != isCursed())
        return false;
    return true;
}
