/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

/* This file contains the protocol numbers. The basic message layout is as
   follows:
       Byte Use
        0   Message length, low byte
        1   Message length, high byte
        2   Message protocol number
        3+  Message information
        n   Set to zero, used to flush message buffer
    Damian Bentley   (dbentley@mcs.une.edu.au)   09-Nov-1998
*/

#ifndef _protocol_h_
#define _protocol_h_

typedef enum E_Protocol {
    /* These are the general login / control commands */
    p_name,	/* Name and password */
    p_type,	/* Player character type - for new players */
    p_stats,	/* Statistics of the player */
        p_ac,       /* Armor class */
        p_hp,       /* Hit points */
    p_list,	/* Information about other players in the game */
    p_player,	/* Update a player on the list */
    p_quit,	/* Player quit / server tells player to quit */
    p_save,	/* Player save command */
    p_level,	/* Level information */
    p_object,	/* Update an object */
    p_monster,	/* Update a monster */
    p_pack,	/* Players pack / objects */
    p_mlist,	/* Monster list / monsters */
    p_cursed,	/* Inform player of cursed object */

    /* Communication commands */
    p_yell, p_talk, p_chat, p_topline,

    /* Movement commands */
    p_move, p_jump, p_wait, p_open, p_close, p_search, p_pickup, p_drop,

    /* General attack commands */
    p_hit, p_throw, p_cast, p_zap, p_kick,

    /* Other action commands */
    p_wield, p_quaff, p_eat, p_engrave, p_pay, p_read, p_apply,
    p_wear, p_remove, p_puton, p_takeoff, p_exchange, p_dip,

    /* Wizard mode commands */
    p_wizard,		/* A client is requesting to enter wizard mode */
    p_wizdetect,	/* Detetect secret doors and traps */
    p_wizmagicmap,	/* Magic mapping */
    p_wizcreatemon,	/* Create monster */
    p_wizidentify,	/* Identify items in pack */
    p_wizlevels,	/* List of special levels in dungeon */
    p_wizteleport,	/* Teleport within a level */
    p_wiztransport,	/* Teleport between levels */
    p_wizwish,		/* Wish for an object */
    p_wizattributes,	/* Show attributes */
    p_wizkill		/* Kill a connection */
} Protocol;

/* These are the directions, which are in relation to the keypad */
typedef enum E_Direction {
    d_southwest,
    d_south,
    d_southeast,
    d_west,
    d_none,
    d_east,
    d_northwest,
    d_north,
    d_northeast,
    d_up,
    d_down,
    d_special,		/* Move to a location x, y */
    d_invalid           /* invalid direction given */
} Direction;

#endif
