/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "player.h"
#include "log.h"
#include "plist.h"

/* Player wants to put an object on */
int CPlayer::puton() {
    ID tid;
    CObject * tobj;
    EObjectType t;
    int loc;
    string tstr;

    *sock >> tid;
    tobj = pack.find(tid);
    if (tobj == NULL) {
        log << "WARNING: player puton invalid object" << '\n';
        return -1;
    }
    if (tobj == amulet || tobj == left_ring || tobj == right_ring) {
        log << "WARNING: player puton object already being worn" << '\n';
        return -1;
    }

    /* Check sureal time! */
    if (players.doMove(id) == false)
        return 0;

    moveCount++;

    t = tobj->getObjectType();
    if (t == Amulet && amulet == NULL) {
        amulet = (CAmulet *)tobj;
        tstr = tobj->getDescription(id);
        tstr += " (being worn)";
        *sock << p_puton << amulet->id << BODY_AMULET
              << tstr << p_end;
    } else if (t == Ring && (left_ring == NULL || right_ring == NULL)) {
        *sock >> loc;
        if (loc == BODY_LEFT_RING && left_ring == NULL) {
            left_ring = (CRing *)tobj;
            tstr = tobj->getDescription(id);
            tstr += " (on left hand)";
            *sock << p_puton << left_ring->id << BODY_LEFT_RING
                  << tstr << p_end;
        } else if (loc == BODY_RIGHT_RING && right_ring == NULL) {
            right_ring = (CRing *)tobj;
            tstr = tobj->getDescription(id);
            tstr += " (on right hand)";
            *sock << p_puton << right_ring->id << BODY_RIGHT_RING
                  << tstr << p_end;
        } else {
            log << "WARNING: player puton ring invalid location" << '\n';
            return -1;
        }
    } else {
        if (t == Amulet || t == Ring)
            log << "WARNING: player puton amulet or ring with no free space" << '\n';
        else
            log << "WARNING: player puton non amulet or ring" << '\n';
        return -1;
    }
    return 0;
}

/* Player wants to take an object off */
int CPlayer::takeoff() {
    ID tid;
    CObject * tobj;
    string tstr;

    *sock >> tid;
    tobj = pack.find(tid);
    if (tobj == NULL) {
        log << "WARNING: player takeoff non-existing object" << '\n';
        return -1;
    }

    /* Check the object is not cursed - not actually counted as a move */
    if (tobj->isCursed() && (amulet == tobj || left_ring == tobj ||
        right_ring == tobj)) {
        tobj->BCidentify(id);
        tstr = tobj->getDescription(id);
        if (amulet == tobj) tstr += " (being worn)";
        if (left_ring == tobj) tstr += " (on left hand)";
        if (right_ring == tobj) tstr += " (on right hand)";
        *sock << p_cursed << tobj->id << tstr << p_end;
        return 0;
    }

    /* Check sureal time! */
    if (players.doMove(id) == false)
        return 0;

    moveCount++;

    tstr = tobj->getDescription(id);

    if (amulet == tobj) {
        amulet = NULL;
        *sock << p_takeoff << BODY_AMULET << tstr << p_end;
    } else if (left_ring == tobj) {
        left_ring = NULL;
        *sock << p_takeoff << BODY_LEFT_RING << tstr << p_end;
    } else if (right_ring == tobj) {
        right_ring = NULL;
        *sock << p_takeoff << BODY_RIGHT_RING << tstr << p_end;
    } else {
        log << "WARNING: player takeoff something not puton" << '\n';
        return -1;
    }
    return 0;
}
