/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "player.h"
#include "log.h"
#include "plist.h"
#include "monster.h"

int CPlayer::quaff() {
    string pmsg;
    ID oid;
    CObject * tempo;
    CPotion * pot;
    *sock >> oid;
    tempo = pack.find(oid);
    if (tempo == NULL) {
        log << "WARNING: player quaffing invalid object\n";
        return -1;
    }
    if (tempo->getObjectType() != Potion) {
        log << "WARNING: player quaffing non-potion\n";
        return -1;
    }

    /* Check sureal time! */
    if (players.doMove(id) == false)
        return 0;

    moveCount++;

    pot = (CPotion *)tempo;
    switch (pot->getPotionType()) {
        case PoBooze:
            log << "TODO: full effects of quaff booze\n";
            log << "TODO: Booze messages\n";
            if (pot->isBlessed())
                nutrition += 30;
            else if (pot->isCursed())
                { nutrition += 10; confused += rn1(3, 20); }
            else
                { nutrition += 20; confused += rn1(3, 20); }
/*            if (pot->isDiluted() && hp < maxhp)
                hp++;*/
            break;

        case PoFruitJuice:
        case PoSeeInvisible:
        case PoSickness:
            log << "TODO: quaffing a potion\n";
            break;

        case PoConfusion:
            log << "TODO: confusion messages\n";
            if (pot->isBlessed())
                confused += rn1(8, 7);
            else if (pot->isCursed())
                confused += rn1(24, 7);
            else
                confused += rn1(16, 7);
            break;

        case PoExtraHealing:
            log << "TODO: Full effects of extra healing\n";
            players.seenMsg("NAME looks much better.",
                "NAME gives you the thumbs up!",
                "You feel much better.", id);
            if (pot->isBlessed()) {
                //hp += rn1(8, 57);
                //int increasemaxhp = hp - maxhp;
                //if (increasemaxhp > 5) increasemaxhp = 5;
                //if (increasemaxhp < 0) increasemaxhp = 0;
                //hp = maxhp = maxhp+increasemaxhp;
                sickness = blind = hallucinating = 0;
            } else if (pot->isCursed()) {
                //hp += rn1(4, 29);
                //if (hp > maxhp) hp = maxhp;
                blind = hallucinating = 0;
            } else {
                //hp += rn1(6, 43);
                //int increasemaxhp = hp - maxhp;
                //if (increasemaxhp > 2) increasemaxhp = 2;
                //if (increasemaxhp < 0) increasemaxhp = 0;
                //hp = maxhp = maxhp+increasemaxhp;
                sickness = blind = hallucinating = 0;
            }
            break;

        case PoHallucination:
            log << "TODO: PoHallucination messages\n";
            log << "TODO: PoHallucination - wield Grayswandir\n";
            if (pot->isBlessed())
                hallucinating += rn1(300, 200);
            else if (pot->isCursed())
                hallucinating += rn1(900, 200);
            else
                hallucinating += rn1(600, 200);
            break;

        case PoHealing:
            log << "TODO: Full effects of healing\n";
            players.seenMsg("NAME looks better.",
                "NAME's face turns into a dasy for a few moments.",
                "You feel better.", id);
            if (pot->isBlessed()) {
                //hp += rn1(8, 25);
                //int increasemaxhp = hp - maxhp;
                //if (increasemaxhp > 1) increasemaxhp = 1;
                //if (increasemaxhp < 0) increasemaxhp = 0;
                //hp = maxhp = maxhp+increasemaxhp;
                sickness = blind = 0;
            } else if (pot->isCursed()) {
                //hp += rn1(4, 13);
                //if (hp > maxhp) hp = maxhp;
            } else {
                //hp += rn1(6, 19);
                //int increasemaxhp = hp - maxhp;
                //if (increasemaxhp > 1) increasemaxhp = 1;
                //if (increasemaxhp < 0) increasemaxhp = 0;
                //hp = maxhp = maxhp+increasemaxhp;
                blind = 0;
            }
            break;

        case PoRestoreAbility:
            log << "TODO: quaffing restore ability\n";
            break;

        case PoSleeping:
            players.seenMsg("NAME falls asleep.", "NAME's eyes disappear!",
                "You suddenly fall asleep!", id);
            log << "TODO: sleep resistance\n";
            if (pot->isBlessed())
                sleeping += rn1(13, 10);
            else if (pot->isCursed())
                sleeping += rn1(37, 10);
            else
                sleeping += rn1(25, 10);
            break;

        case PoBlindness:
            log << "TODO: PoBlindness messages\n";
            if (pot->isBlessed())
                blind += rn1(125, 200);
            else if (pot->isCursed())
                blind += rn1(375, 200);
            else
                blind += rn1(250, 200);
            break;

        case PoGainEnergy:
            log << "TODO: quaffing GainEnergy\n";
            break;

        case PoInvisibility:
            if (invisible) {
                if (hallucinating)
                    pmsg = "You have a peculiar feeling for a moment, then it passes.";
                else
                    pmsg = "You have a normal feeling for a moment, then it passes.";
                players.seenMsg("Something shimmers nearby.",
                    "Wow, a mirage...", pmsg, id);
            } else {
                if (see_invisible) {
                    if (hallucinating)
                        pmsg = "Far out, man!  You can see right through yourself.";
                    else
                        pmsg = "Gee!  All of a sudden, you can see right through yourself.";
                } else {
                    if (hallucinating)
                        pmsg = "Far out, man!  You can't see yourself.";
                    else
                        pmsg = "Gee!  All of a sudden, you can't see yourself.";
                }
                players.seenMsg("NAME suddenly fades!",
                    "NAME vanishes in a puff of smoke!", pmsg, id);
            }
            if (invisible > -1) {
                if (pot->isBlessed())
                    invisible = -1;
                else if (pot->isCursed()) {
                    invisible += rn1(31, 12);
                    aggrivate += rn1(2, 5);
                } else
                    invisible += rn1(31, 12);
            }
            if (pot->isCursed())
                players.seenMsg("You feel aggrivated at NAME.",
                    "You feel annoyed by NAME.",
                    "For some reason, you feel your presence is known.", id);
            players.sendFlags(id);
            break;

        case PoMonsterDetection:
        case PoObjectDetection:
        case PoEnlightenment:
            log << "TODO: quaffing a potion\n";
            break;

        case PoFullHealing:
            log << "TODO: Full effects of full healing\n";
            players.seenMsg("NAME looks healed.",
                "NAME gives you the thumbs up and dances the funky chicken!",
                "You feel completely healed.", id);
            if (pot->isBlessed()) {
                //hp += 400;
                //int increasemaxhp = hp - maxhp;
                //if (increasemaxhp > 8) increasemaxhp = 8;
                //if (increasemaxhp < 0) increasemaxhp = 0;
                //hp = maxhp = maxhp+increasemaxhp;
                sickness = blind = hallucinating = 0;
            } else if (pot->isCursed()) {
                //hp += 400;
                //if (hp > maxhp) hp = maxhp;
                blind = hallucinating = 0;
            } else {
                //hp += 400;
                //int increasemaxhp = hp - maxhp;
                //if (increasemaxhp > 4) increasemaxhp = 4;
                //if (increasemaxhp < 0) increasemaxhp = 0;
                //hp = maxhp = maxhp+increasemaxhp;
                sickness = blind = hallucinating = 0;
            }
            break;

        case PoLevitation:
            log << "TODO: full effects of levitation\n";
            log << "TODO: messages for leviation\n";
            if (pot->isBlessed())
                levitating += rn1(250, 50);
            else if (pot->isCursed())
                levitating += rn1(10, 140);
            else
                levitating += rn1(10, 140);
            break;

        case PoSpeed:
            log << "TODO: temporary effects of speed\n";
            speed = VERY_FAST;
            break;

        case PoPolymorph:
        case PoAcid:
        case PoOil:
        case PoGainAbility:
        case PoGainLevel:
        case PoParalysis:
        case PoWater:
            log << "TODO: quaffing a potion\n";
            break;

        default:
            log << "TODO: quaffing unknown potion\n";
    }

    /* Update the objects details */
    pot->decNumber();
    if (pot->getNumber() < 1) {
        if (pot == pri_weapon) pri_weapon = NULL;
        else if (pot == sec_weapon) sec_weapon = NULL;
    }
    *sock << p_object;
    pack.sendObject(sock, tempo, id);
    *sock << p_end;
    pack.validObjects();

    return 0;
}
