/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

/* Updated to Nethack 3.3.0 */

#define IRON		1
#define GEMSTONE	2
#define WOOD		3
#define MINERAL		4
#define COPPER		5
#define SILVER		6
#define GOLD		7
#define BONE		8

struct S_Ring {
    char *name;
    char *desc;
    int cost;
    int magic;
    int spec;
    int mohs;
    int metal;
    char color;
} SRings[] = {

#define RING(name,power,stone,cost,mgc,spec,mohs,metal,color) \
	{name, stone, cost, mgc, spec, mohs, metal, color}

RING("adornment", ADORNED, "wooden",        100, 1, 1, 2, WOOD, HI_WOOD),
RING("gain strength", 0, "granite",         150, 1, 1, 7, MINERAL, HI_MINERAL),
RING("gain constitution", 0, "opal",        150, 1, 1, 7, MINERAL,  HI_MINERAL),
RING("increase accuracy", 0, "clay",        150, 1, 1, 4, MINERAL, CLR_RED),
RING("increase damage", 0, "coral",         150, 1, 1, 4, MINERAL, CLR_ORANGE),
RING("protection", PROTECTION, "black onyx",100, 1, 1, 7, MINERAL, CLR_BLACK),
RING("regeneration", REGENERATION, "moonstone",
					    200, 1, 0, 6, MINERAL, HI_MINERAL),
RING("searching", SEARCHING, "tiger eye",   200, 1, 0, 6, GEMSTONE, CLR_BROWN),
RING("stealth", STEALTH, "jade",            100, 1, 0, 6, GEMSTONE, CLR_GREEN),
RING("sustain ability", FIXED_ABIL, "bronze",
					    100, 1, 0, 4, COPPER, HI_COPPER),
RING("levitation", LEVITATION, "agate",     200, 1, 0, 7, GEMSTONE, CLR_RED),
RING("hunger", HUNGER, "topaz",             100, 1, 0, 8, GEMSTONE, CLR_CYAN),
RING("aggravate monster", AGGRAVATE_MONSTER, "sapphire",
					    150, 1, 0, 9, GEMSTONE, CLR_BLUE),
RING("conflict", CONFLICT, "ruby",          300, 1, 0, 9, GEMSTONE, CLR_RED),
RING("warning", WARNING, "diamond",         100, 1, 0,10, GEMSTONE, CLR_WHITE),
RING("poison resistance", POISON_RES, "pearl",
					    150, 1, 0, 4, IRON, CLR_WHITE),
RING("fire resistance", FIRE_RES, "iron",   200, 1, 0, 5, IRON, HI_METAL),
RING("cold resistance", COLD_RES, "brass",  150, 1, 0, 4, COPPER, HI_COPPER),
RING("shock resistance", SHOCK_RES, "copper",
					    150, 1, 0, 3, COPPER, HI_COPPER),
RING("free action",     FREE_ACTION, "twisted",
					    200, 1, 0, 6, IRON, HI_METAL),
RING("slow digestion",  SLOW_DIGESTION, "steel",
					    200, 1, 0, 8, IRON, HI_METAL),
RING("teleportation", TELEPORT, "silver",   200, 1, 0, 3, SILVER, HI_SILVER),
RING("teleport control", TELEPORT_CONTROL, "gold",
					    300, 1, 0, 3, GOLD, HI_GOLD),
RING("polymorph", POLYMORPH, "ivory",       300, 1, 0, 4, BONE, CLR_WHITE),
RING("polymorph control", POLYMORPH_CONTROL, "emerald",
					    300, 1, 0, 8, GEMSTONE, CLR_BRIGHT_GREEN),
RING("invisibility", INVIS, "wire",         150, 1, 0, 5, IRON, HI_METAL),
RING("see invisible", SEE_INVIS, "engagement",
					    150, 1, 0, 5, IRON, HI_METAL),
RING("protection from shape changers", PROT_FROM_SHAPE_CHANGERS, "shiny",
					    100, 1, 0, 5, IRON, CLR_BRIGHT_CYAN),
#undef RING
};

#define RINGDESCS	28

static bool ringKnown[RINGDESCS][MAXPLAYERS];
static char SRingRandDesc[RINGDESCS][MAXPLAYERS];

void initRings() {
    for (int x = 0; x < MAXPLAYERS; x++)
        quitRings(x);
}

void loadRings(ID user) {
    /* TODO: load players knowledge of rings */
}

void saveRings(ID user) {
    /* TODO: save players knowledge of rings */
    quitRings(user);  /* Randomise ring descriptions */
}

/* Randomise the ring descriptions for the next player */
void quitRings(ID user) {
    for (int y = 0; y < RINGDESCS; y++) {
        ringKnown[y][user] = false;
        SRingRandDesc[y][user] = y;
    }
    for (int y = 0; y < RINGDESCS; y++) {
        int r = rn2(RINGDESCS);
        char temp = SRingRandDesc[y][user];
        SRingRandDesc[y][user] = SRingRandDesc[r][user];
        SRingRandDesc[r][user] = temp;
    }
}

CRing::CRing() : CObject(Ring, '=', 3) {
    polymorph();
}

CRing::CRing(ERingType t) : CObject(Ring, '=', 3) {
    ringType = t;
}

void CRing::polymorph() {
    ringType = (ERingType)rn2(RINGDESCS);

    setCost(SRings[ringType].cost);

    // Calculate the charge on a ring if it has one

    // Curse / bless the ring
    int r = rn2(100);
    if (ringType == RingHunger || ringType == RingAggravateMonster ||
        ringType == RingTeleportation || ringType == RingPolymorph) {
        if (r < 90) curseIt();
        else neutralIt();
    } else if (ringType == RingAdornment || ringType == RingProtection ||
        ringType == RingGainStrength || ringType == RingIncreaseDamage) {
    } else {
        if (r < 10) curseIt();
        else neutralIt();
    }
    setName("");
}

bool CRing::canEat() {
    return false;
}

string CRing::getDescription(ID user) {
    string s;
    if (getNumber() > 1)
        s = string(number);
    else
        s = "a ";
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
    }
    if (isKnown(user)) {
        s += "ring";
        if (getNumber() > 1) s += "s";
        s += " of ";
        s += SRings[ringType].name;
    } else {
        s += SRings[SRingRandDesc[ringType][user]].desc;
        s += " ring";
        if (getNumber() > 1) s += "s";
    }
    return s;
}

int CRing::getColor(ID user) {
    return SRings[SRingRandDesc[ringType][user]].color;
}

bool CRing::isKnown(ID user) {
   return ringKnown[ringType][user];
}

void CRing::identify(ID user) {
    ringKnown[ringType][user] = true;
    BCidentify(user);
}

/* Keep rings isolated from each other */
bool CRing::operator== (CRing *r) {
    return false;
}
