/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

/* Scrolls are now 3.3.0 compliant */

struct S_Scrolls {
    char *name;
    char *desc;
    int magic;
    int prob;
    int cost;
} SScrolls[] = {

#define SCROLL(name,text,mgc,prob,cost) \
	{name,text,mgc,prob,cost}

SCROLL("enchant armor",         "ZELGO MER",            1,  63,  80),
SCROLL("destroy armor",         "JUYED AWK YACC",       1,  45, 100),
SCROLL("confuse monster",       "NR 9",                 1,  53, 100),
SCROLL("scare monster",         "XIXAXA XOXAXA XUXAXA", 1,  35, 100),
SCROLL("remove curse",          "PRATYAVAYAH",          1,  65,  80),
SCROLL("enchant weapon",        "DAIYEN FOOELS",        1,  80,  60),
SCROLL("create monster",        "LEP GEX VEN ZEA",      1,  45, 200),
SCROLL("taming",                "PRIRUTSENIE",          1,  15, 200),
SCROLL("genocide",              "ELBIB YLOH",           1,  15, 300),
SCROLL("light",                 "VERR YED HORRE",       1,  90,  50),
SCROLL("teleportation",         "VENZAR BORGAVVE",      1,  55, 100),
SCROLL("gold detection",        "THARR",                1,  33, 100),
SCROLL("food detection",        "YUM YUM",              1,  25, 100),
SCROLL("identify",              "KERNOD WEL",           1, 180,  20),
SCROLL("magic mapping",         "ELAM EBOW",            1,  45, 100),
SCROLL("amnesia",               "DUAM XNAHT",           1,  35, 200),
SCROLL("fire",                  "ANDOVA BEGARIN",       1,  30, 100),
SCROLL("earth",                 "KIRJE",                1,  18, 200),
SCROLL("punishment",            "VE FORBRYDERNE",       1,  15, 300),
SCROLL("charging",              "HACKEM MUCHE",         1,  15, 300),
SCROLL("stinking cloud",	"VELOX NEB",            1,  15, 300),
SCROLL((char *)0,		"FOOBIE BLETCH",        1,   0, 100),
SCROLL((char *)0,		"TEMOV",                1,   0, 100),
SCROLL((char *)0,		"GARVEN DEH",           1,   0, 100),
SCROLL((char *)0,		"READ ME",              1,   0, 100),
/* these must come last because they have special descriptions */
#ifdef MAIL
SCROLL("mail",                  "stamped",          0,   0,   0),
#endif
SCROLL("blank paper",           "unlabeled",        0,  28,  60)
#undef SCROLL
};

#ifdef MAIL
#define SCROLLDESCS	26
#else
#define SCROLLDESCS	27
#endif

static bool scrollKnown[SCROLLDESCS][MAXPLAYERS];
static char SScrollRandDesc[SCROLLDESCS][MAXPLAYERS];

void initScrolls() {
    for (int x = 0; x < MAXPLAYERS; x++)
        quitScrolls(x);
}

void loadScrolls(ID user) {
    /* TODO: load users knowledge of scrolls */
}

void saveScrolls(ID user) {
    /* TODO: save users knowledge of potions */
    quitScrolls(user); /* Just randomize the descriptions for next player */
}

void quitScrolls(ID user) {
    for (int y = 0; y < SCROLLDESCS; y++) {
        scrollKnown[y][user] = false;
        SScrollRandDesc[y][user] = y;
    }
    for (int y = 0; y < SCROLLDESCS; y++) {
        int r = rn2(SCROLLDESCS);
        char temp = SScrollRandDesc[y][user];
        SScrollRandDesc[y][user] = SScrollRandDesc[r][user];
        SScrollRandDesc[r][user] = temp;
    }
}

CScroll::CScroll() : CObject(Scroll, '?', 5) {
    polymorph();
}

CScroll::CScroll(EScrollType t) : CObject(Scroll, '?', 5) {
    scrollType = t;
    setCost(SScrolls[scrollType].cost);
    int r = rn2(8);
    if (r < 1) curseIt();
    if (r > 6) blessIt();
}

CScroll::CScroll(EScrollType t, int n) : CObject(Scroll, '?', 5) {
    scrollType = t;
    setCost(SScrolls[scrollType].cost);
    int r = rn2(8);
    if (r < 1) curseIt();
    if (r > 6) blessIt();
    number = n;
}

void CScroll::polymorph() {
    scrollType = (EScrollType)0;
    for(int tprob = rnd(1000);
        (tprob -= SScrolls[scrollType].prob) > 0;
        scrollType = (EScrollType)(scrollType+1));
    setCost(SScrolls[scrollType].cost);
    int r = rn2(8);
    if (r < 1) curseIt();
    if (r > 6) blessIt();
    setName("");
}

string CScroll::getDescription(ID user) {
    string s;
    if (getNumber() > 1) {
        s = string(number);
        s += ' ';
    } else {
        if (isBlessed() || isCursed() || !isKnown(user))
            s = "a ";
        else s = "an ";
    }
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
    }
    if (isKnown(user)) {
        s += "scroll";
        if (getNumber() > 1) s += "s";
        s += " of ";
        s += SScrolls[scrollType].name;
    } else {
        s += "scroll";
        if (getNumber() > 1) s += "s";
        s += " labeled ";
        s += SScrolls[SScrollRandDesc[scrollType][user]].desc;
    }
    return s;
}

int CScroll::getColor(ID user) {
    return HI_PAPER;
}

bool CScroll::isKnown(ID user) {
    return scrollKnown[scrollType][user];
}

void CScroll::identify(ID user) {
    scrollKnown[scrollType][user] = true;
    BCidentify(user);
}

bool CScroll::operator== (CScroll *s) {
    if (s->scrollType != scrollType)
        return false;
    if (s->isBlessed() != isBlessed())
        return false;
    if (s->isCursed() != isCursed())
        return false;
    return true;
}
