/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include <unistd.h>

#include "tcpip.h"
#include "log.h"
#include "version.h"
#include "object.h"
#include "dungeon.h"
#include "plist.h"
#include "hack.h"
#include "vision.h"

int new_connection(int, int, char *);
int incomming_data(int);
int lost_connection(int);
extern int extendedCommand();

CConnection connection(new_connection,incomming_data,lost_connection);

int connCount = 0;

int main(int argc, char**argv) {
    string temp;
    CLevel * tlvl;
    pid_t myid;

    vision.on();

    log << '\n' << LICENSE << "\n"
         << "\thttp://mehta.anu.edu.au/~bendchon/interhack\n" << '\n';

	read_command_line(argc,argv,true);

    vision.msgline("Starting random number generator...");
    seedrand();
    vision.msgline("Initializing Objects...");
    initObjects();
    vision.msgline("Initializing Monsters...");
    initMonsters();
    vision.msgline("Initializing Dungeon...");
    initDungeon();
    vision.msgline("Initializing Players...");
    initPlayers();
    vision.msgline("Initializing Network...");
    if (connection.c_open(server_port) < 0) {
        vision.msgline("Network error.");
        return 0;
    }
    vision.statusline1("Connections: 0");
    vision.msgline("Initialization Complete! Waiting for Hackers!");
    vision.msgline("For help press ?");
    int k;
    while (1) {
        vision.touchall();
        k = vision.inputch("Interhack: ");
        switch (k) {

            case 13: /* ^M */
                vision.lastMsgline();
                break;

            case 16: /* ^P */
                vision.lastTopline();
                break;

            case '#': /* Extended commands */
                if (extendedCommand() < 0) {
                    k = vision.inputch("Are you sure you want to exit? [y/n] ");
                    if (k == 'y' || k == 'Y') goto exitsw;
                    continue;
                }
                break;

            case '&': /* Enter demon mode... :) */
                k = vision.inputch("Really enter demon mode? [yn] (n) ");
                if (k == 'y' || k == 'Y') {
                    log << "Server entering demon mode." << '\n';
                    myid = fork();
                    if (myid < 0) {
                        log << "ERROR: could not enter demon mode." << '\n';
                        vision.msgline("ERROR: could not enter demon mode.");
                        continue;
                    }
                    if (myid != 0) /* If I am the parent */
                        goto exitsw;
                    connection.setDemon();
                    while (1) {    /* Child just listens for conns and data */
                        if (connection.c_listen() < 0) {
                            log << "ERROR: c_listen failed while in demon mode." << '\n';
                            goto exitsw;
                        }
                    }
                }
                break;

            case 'i': /* Enter normal player mode */
                vision.msgline("Server entering interactive mode.");
                hack.client_main(false);
                vision.clean();
                vision.flush();
                vision.msgline("Server returning from interactive mode.");
                continue;
                break;

            case 'k': /* Kill a player */
                vision.touchall();
                temp = vision.inputstr("Kill which player? ");
                if (temp.getNumber() < 0 || temp.getNumber() >= MAXPLAYERS ||
                    plist[temp.getNumber()] == NULL) {
                    vision.msgline("No such player to kill");
                    break; }
                connection.c_close_conn(temp.getNumber());
                break;

            case 'l': /* Display a level */
                vision.touchall();
                temp = vision.inputtopstr("Display which level? ");
                tlvl = dungeon->get(temp);
                if (tlvl == NULL) {
                    vision.msgline("Warning: could not find level to display.");
                    break;
                }
                tlvl->display();
                break;

            case 'S': /* Save the server status */
                k = vision.inputch("Really save? [yn] (n) ");
                if (k == 'y' || k == 'Y') {
                    log << "TODO: save the server" << '\n';
                }
                break;

            case 'u': /* Display User list */
                players.user_list();
                break;

            case 'v': /* Version number */
                vision.msgline(VERSION_SERVER);
                break;

            case 'w': /* Enter wizard mode */
                vision.msgline("Server entering wizard mode.");
                hack.client_main(true);
                vision.clean();
                vision.flush();
                vision.msgline("Server returning from wizard mode.");
                continue;
                break;

            case '?': /* Help */
                vision.serverhelp();
                break;

            default:
                vision.msgline("Enter ? for help");
        }
    }
exitsw:


    vision.msgline("Shutting down...");
    connection.c_close();
    vision.msgline("Thank you for using the Interhack server.");
    quitDungeon();
    return 0;
}

int new_connection(int id, int sockfd, char *from) {
    if (plist[id] != NULL)
        log << "ERROR: new_connection() Player already connected\n";
    else {
        string temp;
        connCount++;
        temp = "Connections: "; temp += connCount;
        vision.statusline1(temp);
        plist[id] = new CPlayer(id, sockfd, from);
        players.arrangeTimers();
    }
    return 0;
}

int incomming_data(int id) {
    if (plist[id] == NULL)
        log << "ERROR: incomming_data() Player not connected\n";
    else {
        int r = plist[id]->incomming_data();
        return r;
    }
    return 0;
}

int lost_connection(int id) {
    if (plist[id] == NULL)
        log << "ERROR: lost_connection() Player not connected\n";
    else {
        string temp;
        connCount--;
        temp = "Connections: "; temp += connCount;
        vision.statusline1(temp);
        plist[id]->lost_connection();
        delete plist[id];
        plist[id] = NULL;
        players.arrangeTimers();
    }
    return 0;
}

/* Turn on wizard mode for socket fd, rand r */
void wizard_mode(int fd, int r) {
    wizardfd = fd;
    wizardr = r;
}

/* Use wizard mode to wish for an object */
int wizard_wish(string tstr) {
    return 0;
}
