/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

/* Objects are up to date as of nethack 3.3.0. This was where the first
   new system was tested. Rather than dozens of different arrays, A single 
   array is constructed, and the definitions can be taken directly from
   the nethack objects.c file. This saves heaps of effort to keep up to
   date with nethack objects. Unlike 3.3.0, we have included the freeze
   and flame sphere spellbooks.

   Unlike Nethack, we can not use the unknown spellbook descriptions as
   they are (ie just randomly change the pointers around), as we have
   multiple players. Hence when constructing the array we ignore the
   spellbook description and colour. This is delt with by a 2D array so
   that each player has different descriptions (the array points to the 1D
   array of objects for ease of compatibility). */

#define RAY		1
#define IMMEDIATE	2
#define NODIR		3

struct S_Spells {
    char *name;
    char *desc;
    char prob;
    char delay;
    char level;
    char magic;
    char dir;
    int color;
    int cost;
} SSpells[] = {

#define SPELL(nam,des,sub,p,del,lvl,mgc,d,col) \
	{nam, des, p, del, lvl, mgc, d, col, lvl*100}

SPELL("dig",             "parchment",   P_MATTER_SPELL, 20,  6, 5, 1, RAY,       HI_PAPER),
SPELL("magic missile",   "vellum",      P_ATTACK_SPELL, 45,  2, 2, 1, RAY,       HI_PAPER),
SPELL("fireball",        "ragged",      P_ATTACK_SPELL, 20,  4, 4, 1, RAY,       HI_PAPER),
SPELL("cone of cold",    "dog eared",   P_ATTACK_SPELL, 10,  7, 4, 1, RAY,       HI_PAPER),
SPELL("sleep",           "mottled",     P_ENCHANTMENT_SPELL, 50,  1, 1, 1, RAY,       HI_PAPER),
SPELL("finger of death", "stained",     P_ATTACK_SPELL,  5, 10, 7, 1, RAY,       HI_PAPER),
SPELL("light",           "cloth",       P_DIVINATION_SPELL, 45,  1, 1, 1, NODIR,     HI_CLOTH),
SPELL("detect monsters", "leather",     P_DIVINATION_SPELL, 43,  1, 1, 1, NODIR,     HI_LEATHER),
SPELL("healing",         "white",       P_HEALING_SPELL, 40,  2, 1, 1, IMMEDIATE, CLR_WHITE),
SPELL("knock",           "pink",        P_MATTER_SPELL, 35,  1, 1, 1, IMMEDIATE, CLR_BRIGHT_MAGENTA),

SPELL("force bolt",      "red",         P_ATTACK_SPELL, 35,  2, 1, 1, IMMEDIATE, CLR_RED),
SPELL("confuse monster", "orange",      P_ENCHANTMENT_SPELL, 30,  2, 2, 1, IMMEDIATE, CLR_ORANGE),
SPELL("cure blindness",  "yellow",      P_HEALING_SPELL, 25,  2, 2, 1, IMMEDIATE, CLR_YELLOW),
SPELL("drain life",      "velvet",      P_ATTACK_SPELL, 10,  2, 2, 1, IMMEDIATE, CLR_MAGENTA),
SPELL("slow monster",    "light green", P_ENCHANTMENT_SPELL, 30,  2, 2, 1, IMMEDIATE, CLR_BRIGHT_GREEN),
SPELL("wizard lock",     "dark green",  P_MATTER_SPELL, 30,  3, 2, 1, IMMEDIATE, CLR_GREEN),
SPELL("create monster",  "turquoise",   P_CLERIC_SPELL, 35,  3, 2, 1, NODIR,     CLR_BRIGHT_CYAN),
SPELL("detect food",     "cyan",        P_DIVINATION_SPELL, 30,  3, 2, 1, NODIR,     CLR_CYAN),
SPELL("cause fear",      "light blue",  P_ENCHANTMENT_SPELL, 25,  3, 3, 1, NODIR,     CLR_BRIGHT_BLUE),
SPELL("clairvoyance",    "dark blue",   P_DIVINATION_SPELL, 15,  3, 3, 1, NODIR,     CLR_BLUE),

SPELL("cure sickness",   "indigo",      P_HEALING_SPELL, 32,  3, 3, 1, NODIR,     CLR_BLUE),
SPELL("charm monster",   "magenta",     P_ENCHANTMENT_SPELL, 20,  3, 3, 1, IMMEDIATE, CLR_MAGENTA),
SPELL("haste self",      "purple",      P_ESCAPE_SPELL, 33,  4, 3, 1, NODIR,     CLR_MAGENTA),
SPELL("detect unseen",   "violet",      P_DIVINATION_SPELL, 20,  4, 3, 1, NODIR,     CLR_MAGENTA),
SPELL("levitation",      "tan",         P_ESCAPE_SPELL, 20,  4, 4, 1, NODIR,     CLR_BROWN),
SPELL("extra healing",   "plaid",       P_HEALING_SPELL, 27,  5, 3, 1, IMMEDIATE, CLR_GREEN),
SPELL("restore ability", "light brown", P_HEALING_SPELL, 25,  5, 4, 1, NODIR,     CLR_BROWN),
SPELL("invisibility",    "dark brown",  P_ESCAPE_SPELL, 25,  5, 4, 1, NODIR,     CLR_BROWN),
SPELL("detect treasure", "gray",        P_DIVINATION_SPELL, 20,  5, 4, 1, NODIR,     CLR_GRAY),
SPELL("remove curse",    "wrinkled",    P_CLERIC_SPELL, 25,  5, 3, 1, NODIR,     HI_PAPER),

SPELL("magic mapping",   "dusty",       P_DIVINATION_SPELL, 18,  7, 5, 1, NODIR,     HI_PAPER),
SPELL("identify",        "bronze",      P_DIVINATION_SPELL, 20,  6, 3, 1, NODIR,     HI_COPPER),
SPELL("turn undead",     "copper",      P_CLERIC_SPELL, 16,  8, 6, 1, IMMEDIATE, HI_COPPER),
SPELL("polymorph",       "silver",      P_MATTER_SPELL, 10,  8, 6, 1, IMMEDIATE, HI_SILVER),
SPELL("teleport away",   "gold",        P_ESCAPE_SPELL, 15,  6, 6, 1, IMMEDIATE, HI_GOLD),
SPELL("create familiar", "glittering",  P_CLERIC_SPELL, 10,  7, 6, 1, NODIR,     CLR_WHITE),
SPELL("cancellation",    "shining",     P_MATTER_SPELL, 15,  8, 7, 1, IMMEDIATE, CLR_WHITE),
SPELL("protection",	 "dull",        P_CLERIC_SPELL, 18,  3, 1, 1, NODIR,     HI_PAPER),
SPELL("jumping",	 "thin",        P_ESCAPE_SPELL, 20,  3, 1, 1, IMMEDIATE, HI_PAPER),
SPELL("stone to flesh",	 "thick",       P_HEALING_SPELL, 15,  1, 3, 1, IMMEDIATE, HI_PAPER),

SPELL("flame sphere",    "canvas",      P_MATTER_SPELL, 20,  2, 1, 1, NODIR, CLR_BROWN),
SPELL("freeze sphere",   "hardcover",   P_MATTER_SPELL, 20,  2, 1, 1, NODIR, CLR_BROWN),

/* These last two are special cases */
SPELL("blank paper",     "plain",       P_NONE, 18,  0, 0, 0, 0,         HI_PAPER),
SPELL("Book of the Dead","papyrus",	P_NONE, 0,   0, 7, 0, 0,	HI_PAPER),
#undef SPELL
};

#define SPELLDESCS	44

static bool spellKnown[SPELLDESCS][MAXPLAYERS];
static char SSpellRandDesc[SPELLDESCS][MAXPLAYERS];

void initSpells() {
    /* Init all spell descriptions to something */
    for (int x = 0; x < MAXPLAYERS; x++)
        quitSpells(x);
}

void loadSpells(ID user) {
    /* TODO: load users knowledge of spells */
}

void saveSpells(ID user) {
    /* TODO: save users knowledge of spells */
    quitSpells(user); /* Randomize the descs for next player */
}

void quitSpells(ID user) {
    for (int y = 0; y < SPELLDESCS-2; y++) {
        spellKnown[y][user] = false;
        SSpellRandDesc[y][user] = y;
    }
    for (int y = 0; y < SPELLDESCS-2; y++) {
        int r = rn2(SPELLDESCS-2);
        char temp = SSpellRandDesc[y][user];
        SSpellRandDesc[y][user] = SSpellRandDesc[r][user];
        SSpellRandDesc[r][user] = temp;
    }
}

CSpell::CSpell() : CObject(Spellbook, '+', 50) {
    polymorph();
}

CSpell::CSpell(ESpellType t) : CObject(Spellbook, '+', 50) {
    spellType = t;
    if (spellType == SpellBookOfTheDead) setWeight(20);
    int r = rn2(34);
    if (r < 1) curseIt();
    if (r > 32) blessIt();
}

void CSpell::polymorph() {
    spellType = (ESpellType)0;
    for(int tprob = rnd(1000);
        (tprob -= SSpells[spellType].prob) > 0;
        spellType = (ESpellType)(spellType+1));
    int r = rn2(34);
    if (r < 1) curseIt();
    if (r > 32) blessIt();
    setName("");
}

string CSpell::getDescription(ID user) {
    string s;
    if (getNumber() > 1) {
        s = string(number);
        s += ' ';
    } else {
        if (isBlessed() || isCursed() || !isKnown(user))
            s = "a ";
        else s = "an ";
    }
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
    }
    if (isKnown(user)) {
        s += "spellbook";
        if (getNumber() > 1) s += "s";
        s += " of ";
        s += SSpells[spellType].name;
    } else {
        s += SSpells[SSpellRandDesc[spellType][user]].desc;
        s += " spellbook";
        if (getNumber() > 1) s += "s";
    }
    return s;
}

int CSpell::getColor(ID user) {
    return SSpells[SSpellRandDesc[spellType][user]].color;
}

bool CSpell::isKnown(ID user) {
    return spellKnown[spellType][user];
}

void CSpell::identify(ID user) {
    spellKnown[spellType][user] = true;
    BCidentify(user);
}

bool CSpell::operator== (CSpell *s) {
    if (s->spellType != spellType)
        return false;
    if (s->isBlessed() != isBlessed())
        return false;
    if (s->isCursed() != isCursed())
        return false;
    return true;
}
