/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifndef _system_h_
#define _system_h_

#include <stdlib.h>
#include <sys/time.h>
#include "mss12/mss.h"

typedef unsigned long ID;

typedef unsigned char uchar;
typedef unsigned long ulong;
typedef signed char schar;
typedef schar xchar;

#define Bitfield(name,bits)     unsigned int name : bits

#define MAXPLAYERS      10
#ifdef SERVERADDR
    #define SERVER_NAME	SERVERADDR
#else
    #define SERVER_NAME	"localhost"
#endif
#ifdef PORT
    #define SERVER_PORT	PORT
#else
    #define SERVER_PORT	3334
#endif
#define PLAYERDIR	"./player/"
#define MAXBACKLOG	5
#define PORTMIN		1000
#define PORTMAX		9999
#define DEADTIME	{300, 0}
#define BUFFER_SIZE     16384

/* This is for the serial port stuff */
#define BAUDRATE	38400
#define SERIAL0		"/dev/ttyS0"
#define SERIAL1		"/dev/ttyS1"

//[991220 OC
extern char* server_name;
extern int   server_port;

#define rn1(x, y)	(rn2(x)+y)
#define rn2(x)		(rand() % (x))
#define rnd(x)		((rand() % (x))+1)
#define seedrand()	(srand(time(NULL)))

enum EObjectType {
    NullObject, Amulet, Armor, Artifact, Food, Gem, Potion, Ring,
    Scroll, Spellbook, Tool, Wand, Weapon, Gold, LevelObject};

#define BODY_AMULET		1
#define BODY_LEFT_RING		2
#define BODY_RIGHT_RING		3
#define BODY_BOOTS		4
#define BODY_GLOVES		5
#define BODY_HELMET		6
#define BODY_SHIRT		7
#define BODY_ARMOR		8
#define BODY_CLOAK		9
#define BODY_SHIELD		10
#define BODY_LEFT_WEPON		11
#define BODY_RIGHT_WEAPON	12

typedef struct timeval Timeval;
            
/* Some sureal time stuff */
extern Timeval LEADWAY;
#define THELEADWAY	{0, 700000}
#define MAXFORCED       10

/* Create a new set of timer functions. Some systems have some odd
   problems with the standard timer defines, so I have copied / added
   the functions below */
#define ih_timerisset(tvp)\
	((tvp).tv_sec || (tvp).tv_usec)
#define ih_timercmp(tvp, uvp, cmp)\
	((tvp).tv_sec cmp (uvp).tv_sec ||\
	(tvp).tv_sec == (uvp).tv_sec &&\
	(tvp).tv_usec cmp (uvp).tv_usec)
#define ih_timerclear(tvp)\
	((tvp)->tv_sec = (tvp)->tv_usec = 0)
#define ih_timershow(tvp)\
	(int)((tvp).tv_sec) << '-' << (int)((tvp).tv_usec)
#define ih_timeradd(tvp, uvp)\
	{(tvp)->tv_sec += (uvp).tv_sec;\
	(tvp)->tv_usec += (uvp).tv_usec;\
	if ((tvp)->tv_usec > 1000000) {\
	    (tvp)->tv_usec -= 1000000;\
	    ((tvp)->tv_sec)++; }; }
#define ih_timersub(tvp, uvp)\
	{(tvp)->tv_sec -= (uvp).tv_sec;\
	(tvp)->tv_usec -= (uvp).tv_usec;\
	if ((tvp)->tv_usec < 0) {\
	    (tvp)->tv_usec += 1000000;\
	    ((tvp)->tv_sec)--; }; }


enum EFloorType {   
    // Nothing on the floor, room floor, corridor
    EFTNoDig, EFTEmpty, EFTFloor, EFTCorridor,
    // Horizontal, Vertical, and four corner walls
    EFTHWall, EFTVWall, EFTTLWall, EFTTRWall, EFTBLWall, EFTBRWall,
    // + wall, and four T join walls
    EFTXWall, EFTSTWall, EFTNTWall, EFTETWall, EFTWTWall,
    // Water and fire
    EFTWater, EFTFire,   
    // Stairs
    EFTUpStair, EFTDownStair, EFTPortal};

typedef enum E_EntryType {
    EEntryPortal, EEntryDown, EEntryUp
} EEntryType;

#define HEIGHT		20
#define WIDTH		80

/* Maximum number of messages kept after being displayed */
#define MAXMSGS		20

#define KEYWORDFILE	"./dat/data.base"

#define SIZE(x) (int)(sizeof(x) / sizeof(x[0]))

#endif
