/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

struct S_Tools {
    char *name;
    char *desc;
    int kn;
    int magic;
    int charge;
    int prob;
    int weight;
    int cost;
    int material;
    int color;
    int sdamage;
    int ldamage;
    int hitbonus;
} STools[] = {

#define METAL		1
#define BONE		2
#define WOOD		3
#define COPPER		4
#define IRON		5
#define LEATHER		6
#define PLASTIC		7
#define CLOTH		8
#define WAX		9
#define GLASS		10
#define MINERAL		11

#define WHACK		1
#define PIERCE		2

/* tools with weapon characteristics come last */
#define TOOL(name,desc,kn,mrg,mgc,chg,prob,wt,cost,mat,color) \
	{name,desc,kn,mgc,chg,prob,wt,cost,mat,color,0,0,0}
#define CONTAINER(name,desc,kn,mgc,chg,prob,wt,cost,mat,color) \
	{name,desc,kn,mgc,chg,prob,wt,cost,mat,color,0,0,0}
#define WEPTOOL(name,desc,kn,mgc,bi,prob,wt,cost,sdam,ldam,hitbon,sub,mat,clr) \
	{name,desc,kn,mgc,0,prob,wt,cost,mat,clr,sdam,ldam,hitbon}

/* containers */
CONTAINER("large box", (char *)0,       1, 0, 0,  40,350,   8, WOOD, HI_WOOD),
CONTAINER("chest", (char *)0,           1, 0, 0,  35,600,  16, WOOD, HI_WOOD),
CONTAINER("ice box", (char *)0,         1, 0, 0,   5,900,  42, PLASTIC, CLR_WHITE),
CONTAINER("sack", "bag",                0, 0, 0,  35, 15,   2, CLOTH, HI_CLOTH),
CONTAINER("oilskin sack", "bag",        0, 0, 0,   5, 15, 100, CLOTH, HI_CLOTH),
CONTAINER("bag of holding", "bag",      0, 1, 0,  20, 15, 100, CLOTH, HI_CLOTH),
CONTAINER("bag of tricks", "bag",       0, 1, 1,  20, 15, 100, CLOTH, HI_CLOTH),
#undef CONTAINER

/* lock opening tools */
TOOL("skeleton key", "key",     0, 0, 0, 0,  80,  3,  10, IRON, HI_METAL),
TOOL("lock pick", (char *)0,    1, 0, 0, 0,  60,  4,  20, IRON, HI_METAL),
TOOL("credit card", (char *)0,  1, 0, 0, 0,  15,  1,  10, PLASTIC, CLR_WHITE),
/* light sources */
TOOL("tallow candle", "candle", 0, 1, 0, 0,  20,  2,  10, WAX, CLR_WHITE),
TOOL("wax candle", "candle",    0, 1, 0, 0,   5,  2,  20, WAX, CLR_WHITE),
TOOL("brass lantern", (char *)0,1, 0, 0, 0,  30, 30,  12, COPPER, CLR_YELLOW),
TOOL("oil lamp", "lamp",        0, 0, 0, 0,  45, 20,  10, COPPER, CLR_YELLOW),
TOOL("magic lamp", "lamp",      0, 0, 1, 0,  15, 20,  50, COPPER, CLR_YELLOW),
/* other tools */
TOOL("expensive camera", (char *)0,
				1, 0, 0, 1,  15, 12, 200, PLASTIC, CLR_BLACK),
TOOL("mirror", "looking glass", 0, 0, 0, 0,  45, 13,  10, GLASS, HI_SILVER),
TOOL("crystal ball", "glass orb",
				0, 0, 1, 1,  15,150,  60, GLASS, HI_GLASS),
TOOL("lenses", (char *)0,	1, 0, 0, 0,   5,  3,  80, GLASS, HI_GLASS),
TOOL("blindfold", (char *)0,    1, 0, 0, 0,  50,  2,  20, CLOTH, CLR_BLACK),
TOOL("towel", (char *)0,        1, 0, 0, 0,  50,  2,  50, CLOTH, CLR_MAGENTA),
TOOL("saddle", (char *)0,       1, 0, 0, 0,   5,200, 150, LEATHER, HI_LEATHER),
TOOL("leash", (char *)0,        1, 0, 0, 0,  65, 12,  20, LEATHER, HI_LEATHER),
TOOL("stethoscope", (char *)0,  1, 0, 0, 0,  25,  4,  75, IRON, HI_METAL),
TOOL("tinning kit", (char *)0,  1, 0, 0, 1,  15,100,  30, IRON, HI_METAL),
TOOL("tin opener", (char *)0,   1, 0, 0, 0,  35,  4,  30, IRON, HI_METAL),
TOOL("can of grease", (char *)0,1, 0, 0, 1,  15, 15,  20, IRON, HI_METAL),
TOOL("figurine", (char *)0,     1, 0, 1, 0,  25, 50,  80, MINERAL, HI_MINERAL),
TOOL("magic marker", (char *)0, 1, 0, 1, 1,  15,  2,  50, PLASTIC, CLR_RED),
/* traps */
TOOL("land mine",(char *)0,     1, 0, 0, 0,   0,300, 180, IRON, CLR_RED),
TOOL("beartrap", (char *)0,     1, 0, 0, 0,   0,200,  60, IRON, HI_METAL),
/* instruments */
TOOL("tin whistle", "whistle",  0, 0, 0, 0, 100,  3,  10, METAL, HI_METAL),
TOOL("magic whistle", "whistle",0, 0, 1, 0,  30,  3,  10, METAL, HI_METAL),
/* "If tin whistles are made out of tin, what do they make foghorns out of?" */
TOOL("wooden flute", "flute",   0, 0, 0, 0,   4,  5,  12, WOOD, HI_WOOD),
TOOL("magic flute", "flute",    0, 0, 1, 1,   2,  5,  36, WOOD, HI_WOOD),
TOOL("tooled horn", "horn",     0, 0, 0, 0,   5, 18,  15, BONE, CLR_WHITE),
TOOL("frost horn", "horn",      0, 0, 1, 1,   2, 18,  50, BONE, CLR_WHITE),
TOOL("fire horn", "horn",       0, 0, 1, 1,   2, 18,  50, BONE, CLR_WHITE),
TOOL("horn of plenty", "horn",  0, 0, 1, 1,   2, 18,  50, BONE, CLR_WHITE),
TOOL("wooden harp", "harp",     0, 0, 1, 0,   4, 30,  50, WOOD, HI_WOOD),
TOOL("magic harp", "harp",      0, 0, 1, 1,   2, 30,  50, WOOD, HI_WOOD),
TOOL("bell", (char *)0,         1, 0, 0, 0,   2, 30,  50, COPPER, HI_COPPER),
TOOL("bugle", (char *)0,        1, 0, 0, 0,   4, 10,  15, COPPER, HI_COPPER),
TOOL("leather drum", "drum",    0, 0, 0, 0,   4, 25,  25, LEATHER, HI_LEATHER),
TOOL("drum of earthquake", "drum",
				0, 0, 1, 1,   2, 25,  25, LEATHER, HI_LEATHER),
/* tools useful as weapons */
WEPTOOL("pick-axe", (char *)0,
	1, 0, 0, 20, 100,  50,	6,  3, WHACK,  P_PICK_AXE, IRON, HI_METAL),
WEPTOOL("grappling hook", "iron hook",
	0, 0, 0,  5,  30,  50,  2,  6, WHACK,  P_FLAIL, IRON, HI_METAL),
WEPTOOL("unicorn horn", (char *)0,
	1, 1, 1,  0,  20, 100, 12, 12, PIERCE, P_UNICORN_HORN, BONE, CLR_WHITE),

#undef TOOL
#undef WEPTOOL
};

#define TOOLDESCS	36

static bool toolKnown[TOOLDESCS][MAXPLAYERS];

void initTools() {
    for (int x = 0; x < MAXPLAYERS; x++)
        quitTools(x);
}

void loadTools(ID user) {
}

void saveTools(ID user) {
}

void quitTools(ID user) {
}

CTool::CTool() : CObject(Tool, ')', 50) {
    contents = NULL;
    polymorph();
}

CTool::CTool(EToolType t) : CObject(Tool, ')', 50) {
    toolType = t;
    int r = rn2(34);
    if (r < 1) curseIt();
    if (r > 32) blessIt();
}

CTool::CTool(EToolType t, int n, int s) : CObject(Tool, ')', 50) {
    toolType = t;
    int r = rn2(34);
    if (r < 1) curseIt();
    if (r > 32) blessIt();
    number = n;
}

void CTool::polymorph() {
    toolType = (EToolType)0;
    for(int tprob = rnd(1000);
        (tprob -= STools[toolType].prob) > 0;
        toolType = (EToolType)(toolType+1));
    int r = rn2(34);
    if (r < 1) curseIt();
    if (r > 32) blessIt();
    setName("");
}

bool CTool::canEat() {
    return false;
}

string CTool::getDescription(ID user) {
    string s;
    if (getNumber() > 1)
        s = string(number);
    else {
        if (isBlessed() || isCursed() || !isKnown(user))
            s = "a ";
        else s = "an ";
    }
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
    }
    if (isKnown(user)) {
        s += STools[toolType].name;
        if (getNumber() > 1) s += "s";
    } else {
        if (STools[toolType].desc != NULL)
            s += STools[toolType].desc;
        else
            s += STools[toolType].name;
    }
    return s;
}

int CTool::getColor(ID user) {
    return STools[toolType].color;
}

bool CTool::isKnown(ID user) {
    return toolKnown[toolType][user];
}

void CTool::identify(ID user) {
    toolKnown[toolType][user] = true;
    BCidentify(user);
}

/* Keep all tools isolated from each other */
bool CTool::operator== (CTool *t) {
    return false;
}
