/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include <termios.h>
#include <unistd.h>
#include <fstream.h>

#include "vision.h"
#include "color.h"
#include "tcpip.h"
#include "log.h"
#include "string.h"

CVision vision;

/******************************************************************************/
/* Setup the players vision */
CVision::CVision() {
    struct termios term;

    initscr();
    cbreak(); noecho();
    nonl();
    intrflush(stdscr, FALSE);
    keypad(stdscr, TRUE);
    wtop = wmap = wstat = NULL;
    topCurrent = msgCurrent = 0;
    topViewed = msgViewed = -1;
    tcgetattr(STDIN_FILENO, &term);
    term.c_lflag &= ~(ICANON | ECHO | ISIG | 1);
    tcsetattr(STDIN_FILENO, TCSAFLUSH, &term);
//    fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
}

WINDOW * CVision::colorwin(int nlines, int ncols, int begin_y, int begin_x) {
    WINDOW * temp = newwin(nlines, ncols, begin_y, begin_x);
    if ((useColor = has_colors()) == TRUE) start_color();
    /* Setup the color pairs */
    init_pair(CLR_BLACK, COLOR_BLACK, COLOR_BLACK);
    init_pair(CLR_RED, COLOR_RED, COLOR_BLACK);
    init_pair(CLR_GREEN, COLOR_GREEN, COLOR_BLACK);
    init_pair(CLR_BROWN, COLOR_YELLOW, COLOR_BLACK);
    init_pair(CLR_BLUE, COLOR_BLUE, COLOR_BLACK);
    init_pair(CLR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
    init_pair(CLR_CYAN, COLOR_CYAN, COLOR_BLACK);
    init_pair(CLR_GRAY, COLOR_WHITE, COLOR_BLACK);
    init_pair(NO_COLOR, COLOR_BLACK, COLOR_BLACK);
    init_pair(CLR_ORANGE, COLOR_RED, COLOR_BLACK);
    init_pair(CLR_BRIGHT_GREEN, COLOR_GREEN, COLOR_BLACK);
    init_pair(CLR_YELLOW, COLOR_YELLOW, COLOR_BLACK);
    init_pair(CLR_BRIGHT_BLUE, COLOR_BLUE, COLOR_BLACK);
    init_pair(CLR_BRIGHT_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
    init_pair(CLR_BRIGHT_CYAN, COLOR_CYAN, COLOR_BLACK);
    init_pair(CLR_WHITE, COLOR_WHITE, COLOR_BLACK);
    return temp;
}

void CVision::on() {
    clear();
    if (wtop == NULL)
        wtop = colorwin(3,0,0,0);
    if (wmap == NULL)
        wmap = colorwin(19,0,3,0);
    if (wstat == NULL) {
        wstat = colorwin(3,0,22,0);
        if (wstat == NULL)
            wstat = colorwin(2, 0, 22, 0);
    }
    refresh();
	if (wmap == NULL || wtop == NULL || wstat == NULL) exit(1);

}

/******************************************************************************/
/* Close down the vision system */
CVision::~CVision() {
    refresh();
    endwin();
}

void CVision::off() {
    if (wtop != NULL) delwin(wtop);
    if (wmap != NULL) delwin(wmap);
    if (wstat != NULL) delwin(wstat);
    wtop = wmap = wstat = NULL;
    clear();
    refresh();
}

/******************************************************************************/
/* The functions for deal with the top message window */
void CVision::topline(string s) {
    toplines[topCurrent] = s;
    topViewed = topCurrent;
    mvwaddnstr(wtop, 0, 0, s.getStr(), WIDTH);
    wclrtoeol(wtop);
    topCurrent = (topCurrent + 1) % MAXMSGS;
    wrefresh(wtop);
}

void CVision::msgline(string s) {
    int x, y;
    getyx(wtop, y, x);
    msglines[msgCurrent] = s;
    msgViewed = msgCurrent;
    if (msgViewed == 0)
        mvwaddnstr(wtop, 1, 0, msglines[MAXMSGS-1].getStr(), WIDTH);
    else
        mvwaddnstr(wtop, 1, 0, msglines[msgViewed-1].getStr(), WIDTH);
    wclrtoeol(wtop);
    mvwaddnstr(wtop, 2, 0, s.getStr(), WIDTH);
    wclrtoeol(wtop);
    msgCurrent = (msgCurrent + 1) % MAXMSGS;
    wmove(wtop, y, x);
    wrefresh(wtop);
}

void CVision::lastTopline() {
    if (topViewed < 0) return;
    topViewed--;
    if (topViewed < 0) topViewed = MAXMSGS - 1;
    if (toplines[topViewed].size() == 0) topViewed = topCurrent-1;
    if (topViewed < 0) topViewed = MAXMSGS - 1;
    mvwaddnstr(wtop, 0, 0, toplines[topViewed].getStr(), WIDTH);
    wclrtoeol(wtop);
    wrefresh(wtop);
}

void CVision::lastMsgline() {
    if (msgViewed < 0) return;
    int x, y;
    getyx(wtop, y, x);
    msgViewed--;
    if (msgViewed < 0) msgViewed = MAXMSGS - 1;
    if (msglines[msgViewed].size() == 0) msgViewed = msgCurrent-1;
    if (msgViewed < 0) msgViewed = MAXMSGS - 1;
    if (msgViewed == 0) {
        if (msglines[MAXMSGS-1].size() == 0)
            mvwaddnstr(wtop, 1, 0, msglines[msgCurrent].getStr(), WIDTH);
        else
            mvwaddnstr(wtop, 1, 0, msglines[MAXMSGS-1].getStr(), WIDTH);
    } else
        mvwaddnstr(wtop, 1, 0, msglines[msgViewed-1].getStr(), WIDTH);
    wclrtoeol(wtop);
    if (msglines[msgViewed].size() == 0) {
        if (msgCurrent-1 < 0)
            mvwaddnstr(wtop, 2, 0, msglines[MAXMSGS-1].getStr(), WIDTH);
        else
            mvwaddnstr(wtop, 2, 0, msglines[msgCurrent-1].getStr(), WIDTH);
    } else
        mvwaddnstr(wtop, 2, 0, msglines[msgViewed].getStr(), WIDTH);
    wclrtoeol(wtop);
    wmove(wtop, y, x);
    wrefresh(wtop);
}

/******************************************************************************/
/* To display something to the status lines: */
void CVision::statusline1(string s) {
    mvwaddnstr(wstat, 0, 0, s.getStr(), WIDTH);
    wclrtoeol(wstat);
    wrefresh(wstat);
}

void CVision::statusline2(string s) {
    mvwaddnstr(wstat, 1, 0, s.getStr(), WIDTH);
    wclrtoeol(wstat);
    wrefresh(wstat);
}

/******************************************************************************/
/* To display a file: */
#define HLINESIZE	80
#define DELIMETER	'\n'
int CVision::displayFile(string fn) {
    ifstream HelpFile;
    char inbuffer[HLINESIZE];
    int linenum;
    int keyinp;

    HelpFile.open(fn.getStr(), ios::in);
    if (HelpFile.fail())
        return -1;

    linenum = 0;
    wclear(stdscr);
    wrefresh(stdscr);
    while (1) {
        linenum++;
        HelpFile.getline(inbuffer, HLINESIZE, DELIMETER);
        if (HelpFile.fail())
            break;
        mvwaddstr(stdscr, linenum, 0, inbuffer);
        if (linenum == 23) {
            mvwaddstr(stdscr, linenum+1, 0, "-- more --");
            wrefresh(stdscr);
            keyinp = connection.c_listen();
            wclear(stdscr);
            wrefresh(stdscr);
            refresh();
            if (keyinp == 'q') {
                HelpFile.close();
                touchall();
                return 0;
            }
            linenum = 0;
        }
    }
    mvwaddstr(stdscr, linenum+1, 0, "-- end --");
    wrefresh(stdscr);
    inputch();

    HelpFile.close();
    wclear(stdscr);
    wrefresh(stdscr);
    touchall();

    flush();

    return 0;
}

/******************************************************************************/
/* To read input: */
int CVision::inputch() {
    return connection.c_listen();
}

int CVision::inputch(string prompt) {
    topline(prompt);
    return connection.c_listen();
}

string CVision::inputtopstr(string prompt) {
    string temp;
    int charCount = 0;

    topline(prompt);
    wclrtoeol(wtop);
    wrefresh(wtop);

    while (1) {
        int c = connection.c_listen();
        if (c < 0) break;
        if (c == '\r') break;
        if (c == KEY_BACKSPACE) {
            if (charCount < 1) continue;
            charCount--;
            --temp;
            waddch(wtop, 8); waddch(wtop, ' ');
            waddch(wtop, 8); wrefresh(wtop); }
        if (c < 32 || c > 126) continue;
        if (charCount >= (79 - prompt.size())) continue;
        charCount++;
        temp += (char)c;
        waddch(wtop, c);
        wrefresh(wtop);
    }

    return temp;
}

string CVision::inputstr(string prompt) {
    string temp;
    int x, y;
    char buffer[80];
    getyx(stdscr, y, x);
    waddstr(stdscr, prompt.getStr());
    wclrtoeol(stdscr);
    echo();
    wgetnstr(stdscr, buffer, 79 - prompt.size());
    noecho();
    wmove(stdscr, y, x);
    temp = buffer;
    return temp;
}

void CVision::outputstr(string s) {
    /* Can not use waddstr as it ignores cr / lf */
    int y, x;
    for (int i = 0; i < s.size(); i++) {
        if (s[i] == '\r') {
            getyx(stdscr, y, x);
            wmove(stdscr, y+1, 0);
            continue;
        }
        waddch(stdscr, s[i]);
    }
    touchwin(stdscr);
    wrefresh(stdscr);
}

string CVision::inputpasswd(string prompt) {
    int x, y;
    string temp;
    int charCount = 0;

    getyx(stdscr, y, x);
    waddstr(stdscr, prompt.getStr());
    wclrtoeol(stdscr);

    while (1) {
        int c = getch();
        if (c == '\r') break;
        if (c == KEY_BACKSPACE) {
            if (charCount < 1) continue;
            charCount--;
            --temp;
            waddch(stdscr, 8); waddch(stdscr, ' ');
            waddch(stdscr, 8); wrefresh(stdscr); }
        if (c < 32 || c > 126) continue;
        if (charCount >= (79 - prompt.size())) continue;
        charCount++;
        temp += (char)c;
        waddch(stdscr, '*');
        wrefresh(stdscr);
    }
    wmove(stdscr, y, x);
    return temp;
}

/* Return a location based on players keypresses */
CLocation CVision::inputloc(CLocation start) {
    CLocation temp;
    int y = start.y, x = start.x;
    topline("Use movement keys to select location, '.' when ready:");
    while (1) {
        wmove(wmap, y, x);
        wrefresh(wmap);
        int kin = inputch();
        if (kin < 0) break;
        if (kin == '.') break;
        switch (kin) {
            case KEY_UP:    kin = '8'; break;
            case KEY_DOWN:  kin = '2'; break;
            case KEY_RIGHT: kin = '6'; break;
            case KEY_LEFT:  kin = '4'; break;
            case KEY_HOME:  kin = '7'; break;
            case KEY_PPAGE: kin = '9'; break;
            case KEY_B2:    kin = '5'; break;
            case KEY_END:   kin = '1'; break;
            case KEY_NPAGE: kin = '3'; break;
        }
        if (kin < '4' && kin > '0')   y++;
        if (kin > '6' && kin < '9'+1) y--;
        if ((kin-'1')%3 == 0)         x--;
        if ((kin-'0')%3 == 0)         x++;
    }
    temp.set(x, y);
    return temp;
}

/******************************************************************************/
/* Menu system */
static char * prompt = "--more-- ^N ^P ^X: ";

int CVision::menu(char **themenu) {
    /* Find the width of the menu, and the number of entries */
    int mwidth = 0, mentries = 0;
    for (mentries = 0; themenu[mentries] != NULL; mentries++)
        if (mystrlen(themenu[mentries]) > mwidth)
            mwidth = mystrlen(themenu[mentries]);
    mwidth += 2;
    if (mwidth > 60) mwidth = 80;

    /* Create the menu window */
    WINDOW *wmenu;
    if (mentries < 24)
        wmenu = colorwin(mentries, mwidth, 0, 80 - mwidth);
    else
        wmenu = colorwin(24, mwidth, 0, 80 - mwidth);
	
	if (wmenu == NULL) exit(1);  // this is the line of code

    wclear(wmenu);

    /* Write the menu options to the window */
    int item = 0;
    int c = 0;
    for (int i = 0; i < mentries; i++) {
        wattron(wmenu, COLOR_PAIR(CLR_WHITE));
        if (themenu[i][2] != '-') wattron(wmenu, COLOR_PAIR(CLR_BLUE));
        mvwaddstr(wmenu, item++, 1, themenu[i]);
        /* Generic menu that is longer than current screen */
        if (((i%22) == 0 && i < mentries-2 && i != 0) ||
            (i == mentries - 1)) {
            if (i != mentries - 1)
                mvwaddstr(wmenu, item++, 1, prompt);
            wrefresh(wmenu);
            c = connection.c_listen();
            if (c == KEY_ENTER || c == '\n' || c == '\r') break;
            if (c == 24) break;
            if (c == 16) i -= 23;
            if (i < 0) i = -1;
            /* Check for valid input */
            int j;
            for (j = 0; j < item; j++)
                if (((char)mvwinch(wmenu, j, 3) == '-' ||
                     (char)mvwinch(wmenu, j, 3) == '+' ||
                     (char)mvwinch(wmenu, j, 3) == '#') &&
                    ((char)mvwinch(wmenu, j, 1) == c))
                    break;
            wclear(wmenu);
            if (j < item) break;
            if (i == mentries - 1) {
                if (c == ' ') break;
                i = -1;
            }
            item = 0;
        }
    }

    /* Delete and refresh the screen */
    delwin(wmenu);
    touchall();
    if (c == 24) c = -1;
    color(CLR_WHITE);
    return c;
}

string CVision::multiMenu(char **themenu) {
    string tempstr;

    /* Find the width of the menu, and the number of entries */
    int mwidth = 0, mentries = 0;
    for (mentries = 0; themenu[mentries] != NULL; mentries++)
        if (mystrlen(themenu[mentries]) > mwidth)
            mwidth = mystrlen(themenu[mentries]);
    mwidth += 2;
    if (mwidth > 60) mwidth = 80;

    /* Create the menu window */
    WINDOW *wmenu;
    if (mentries < 24)
        wmenu = colorwin(mentries, mwidth, 0, 80 - mwidth);
    else
        wmenu = colorwin(24, mwidth, 0, 80 - mwidth);
	
	if (wmenu == NULL) exit(1);  // this is the line of code

    wclear(wmenu);
    touchwin(wmenu);
    wrefresh(wmenu);

    /* Write the menu options to the window */
  while (1) {
    int item = 0;
    int c = 0;
    int j;
    for (int i = 0; i < mentries; i++) {
        wattron(wmenu, COLOR_PAIR(CLR_WHITE));
        if (themenu[i][2] != '-') wattron(wmenu, COLOR_PAIR(CLR_BLUE));
        mvwaddstr(wmenu, item, 1, themenu[i]);
        if (themenu[i][2] == '-' && tempstr.contains(themenu[i][0]))
            mvwaddch(wmenu, item, 3, '+');
        item++;
        /* Generic menu that is longer than current screen */
        if (((i%22) == 0 && i < mentries-2 && i != 0) ||
            (i == mentries - 1)) {
            if (i != mentries - 1)
                mvwaddstr(wmenu, item++, 1, prompt);
            wrefresh(wmenu);
            c = connection.c_listen();
            if (c == KEY_ENTER || c == '\n' || c == '\r') break;
            if (c == 24) break;
            if (c == 16) i -= 23;
            if (i < 0) i = -1;
            /* Check for valid input */
            for (j = 0; j < item; j++)
                if (((char)mvwinch(wmenu, j, 3) == '-' ||
                     (char)mvwinch(wmenu, j, 3) == '+' ||
                     (char)mvwinch(wmenu, j, 3) == '#') &&
                    ((char)mvwinch(wmenu, j, 1) == c))
                    break;
            wclear(wmenu);
            if (j < item) break;
            if (i == mentries - 1) {
                if (c == ' ') break;
                i = -1;
            }
            item = 0;
        }
    }
    if (c == 24) break;
    if (c == KEY_ENTER || c == '\n' || c == '\r') break;

    /* Change the '-' to a '+' or back */
    tempstr += (char)c;
  }

    /* Delete and refresh the screen */
    delwin(wmenu);
    touchall();
    color(CLR_WHITE);

    return tempstr;
}

/******************************************************************************/
/* Help functions */

static char * clientmenu[] = {
    "Select one item:",
    "",
    "a - Long description of the game and commands.",
    "b - List of game commands.",
    "c - Concise history of Interhack.",
    "d - Concise history of NetHack.",
    "e - Info on a character in the game display.",
    "f - Info on what a given key does.",
    "g - List of game options.",
    "h - Longer explanation of game options.",
    "i - List of extended commands.",
    "j - The Interhack license.",
    "k - Multi-player rules.",
    "(end) ",
    (char *)NULL };

/* Displays the main help menu to the screen, obtains input from the
   keyboard, and displays the apropriate file / information. */
void CVision::clienthelp(CLocation ploc) {
    int kin = menu(clientmenu);

    /* Act on input, calling display_file(), or some other function */
    switch (kin) {
        case 'a': displayFile("./dat/help");     break;
        case 'b': displayFile("./dat/hh");       break;
        case 'c': displayFile("./dat/ihistory"); break;
        case 'd': displayFile("./dat/nhistory"); break;
        case 'e': info_on_char(ploc);            break;
        case 'f': info_on_key();                 break;
        case 'g': displayFile("./dat/gameopt");  break;
        case 'h': displayFile("./dat/opthelp");  break;
        case 'i': displayFile("./dat/cextend");  break;
        case 'j': displayFile("./dat/license");  break;
        case 'k': displayFile("./dat/rules");    break;
    }
}

/* Tells the user what a given key does */
void CVision::info_on_key() {
    topline("TODO: Info on a key pressed");
    (void) connection.c_listen();
}

/* Tells the user what a character in the display, or a word is */
void CVision::info_on_char(CLocation ploc) {
    int kin;
    string buffer;

    topline("Specify unknown object by cursor? [ynq] (q) ");
    do {
        kin = connection.c_listen();
        kin = uppercase(kin);
    } while (kin != 'Y' && kin != 'N' && kin != 'Q' && kin != '\r');

    switch (kin) {
        case 'Q':
        case '\r':
            return;
        case 'Y':
            inputloc(ploc);
            break;
        case 'N':
            buffer = inputtopstr("Specify what? (type the word) ");
            if (buffer.size() < 1)
                topline("");
            keyword_search(buffer);
            break;
    }
}

/* Search the keyword search help file for info on a keyword */
void CVision::keyword_search(string keyword) {
    ifstream KeywordFile;
    char inbuffer[HLINESIZE];
    int linenum = 0;

    log << "FIXME: Proper keyword help search\n";

    if (keyword.size() < 1) {
        topline("Never mind.");
        return;
    }

    KeywordFile.open(KEYWORDFILE, ios::in);
    if (KeywordFile.fail()) {
        log << "ERROR: Help::keyword_search- "
             << "could not open keyword file: " << KEYWORDFILE << '\n';
        return;
    }

    wclear(stdscr);
    wrefresh(stdscr);
    while (1) {
        KeywordFile.getline(inbuffer, HLINESIZE, DELIMETER);
        if (KeywordFile.fail()) {
            topline("I don't have any information on those things.");
            KeywordFile.close();
            touchall();
            return;
        }
        if (inbuffer[0] == '\t' || inbuffer[0] == '\0' ||
            inbuffer[0] == '#') continue;
        if (mystrstr(inbuffer, keyword.getStr()) != NULL) {
            /* Find start of database entry */
            do {
                KeywordFile.getline(inbuffer, HLINESIZE, DELIMETER);
            } while (inbuffer[0] != '\t');
            /* Display the database entry */
            do {
                if (inbuffer[0] == '\t' || inbuffer[0] == '\0') {
                    linenum++;
                    outputstr(inbuffer);
                    outputstr("\r");
                    if (linenum > 22) {
                        outputstr("-- more --");
                        inputch();
                        wclear(stdscr);
                        wmove(stdscr, 0, 0);
                        linenum = 0;
                    }
                }
                KeywordFile.getline(inbuffer, HLINESIZE, DELIMETER);
            } while (inbuffer[0] == '\t' || inbuffer[0] == '\0');
            break;
        }
    }
    outputstr("-- end --");
    inputch();

    KeywordFile.close();
    touchall();
}

static char * servermenu[] = {
    "Select one item:",
    "",
    "Information:",
    "l - Display Level",
    "u - User List",
    "v - Version",
    "Server Control:",
    "k - Kill Player",
    "Server Modes:",
    "& - Demon Mode",
    "i - Interactive Mode",
    "w - Wizard Mode",
    "Other commands:",
    "# - Extended Commands",
    "(end) ",
    (char *)NULL };

int CVision::serverhelp() {
    return menu(servermenu);
}

/******************************************************************************/

void CVision::options() {
}
