/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#ifndef _vision_h_
#define _vision_h_

#include "string.h"
#include "location.h"
#include "color.h"

#include <curses.h>

class CVision {
public:
    /* General vision on / off control */
    CVision();
    ~CVision();
    void on();
    void off();

    /* These are for accessing the map window */
    void mapxy(int x, int y, chtype c) { mvwaddch(wmap, y, x, c); };
    void playerxy(int x, int y, chtype c) {
        mvwaddch(wmap, y, x, c); wmove(wmap, y, x); wrefresh(wmap); };
    void color(int c) {
        if (useColor) {
            if (c == CLR_YELLOW || c == CLR_BRIGHT_GREEN ||
                c == CLR_BRIGHT_BLUE || c == CLR_BRIGHT_MAGENTA ||
                c == CLR_BRIGHT_CYAN)
                wattron(wmap, A_BOLD);
            else
                wattroff(wmap, A_BOLD);
            wattron(wmap, COLOR_PAIR(c));
        }
    };
    void bold() { wattron(wmap, A_BOLD); };
    void dim() { wattron(wmap, A_DIM); };
    void reverse() { wattron(wmap, A_REVERSE); };
    void blink() { wattron(wmap, A_BLINK); };
    void underline() { wattron(wmap, A_UNDERLINE); };
    void normal() { wattrset(wmap, 0); };

    /* These are for accessing the top message lines */
    void topline(string);
    void msgline(string);
    void lastTopline();
    void lastMsgline();

    /* Some input functions */
    int inputch();
    int inputch(string);
    string inputtopstr(string);
    string inputstr(string);
    string inputpasswd(string);
    void outputstr(string);
    CLocation inputloc(CLocation);

    /* These are for accessing the bottom status lines */
    void statusline1(string);
    void statusline2(string);

    /* These are the flush functions to do the actual update */
    void flush() { wrefresh(wtop); wrefresh(wmap); wrefresh(wstat); };
    void flushtop() { wrefresh(wtop); };
    void flushmap() { wrefresh(wmap); };
    void flushstat() { wrefresh(wstat); };

    /* Some cleaning functions: */
    void clean() { wclear(wtop); wclear(wmap); wclear(wstat); };
    void cleantop() { wclear(wtop); };
    void cleanmap() { wclear(wmap); };
    void cleanstat() { wclear(wstat); };

    /* Touch all the main screens */
    void touchall() {
        touchwin(wtop);  wrefresh(wtop);
        touchwin(wstat); wrefresh(wstat);
        touchwin(wmap);  wrefresh(wmap); };
    void touchtop() {
        touchwin(wtop);  wrefresh(wtop); };


    /* Functions related to displaying files */
    int displayFile(string);

    /* Finally we have some menu functions */
    int menu(char **);
    string multiMenu(char **);

    /* Conversion functions */
    char uppercase(char c) {
        if (c >= 'a' && c <= 'z') return (c + ('A'-'a')); return c; };

    /* Help functions */
    void clienthelp(CLocation);
    int serverhelp();
    void info_on_key();
    void info_on_char(CLocation);
    void keyword_search(string);
    void options();

    /* Toggle functions */
    void Keypad(bool f) { keypad(stdscr, f); };
    void Colour(bool f) {
        if (has_colors() == false) return;
        useColor = f; };
private:
    WINDOW * wtop, * wmap, * wstat;
    string toplines[MAXMSGS];
    string msglines[MAXMSGS];
    int topCurrent, msgCurrent;
    int topViewed, msgViewed;
    bool useColor;
    WINDOW * colorwin(int, int, int, int);
protected:
};

extern CVision vision;

#endif
