/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "object.h"

/* Wands are nethack 3.3.0 compliant now */

struct S_Wands {
    char *name;
    char *desc;
    int prob;
    int cost;
    int magic;
    int direction;
    int metal;
    int color;
} SWands[] = {

#define IRON		1
#define METAL		2
#define WOOD		3
#define COPPER		4
#define PLATINUM	5
#define SILVER		6
#define GLASS		7
#define MINERAL		8

#define IMMEDIATE	1
#define RAY		2
#define NODIR		3

#define WAND(name,typ,prob,cost,mgc,dir,metal,color) \
	{name,typ,prob,cost,mgc,dir,metal,color}

WAND("light",          "glass",    95, 100, 1, NODIR,     GLASS,    HI_GLASS),
WAND("secret door detection", "balsa",
				   50, 150, 1, NODIR,	  WOOD,     HI_WOOD),
WAND("enlightenment",  "crystal",  15, 150, 1, NODIR,     GLASS,    HI_GLASS),
WAND("create monster", "maple",    45, 200, 1, NODIR,     WOOD,     HI_WOOD),
WAND("wishing",        "pine",      5, 500, 1, NODIR,     WOOD,     HI_WOOD),
WAND("nothing",        "oak",      25, 100, 0, IMMEDIATE, WOOD,     HI_WOOD),
WAND("striking",       "ebony",    75, 150, 1, IMMEDIATE, WOOD,     HI_WOOD),
WAND("make invisible", "marble",   45, 150, 1, IMMEDIATE, MINERAL,  HI_MINERAL),
WAND("slow monster",   "tin",      50, 150, 1, IMMEDIATE, METAL,    HI_METAL),
WAND("speed monster",  "brass",    50, 150, 1, IMMEDIATE, COPPER,   HI_COPPER),
WAND("undead turning", "copper",   50, 150, 1, IMMEDIATE, COPPER,   HI_COPPER),
WAND("polymorph",      "silver",   45, 200, 1, IMMEDIATE, SILVER,   HI_SILVER),
WAND("cancellation",   "platinum", 45, 200, 1, IMMEDIATE, PLATINUM, CLR_WHITE),
WAND("teleportation",  "iridium",  45, 200, 1, IMMEDIATE, METAL,    CLR_BRIGHT_CYAN),
WAND("opening",        "zinc",     25, 150, 1, IMMEDIATE, METAL,    HI_METAL),
WAND("locking",        "aluminum", 25, 150, 1, IMMEDIATE, METAL,    HI_METAL),
WAND("probing",        "uranium",  30, 150, 1, IMMEDIATE, METAL,    HI_METAL),
WAND("digging",        "iron",     55, 150, 1, RAY,       IRON,     HI_METAL),
WAND("magic missile",  "steel",    50, 150, 1, RAY,       IRON,     HI_METAL),
WAND("fire",           "hexagonal",40, 175, 1, RAY,       IRON,     HI_METAL),
WAND("cold",           "short",    40, 175, 1, RAY,       IRON,     HI_METAL),
WAND("sleep",          "runed",    50, 175, 1, RAY,       IRON,     HI_METAL),
WAND("death",          "long",      5, 500, 1, RAY,       IRON,     HI_METAL),
WAND("lightning",      "curved",   40, 175, 1, RAY,       IRON,     HI_METAL),
WAND((char *)0,        "forked",    0, 150, 1, 0,         WOOD,     HI_WOOD),
WAND((char *)0,        "spiked",    0, 150, 1, 0,         IRON,     HI_METAL),
WAND((char *)0,        "jeweled",   0, 150, 1, 0,         IRON,     HI_MINERAL)
#undef WAND
};

#define WANDDESCS	27

static bool wandKnown[WANDDESCS][MAXPLAYERS];
static char SWandRandDesc[WANDDESCS][MAXPLAYERS];

void initWands() {
    for (int x = 0; x < MAXPLAYERS; x++)
        quitWands(x);
}

void loadWands(ID user) {
    /* TODO: load wand details for a user */
}

void saveWands(ID user) {
    /* TODO: save wand details for a user */
    quitWands(user);
}

void quitWands(ID user) {
    for (int y = 0; y < WANDDESCS; y++) {
        wandKnown[y][user] = false;
        SWandRandDesc[y][user] = y;
    }
    for (int y = 0; y < WANDDESCS; y++) {
        int r = rn2(WANDDESCS);
        char temp = SWandRandDesc[y][user];
        SWandRandDesc[y][user] = SWandRandDesc[r][user];
        SWandRandDesc[r][user] = temp;
    }
}

CWand::CWand() : CObject(Wand, '\\', 50) {
    polymorph();
}

CWand::CWand(EWandType t) : CObject(Wand, '\\', 50) {
    wandType = t;
    charge = 0;
}

void CWand::polymorph() {
    wandType = (EWandType)0;
    for(int tprob = rnd(1000);
        (tprob -= SWands[wandType].prob) > 0;
        wandType = (EWandType)(wandType+1));
    charge = 0;
    setName("");
}

bool CWand::canEat() {
    return false;
}

string CWand::getDescription(ID user) {
    string s;
    if (getNumber() > 1)
        s = string(number);
    else {
        if (isBlessed() || isCursed() || !isKnown(user))
            s = "a ";
        else s = "an ";
    }
    if (BCisKnown(user)) {
        if (isBlessed()) s += "blessed ";
        else if (isCursed()) s += "cursed ";
        else s += "uncursed ";
    }
    if (isKnown(user)) {
        s += "wand";
        if (getNumber() > 1) s += "s";
        s += " of ";
        s += SWands[wandType].name;
        s += " (_:";
        s += charge; s += ')';
    } else {
        s += SWands[SWandRandDesc[wandType][user]].desc;
        s += " wand";
        if (getNumber() > 1) s += "s";
    }
    return s;
}

int CWand::getColor(ID user) {
    return SWands[wandType].color;
}

bool CWand::isKnown(ID user) {
    return wandKnown[wandType][user];
}

void CWand::identify(ID user) {
    wandKnown[wandType][user] = true;
    BCidentify(user);
}

/* Keep all wands isolated from each other */
bool CWand::operator== (CWand *w) {
    return false;
}
