/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "player.h"
#include "log.h"
#include "plist.h"

int CPlayer::wear() {
    ID tid;
    CObject * tobj;
    int loc;
    string tstr;

    *sock >> tid;
    tobj = pack.find(tid);
    if (tobj == NULL) {
        log << "WARNING: player wear invalid object" << '\n';
        return -1;
    }
    if (tobj == boots || tobj == gloves || tobj == helmet ||
        tobj == shirt || tobj == armor || tobj == cloak ||
        tobj == shield) {
        log << "WARNING: player wear object already being worn" << '\n';
        return -1;
    }
    if (tobj->getObjectType() != Armor) {
        log << "WARNING: player wear non-armor object" << '\n';
        return -1;
    }

    CArmor * tptr = (CArmor *)tobj;
    EArmorType a = tptr->getArmorType();
    if (((a == ShirtHawaiian || a == ShirtT) && shirt != NULL) ||
        ((a >= DragonGray && a <= ArmLeatherJacket) && armor != NULL) ||
        ((a >= CloakMummyWrapping && a <= CloakDisplacement) && cloak != NULL) ||
        ((a >= HelmElvenLeather && a <= HelmTelepathy) && helmet != NULL) ||
        ((a >= GloveLeather && a <= GloveDexterity) && gloves != NULL) ||
        ((a >= ShieldSmall && a <= ShieldReflection) && shield != NULL) ||
        ((a >= BootsLow && a <= BootsLevitation) && boots != NULL)) {
        log << "WARNING: player wearing armor already worn, or unknown armor type" << '\n';
        return -1;
    }

    /* Check sureal time! */
    if (players.doMove(id) == false)
        return 0;

    moveCount++;

    /* Wear the armor */
    if (a == ShirtHawaiian || a == ShirtT) {
        shirt = tptr; loc = BODY_SHIRT;
        if (armor != NULL || cloak != NULL) {
            log << "WARNING: player wear armor under something bigger" << '\n';
            return -1;
        }
        attr.subAC(shirt->getAC());
    } else if (a >= DragonGray && a <= ArmLeatherJacket) {
        armor = tptr; loc = BODY_ARMOR;
        if (cloak != NULL) {
            log << "WARNING: player wear armor under something bigger" << '\n';
            return -1;
        }
        attr.subAC(armor->getAC());
    } else if (a >= CloakMummyWrapping && a <= CloakDisplacement) {
        cloak = tptr; loc = BODY_CLOAK;
        attr.subAC(cloak->getAC());
    } else if (a >= HelmElvenLeather && a <= HelmTelepathy) {
        helmet = tptr; loc = BODY_HELMET;
        attr.subAC(helmet->getAC());
    } else if (a >= GloveLeather && a <= GloveDexterity) {
        gloves = tptr; loc = BODY_GLOVES;
        attr.subAC(gloves->getAC());
    } else if (a >= ShieldSmall && a <= ShieldReflection) {
        shield = tptr; loc = BODY_SHIELD;
        attr.subAC(shield->getAC());
    } else if (a >= BootsLow && a <= BootsLevitation) {
        boots = tptr; loc = BODY_BOOTS;
        attr.subAC(boots->getAC());
    }

    /* Tell the client, sending new object description */
    tstr = tptr->getDescription(id);
    tstr += " (being worn)";
    *sock << p_wear << tptr->id << loc << tstr << p_end;
    *sock << p_ac << attr.getac() << p_end;

    return 0;
}

int CPlayer::remove() {
    ID tid;
    CObject * tobj;
    string tstr;

    *sock >> tid;
    tobj = pack.find(tid);
    if (tobj == NULL) {
        log << "WARNING: player remove non-existing object" << '\n';
        return -1;
    }

    /* Check the object is not cursed - not actually counted as a move */
    if (tobj->isCursed() && (boots == tobj || gloves == tobj ||
        helmet == tobj || shirt == tobj || armor == tobj ||
        cloak == tobj)) {
        tobj->BCidentify(id);
        tstr = tobj->getDescription(id);
        if (amulet == tobj) tstr += " (being worn)";
        if (left_ring == tobj) tstr += " (on left hand)";
        if (right_ring == tobj) tstr += " (on right hand)";
        *sock << p_cursed << tobj->id << tstr << p_end;
        return 0;
    }

    /* Check sureal time! */
    if (players.doMove(id) == false)
        return 0;

    moveCount++;

    tstr = tobj->getDescription(id);

    if (boots == tobj) {
        attr.addAC(boots->getAC());
        boots = NULL;
        *sock << p_remove << BODY_BOOTS << tstr << p_end;
    } else if (gloves == tobj) {
        attr.addAC(gloves->getAC());
        gloves = NULL;
        *sock << p_remove << BODY_GLOVES << tstr << p_end;
    } else if (helmet == tobj) {
        attr.addAC(helmet->getAC());
        helmet = NULL;
        *sock << p_remove << BODY_HELMET << tstr << p_end;
    } else if (shield == tobj) {
        attr.addAC(shield->getAC());
        shield = NULL;
        *sock << p_remove << BODY_SHIELD << tstr << p_end;
    } else if (shirt == tobj && armor == NULL && cloak == NULL) {
        attr.addAC(shirt->getAC());
        shirt = NULL;
        *sock << p_remove << BODY_SHIRT << tstr << p_end;
    } else if (armor == tobj && cloak == NULL) {
        attr.addAC(armor->getAC());
        armor = NULL;
        *sock << p_remove << BODY_ARMOR << tstr << p_end;
    } else if (cloak == tobj) {
        attr.addAC(cloak->getAC());
        cloak = NULL;
        *sock << p_remove << BODY_CLOAK << tstr << p_end;
    } else {
        log << "WARNING: player remove something not puton" << '\n';
        log << "         or removing armor under other armor" << '\n';
        return -1;
    }
    *sock << p_ac << attr.getac() << p_end;

    return 0;
}
