/* Interhack   Multi-player Rougelike game (based on Nethack)
   Copyright (C) 1996-2000  Damian Bentley

   See license.txt for details. If you do not have a copy of the license,
   please email bendchon@mehta.anu.edu.au */

#include "player.h"
#include "log.h"
#include "plist.h"

/* Wield an object as a weapon */
int CPlayer::wield() {
    ID tid;
    CObject * tobj;
    string tstr, ostr;

    *sock >> tid;
    tobj = pack.find(tid);
    if (tobj == NULL) {
        log << "WARNING: player wield invalid object" << '\n';
        return -1;
    }
    /* check object is not currently worn */
    if (tobj == amulet || tobj == left_ring || tobj == right_ring ||
        tobj == boots || tobj == gloves || tobj == helmet ||
        tobj == shirt || tobj == armor || tobj == cloak ||
        tobj == shield || tobj == blindfold) {
        log << "WARNING: player wield object being worn" << '\n';
        return -1;
    }
    if (tobj == pri_weapon || tobj == sec_weapon) {
        log << "WARNING: player wield already wielded object" << '\n';
        return -1;
    }

    /* Check sureal time! */
    if (players.doMove(id) == false)
        return 0;

    moveCount++;

    /* Now we can wield the object, no matter what it is. */
    if (pri_weapon != NULL) ostr = pri_weapon->getDescription(id);
    else ostr = "null";
    pri_weapon = tobj;
    tstr = tobj->getDescription(id);
    tstr += " (weapon in hand)";
    *sock << p_wield << tobj->id << ostr << tstr << p_end;
    return 0;
}

/* Exchange primary and secondary weapons */
int CPlayer::exchange() {
    CObject * tobj;
    int weps = 0;
    string tstr;

    /* Exchange the weapons */
    if (pri_weapon == NULL && sec_weapon == NULL) return 0;
    tobj = pri_weapon;
    pri_weapon = sec_weapon;
    sec_weapon = tobj;

    /* Tell how many bits of info are coming: 1 = primary, 2 = secondary,
       3 = both */
    *sock << p_exchange;
    if (pri_weapon != NULL) weps += 1;
    if (sec_weapon != NULL) weps += 2;
    *sock << weps;

    /* Send the weapons and their new descriptions */
    if (pri_weapon != NULL) {
        tstr = pri_weapon->getDescription(id);
        tstr += " (weapon in hand)";
        *sock << pri_weapon->id << tstr;
    }
    if (sec_weapon != NULL) {
        tstr = sec_weapon->getDescription(id);
        tstr += " (secondary weapon)";
        *sock << sec_weapon->id << tstr;
    }
    *sock << p_end;

    return 0;
}
